// 
// ClassID.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Text;

using Poi.Net.Util;

namespace Poi.Net.HPSF
{
	
	/// <summary> Represents a class ID (16 bytes). Unlike other little-endian
	/// type the ClassID is not just 16 bytes stored in the wrong
	/// order. Instead, it is a double word (4 bytes) followed by two
	/// words (2 bytes each) followed by 8 bytes.
	/// 
	/// </summary>

	public class ClassID
	{
		/// <summary> Gets the bytes making out the class ID. They are returned in
		/// correct order, i.e. big-endian.
		/// 
		/// </summary>
		/// <returns> the bytes making out the class ID.
		/// </returns>

		virtual public byte[] Bytes {
			get { return bytes; }			
		}
		
		/// <summary> The bytes making out the class ID in correct order,
		/// i.e. big-endian.
		/// </summary>
		protected internal byte[] bytes;
		
		/// <summary>  Creates a ClassID and reads its value from a byte
		/// array.
		/// </summary>
		/// <param name="src">The byte array to read from.
		/// </param>
		/// <param name="offset">The offset of the first byte to read.
		/// </param>
		public ClassID(byte[] src, int offset)
		{
			Read(src, offset);
		}
		
		
		/// <summary>  Creates a ClassID and initializes its value with
		/// 0x00 bytes.
		/// </summary>
		public ClassID()
		{
			bytes = new byte[LENGTH];
			for (int i = 0; i < LENGTH; i++)
				bytes[i] = 0x00;
		}
		
		
		
		/// <summary>The number of bytes occupied by this object in the byte
		/// stream. 
		/// </summary>
		public const int LENGTH = 16;
		
		/// <returns> The number of bytes occupied by this object in the byte
		/// stream.
		/// </returns>
		/// 

		public virtual int Length {
			get { return LENGTH; }
		}
		
		
		
		/// <summary> Reads the class ID's value from a byte array by turning
		/// little-endian into big-endian.
		/// 
		/// </summary>
		/// <param name="src">The byte array to read from
		/// 
		/// </param>
		/// <param name="offset">The offset within the <var>src</var> byte array
		/// 
		/// </param>
		/// <returns> A byte array containing the class ID.
		/// </returns>

		public virtual byte[] Read(byte[] src, int offset)
		{
			bytes = new byte[16];
			
			/* Read double word. */
			bytes[0] = src[3 + offset];
			bytes[1] = src[2 + offset];
			bytes[2] = src[1 + offset];
			bytes[3] = src[0 + offset];
			
			/* Read first word. */
			bytes[4] = src[5 + offset];
			bytes[5] = src[4 + offset];
			
			/* Read second word. */
			bytes[6] = src[7 + offset];
			bytes[7] = src[6 + offset];
			
			/* Read 8 bytes. */
			for (int i = 8; i < 16; i++)
				bytes[i] = src[i + offset];
			
			return bytes;
		}
		
		
		
		/// <summary> Writes the class ID to a byte array in the
		/// little-endian.
		/// 
		/// </summary>
		/// <param name="dst">The byte array to write to.
		/// 
		/// </param>
		/// <param name="offset">The offset within the <var>dst</var> byte array.
		/// 
		/// </param>
		/// <exception cref=""> ArrayStoreException if there is not enough room for the class 
		/// ID 16 bytes in the byte array after the <var>offset</var> position.
		/// </exception>
		public virtual void Write(byte[] dst, int offset)
		{
			/* Check array size: */
			if (dst.Length < 16)
				throw new ArrayTypeMismatchException(
					"Destination byte[] must have room for " + 
					"at least 16 bytes, but has a length of only " + 
					dst.Length + ".");

			/* Write double word. */
			dst[0 + offset] = bytes[3];
			dst[1 + offset] = bytes[2];
			dst[2 + offset] = bytes[1];
			dst[3 + offset] = bytes[0];
			
			/* Write first word. */
			dst[4 + offset] = bytes[5];
			dst[5 + offset] = bytes[4];
			
			/* Write second word. */
			dst[6 + offset] = bytes[7];
			dst[7 + offset] = bytes[6];
			
			/* Write 8 bytes. */
			for (int i = 8; i < 16; i++)
				dst[i + offset] = bytes[i];
		}
		
		
		
		/// <summary> Checks whether this <code>ClassID</code> is equal to another
		/// object.
		/// 
		/// </summary>
		/// <param name="o">the object to compare this <code>PropertySet</code> with
		/// </param>
		/// <returns> <code>true</code> if the objects are equal, else
		/// <code>false</code>.
		/// </returns>
		public  override bool Equals(object o)
		{
			if (o == null || !(o is ClassID))
				return false;
			
			ClassID cid = (ClassID) o;
			if (bytes.Length != cid.Bytes.Length)
				return false;
			for (int i = 0; i < Bytes.Length; i++)
				if (Bytes[i] != cid.Bytes[i])
					return false;
			return true;
		}
		/// <summary> Returns a human readable representation of the Class ID
		/// in standard format <code>"{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}"</code>
		/// </summary>
		/// <returns> String representation of the Class ID represented
		/// by this object.
		/// </returns>
		public override string ToString()
		{
			StringBuilder sbClassId = new StringBuilder(38);
			sbClassId.Append('{');
			for (int i = 0; i < 16; i++)
			{
				sbClassId.Append(HexDump.ToHex(bytes[i]));
				if (i == 3 || i == 5 || i == 7 || i == 9)
				{
					sbClassId.Append('-');
				}
			}
			sbClassId.Append('}');
			return sbClassId.ToString();
		}
	}
}