// 
// DocumentSummaryInformation.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Author:
//   Rainer Klute (klute@rainer-klute.de)
//   Drew Varner (Drew.Varner closeTo sc.edu)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using Poi.Net.HPSF.Wellknown;
namespace Poi.Net.HPSF
{
	
	/// <summary> Convenience class representing a DocumentSummary Information stream in a
	/// Microsoft Office document.
	/// 
	/// </summary>
	/// <seealso cref="SummaryInformation">
	/// </seealso>
	public class DocumentSummaryInformation:SpecialPropertySet
	{
		/// <summary> Returns the stream's category (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The category value
		/// </returns>
		virtual public string Category {
			get {
				return (string) GetProperty(PropertyIDMap.PID_CATEGORY);
			}
			
		}
		/// <summary> Returns the stream's presentation format (or
		/// <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The presentationFormat value
		/// </returns>
		virtual public string PresentationFormat {
			get {
				return (string) GetProperty(PropertyIDMap.PID_PRESFORMAT);
			}
			
		}
		/// <summary> Returns the stream's byte count or 0 if the 
		/// DocumentSummaryInformation} does not contain a byte count.
		/// 
		/// </summary>
		/// <returns> The byteCount value
		/// </returns>
		virtual public int ByteCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_BYTECOUNT);
			}
			
		}
		/// <summary> Returns the stream's line count or 0 if the 
		/// DocumentSummaryInformation} does not contain a line count.
		/// 
		/// </summary>
		/// <returns> The lineCount value
		/// </returns>
		virtual public int LineCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_LINECOUNT);
			}
			
		}
		/// <summary> Returns the stream's par count or 0 if the 
		/// DocumentSummaryInformation} does not contain a par count.
		/// 
		/// </summary>
		/// <returns> The parCount value
		/// </returns>
		virtual public int ParCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_PARCOUNT);
			}
			
		}
		/// <summary> Returns the stream's slide count or 0 if the 
		/// DocumentSummaryInformation} does not contain a slide count.
		/// 
		/// </summary>
		/// <returns> The slideCount value
		/// </returns>
		virtual public int SlideCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_SLIDECOUNT);
			}
			
		}
		/// <summary> Returns the stream's note count or 0 if the 
		/// DocumentSummaryInformation} does not contain a note count.
		/// 
		/// </summary>
		/// <returns> The noteCount value
		/// </returns>
		virtual public int NoteCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_NOTECOUNT);
			}
			
		}
		/// <summary> Returns the stream's hidden count or 0 if the 
		/// DocumentSummaryInformation} does not contain a hidden
		/// count.
		/// 
		/// </summary>
		/// <returns> The hiddenCount value
		/// </returns>
		virtual public int HiddenCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_HIDDENCOUNT);
			}
			
		}
		/// <summary> Returns the stream's mmclip count or 0 if the 
		/// DocumentSummaryInformation} does not contain a mmclip
		/// count.
		/// 
		/// </summary>
		/// <returns> The mMClipCount value
		/// </returns>
		virtual public int MMClipCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_MMCLIPCOUNT);
			}
			
		}
		/// <summary> Returns <code>true</code> when scaling of the thumbnail is
		/// desired, <code>false</code> if cropping is desired.
		/// 
		/// </summary>
		/// <returns> The scale value
		/// </returns>
		virtual public bool Scale {
			get {
				return GetPropertyBooleanValue(PropertyIDMap.PID_SCALE);
			}
			
		}
		/// <summary> Returns the stream's heading pair (or <code>null</code>)
		/// <strong>when this method is implemented. Please note that the
		/// return type is likely to change!</strong>
		/// 
		/// </summary>
		/// <returns> The headingPair value
		/// </returns>
		virtual public byte[] HeadingPair {
			get {
				if (true)
					throw new NotSupportedException("FIXME");
				// return (byte[]) GetProperty(PropertyIDMap.PID_HEADINGPAIR);
			}
			
		}
		/// <summary> Returns the stream's doc parts (or <code>null</code>)
		/// <strong>when this method is implemented. Please note that the
		/// return type is likely to change!</strong>
		/// 
		/// </summary>
		/// <returns> The docparts value
		/// </returns>
		virtual public byte[] Docparts {
			get {
				if (true)
					throw new NotSupportedException("FIXME");
				// return (byte[]) GetProperty(PropertyIDMap.PID_DOCPARTS);
			}
			
		}
		/// <summary> Returns the stream's manager (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The manager value
		/// </returns>
		virtual public string Manager {
			get {
				return (string) GetProperty(PropertyIDMap.PID_MANAGER);
			}
			
		}
		/// <summary> Returns the stream's company (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The company value
		/// </returns>
		virtual public string Company {
			get {
				return (string) GetProperty(PropertyIDMap.PID_COMPANY);
			}
			
		}
		/// <summary> Returns <code>true</code> if the custom links are hampered
		/// by excessive noise, for all applications. 
		/// 
		/// <strong>FIXME:</strong> Explain this some more! I (Rainer)
		/// don't understand it.
		/// 
		/// </summary>
		/// <returns> The linksDirty value
		/// </returns>
		virtual public bool LinksDirty {
			get {
				return GetPropertyBooleanValue(PropertyIDMap.PID_LINKSDIRTY);
			}
			
		}
		
		/// <summary> The document name a document summary information stream
		/// usually has in a POIFS filesystem.
		/// </summary>
		public const string DEFAULT_STREAM_NAME = "\x0005DocumentSummaryInformation";
		
		
		
		/// <summary> Creates a DocumentSummaryInformation from a given
		///  PropertySet}.
		/// 
		/// </summary>
		/// <param name="ps">A property set which should be created from a
		/// document summary information stream.
		/// </param>
		/// <throws>  UnexpectedPropertySetTypeException if <var>ps</var> </throws>
		/// <summary> does not contain a document summary information stream.
		/// </summary>
		public DocumentSummaryInformation(PropertySet ps):base(ps)
		{
			if (!DocumentSummaryInformation)				
				throw new UnexpectedPropertySetTypeException("Not a " +
													GetType().FullName);
		}
	}
}