// 
// Property.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//   Drew Varner (Drew.Varner InAndAround sc.edu)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


using System;
using System.Text;
using System.Collections;

using Poi.Net.Util;

namespace Poi.Net.HPSF
{
	
	/// <summary> A property in a  Section} of a  PropertySet}.
	/// 
	/// The property's <strong>ID</strong> gives the property a meaning
	/// in the context of its  Section}. Each  Section} spans
	/// its own name space of property IDs.
	/// 
	/// The property's <strong>type</strong> determines how its
	/// <strong>value </strong> is interpreted. For example, if the type is
	///  Variant#VT_LPSTR} (byte string), the value consists of a
	/// DWord telling how many bytes the string contains. The bytes follow
	/// immediately, including any null bytes that terminate the
	/// string. The type  Variant#VT_I4} denotes a four-byte integer
	/// value,  Variant#VT_FILETIME} some date and time (of a
	/// file).
	/// 
	/// <strong>FIXME:</strong> Reading is not implemented for all
	///  Variant} types yet. Feel free to submit error reports or
	/// patches for the types you need.
	/// 
	/// </summary>
	/// <seealso cref="Section">
	/// </seealso>
	/// <seealso cref="Variant">
	/// </seealso>
	/// <version>  $Id: Property.java,v 1.10.6.1 2004/02/22 11:54:45 glens Exp $
	/// </version>
	/// <since> 2002-02-09
	/// </since>
	public class Property
	{
		/// <summary> Returns the property's ID.
		/// 
		/// </summary>
		/// <returns> The ID value
		/// </returns>
		virtual public int ID {
			get {
				return id;
			}
			
		}
		/// <summary> Returns the property's type.
		/// 
		/// </summary>
		/// <returns> The type value
		/// </returns>
		virtual public long Type {
			get {
				return type;
			}
			
		}
		/// <summary> Returns the property's value.
		/// 
		/// </summary>
		/// <returns> The property's value
		/// </returns>
		virtual public object Value {
			get {
				return _value;
			}
			
		}
		
		/* Codepage 1200 denotes Unicode. */
		private static int CP_UNICODE = 1200;
		
		private int id;
		
		
		
		private long type;
		
		
		
		private object _value;
		
		
		
		/// <summary> Creates a  Property} instance by reading its bytes
		/// from the property set stream.
		/// 
		/// </summary>
		/// <param name="id">The property's ID.
		/// </param>
		/// <param name="src">The bytes the property set stream consists of.
		/// </param>
		/// <param name="offset">The property's type/value pair's offset in the
		/// section.
		/// </param>
		/// <param name="length">The property's type/value pair's length in bytes.
		/// </param>
		/// <param name="codepage">The section's and thus the property's
		/// codepage. It is needed only when reading string values.
		/// </param>
		public Property(int id, byte[] src, long offset, int length, int codepage)
		{
			this.id = id;
			
			/*
			* ID 0 is a special case since it specifies a dictionary of
			* property IDs and property names.
			*/
			if (id == 0)
			{
				_value = ReadDictionary(src, offset, length, codepage);
				return ;
			}
			
			int o = (int) offset;
			type = LittleEndian.GetUInt(src, o);
			o += LittleEndian.INT_SIZE;
			
			try
			{
				_value = TypeReader.Read(src, o, length, (int) type, codepage);
			}
			
			catch (Exception t)
			{
				//HelperClass
				Console.Error.Write(t.StackTrace);
				Console.Error.Flush();
				_value = "*** null ***";
			}
		}
		
		
		
		/// <summary> Reads a dictionary.
		/// 
		/// </summary>
		/// <param name="src">The byte array containing the bytes making out the
		/// dictionary.
		/// </param>
		/// <param name="offset">At this offset within <var>src</var> the
		/// dictionary starts.
		/// </param>
		/// <param name="length">The dictionary contains at most this many bytes.
		/// </param>
		/// <param name="codepage">The codepage of the string values.
		/// </param>
		/// <returns> The dictonary
		/// </returns>
		protected internal virtual IDictionary ReadDictionary(byte[] src, long offset, int length, int codepage)
		{
			/* Check whether "offset" points into the "src" array". */
			if (offset < 0 || offset > src.Length)
				throw new HPSFRuntimeException("Illegal offset " + offset + " while HPSF stream contains " + length + " bytes.");
			int o = (int) offset;
			
			/*
			* Read the number of dictionary entries.
			*/
			
			long nrEntries = LittleEndian.GetUInt(src, o);
			o += LittleEndian.INT_SIZE;
			
			
			
			IDictionary m = new Hashtable((int) nrEntries, (float) 1.0);
			for (int i = 0; i < nrEntries; i++)
			{
				/* The key. */
				
				long id = (long) LittleEndian.GetUInt(src, o);
				o += LittleEndian.INT_SIZE;
				
				/* The value (a string). The length is the either the
				* number of characters if the character set is Unicode or
				* else the number of bytes. The length includes
				* terminating 0x00 bytes which we have to strip off to
				* create a Java string. */
				long sLength = LittleEndian.GetUInt(src, o);
				o += LittleEndian.INT_SIZE;
				
				/* Read the bytes or characters depending on whether the
				* character set is Unicode or not. */
				StringBuilder b = new StringBuilder((int) sLength);
				for (int j = 0; j < sLength; j++)
				{
					if(codepage == CP_UNICODE)
					{
						
						int i1 = o + (j * 2);
						
							int i2 = i1 + 1;
						b.Append((char) ((src[i2] << 8) + src[i1]));
					}
					else
							b.Append((char) src[o + j]);
				}
				/* Strip 0x00 characters from the end of the string: */
				while (b[b.Length - 1] == 0x00)
					b.Length -= 1;
				if (codepage == CP_UNICODE)
				{
					if (sLength % 2 == 1)
						sLength++;
					o = (int) (o + (sLength + sLength));
				}
				else
					o = (int) (o + sLength);
				m[id] = b.ToString();
			}
			return m;
		}
	}
}