// 
// PropertySet.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//   Drew Varner (Drew.Varner hanginIn sc.edu)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


using System;
using System.Collections;
using System.IO;

using Poi.Net.HPSF.Wellknown;
using Poi.Net.PoiFS.FileSystem;
using Poi.Net.Util;

namespace Poi.Net.HPSF
{
	
	/// <summary> Represents a property set in the Horrible Property Set Format
	/// (HPSF). These are usually metadata of a Microsoft Office
	/// document.
	/// 
	/// An application that wants to access these metadata should create
	/// an instance of this class or one of its subclasses by calling the
	/// factory method  PropertySetFactory#create} and then retrieve
	/// the information its needs by calling appropriate methods.
	/// 
	///  PropertySetFactory#create} does its work by calling one
	/// of the constructors  PropertySet#PropertySet(InputStream)} or
	///  PropertySet#PropertySet(byte[])}. If the constructor's
	/// argument is not in the Horrible Property Set Format, i.e. not a
	/// property set stream, or if any other error occurs, an appropriate
	/// exception is thrown.
	/// 
	/// A  PropertySet} has a list of  Section}s, and each
	///  Section} has a  Property} array. Use 
	/// #getSections} to retrieve the  Section}s, then call 
	/// Section#getProperties} for each  Section} to get hold of the
	///  Property} arrays. Since the vast majority of 
	/// PropertySet}s contains only a single  Section}, the
	/// convenience method  #getProperties} returns the properties of
	/// a  PropertySet}'s  Section} (throwing a 
	/// NoSingleSectionException} if the  PropertySet} contains more
	/// (or less) than exactly one  Section}).
	/// 
	/// </summary>
	/// <version>  $Id: PropertySet.java,v 1.9.4.1 2004/02/22 11:54:45 glens Exp $
	/// </version>
	/// <since> 2002-02-09
	/// </since>
	public class PropertySet
	{
		/// <summary> Returns the property set stream's low-level "byte order"
		/// field. It is always <tt>0xFFFE</tt> .
		/// 
		/// </summary>
		/// <returns> The property set stream's low-level "byte order" field.
		/// </returns>
		virtual public int ByteOrder {
			get {
				return byteOrder;
			}
			
		}
		/// <summary> Returns the property set stream's low-level "format"
		/// field. It is always <tt>0x0000</tt> .
		/// 
		/// </summary>
		/// <returns> The property set stream's low-level "format" field.
		/// </returns>
		virtual public int Format {
			get {
				return format;
			}
			
		}
		/// <summary> Returns the property set stream's low-level "OS version"
		/// field.
		/// 
		/// <strong>FIXME:</strong> Return an <code>int</code> instead
		/// of a <code>long</code> in the next major version, i.e. when
		/// incompatible changes are allowed.
		/// 
		/// </summary>
		/// <returns> The property set stream's low-level "OS version" field.
		/// </returns>
		virtual public long OSVersion {
			get {
				return osVersion;
			}
			
		}
		/// <summary> Returns the property set stream's low-level "class ID"
		/// field.
		/// 
		/// </summary>
		/// <returns> The property set stream's low-level "class ID" field.
		/// </returns>
		virtual public ClassID ClassID {
			get {
				return classID;
			}
			
		}
		/// <summary> Returns the number of  Section}s in the property
		/// set.
		/// 
		/// <strong>FIXME:</strong> Return an <code>int</code> instead
		/// of a <code>long</code> in the next major version, i.e. when
		/// incompatible changes are allowed.
		/// 
		/// </summary>
		/// <returns> The number of  Section}s in the property set.
		/// </returns>
		virtual public long SectionCount {
			get {
				return sectionCount;
			}
			
		}
		/// <summary> Returns the  Section}s in the property set.
		/// 
		/// </summary>
		/// <returns> The  Section}s in the property set.
		/// </returns>
		virtual public IList Sections {
			get {
				return sections;
			}
			
		}
		/// <summary> Checks whether this  PropertySet} represents a Summary
		/// Information.
		/// 
		/// </summary>
		/// <returns> <code>true</code> if this  PropertySet}
		/// represents a Summary Information, else <code>false</code>.
		/// </returns>
		virtual public bool SummaryInformation {
			get {
				return Util.Equal(((Section) sections[0]).FormatID.Bytes, SectionIDMap.SUMMARY_INFORMATION_ID);
			}
			
		}
		/// <summary> Checks whether this  PropertySet} is a Document
		/// Summary Information.
		/// 
		/// </summary>
		/// <returns> <code>true</code> if this  PropertySet}
		/// represents a Document Summary Information, else <code>false</code>.
		/// </returns>
		virtual public bool DocumentSummaryInformation {
			get {
				return Util.Equal(((Section) sections[0]).FormatID.Bytes, SectionIDMap.DOCUMENT_SUMMARY_INFORMATION_ID);
			}
			
		}
		/// <summary> Convenience method returning the  Property} array
		/// contained in this property set. It is a shortcut for getting
		/// the  PropertySet}'s  Section}s list and then
		/// getting the  Property} array from the first 
		/// Section}. However, it can only be used if the 
		/// PropertySet} contains exactly one  Section}, so check
		///  #getSectionCount} first!
		/// 
		/// </summary>
		/// <returns> The properties of the only  Section} of this
		///  PropertySet}.
		/// </returns>
		/// <throws>  NoSingleSectionException if the  PropertySet} has </throws>
		/// <summary> more or less than one  Section}.
		/// </summary>
		virtual public Property[] Properties {
			get {
				return SingleSection.Properties;
			}
			
		}
		/// <summary> If the  PropertySet} has only a single section this
		/// method returns it.
		/// 
		/// </summary>
		/// <returns> The singleSection value
		/// </returns>
		/// <throws>  NoSingleSectionException if the  PropertySet} has </throws>
		/// <summary>more or less than exactly one  Section}.
		/// </summary>
		virtual public Section SingleSection
		{
			get	
			{
				//BUG
				// if(sectionCount != 1)
				//	throw new NoSingleSectionException("Property set contains " + sectionCount + " sections.");
				return ((Section) sections[0]);
			}
			
		}
		
		/// <summary> The "byteOrder" field must equal this value.</summary>
		
		internal static readonly byte[] BYTE_ORDER_ASSERTION = new byte[]{unchecked((byte) 0xFE), unchecked((byte) 0xFF)};
		
		/// <summary> Specifies this  PropertySet}'s byte order. See the
		/// HPFS documentation for details!
		/// </summary>
		protected internal int byteOrder;
		
		
		
		/// <summary> The "format" field must equal this value.</summary>
		
		internal static readonly byte[] FORMAT_ASSERTION = new byte[]{(byte) 0x00, (byte) 0x00};
		
		/// <summary> Specifies this  PropertySet}'s format. See the HPFS
		/// documentation for details!
		/// </summary>
		protected internal int format;
		
		
		
		/// <summary> Specifies the version of the operating system that created
		/// this  PropertySet}. See the HPFS documentation for
		/// details!
		/// </summary>
		protected internal int osVersion;
		
		
		public const int OS_WIN16 = 0x0000;
		public const int OS_MACINTOSH = 0x0001;
		public const int OS_WIN32 = 0x0002;
		
		
		
		/// <summary> Specifies this  PropertySet}'s "classID" field. See
		/// the HPFS documentation for details!
		/// </summary>
		protected internal ClassID classID;
		
		
		
		/// <summary> The number of sections in this  PropertySet}.</summary>
		protected internal int sectionCount;
		
		
		
		/// <summary> The sections in this  PropertySet}.</summary>
		protected internal IList sections;
		
		
		
		/// <summary> Creates an empty (uninitialized)  PropertySet}.
		/// 
		/// <strong>Please note:</strong> For the time being this
		/// constructor is protected since it is used for internal purposes
		/// only, but expect it to become public once the property set's
		/// writing functionality is implemented.
		/// </summary>
		protected internal PropertySet()
		{
		}
		
		
		
		/// <summary> Creates a  PropertySet} instance from an 
		/// InputStream} in the Horrible Property Set Format.
		/// 
		/// The constructor reads the first few bytes from the stream
		/// and determines whether it is really a property set stream. If
		/// it is, it parses the rest of the stream. If it is not, it
		/// resets the stream to its beginning in order to let other
		/// components mess around with the data and throws an
		/// exception.
		/// 
		/// </summary>
		/// <param name="stream">Holds the data making out the property set
		/// stream.
		/// </param>
		/// <throws>  MarkUnsupportedException if the stream does not support </throws>
		/// <summary> the  InputStream#markSupported} method.
		/// </summary>
		/// <throws>  IOException if the  InputStream} cannot not be </throws>
		/// <summary> accessed as needed.
		/// </summary>
		public PropertySet(Stream stream)
		{
			if (IsPropertySetStream(stream))
			{				
				long available = stream.Length - stream.Position;
				
				byte[] buffer = new byte[available];

				int bytesRead  = stream.Read(buffer, 0, buffer.Length);
			
				Init(buffer, 0, buffer.Length);
			}
			else
				throw new NoPropertySetStreamException();
		}
		
		
		
		/// <summary> Creates a  PropertySet} instance from a byte array
		/// that represents a stream in the Horrible Property Set
		/// Format.
		/// 
		/// </summary>
		/// <param name="stream">The byte array holding the stream data.
		/// </param>
		/// <param name="offset">The offset in <var>stream</var> where the stream
		/// data begin. If the stream data begin with the first byte in the
		/// array, the <var>offset</var> is 0.
		/// </param>
		/// <param name="length">The length of the stream data.
		/// </param>
		/// <throws>  NoPropertySetStreamException if the byte array is not a </throws>
		/// <summary> property set stream.
		/// </summary>
		public PropertySet(byte[] stream, int offset, int length)
		{
			if (IsPropertySetStream(stream, offset, length))
				Init(stream, offset, length);
			else
				throw new NoPropertySetStreamException();
		}
		
		
		
		/// <summary> Creates a  PropertySet} instance from a byte array
		/// that represents a stream in the Horrible Property Set
		/// Format.
		/// 
		/// </summary>
		/// <param name="stream">The byte array holding the stream data. The
		/// complete byte array contents is the stream data.
		/// </param>
		/// <throws>  NoPropertySetStreamException if the byte array is not a </throws>
		/// <summary> property set stream.
		/// </summary>
		public PropertySet(byte[] stream):this(stream, 0, stream.Length)
		{
		}
		
		
		
		/// <summary> Checks whether an  InputStream} is in the Horrible
		/// Property Set Format.
		/// 
		/// </summary>
		/// <param name="stream">The  InputStream} to check. In order to
		/// perform the check, the method reads the first bytes from the
		/// stream. After reading, the stream is reset to the position it
		/// had before reading. The  InputStream} must support the
		///  InputStream#mark} method.
		/// </param>
		/// <returns> <code>true</code> if the stream is a property set
		/// stream, else <code>false</code>.
		/// </returns>
		/// <throws>  MarkUnsupportedException if the  InputStream} </throws>
		/// <summary> does not support the  InputStream#mark} method.
		/// </summary>
		public static bool IsPropertySetStream(Stream stream)
		{
			/*
			* Read at most this many bytes.
			*/
			
			int BUFFER_SIZE = 50;
			
			/*
			* Mark the current position in the stream so that we can
			* reset to this position if the stream does not contain a
			* property set.
			*/

			if (!stream.CanSeek)
			{
				throw new MarkUnsupportedException(stream.GetType().FullName);
			}
			
			// Do not mark! Position set at the end of the method
			
			/*
			* Read a couple of bytes from the stream.
			*/
			
			byte[] buffer = new byte[BUFFER_SIZE];
			
			long available = stream.Length - stream.Position;

			int bytes = stream.Read(buffer,0, buffer.Length);

			bool isPropertySetStream_ = IsPropertySetStream(buffer, 0, bytes);
			
			stream.Position = 0;

			return isPropertySetStream_;
		}
		
		
		
		/// <summary> Checks whether a byte array is in the Horrible Property Set
		/// Format.
		/// 
		/// </summary>
		/// <param name="src">The byte array to check.
		/// </param>
		/// <param name="offset">The offset in the byte array.
		/// </param>
		/// <param name="length">The significant number of bytes in the byte
		/// array. Only this number of bytes will be checked.
		/// </param>
		/// <returns> <code>true</code> if the byte array is a property set
		/// stream, <code>false</code> if not.
		/// </returns>
		public static bool IsPropertySetStream(byte[] src, int offset, int length)
		{
			/*
			* Read the header fields of the stream. They must always be
			* there.
			*/
			
			int byteOrder = LittleEndian.GetUShort(src, offset);
			offset += LittleEndian.SHORT_SIZE;
			byte[] temp = new byte[LittleEndian.SHORT_SIZE];
			LittleEndian.PutShort(temp, (short) byteOrder);
			if (!Util.Equal(temp, BYTE_ORDER_ASSERTION))
				return false;
			
			int format = LittleEndian.GetUShort(src, offset);
			offset += LittleEndian.SHORT_SIZE;
			temp = new byte[LittleEndian.SHORT_SIZE];
			LittleEndian.PutShort(temp, (short) format);
			if (!Util.Equal(temp, FORMAT_ASSERTION))
				return false;
			
			long osVersion = LittleEndian.GetUInt(src, offset);
			offset += LittleEndian.INT_SIZE;
			
			ClassID classID = new ClassID(src, offset);
			offset += ClassID.LENGTH;
			
			long sectionCount = LittleEndian.GetUInt(src, offset);
			offset += LittleEndian.INT_SIZE;
			if (sectionCount < 1)
				return false;
			return true;
		}
		
		
		
		/// <summary> Initializes this  PropertySet} instance from a byte
		/// array. The method assumes that it has been checked already that
		/// the byte array indeed represents a property set stream. It does
		/// no more checks on its own.
		/// 
		/// </summary>
		/// <param name="src">Byte array containing the property set stream
		/// </param>
		/// <param name="offset">The property set stream starts at this offset
		/// from the beginning of <var>src</src>
		/// </param>
		/// <param name="length">Length of the property set stream.
		/// </param>
		private void Init(byte[] src, int offset, int length)
		{
			/*
			* Read the stream's header fields.
			*/
			byteOrder = LittleEndian.GetUShort(src, offset);
			offset += LittleEndian.SHORT_SIZE;
			format = LittleEndian.GetUShort(src, offset);
			offset += LittleEndian.SHORT_SIZE;
			osVersion = (int) LittleEndian.GetUInt(src, offset);
			offset += LittleEndian.INT_SIZE;
			classID = new ClassID(src, offset);
			offset += ClassID.Length;
			sectionCount = LittleEndian.GetInt(src, offset);
			offset += LittleEndian.INT_SIZE;
			if (sectionCount <= 0)
				throw new HPSFRuntimeException("Section count " + sectionCount + " must be greater than 0.");
			
			/*
			* Read the sections, which are following the header. They
			* start with an array of section descriptions. Each one
			* consists of a format ID telling what the section contains
			* and an offset telling how many bytes from the start of the
			* stream the section begins.
			*/
			/*
			* Most property sets have only one section. The Document
			* Summary Information stream has 2. Everything else is a rare
			* exception and is no longer fostered by Microsoft.
			*/
			sections = new ArrayList(2);
			
			/*
			* Loop over the section descriptor array. Each descriptor
			* consists of a ClassID and a DWord, and we have to increment
			* "offset" accordingly.
			*/
			for (int i = 0; i < sectionCount; i++)
			{
				
				Section s = new Section(src, offset);
				offset += ClassID.Length + LittleEndian.INT_SIZE;
				sections.Add(s);
			}
		}
		
		
		
		/// <summary> Convenience method returning the value of the property with
		/// the specified ID. If the property is not available,
		/// <code>null</code> is returned and a subsequent call to 
		/// #wasNull} will return <code>true</code> .
		/// 
		/// </summary>
		/// <param name="id">The property ID
		/// </param>
		/// <returns> The property value
		/// </returns>
		/// <throws>  NoSingleSectionException if the  PropertySet} has </throws>
		/// <summary> more or less than one  Section}.
		/// </summary>
		protected internal virtual object GetProperty(int id)
		{
			return SingleSection.GetProperty(id);
		}
		
		
		
		/// <summary> Convenience method returning the value of a boolean property
		/// with the specified ID. If the property is not available,
		/// <code>false</code> is returned. A subsequent call to 
		/// #wasNull} will return <code>true</code> to let the caller
		/// distinguish that case from a real property value of
		/// <code>false</code>.
		/// 
		/// </summary>
		/// <param name="id">The property ID
		/// </param>
		/// <returns> The property value
		/// </returns>
		/// <throws>  NoSingleSectionException if the  PropertySet} has </throws>
		/// <summary> more or less than one  Section}.
		/// </summary>
		protected internal virtual bool GetPropertyBooleanValue(int id)
		{
			return SingleSection.GetPropertyBooleanValue(id);
		}
		
		
		
		/// <summary> Convenience method returning the value of the numeric
		/// property with the specified ID. If the property is not
		/// available, 0 is returned. A subsequent call to  #wasNull}
		/// will return <code>true</code> to let the caller distinguish
		/// that case from a real property value of 0.
		/// 
		/// </summary>
		/// <param name="id">The property ID
		/// </param>
		/// <returns> The propertyIntValue value
		/// </returns>
		/// <throws>  NoSingleSectionException if the  PropertySet} has </throws>
		/// <summary> more or less than one  Section}.
		/// </summary>
		protected internal virtual int GetPropertyIntValue(int id)
		{
			return SingleSection.GetPropertyIntValue(id);
		}
		
		
		
		/// <summary> Checks whether the property which the last call to 
		/// #getPropertyIntValue} or  #getProperty} tried to access
		/// was available or not. This information might be important for
		/// callers of  #getPropertyIntValue} since the latter
		/// returns 0 if the property does not exist. Using 
		/// #wasNull}, the caller can distiguish this case from a
		/// property's real value of 0.
		/// 
		/// </summary>
		/// <returns> <code>true</code> if the last call to 
		/// #getPropertyIntValue} or  #getProperty} tried to access a
		/// property that was not available, else <code>false</code>.
		/// </returns>
		/// <throws>  NoSingleSectionException if the  PropertySet} has </throws>
		/// <summary> more than one  Section}.
		/// </summary>
		public virtual bool WasNull
		{
			get 
			{
				return SingleSection.WasNull;
			}

		}
	}
}