// 
// Section.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//   Drew Varner (Drew.Varner allUpIn sc.edu)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Collections;

using Poi.Net.HPSF.Wellknown;
using Poi.Net.Util;

namespace Poi.Net.HPSF
{
	
	/// <summary> Represents a section in a  PropertySet}.
	/// 
	/// </summary>
	public class Section
	{
		/// <summary> Returns the format ID. The format ID is the "type" of the
		/// section. For example, if the format ID of the first 
		/// Section} contains the bytes specified by
		/// <code>Poi.Net.HPSF.Wellknown.SectionIDMap.SUMMARY_INFORMATION_ID</code>
		/// the section (and thus the property set) is a
		/// SummaryInformation.
		/// 
		/// </summary>
		/// <returns> The format ID
		/// </returns>
		virtual public ClassID FormatID {
			get {
				return formatID;
			}
			
		}
		/// <summary> Returns the offset of the section in the stream.
		/// 
		/// </summary>
		/// <returns> The offset of the section in the stream.
		/// </returns>
		virtual public long Offset {
			get {
				return offset;
			}
			
		}
		/// <summary> Returns the section's size in bytes.
		/// 
		/// </summary>
		/// <returns> The section's size in bytes.
		/// </returns>
		virtual public int Size {
			get {
				return size;
			}
			
		}
		/// <summary> Returns the number of properties in this section.
		/// 
		/// </summary>
		/// <returns> The number of properties in this section.
		/// </returns>
		virtual public int PropertyCount {
			get {
				return propertyCount;
			}
			
		}
		/// <summary> Returns this section's properties.
		/// 
		/// </summary>
		/// <returns> This section's properties.
		/// </returns>
		virtual public Property[] Properties {
			get {
				return properties;
			}
			
		}
		
		/// <summary> Maps property IDs to section-private PID strings. These
		/// strings can be found in the property with ID 0.
		/// </summary>
		protected internal IDictionary dictionary;
		
		protected internal ClassID formatID;
		
		
		
		protected internal long offset;
		
		
		
		protected internal int size;
		
		
		
		protected internal int propertyCount;
		
		
		
		protected internal Property[] properties;
		
		
		
		/// <summary> Creates an empty and uninitialized  Section}.</summary>
		protected internal Section()
		{
		}
		
		
		
		/// <summary> Creates a  Section} instance from a byte array.
		/// 
		/// </summary>
		/// <param name="src">Contains the complete property set stream.
		/// </param>
		/// <param name="offset">The position in the stream that points to the
		/// section's format ID.
		/// </param>
		public Section(byte[] src, int offset)
		{
			
			/*
			* Read the format ID.
			*/
			formatID = new ClassID(src, offset);
			offset += ClassID.LENGTH;
			
			/*
			* Read the offset from the stream's start and positions to
			* the section header.
			*/
			this.offset = LittleEndian.GetUInt(src, offset);
			offset = (int) this.offset;			

			/*
			* Read the section length.
			*/
			size = (int) LittleEndian.GetUInt(src, offset);
			offset += LittleEndian.INT_SIZE;
			
			/*
			* Read the number of properties.
			*/
			propertyCount = (int) LittleEndian.GetUInt(src, offset);
			offset += LittleEndian.INT_SIZE;
			
			/*
			* Read the properties. The offset is positioned at the first
			* entry of the property list. The problem is that we have to
			* read the property with ID 1 before we read other
			* properties, at least before other properties containing
			* strings. The reason is that property 1 specifies the
			* codepage. If it is 1200, all strings are in Unicode. In
			* other words: Before we can read any strings we have to know
			* whether they are in Unicode or not. Unfortunately property
			* 1 is not guaranteed to be the first in a section.
			*
			* The algorithm below reads the properties in two passes: The
			* first one looks for property ID 1 and extracts the codepage
			* number. The seconds pass reads the other properties.
			*/
			properties = new Property[propertyCount];
			//unused: Property propertyOne;
			
			/* Pass 1: Look for the codepage. */
			int codepage = - 1;
			int pass1Offset = offset;
			for (int i = 0; i < properties.Length; i++)
			{
				/* Read the property ID. */
				
				int id = (int) LittleEndian.GetUInt(src, pass1Offset);
				pass1Offset += LittleEndian.INT_SIZE;
				
				/* Offset from the section's start. */
				
				int sOffset = (int) LittleEndian.GetUInt(src, pass1Offset);
				pass1Offset += LittleEndian.INT_SIZE;
				
				/* Calculate the length of the property. */
				int length;
				if (i == properties.Length - 1)
					length = (int) (src.Length - this.offset - sOffset);
				else
					length = (int) LittleEndian.GetUInt(src, pass1Offset + LittleEndian.INT_SIZE) - sOffset;
				
				if (id == PropertyIDMap.PID_CODEPAGE)
				{
					/* Read the codepage if the property ID is 1. */
					
					/* Read the property's value type. It must be
					* VT_I2. */
					int o = (int) (this.offset + sOffset);
					
					long type = LittleEndian.GetUInt(src, o);
					o += LittleEndian.INT_SIZE;
					
					if (type != Variant.VT_I2)
						throw new HPSFRuntimeException("Value type of property ID 1 is not VT_I2 but " + type + ".");
					
					/* Read the codepage number. */
					codepage = LittleEndian.GetUShort(src, o);
				}
			}
			
			/* Pass 2: Read all properties, including 1. */
			for (int i = 0; i < properties.Length; i++)
			{

				/* Read the property ID. */
				
				int id = (int) LittleEndian.GetUInt(src, offset);
				offset += LittleEndian.INT_SIZE;
				
				/* Offset from the section. */
				
				int sOffset = (int) LittleEndian.GetUInt(src, offset);
				offset += LittleEndian.INT_SIZE;
				
				/* Calculate the length of the property. */
				int length;
				if (i == properties.Length - 1)
					length = (int) (src.Length - this.offset - sOffset);
				else
					length = (int) LittleEndian.GetUInt(src, offset + LittleEndian.INT_SIZE) - sOffset;
				
				/* Create it. */
				properties[i] = new Property(id, src, this.offset + sOffset, length, codepage);
			}
			
			/*
			* Extract the dictionary (if available).
			*/
			dictionary = (IDictionary) GetProperty(0);

		}
		
		
		
		/// <summary> Returns the value of the property with the specified ID. If
		/// the property is not available, <code>null</code> is returned
		/// and a subsequent call to  #wasNull} will return
		/// <code>true</code>.
		/// 
		/// </summary>
		/// <param name="id">The property's ID
		/// 
		/// </param>
		/// <returns> The property's value
		/// </returns>
		public virtual object GetProperty(int id)
		{
			wasNull = false;
			for (int i = 0; i < properties.Length; i++)
				if (id == properties[i].ID)
					return properties[i].Value;
			wasNull = true;
			return null;
		}		
		
		
		/// <summary> Returns the value of the numeric property with the specified
		/// ID. If the property is not available, 0 is returned. A
		/// subsequent call to  #wasNull} will return
		/// <code>true</code> to let the caller distinguish that case from
		/// a real property value of 0.
		/// 
		/// </summary>
		/// <param name="id">The property's ID
		/// 
		/// </param>
		/// <returns> The property's value
		/// </returns>
		protected internal virtual int GetPropertyIntValue(int id)
		{
			/* FIXME: Find out why the following is a Long instead of an
			* Integer! */
			
			object i = GetProperty(id);
			
			//TODO: Solve in calling code

			if (i != null)
				return Convert.ToInt32(i);
			else
				return 0;
		}
		
		
		
		/// <summary> Returns the value of the boolean property with the specified
		/// ID. If the property is not available, <code>false</code> is
		/// returned. A subsequent call to  #wasNull} will return
		/// <code>true</code> to let the caller distinguish that case from
		/// a real property value of <code>false</code>.
		/// 
		/// </summary>
		/// <param name="id">The property's ID
		/// 
		/// </param>
		/// <returns> The property's value
		/// </returns>
		protected internal virtual bool GetPropertyBooleanValue(int id)
		{
			
			bool b = (bool) GetProperty(id);
			
			//TODO: Solve in calling code
			
			if ((object) b != null)
				return b;
			else
				return false;
		}
		
		
		
		private bool wasNull;
		
		
		/// <summary> Checks whether the property which the last call to 
		/// #getPropertyIntValue} or  #getProperty} tried to access
		/// was available or not. This information might be important for
		/// callers of  #getPropertyIntValue} since the latter
		/// returns 0 if the property does not exist. Using 
		/// #wasNull} the caller can distiguish this case from a property's
		/// real value of 0.
		/// 
		/// </summary>
		/// <returns> <code>true</code> if the last call to 
		/// #getPropertyIntValue} or  #getProperty} tried to access a
		/// property that was not available, else <code>false</code>.
		/// </returns>
		public virtual bool WasNull {		
			get { return wasNull; }
		}
		
		
		
		/// <summary> Returns the PID string associated with a property ID. The ID
		/// is first looked up in the  Section}'s private
		/// dictionary. If it is not found there, the method calls 
		/// SectionIDMap#getPIDString}.
		/// 
		/// </summary>
		/// <param name="pid">The property ID
		/// 
		/// </param>
		/// <returns> The property ID's string value
		/// </returns>
		public virtual string GetPIDString(int pid)
		{
			string s = null;
			if (dictionary != null)
				s = ((string) dictionary[(int) pid]);
			if ((object) s == null)
				s = SectionIDMap.GetPIDString(FormatID.Bytes, pid);
			if ((object) s == null)
				s = SectionIDMap.UNDEFINED;
			return s;
		}
	}
}