// 
// SpecialPropertySet.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Collections;

using Poi.Net.Util;

namespace Poi.Net.HPSF
{
	
	/// <summary> Abstract superclass for the convenience classes 
	/// SummaryInformation} and  DocumentSummaryInformation}.
	/// 
	/// The motivation behind this class is quite nasty if you look
	/// behind the scenes, but it serves the application programmer well by
	/// providing him with the easy-to-use  SummaryInformation} and
	///  DocumentSummaryInformation} classes. When parsing the data a
	/// property set stream consists of (possibly coming from an 
	/// java.io.InputStream}) we want to read and process each byte only
	/// once. Since we don't know in advance which kind of property set we
	/// have, we can expect only the most general 
	/// PropertySet}. Creating a special subclass should be as easy as
	/// calling the special subclass' constructor and pass the general
	///  PropertySet} in. To make things easy internally, the special
	/// class just holds a reference to the general  PropertySet} and
	/// delegates all method calls to it.
	/// 
	/// A cleaner implementation would have been like this: The 
	/// PropertySetFactory} parses the stream data into some internal
	/// object first.  Then it finds out whether the stream is a 
	/// SummaryInformation}, a  DocumentSummaryInformation} or a
	/// general  PropertySet}.  However, the current implementation
	/// went the other way round historically: the convenience classes came
	/// only late to my mind.
	/// 
	/// </summary>
	public abstract class SpecialPropertySet:PropertySet
	{
		override public int ByteOrder {
			get {
				return _delegate.ByteOrder;
			}
			
		}
		override public int Format {
			get {
				return _delegate.Format;
			}
			
		}
		override public long OSVersion
		{
			/*
			* This is intentionally no javadoc comment.
			*
			* FIXME: Return an <code>int</code> instead of a
			* <code>long</code> in the next major version, i.e. when
			* incompatible changes are allowed.
			*/
			
			get
			{
				return _delegate.OSVersion;
			}
			
		}
		override public ClassID ClassID {
			get {
				return _delegate.ClassID;
			}
			
		}
		override public long SectionCount
		{
			/*
			* This is intentionally no javadoc comment.
			*
			* FIXME: Return an <code>int</code> instead of a
			* <code>long</code> in the next major version, i.e. when
			* incompatible changes are allowed.
			*/
			
			get
			{
				return _delegate.SectionCount;
			}
			
		}
		override public IList Sections {
			get {
				return _delegate.Sections;
			}
			
		}
		override public bool SummaryInformation {
			get {
				return _delegate.SummaryInformation;
			}
			
		}
		override public bool DocumentSummaryInformation {
			get {
				return _delegate.DocumentSummaryInformation;
			}
			
		}
		override public Section SingleSection {
			get {
				return _delegate.SingleSection;
			}
			
		}
		
		private PropertySet _delegate;
		
		
		
		public SpecialPropertySet(PropertySet ps)
		{
			_delegate = ps;
		}
	}
}