// 
// SummaryInformation.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


using System;
using Poi.Net.HPSF.Wellknown;

namespace Poi.Net.HPSF
{
	
	/// <summary> Convenience class representing a Summary Information stream in a
	/// Microsoft Office document.
	/// 
	/// See <a
	/// href="http://msdn.microsoft.com/library/default.asp?url=/library/en-us/com/stgu_8910.asp">http://msdn.microsoft.com/library/default.asp?url=/library/en-us/com/stgu_8910.asp</a>
	/// for documentation from That Redmond Company.
	/// 
	/// </summary>
	/// <seealso cref="DocumentSummaryInformation">
	/// </seealso>
	public class SummaryInformation:SpecialPropertySet
	{
		/// <summary> Returns the stream's title (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The title or <code>null</code>
		/// </returns>
		virtual public string Title {
			get {
				return (string) GetProperty(PropertyIDMap.PID_TITLE);
			}
			
		}
		/// <summary> Returns the stream's subject (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The subject or <code>null</code>
		/// </returns>
		virtual public string Subject {
			get {
				return (string) GetProperty(PropertyIDMap.PID_SUBJECT);
			}
			
		}
		/// <summary> Returns the stream's author (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The author or <code>null</code>
		/// </returns>
		virtual public string Author {
			get {
				return (string) GetProperty(PropertyIDMap.PID_AUTHOR);
			}
			
		}
		/// <summary> Returns the stream's keywords (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The keywords or <code>null</code>
		/// </returns>
		virtual public string Keywords {
			get {
				return (string) GetProperty(PropertyIDMap.PID_KEYWORDS);
			}
			
		}
		/// <summary> Returns the stream's comments (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The comments or <code>null</code>
		/// </returns>
		virtual public string Comments {
			get {
				return (string) GetProperty(PropertyIDMap.PID_COMMENTS);
			}
			
		}
		/// <summary> Returns the stream's template (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The template or <code>null</code>
		/// </returns>
		virtual public string Template {
			get {
				return (string) GetProperty(PropertyIDMap.PID_TEMPLATE);
			}
			
		}
		/// <summary> Returns the stream's last author (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The last author or <code>null</code>
		/// </returns>
		virtual public string LastAuthor {
			get {
				return (string) GetProperty(PropertyIDMap.PID_LASTAUTHOR);
			}
			
		}
		/// <summary> Returns the stream's revision number (or
		/// <code>null</code>). 
		/// 
		/// </summary>
		/// <returns> The revision number or <code>null</code>
		/// </returns>
		virtual public string RevNumber {
			get {
				return (string) GetProperty(PropertyIDMap.PID_REVNUMBER);
			}
			
		}
		/// <summary> Returns the stream's edit time (or <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The edit time or <code>null</code>
		/// </returns>
		virtual public Object EditTime {
			get {
				object o = GetProperty(PropertyIDMap.PID_EDITTIME);
				if(o != null)
					return (DateTime) o;
				else
					return null;
			}
			
		}
		/// <summary> Returns the stream's last printed time (or
		/// <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The last printed time or <code>null</code>
		/// </returns>
		virtual public object LastPrinted {
			get { 
				object o = GetProperty(PropertyIDMap.PID_LASTPRINTED);
				if(o != null)
					return (DateTime) o;
				else
					return null;
			}
			
		}
		/// <summary> Returns the stream's creation time (or
		/// <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The creation time or <code>null</code>
		/// </returns>
		virtual public object CreateDateTime {
			get {
				object o = GetProperty(PropertyIDMap.PID_CREATE_DTM);
				if(o != null)
					return (DateTime) o;
				else
					return null;
			}
			
		}
		/// <summary> Returns the stream's last save time (or
		/// <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The last save time or <code>null</code>
		/// </returns>
		virtual public object LastSaveDateTime {
			get {
				object o = GetProperty(PropertyIDMap.PID_LASTSAVE_DTM);
				if(o != null)
					return (DateTime) o;
				else
					return null;
			}
			
		}
		/// <summary> Returns the stream's page count or 0 if the 
		/// SummaryInformation} does not contain a page count.
		/// 
		/// </summary>
		/// <returns> The page count or <code>null</code>
		/// </returns>
		virtual public int PageCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_PAGECOUNT);
			}
			
		}
		/// <summary> Returns the stream's word count or 0 if the 
		/// SummaryInformation} does not contain a word count.
		/// 
		/// </summary>
		/// <returns> The word count or <code>null</code>
		/// </returns>
		virtual public int WordCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_WORDCOUNT);
			}
			
		}
		/// <summary> Returns the stream's character count or 0 if the 
		/// SummaryInformation} does not contain a char count.
		/// 
		/// </summary>
		/// <returns> The character count or <code>null</code>
		/// </returns>
		virtual public int CharCount {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_CHARCOUNT);
			}
			
		}
		/// <summary> Returns the stream's thumbnail (or <code>null</code>)
		/// <strong>when this method is implemented. Please note that the
		/// return type is likely to change!</strong>
		/// 
		/// <strong>FIXME / Hint to developers:</strong> Drew Varner
		/// &lt;Drew.Varner -at- sc.edu&gt; said that this is an image in
		/// WMF or Clipboard (BMP?) format. He also provided two links that
		/// might be helpful: <a
		/// href="http://www.csn.ul.ie/~caolan/publink/file/OLE2SummaryAgainst_file-3.27.patch"
		/// target="_blank">http://www.csn.ul.ie/~caolan/publink/file/OLE2SummaryAgainst_file-3.27.patch
		/// </a> and <a
		/// href="http://msdn.microsoft.com/library/en-us/dno97ta/html/msdn_docprop.asp"
		/// target="_blank">http://msdn.microsoft.com/library/en-us/dno97ta/html/msdn_docprop.asp
		/// </a>. However, we won't do any conversion into any image type
		/// but instead just return a byte array.
		/// 
		/// </summary>
		/// <returns> The thumbnail or <code>null</code>
		/// </returns>
		virtual public byte[] Thumbnail {
			get {
				return (byte[]) GetProperty(PropertyIDMap.PID_THUMBNAIL);
			}
			
		}
		/// <summary> Returns the stream's application name (or
		/// <code>null</code>).
		/// 
		/// </summary>
		/// <returns> The application name or <code>null</code>
		/// </returns>
		virtual public string ApplicationName {
			get {
				return (string) GetProperty(PropertyIDMap.PID_APPNAME);
			}
			
		}
		/// <summary> Returns a security code which is one of the following
		/// values:
		/// 
		/// <ul>
		/// <li>
		/// 0 if the  SummaryInformation} does not contain a
		/// security field or if there is no security on the
		/// document. Use  #wasNull} to distinguish between the
		/// two cases!
		/// </li>
		/// 
		/// <li>
		/// 1 if the document is password protected
		/// </li>
		/// 
		/// <li>
		/// 2 if the document is read-only recommended
		/// </li>
		/// 
		/// <li>
		/// 4 if the document is read-only enforced
		/// </li>
		/// 
		/// <li>
		/// 8 if the document is locked for annotations
		/// </li>
		/// 
		/// </ul>
		/// 
		/// </summary>
		/// <returns> The security code or <code>null</code>
		/// </returns>
		virtual public int Security {
			get {
				return GetPropertyIntValue(PropertyIDMap.PID_SECURITY);
			}
			
		}
		
		/// <summary> The document name a summary information stream usually has
		/// in a POIFS filesystem.
		/// </summary>
		public const string DEFAULT_STREAM_NAME = "\x0005SummaryInformation";
		
		
		
		/// <summary> Creates a  SummaryInformation} from a given 
		/// PropertySet}.
		/// 
		/// </summary>
		/// <param name="ps">A property set which should be created from a summary
		/// information stream.
		/// </param>
		/// <throws>  UnexpectedPropertySetTypeException if <var>ps</var> </throws>
		/// <summary> does not contain a summary information stream.
		/// </summary>
		public SummaryInformation(PropertySet ps):base(ps)
		{
			if (!SummaryInformation)
			{
				throw new UnexpectedPropertySetTypeException("Not a " + GetType().FullName);
			}
		}
	}
}