// 
// Thumbnail.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Drew Varner (Drew.Varner inOrAround sc.edu)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using Poi.Net.Util;

namespace Poi.Net.HPSF
{
	/// <summary> Class to manipulate data in the Clipboard Variant (
	/// Variant#VT_CF VT_CF}) format.
	/// 
	/// </summary>
	/// <seealso cref="SummaryInformation#getThumbnail()">
	/// </seealso>
	public class Thumbnail
	{
		/// <summary> Returns an <code>int</code> representing the Clipboard
		/// Format Tag
		/// 
		/// Possible return values are:
		/// <ul>
		/// <li> #CFTAG_WINDOWS CFTAG_WINDOWS}</li>
		/// <li> #CFTAG_MACINTOSH CFTAG_MACINTOSH}</li>
		/// <li> #CFTAG_FMTID CFTAG_FMTID}</li>
		/// <li> #CFTAG_NODATA CFTAG_NODATA}</li>
		/// </ul>
		/// 
		/// </summary>
		/// <returns> A flag indicating the Clipboard Format Tag
		/// </returns>
		virtual public long ClipboardFormatTag {
			get {
				long clipboardFormatTag = LittleEndian.GetUInt(ThumbnailData, OFFSET_CFTAG);
				return clipboardFormatTag;
			}
			
		}
		/// <summary> Returns an <code>int</code> representing the Clipboard
		/// Format
		/// 
		/// Will throw an exception if the Thumbnail's Clipboard Format
		/// Tag is not  Thumbnail#CFTAG_WINDOWS CFTAG_WINDOWS}.
		/// 
		/// Possible return values are:
		/// 
		/// <ul>
		/// <li> #CF_METAFILEPICT CF_METAFILEPICT}</li>
		/// <li> #CF_DIB CF_DIB}</li>
		/// <li> #CF_ENHMETAFILE CF_ENHMETAFILE}</li>
		/// <li> #CF_BITMAP CF_BITMAP}</li>
		/// </ul>
		/// 
		/// </summary>
		/// <returns> a flag indicating the Clipboard Format
		/// </returns>
		/// <throws>  HPSFException if the Thumbnail isn't CFTAG_WINDOWS </throws>
		virtual public long ClipboardFormat {
			get {
				if (!(ClipboardFormatTag == CFTAG_WINDOWS))
					throw new HPSFException("Clipboard Format Tag of Thumbnail must " + "be CFTAG_WINDOWS.");
				
				return LittleEndian.GetUInt(ThumbnailData, OFFSET_CF);
			}
			
		}
		/// <summary> Returns the Thumbnail as a <code>byte[]</code> of WMF data
		/// if the Thumbnail's Clipboard Format Tag is 
		/// #CFTAG_WINDOWS CFTAG_WINDOWS} and its Clipboard Format is
		///  #CF_METAFILEPICT CF_METAFILEPICT} This
		/// <code>byte[]</code> is in the traditional WMF file, not the
		/// clipboard-specific version with special headers.
		/// 
		/// See <a href="http://www.wvware.com/caolan/ora-wmf.html"
		/// target="_blank">http://www.wvware.com/caolan/ora-wmf.html</a>
		/// for more information on the WMF image format.
		/// 
		/// </summary>
		/// <returns> A WMF image of the Thumbnail
		/// </returns>
		/// <throws>  HPSFException if the Thumbnail isn't CFTAG_WINDOWS and </throws>
		/// <summary> CF_METAFILEPICT
		/// </summary>
		virtual public byte[] ThumbnailAsWMF {
			get {
				if (!(ClipboardFormatTag == CFTAG_WINDOWS))
					throw new HPSFException("Clipboard Format Tag of Thumbnail must " + "be CFTAG_WINDOWS.");
				if (!(ClipboardFormat == CF_METAFILEPICT))
					throw new HPSFException("Clipboard Format of Thumbnail must " + "be CF_METAFILEPICT.");
				else
				{
					byte[] thumbnail = ThumbnailData;
					int wmfImageLength = thumbnail.Length - OFFSET_WMFDATA;
					byte[] wmfImage = new byte[wmfImageLength];
					Array.Copy(thumbnail, OFFSET_WMFDATA, wmfImage, 0, wmfImageLength);
					return wmfImage;
				}
			}
			
		}
		
		/// <summary> Offset in bytes where the Clipboard Format Tag starts in the
		/// <code>byte[]</code> returned by 
		/// SummaryInformation#getThumbnail()}
		/// </summary>
		public static int OFFSET_CFTAG = 4;
		
		/// <summary> Offset in bytes where the Clipboard Format starts in the
		/// <code>byte[]</code> returned by 
		/// SummaryInformation#getThumbnail()}
		/// 
		/// This is only valid if the Clipboard Format Tag is 
		/// #CFTAG_WINDOWS}
		/// </summary>
		public static int OFFSET_CF = 8;
		
		/// <summary> Offset in bytes where the Windows Metafile (WMF) image data
		/// starts in the <code>byte[]</code> returned by 
		/// SummaryInformation#getThumbnail()}
		/// 
		/// There is only WMF data at this point in the
		/// <code>byte[]</code> if the Clipboard Format Tag is 
		/// #CFTAG_WINDOWS} and the Clipboard Format is 
		/// #CF_METAFILEPICT}.
		/// 
		/// Note: The <code>byte[]</code> that starts at
		/// <code>OFFSET_WMFDATA</code> and ends at
		/// <code>getThumbnail().length - 1</code> forms a complete WMF
		/// image. It can be saved to disk with a <code>.wmf</code> file
		/// type and read using a WMF-capable image viewer.
		/// </summary>
		public static int OFFSET_WMFDATA = 20;
		
		/// <summary> Clipboard Format Tag - Windows clipboard format
		/// 
		/// A <code>DWORD</code> indicating a built-in Windows clipboard
		/// format value
		/// 
		/// See: <a
		/// href="http://msdn.microsoft.com/library/en-us/dnolegen/html/msdn_propset.asp"
		/// target="_blank">http://msdn.microsoft.com/library/en-us/dnolegen/html/msdn_propset.asp</a>.
		/// </summary>
		public static int CFTAG_WINDOWS = - 1;
		
		/// <summary> Clipboard Format Tag - Macintosh clipboard format
		/// 
		/// A <code>DWORD</code> indicating a Macintosh clipboard format
		/// value
		/// 
		/// See: <a
		/// href="http://msdn.microsoft.com/library/en-us/dnolegen/html/msdn_propset.asp"
		/// target="_blank">http://msdn.microsoft.com/library/en-us/dnolegen/html/msdn_propset.asp</a>.
		/// </summary>
		public static int CFTAG_MACINTOSH = - 2;
		
		/// <summary> Clipboard Format Tag - Format ID
		/// 
		/// A GUID containing a format identifier (FMTID). This is
		/// rarely used.
		/// 
		/// See: <a
		/// href="http://msdn.microsoft.com/library/en-us/dnolegen/html/msdn_propset.asp"
		/// target="_blank">http://msdn.microsoft.com/library/en-us/dnolegen/html/msdn_propset.asp</a>.
		/// </summary>
		public static int CFTAG_FMTID = - 3;
		
		/// <summary> Clipboard Format Tag - No Data
		/// 
		/// A <code>DWORD</code> indicating No data. This is rarely
		/// used.
		/// 
		/// See: <a
		/// href="http://msdn.microsoft.com/library/en-us/dnolegen/html/msdn_propset.asp"
		/// target="_blank">
		/// http://msdn.microsoft.com/library/en-us/dnolegen/html/msdn_propset.asp</a>.
		/// </summary>
		public static int CFTAG_NODATA = 0;
		
		/// <summary> Clipboard Format - Windows metafile format. This is the
		/// recommended way to store thumbnails in Property Streams.
		/// 
		/// <strong>Note:</strong> This is not the same format used in
		/// regular WMF images. The clipboard version of this format has an
		/// extra clipboard-specific header.
		/// </summary>
		public static int CF_METAFILEPICT = 3;
		
		/// <summary> Clipboard Format - Device Independent Bitmap</summary>
		public static int CF_DIB = 8;
		
		/// <summary> Clipboard Format - Enhanced Windows metafile format</summary>
		public static int CF_ENHMETAFILE = 14;
		
		/// <summary> Clipboard Format - Bitmap
		/// 
		/// Obsolete, see <a 
		/// href="msdn.microsoft.com/library/en-us/dnw98bk/html/clipboardoperations.asp"
		/// target="_blank">msdn.microsoft.com/library/en-us/dnw98bk/html/clipboardoperations.asp</a>.
		/// </summary>
		public static int CF_BITMAP = 2;
		
		/// <summary> A <code>byte[]</code> to hold a thumbnail image in (
		/// Variant#VT_CF VT_CF}) format.
		/// </summary>
		private byte[] thumbnailData = null;
		
		
		
		/// <summary> Default Constructor. If you use it then one you'll have to add
		/// the thumbnail <code>byte[]</code> from 
		/// SummaryInformation#getThumbnail()} to do any useful
		/// manipulations, otherwise you'll get a
		/// <code>NullPointerException</code>.
		/// </summary>
		public Thumbnail():base()
		{
		}
		
		
		
		/// <summary> Creates a <code>Thumbnail</code> instance and initializes
		/// with the specified image bytes.
		/// 
		/// </summary>
		/// <param name="thumbnailData">The thumbnail data
		/// </param>
		public Thumbnail(byte[] thumbnailData)
		{
			this.thumbnailData = thumbnailData;
		}
		
		
		
		/// <summary> Returns the thumbnail as a <code>byte[]</code> in 
		/// Variant#VT_CF VT_CF} format.
		/// 
		/// </summary>
		/// <returns> The thumbnail value
		/// </returns>
		/// <seealso cref="SummaryInformation#getThumbnail()">
		/// </seealso>
		/// 
		
		
		
		/// <summary> Sets the Thumbnail's underlying <code>byte[]</code> in
		///  Variant#VT_CF VT_CF} format.
		/// 
		/// </summary>
		/// <param name="thumbnail">The new thumbnail value
		/// </param>
		/// <seealso cref="SummaryInformation#getThumbnail()">
		/// </seealso>

		public virtual byte[] ThumbnailData{
			get {
				return thumbnailData;
			}
			set {
				this.thumbnailData = value;
			}
		}
	}
}