// 
// TypeReader.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


using System;
using System.Text;

using Poi.Net.Util;

namespace Poi.Net.HPSF
{
	
	/// <summary> Reader for specific data types.
	/// 
	/// </summary>
	/// <seealso cref="Property">
	/// </seealso>
	/// <seealso cref="Variant">
	/// </seealso>
	public class TypeReader
	{
		
		/// <summary> Reads a variant data type from a byte array.
		/// 
		/// </summary>
		/// <param name="src">The byte array
		/// </param>
		/// <param name="offset">The offset in the byte array where the variant
		/// starts
		/// </param>
		/// <param name="length">The length of the variant including the variant
		/// type field
		/// </param>
		/// <param name="type">The variant type to read
		/// </param>
		/// <returns> A Java object that corresponds best to the variant
		/// field. For example, a VT_I4 is returned as a  Long}, a
		/// VT_LPSTR as a  String}.
		/// 
		/// </returns>
		/// <seealso cref="Variant">
		/// </seealso>
		/// 

		public static object Read(byte[] src, int offset, int length, int type)
		{
			return Read(src, offset, length, type, -1);
		}
		public static object Read(byte[] src, int offset, int length, int type, int codepage)
		{
			/*
			* FIXME: Support reading more types and clean up this code!
			*/
			object _value;
			length = length - LittleEndian.INT_SIZE;
			switch (type)
			{
				
				case Variant.VT_EMPTY: 
				{
					/*
					* FIXME: The value returned by this case relies on the
					* assumption that the value VT_EMPTY denotes consists of zero 
					* bytes. I'd be glad if some could confirm or correct this. 
					*/
					_value = null;
					break;
				}
				
				case Variant.VT_I2: 
				{
					/*
					* Read a short. In Java it is represented as an
					* Integer object.
					*/
					_value = (int) LittleEndian.GetUShort(src, offset);
					break;
				}
				
				case Variant.VT_I4: 
				{
					/*
					* Read a word. In Java it is represented as a
					* Long object.
					*/
					_value = (long) LittleEndian.GetUInt(src, offset);
					break;
				}
				
				case Variant.VT_FILETIME: 
				{
					/*
					* Read a FILETIME object. In Java it is represented
					* as a Date object.
					*/
					
					long low = LittleEndian.GetUInt(src, offset);
					offset += LittleEndian.INT_SIZE;
					
					long high = LittleEndian.GetUInt(src, offset);
					_value = Util.FiletimeToDate((int) high, (int) low);
					break;
				}
				
				case Variant.VT_LPSTR: 
				{
					/*
					* Read a byte string. In Java it is represented as a
					* String object. The 0x00 bytes at the end must be
					* stripped.
					*
					* FIXME: Reading an 8-bit string should pay attention
					* to the codepage. Currently the byte making out the
					* property's value are interpreted according to the
					* platform's default character set.
					*/
					
					int first = offset + LittleEndian.INT_SIZE;
					long last = first + LittleEndian.GetUInt(src, offset) - 1;
					offset += LittleEndian.INT_SIZE;
					while (src[(int) last] == 0 && first <= last)
						last--;

					//if (codepage < 1)
						codepage = 1252;
					//Console.WriteLine(codepage);

					char[] bsrc = Encoding.GetEncoding("windows-1252").GetChars(src);

					_value = new string(bsrc, (int) first, (int) (last - first + 1));
					break;
				}
				
				case Variant.VT_LPWSTR: 
				{
					/*
					* Read a Unicode string. In Java it is represented as
					* a String object. The 0x00 bytes at the end must be
					* stripped.
					*/
					
					int first = offset + LittleEndian.INT_SIZE;
					long last = first + LittleEndian.GetUInt(src, offset) - 1;
					long l = last - first;
					offset += LittleEndian.INT_SIZE;
					StringBuilder b = new StringBuilder((int) (last - first));
					for (int i = 0; i <= l; i++)
					{
						
						int i1 = offset + (i * 2);
						
						int i2 = i1 + 1;
						b.Append((char) ((src[i2] << 8) + src[i1]));
					}
					/* Strip 0x00 characters from the end of the string: */
					while (b[b.Length - 1] == 0x00)
						b.Length -= 1;
					_value = b.ToString();
					break;
				}
				
				case Variant.VT_CF: 
				{
					
					byte[] v = new byte[length];
					for (int i = 0; i < length; i++)
						v[i] = src[(int) (offset + i)];
					_value = v;
					break;
				}
				
				case Variant.VT_BOOL: 
				{
					/*
					* The first four bytes in src, from src[offset] to
					* src[offset + 3] contain the DWord for VT_BOOL, so
					* skip it, we don't need it.
					*/
					// final int first = offset + LittleEndian.INT_SIZE;
					long _bool = LittleEndian.GetUInt(src, offset);
					if (_bool != 0)
						_value = true;
					else
						_value = false;
					break;
				}
				
				default: 
				{
					
					byte[] v = new byte[length];
					for (int i = 0; i < length; i++)
						v[i] = src[(int) (offset + i)];
					_value = v;
					break;
				}
				
			}
			return _value;
		}
	}
}