// 
// Util.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;

namespace Poi.Net.HPSF
{
	
	/// <summary> Provides various static utility methods.
	/// 
	/// </summary>
	public class Util
	{
		
		/// <summary> Checks whether two byte arrays <var>a</var> and <var>b</var>
		/// are equal. They are equal
		/// 
		/// <ul>
		/// 
		/// <li>if they have the same length and</li>
		/// 
		/// <li>if for each <var>i</var> with
		/// <var>i</var>&nbsp;&gt;=&nbsp;0 and
		/// <var>i</var>&nbsp;&lt;&nbsp;<var>a.length</var> holds
		/// <var>a</var>[<var>i</var>]&nbsp;==&nbsp;<var>b</var>[<var>i</var>].</li>
		/// 
		/// </ul>
		/// 
		/// </summary>
		/// <param name="a">The first byte array
		/// </param>
		/// <param name="b">The first byte array
		/// </param>
		/// <returns> <code>true</code> if the byte arrays are equal, else
		/// <code>false</code>
		/// </returns>
		public static bool Equal(byte[] a, byte[] b)
		{
			if (a.Length != b.Length)
				return false;
			for (int i = 0; i < a.Length; i++)
				if (a[i] != b[i])
					return false;
			return true;
		}
		
		
		
		/// <summary> Copies a part of a byte array into another byte array.
		/// 
		/// </summary>
		/// <param name="src">The source byte array.
		/// </param>
		/// <param name="srcOffset">Offset in the source byte array.
		/// </param>
		/// <param name="length">The number of bytes to copy.
		/// </param>
		/// <param name="dst">The destination byte array.
		/// </param>
		/// <param name="dstOffset">Offset in the destination byte array.
		/// </param>
		public static void Copy(byte[] src, int srcOffset, int length, byte[] dst, int dstOffset)
		{
			for (int i = 0; i < length; i++)
				dst[dstOffset + i] = src[srcOffset + i];
		}
		
		
		
		/// <summary> Concatenates the contents of several byte arrays into a
		/// single one.
		/// 
		/// </summary>
		/// <param name="byteArrays">The byte arrays to be concatened.
		/// </param>
		/// <returns> A new byte array containing the concatenated byte
		/// arrays.
		/// </returns>
		public static byte[] Cat(byte[][] byteArrays)
		{
			int capacity = 0;
			for (int i = 0; i < byteArrays.Length; i++)
				capacity += byteArrays[i].Length;
			
			byte[] result = new byte[capacity];
			int r = 0;
			for (int i = 0; i < byteArrays.Length; i++)
				for (int j = 0; j < byteArrays[i].Length; j++)
					result[r++] = byteArrays[i][j];
			return result;
		}
		
		
		
		/// <summary> Copies bytes from a source byte array into a new byte
		/// array.
		/// 
		/// </summary>
		/// <param name="src">Copy from this byte array.
		/// </param>
		/// <param name="offset">Start copying here.
		/// </param>
		/// <param name="length">Copy this many bytes.
		/// </param>
		/// <returns> The new byte array. Its length is number of copied bytes.
		/// </returns>
		public static byte[] Copy(byte[] src, int offset, int length)
		{
			
			byte[] result = new byte[length];
			Copy(src, offset, length, result, 0);
			return result;
		}
		
		
		
		/// <summary> The difference between the Windows epoch (1601-01-01
		/// 00:00:00) and the Unix epoch (1970-01-01 00:00:00) in
		/// milliseconds: 11644473600000L. (Use your favorite spreadsheet
		/// program to verify the correctness of this value. By the way,
		/// did you notice that you can tell from the epochs which
		/// operating system is the modern one? :-))
		/// </summary>
		public const long EPOCH_DIFF = 11644473600000L;
		
		
		/// <summary> Converts a Windows FILETIME into a  Date}. The Windows
		/// FILETIME structure holds a date and time associated with a
		/// file. The structure identifies a 64-bit integer specifying the
		/// number of 100-nanosecond intervals which have passed since
		/// January 1, 1601. This 64-bit value is split into the two double
		/// words stored in the structure.
		/// 
		/// </summary>
		/// <param name="high">The higher double word of the FILETIME structure.
		/// </param>
		/// <param name="low">The lower double word of the FILETIME structure.
		/// </param>
		/// <returns> The Windows FILETIME as a  Date}.
		/// </returns>
		public static DateTime FiletimeToDate(int high, int low)
		{
			
			long filetime = ((long) high) << 32 | (low & unchecked((int) 0xffffffffL));
			long ms_since_16010101 = filetime / (1000 * 10);
			long ms_since_19700101 = ms_since_16010101 - EPOCH_DIFF;
			
			return new DateTime((ms_since_19700101 * TimeSpan.TicksPerMillisecond) + 621355968000000000);
		}
	}
}