// 
// PropertyIDMap.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


using System;
using System.Collections;

namespace Poi.Net.HPSF.Wellknown
{
	
	/// <summary> This is a dictionary which maps property ID values to property
	/// ID strings.
	/// 
	/// The methods  #getSummaryInformationProperties} and 
	/// #getDocumentSummaryInformationProperties} return singleton 
	/// PropertyIDMap}s. An application that wants to extend these maps
	/// should treat them as unmodifiable, copy them and modifiy the
	/// copies.
	/// 
	/// <strong>FIXME:</strong> Make the singletons
	/// unmodifiable. However, since this requires to use a  HashMap}
	/// delegate instead of extending  HashMap} and thus requires a
	/// lot of stupid typing. I won't do that for the time being.
	/// 
	/// </summary>
	
	[Serializable]
	public class PropertyIDMap:Hashtable
	{
		/*
		* The following definitions are for property IDs in the first
		* (and only) section of the Summary Information property set.
		*/
		public const int PID_TITLE = 2;
		public const int PID_SUBJECT = 3;
		public const int PID_AUTHOR = 4;
		public const int PID_KEYWORDS = 5;
		public const int PID_COMMENTS = 6;
		public const int PID_TEMPLATE = 7;
		public const int PID_LASTAUTHOR = 8;
		public const int PID_REVNUMBER = 9;
		public const int PID_EDITTIME = 10;
		public const int PID_LASTPRINTED = 11;
		public const int PID_CREATE_DTM = 12;
		public const int PID_LASTSAVE_DTM = 13;
		public const int PID_PAGECOUNT = 14;
		public const int PID_WORDCOUNT = 15;
		public const int PID_CHARCOUNT = 16;
		public const int PID_THUMBNAIL = 17;
		public const int PID_APPNAME = 18;
		public const int PID_SECURITY = 19;
		
		
		
		/*
		* The following definitions are for property IDs in the first
		* section of the Document Summary Information property set.
		*/
		
		/// <summary> The entry is a dictionary.</summary>
		public const int PID_DICTIONARY = 0;
		
		/// <summary> The entry denotes a code page.</summary>
		public const int PID_CODEPAGE = 1;
		
		/// <summary> The entry is a string denoting the category the file belongs
		/// to, e.g. review, memo, etc. This is useful to find documents of
		/// same type.
		/// </summary>
		public const int PID_CATEGORY = 2;
		
		/// <summary> Target format for power point presentation, e.g. 35mm,
		/// printer, video etc.
		/// </summary>
		public const int PID_PRESFORMAT = 3;
		
		/// <summary> Number of bytes.</summary>
		public const int PID_BYTECOUNT = 4;
		
		/// <summary> Number of lines.</summary>
		public const int PID_LINECOUNT = 5;
		
		/// <summary> Number of paragraphs.</summary>
		public const int PID_PARCOUNT = 6;
		
		/// <summary> Number of slides in a power point presentation.</summary>
		public const int PID_SLIDECOUNT = 7;
		
		/// <summary> Number of slides with notes.</summary>
		public const int PID_NOTECOUNT = 8;
		
		/// <summary> Number of hidden slides.</summary>
		public const int PID_HIDDENCOUNT = 9;
		
		/// <summary> Number of multimedia clips, e.g. sound or video.</summary>
		public const int PID_MMCLIPCOUNT = 10;
		
		/// <summary> This entry is set to -1 when scaling of the thumbnail is
		/// desired. Otherwise the thumbnail should be cropped.
		/// </summary>
		public const int PID_SCALE = 11;
		
		/// <summary> This entry denotes an internally used property. It is a
		/// vector of variants consisting of pairs of a string (VT_LPSTR)
		/// and a number (VT_I4). The string is a heading name, and the
		/// number tells how many document parts are under that
		/// heading.
		/// </summary>
		public const int PID_HEADINGPAIR = 12;
		
		/// <summary> This entry contains the names of document parts (word: names
		/// of the documents in the master document, excel: sheet names,
		/// power point: slide titles, binder: document names).
		/// </summary>
		public const int PID_DOCPARTS = 13;
		
		/// <summary> This entry contains the name of the project manager.</summary>
		public const int PID_MANAGER = 14;
		
		/// <summary> This entry contains the company name.</summary>
		public const int PID_COMPANY = 15;
		
		/// <summary> If this entry is -1 the links are dirty and should be
		/// re-evaluated.
		/// </summary>
		public const int PID_LINKSDIRTY = 16;
		
		
		
		/// <summary> Returns the Summary Information properties singleton.</summary>
		public static PropertyIDMap SummaryInformationProperties {
			get {
				if (summaryInformationProperties == null)
				{
					PropertyIDMap m = new PropertyIDMap(18, (float) 1.0);
					m.Put(PID_TITLE, "PID_TITLE");
					m.Put(PID_SUBJECT, "PID_SUBJECT");
					m.Put(PID_AUTHOR, "PID_AUTHOR");
					m.Put(PID_KEYWORDS, "PID_KEYWORDS");
					m.Put(PID_COMMENTS, "PID_COMMENTS");
					m.Put(PID_TEMPLATE, "PID_TEMPLATE");
					m.Put(PID_LASTAUTHOR, "PID_LASTAUTHOR");
					m.Put(PID_REVNUMBER, "PID_REVNUMBER");
					m.Put(PID_EDITTIME, "PID_EDITTIME");
					m.Put(PID_LASTPRINTED, "PID_LASTPRINTED");
					m.Put(PID_CREATE_DTM, "PID_CREATE_DTM");
					m.Put(PID_LASTSAVE_DTM, "PID_LASTSAVE_DTM");
					m.Put(PID_PAGECOUNT, "PID_PAGECOUNT");
					m.Put(PID_WORDCOUNT, "PID_WORDCOUNT");
					m.Put(PID_CHARCOUNT, "PID_CHARCOUNT");
					m.Put(PID_THUMBNAIL, "PID_THUMBNAIL");
					m.Put(PID_APPNAME, "PID_APPNAME");
					m.Put(PID_SECURITY, "PID_SECURITY");
					summaryInformationProperties = m;
				}
				return summaryInformationProperties;
			}
			
		}
		/// <summary> Returns the Document Summary Information properties
		/// singleton.
		/// 
		/// </summary>
		/// <returns> The Document Summary Information properties singleton.
		/// </returns>
		public static PropertyIDMap DocumentSummaryInformationProperties 
		{
			get 
			{
				if (documentSummaryInformationProperties == null)
				{
					PropertyIDMap m = new PropertyIDMap(17, (float) 1.0);
					m.Put(PID_DICTIONARY, "PID_DICTIONARY");
					m.Put(PID_CODEPAGE, "PID_CODEPAGE");
					m.Put(PID_CATEGORY, "PID_CATEGORY");
					m.Put(PID_PRESFORMAT, "PID_PRESFORMAT");
					m.Put(PID_BYTECOUNT, "PID_BYTECOUNT");
					m.Put(PID_LINECOUNT, "PID_LINECOUNT");
					m.Put(PID_PARCOUNT, "PID_PARCOUNT");
					m.Put(PID_SLIDECOUNT, "PID_SLIDECOUNT");
					m.Put(PID_NOTECOUNT, "PID_NOTECOUNT");
					m.Put(PID_HIDDENCOUNT, "PID_HIDDENCOUNT");
					m.Put(PID_MMCLIPCOUNT, "PID_MMCLIPCOUNT");
					m.Put(PID_SCALE, "PID_SCALE");
					m.Put(PID_HEADINGPAIR, "PID_HEADINGPAIR");
					m.Put(PID_DOCPARTS, "PID_DOCPARTS");
					m.Put(PID_MANAGER, "PID_MANAGER");
					m.Put(PID_COMPANY, "PID_COMPANY");
					m.Put(PID_LINKSDIRTY, "PID_LINKSDIRTY");
					documentSummaryInformationProperties = m;
				}
				return documentSummaryInformationProperties;
			}
			
		}
		

		/// <summary> Contains the summary information property ID values and
		/// associated strings. See the overall HPSF documentation for
		/// details!
		/// </summary>
		private static PropertyIDMap summaryInformationProperties;
		
		/// <summary> Contains the summary information property ID values and
		/// associated strings. See the overall HPSF documentation for
		/// details!
		/// </summary>
		private static PropertyIDMap documentSummaryInformationProperties;
		
		
		
		/// <summary> Creates a  PropertyIDMap}.</summary>
		
		public PropertyIDMap(int initialCapacity, float loadFactor):base(initialCapacity, loadFactor)
		{
		}
		
		
		
		/// <summary> Puts a ID string for an ID into the 
		/// PropertyIDMap}.
		/// 
		/// </summary>
		/// <param name="id">The ID.
		/// </param>
		/// <param name="idString">The ID string.
		/// </param>
		/// <returns> As specified by the  Map} interface, this method
		/// returns the previous value associated with the specified
		/// <var>id</var>, or <code>null</code> if there was no mapping for
		/// key.
		/// </returns>
		public virtual object Put(int id, string idString)
		{
			object tempObject;
			tempObject = this[(int) id];
			this[(int) id] = idString;
			return tempObject;
		}
		
		
		
		/// <summary> Gets the ID string for an ID from the 
		/// PropertyIDMap.
		/// 
		/// </summary>
		/// <param name="id">The ID.
		/// </param>
		/// <returns> The ID string associated with <var>id</var>.
		/// </returns>
		public virtual object Get(int id) {
			return this[(int) id];
		}
		
		
		
		/// <summary> For the most basic testing.</summary>
		[STAThread]
		public static void Main2(string[] args)
		{
			PropertyIDMap s1 = SummaryInformationProperties;
			PropertyIDMap s2 = DocumentSummaryInformationProperties;
			//Console.WriteLine("s1: " + HelperClass.CollectionToString(s1));
			//Console.WriteLine("s2: " + HelperClass.CollectionToString(s2));
		}
	}
}