// 
// SectionIDMap.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


using System;
using System.Text;
using System.Collections;

namespace Poi.Net.HPSF.Wellknown
{
	
	/// <summary> Maps section format IDs to PropertyIDMaps. It is
	/// initialized with two well-known section format IDs: those of the
	/// <tt>\005SummaryInformation</tt> stream and the
	/// <tt>\005DocumentSummaryInformation</tt> stream.
	/// 
	/// If you have a section format ID you can use it as a key to query
	/// this map.  If you get a  PropertyIDMap returned your section
	/// is well-known and you can query the  PropertyIDMap for PID
	/// strings. If you get back <code>null</code> you are on your own.
	/// 
	/// This  Map} expects the byte arrays of section format IDs
	/// as keys. A key maps to a  PropertyIDMap describing the
	/// property IDs in sections with the specified section format ID.
	/// 
	/// </summary>
	
	[Serializable]
	public class SectionIDMap:Hashtable
	{
		/// <summary> Returns the singleton instance of the default 
		/// SectionIDMap}.
		/// 
		/// </summary>
		/// <returns> The instance value
		/// </returns>
		public static SectionIDMap Instance {
			get {
				if (defaultMap == null)
				{
					
					SectionIDMap m = new SectionIDMap();
					m.Put(SUMMARY_INFORMATION_ID, PropertyIDMap.SummaryInformationProperties);
					m.Put(DOCUMENT_SUMMARY_INFORMATION_ID, PropertyIDMap.DocumentSummaryInformationProperties);
					defaultMap = m;
				}
				return defaultMap;
			}
			
		}
		/// <summary> Returns the  PropertyIDMap for a given section format
		/// ID.
		/// 
		/// </summary>
		/// <param name="sectionFormatID">A section format ID as a <tt>byte[]</tt> .
		/// </param>
		/// <deprecated> Use  #get(byte[]) instead!
		/// </deprecated>
		
		public override object this[object sectionFormatID] {
			get {
				return Get((byte[]) sectionFormatID);
			}
			
			set {
				base[sectionFormatID] = value;
			}
			
		}
		
		/// <summary> The SummaryInformation's section's format ID.</summary>
		
		public static readonly byte[] SUMMARY_INFORMATION_ID = new byte[]{unchecked((byte) 0xF2), 
				unchecked((byte) 0x9F), unchecked((byte) 0x85), unchecked((byte) 0xE0), (byte) 0x4F, 
				unchecked((byte) 0xF9), (byte) 0x10, (byte) 0x68, unchecked((byte) 0xAB), 
				unchecked((byte) 0x91), (byte) 0x08, (byte) 0x00, (byte) 0x2B, (byte) 0x27, 
				unchecked((byte) 0xB3), unchecked((byte) 0xD9)};
		
		/// <summary> The DocumentSummaryInformation's first section's format
		/// ID. The second section has a different format ID which is not
		/// well-known.
		/// </summary>
		
		public static readonly byte[] DOCUMENT_SUMMARY_INFORMATION_ID = new byte[]{unchecked((byte) 0xD5), 
				unchecked((byte) 0xCD), unchecked((byte) 0xD5), (byte) 0x02, (byte) 0x2E, unchecked((byte) 0x9C), 
				(byte) 0x10, (byte) 0x1B, unchecked((byte) 0x93), unchecked((byte) 0x97), (byte) 0x08, (byte) 0x00, 
				(byte) 0x2B, (byte) 0x2C, unchecked((byte) 0xF9), unchecked((byte) 0xAE)};
		
		public const string UNDEFINED = "[undefined]";
		
		private static SectionIDMap defaultMap;
		
		
		
		/// <summary> Returns the property ID string that is associated with a
		/// given property ID in a section format ID's namespace.
		/// 
		/// </summary>
		/// <param name="sectionFormatID">Each section format ID has its own name
		/// space of property ID strings and thus must be specified.
		/// </param>
		/// <param name="pid">The property ID
		/// </param>
		/// <returns> The well-known property ID string associated with the
		/// property ID <var>pid</var> in the name space spanned by <var>
		/// sectionFormatID</var> . If the <var>pid</var>
		/// /<var>sectionFormatID </var> combination is not well-known, the
		/// string "[undefined]" is returned.
		/// </returns>
		public static string GetPIDString(byte[] sectionFormatID, int pid)
		{
			
			PropertyIDMap m = (PropertyIDMap) Instance.Get(sectionFormatID);
			if (m == null)
				return UNDEFINED;
			else
			{
				
				string s = (string) m.Get(pid);
				if ((object) s == null)
					return UNDEFINED;
				return s;
			}
		}
		
		
		
		/// <summary> Returns the  PropertyIDMap for a given section format
		/// ID.
		/// </summary>
		public PropertyIDMap Get(byte[] sectionFormatID)
		{
			return (PropertyIDMap) base[new string(UTF8Encoding.UTF8.GetChars(sectionFormatID))];
		}
		
		
		
		/// <summary> Associates a section format ID with a 
		/// PropertyIDMap.
		/// </summary>
		public object Put(byte[] sectionFormatID, PropertyIDMap propertyIDMap)
		{
			object tempObject;
			
			tempObject = base[new string(UTF8Encoding.UTF8.GetChars(sectionFormatID))];
			base[new string(UTF8Encoding.UTF8.GetChars(sectionFormatID))] = propertyIDMap;
			return tempObject;
		}
		
		
		
		/// <deprecated> Use  #put(byte[], PropertyIDMap) instead!
		/// </deprecated>
		
		public object Put(object key, object _value)
		{
			return Put((byte[]) key, (PropertyIDMap) _value);
		}
	}
}