// 
// HWPFDocument.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.FileSystem;

using Poi.Net.PoiFS.Common;
using Poi.Net.HWPF.UserModel;
using Poi.Net.HWPF.SPRM;
using Poi.Net.HWPF.Model;
using Poi.Net.HWPF.Model.IO;

namespace Poi.Net.HWPF
{
	
	
	/// <summary> 
	/// This class acts as the bucket that we throw all of the Word data structures
	/// into.
	/// 
	/// </summary>
	/// 
	public class HWPFDocument : ICloneable
	{
		virtual public StyleSheet StyleSheet {
			get { return _ss; }
			
		}
		virtual public FileInformationBlock FileInformationBlock {
			get { return _fib; }
			
		}
		virtual public DocumentProperties DocProperties {
			get { return _dop; }
			
		}
		virtual public Range Range {
			get {
				// hack to get the ending cp of the document, Have to revisit this.
				IList text = _tpt.TextPieces;
				PropertyNode p = (PropertyNode) text[text.Count - 1];
				
				return new Range(0, p.End, this);
			}
			
		}
		virtual public ListTables ListTables {
			get { return _lt; }
			
		}
		virtual public CHPBinTable CharacterTable {
			get { return _cbt; }
			
		}
		virtual public PAPBinTable ParagraphTable {
			get { return _pbt; }
			
		}
		virtual public SectionTable SectionTable {
			get { return _st; }
			
		}
		virtual public TextPieceTable TextTable {
			get { return _cft.TextPieceTable; }
			
		}
		virtual public byte[] DataStream {
			get { return _dataStream; }
			
		}
		virtual public FontTable FontTable {
			get {
				return _ft;
			}
			
		}
		/// <summary>The FIB</summary>
		protected internal FileInformationBlock _fib;
		
		/// <summary>main document stream buffer</summary>
		private byte[] _mainStream;
		
		/// <summary>table stream buffer</summary>
		private byte[] _tableStream;
		
		/// <summary>data stream buffer</summary>
		protected internal byte[] _dataStream;
		
		/// <summary>Document wide Properties</summary>
		protected internal DocumentProperties _dop;
		
		/// <summary>Contains text of the document wrapped in a obfuscated Word data
		/// structure
		/// </summary>
		protected internal ComplexFileTable _cft;
		
		protected internal TextPieceTable _tpt;
		
		/// <summary>Contains formatting properties for text</summary>
		protected internal CHPBinTable _cbt;
		
		/// <summary>Contains formatting properties for paragraphs</summary>
		protected internal PAPBinTable _pbt;
		
		/// <summary>Contains formatting properties for sections.</summary>
		protected internal SectionTable _st;
		
		/// <summary>Holds styles for this document.</summary>
		protected internal StyleSheet _ss;
		
		/// <summary>Holds fonts for this document.</summary>
		protected internal FontTable _ft;
		
		/// <summary>Hold list tables </summary>
		protected internal ListTables _lt;
		
		protected internal HWPFDocument()
		{
		}
		
		/// <summary> This constructor loads a Word document from an InputStream.
		/// 
		/// </summary>
		/// <param name="istream">The InputStream that contains the Word document.
		/// </param>
		/// <throws>  IOException If there is an unexpected IOException from the passed </throws>
		/// <summary>         in InputStream.
		/// </summary>
		public HWPFDocument(Stream istream)
		{
			//do Ole stuff
			POIFSFileSystem filesystem = new POIFSFileSystem(istream);
			Init(filesystem);
		}

		public HWPFDocument(POIFSFileSystem filesystem)
		{
			Init(filesystem);
		}

		public void Init(POIFSFileSystem filesystem)
		{
			
			// read in the main stream.
			DocumentEntry documentProps = (DocumentEntry) filesystem.Root.GetEntry("WordDocument");

			_mainStream = new byte[documentProps.Size];
			filesystem.CreateDocumentInputStream("WordDocument").Read(_mainStream);
			
			// use the fib to determine the name of the table stream.
			_fib = new FileInformationBlock(_mainStream);
			
			string name = "0Table";
			if (_fib.FWhichTblStm)
			{
				name = "1Table";
			}
			
			// read in the table stream.
			DocumentEntry tableProps = (DocumentEntry) filesystem.Root.GetEntry(name);
			_tableStream = new byte[tableProps.Size];
			filesystem.CreateDocumentInputStream(name).Read(_tableStream);
			
			_fib.FillVariableFields(_mainStream, _tableStream);
			
			// read in the data stream.
			try
			{
				DocumentEntry dataProps = (DocumentEntry) filesystem.Root.GetEntry("Data");
				_dataStream = new byte[dataProps.Size];
				filesystem.CreateDocumentInputStream("Data").Read(_dataStream);
			}
			catch (FileNotFoundException ignored)
			{
				_dataStream = new byte[0];
			}
			
			// get the start of text in the main stream
			int fcMin = _fib.FcMin;
			
			// load up our standard structures.
			_dop = new DocumentProperties(_tableStream, _fib.FcDop);
			_cft = new ComplexFileTable(_mainStream, _tableStream, _fib.FcClx, fcMin);
			_tpt = _cft.TextPieceTable;
			_cbt = new CHPBinTable(_mainStream, _tableStream, _fib.FcPlcfbteChpx, _fib.LcbPlcfbteChpx, fcMin);
			_pbt = new PAPBinTable(_mainStream, _tableStream, _dataStream, _fib.FcPlcfbtePapx, _fib.LcbPlcfbtePapx, fcMin);
			
			// Word XP puts in a zero filled buffer in front of the text and it screws
			// up my system for offsets. This is an adjustment.
			int cpMin = _tpt.CpMin;
			if (cpMin > 0)
			{
				_cbt.AdjustForDelete(0, 0, cpMin);
				_pbt.AdjustForDelete(0, 0, cpMin);
			}
			
			_st = new SectionTable(_mainStream, _tableStream, _fib.FcPlcfsed, _fib.LcbPlcfsed, fcMin, TextTable.TextPieces);
			_ss = new StyleSheet(_tableStream, _fib.FcStshf);
			_ft = new FontTable(_tableStream, _fib.FcSttbfffn, _fib.LcbSttbfffn);
			
			int listOffset = _fib.FcPlcfLst;
			int lfoOffset = _fib.FcPlfLfo;
			if (listOffset != 0 && _fib.LcbPlcfLst != 0)
			{
				_lt = new ListTables(_tableStream, _fib.FcPlcfLst, _fib.FcPlfLfo);
			}
			
			PlexOfCps plc = new PlexOfCps(_tableStream, _fib.FcPlcffldMom, _fib.LcbPlcffldMom, 2);
			for (int x = 0; x < plc.Length; x++)
			{
				GenericPropertyNode node = plc.GetProperty(x);
				byte[] fld = node.Bytes;
				//unused: int breakpoint = 0;
			}
		}
		
		/// <summary> Returns the character length of a document.</summary>
		/// <returns>
		/// </returns>
		public int CharacterLength {
			get {

				IList textPieces = _tpt.TextPieces;
			
				int length = 0;
			
				foreach(TextPiece tp in textPieces)
				{
					length += tp.CharacterLength;
				}
				return length;
			}
		}
		/// <summary> Writes out the word file that is represented by an instance of this class.
		/// 
		/// </summary>
		/// <param name="out">The OutputStream to write to.
		/// </param>
		/// <throws>  IOException If there is an unexpected IOException from the passed </throws>
		/// <summary>         in OutputStream.
		/// </summary>
		public virtual void Write(Stream outStream)
		{
			// initialize our streams for writing.
			HWPFFileSystem docSys = new HWPFFileSystem();
			HWPFOutputStream mainStream = docSys.GetStream("WordDocument");
			HWPFOutputStream tableStream = docSys.GetStream("1Table");
			//HWPFOutputStream dataStream = docSys.GetStream("Data");
			int tableOffset = 0;
			
			// FileInformationBlock fib = (FileInformationBlock)_fib.clone();
			// clear the offsets and sizes in our FileInformationBlock.
			_fib.ClearOffsetsSizes();
			
			// determine the FileInformationBLock size
			int fibSize = _fib.Size;
			fibSize += POIFSConstants.BIG_BLOCK_SIZE - (fibSize % POIFSConstants.BIG_BLOCK_SIZE);
			
			// preserve space for the FileInformationBlock because we will be writing
			// it after we write everything else.
			byte[] placeHolder = new byte[fibSize];
			mainStream.Write(placeHolder, 0, placeHolder.Length);
			int mainOffset = mainStream.Offset;
			
			// write out the StyleSheet.
			_fib.FcStshf = tableOffset;
			_ss.WriteTo(tableStream);
			_fib.LcbStshf = tableStream.Offset - tableOffset;
			tableOffset = tableStream.Offset;
			
			// get fcMin and fcMac because we will be writing the actual text with the
			// complex table.
			int fcMin = mainOffset;
			
			// write out the Complex table, includes text.
			_fib.FcClx = tableOffset;
			_cft.WriteTo(docSys);
			_fib.LcbClx = tableStream.Offset - tableOffset;
			tableOffset = tableStream.Offset;
			int fcMac = mainStream.Offset;
			
			// write out the CHPBinTable.
			_fib.FcPlcfbteChpx = tableOffset;
			_cbt.WriteTo(docSys, fcMin);
			_fib.LcbPlcfbteChpx = tableStream.Offset - tableOffset;
			tableOffset = tableStream.Offset;
			
			// write out the PAPBinTable.
			_fib.FcPlcfbtePapx = tableOffset;
			_pbt.WriteTo(docSys, fcMin);
			_fib.LcbPlcfbtePapx = tableStream.Offset - tableOffset;
			tableOffset = tableStream.Offset;
			
			// write out the SectionTable.
			_fib.FcPlcfsed = tableOffset;
			_st.WriteTo(docSys, fcMin);
			_fib.LcbPlcfsed = tableStream.Offset - tableOffset;
			tableOffset = tableStream.Offset;
			
			// write out the list tables
			if (_lt != null)
			{
				_fib.FcPlcfLst = tableOffset;
				_lt.WriteListDataTo(tableStream);
				_fib.LcbPlcfLst = tableStream.Offset - tableOffset;
				
				_fib.FcPlfLfo = tableStream.Offset;
				_lt.WriteListOverridesTo(tableStream);
				_fib.LcbPlfLfo = tableStream.Offset - tableOffset;
				tableOffset = tableStream.Offset;
			}
			
			// write out the FontTable.
			_fib.FcSttbfffn = tableOffset;
			_ft.WriteTo(docSys);
			_fib.LcbSttbfffn = tableStream.Offset - tableOffset;
			tableOffset = tableStream.Offset;
			
			// write out the DocumentProperties.
			_fib.FcDop = tableOffset;
			byte[] buf = new byte[_dop.Size];
			_fib.LcbDop = _dop.Size;
			_dop.Serialize(buf, 0);
			tableStream.Write(buf, 0, buf.Length);
			
			// set some variables in the FileInformationBlock.
			_fib.FcMin = fcMin;
			_fib.FcMac = fcMac;
			_fib.CbMac = mainStream.Offset;
			
			// make sure that the table, doc and data streams use big blocks.
			byte[] mainBuf = mainStream.ToArray();
			if (mainBuf.Length < 4096)
			{
				byte[] tempBuf = new byte[4096];
				Array.Copy(mainBuf, 0, tempBuf, 0, mainBuf.Length);
				mainBuf = tempBuf;
			}
			
			// write out the FileInformationBlock.
			//_fib.Serialize(mainBuf, 0);
			_fib.WriteTo(mainBuf, tableStream);
			
			byte[] tableBuf = tableStream.ToArray();
			if (tableBuf.Length < 4096)
			{
				byte[] tempBuf = new byte[4096];
				Array.Copy(tableBuf, 0, tempBuf, 0, tableBuf.Length);
				tableBuf = tempBuf;
			}
			
			byte[] dataBuf = _dataStream;
			if (dataBuf == null)
			{
				dataBuf = new byte[4096];
			}
			if (dataBuf.Length < 4096)
			{
				byte[] tempBuf = new byte[4096];
				Array.Copy(dataBuf, 0, tempBuf, 0, dataBuf.Length);
				dataBuf = tempBuf;
			}
			
			
			// spit out the Word document.
			POIFSFileSystem pfs = new POIFSFileSystem();
			pfs.CreateDocument(new MemoryStream(mainBuf), "WordDocument");
			pfs.CreateDocument(new MemoryStream(tableBuf), "1Table");
			pfs.CreateDocument(new MemoryStream(dataBuf), "Data");
			
			pfs.WriteFileSystem(outStream);
		}
		
		public virtual int RegisterList(HWPFList list)
		{
			if (_lt == null)
			{
				_lt = new ListTables();
			}
			return _lt.AddList(list.ListData, list.Override);
		}
		
		public virtual void Delete(int start, int length)
		{
			Range r = new Range(start, start + length, this);
			r.Delete();
		}
		
		/// <summary> Takes two arguments, 1) name of the Word file to read in 2) location to
		/// write it out at.
		/// </summary>
		/// <param name="">args
		/// </param>
		[STAThread]
		public static void _Main(string[] args)
		{
			
			try
			{
				
				HWPFDocument doc = new HWPFDocument(new FileStream(args[0], FileMode.Open, FileAccess.Read));
				Range r = doc.Range;
				string str = r.Text;

				//      CharacterRun run = new CharacterRun();
				//      run.SetBold(true);
				//      run.SetItalic(true);
				//      run.SetCapitalized(true);
				//
				//      Range range = doc.getRange();
				//      range.InsertBefore("Hello World!!! HAHAHAHAHA I DID IT!!!", run);
				//
				//      OutputStream out = new FileOutputStream(args[1]);
				//      doc.Write(out);
				//
				//      out.flush();
				//      out.close();
			}
			
			catch
			{
			}
		}

		public Object Clone() 
		{
			return this.MemberwiseClone();
		}

		//  public Object clone()
		//    throws CloneNotSupportedException
		//  {
		//    _tpt;
		//
		//    _cbt;
		//
		//    _pbt;
		//
		//    _st;
		//
		//  }
	}
}