// 
// CHPBinTable.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.Common;
using Poi.Net.Util;
using Poi.Net.HWPF.Model;
using Poi.Net.HWPF.Model.IO;

using Poi.Net.HWPF.SPRM;

namespace Poi.Net.HWPF.Model
{
	
	/// <summary> This class holds all of the character formatting properties.
	/// 
	/// </summary>

	public class CHPBinTable
	{
		/// <summary>List of character properties.</summary>
		protected internal ArrayList _textRuns = new ArrayList();

		virtual public IList TextRuns 
		{
			get {
				return _textRuns;
			}
			
		}
		
		public CHPBinTable()
		{
		}
		
		/// <summary> Constructor used to read a binTable in from a Word document.
		/// 
		/// </summary>
		/// <param name="">documentStream
		/// </param>
		/// <param name="">tableStream
		/// </param>
		/// <param name="">offset
		/// </param>
		/// <param name="">size
		/// </param>
		/// <param name="">fcMin
		/// </param>
		public CHPBinTable(byte[] documentStream, byte[] tableStream, int offset, int size, int fcMin)
		{
			PlexOfCps binTable = new PlexOfCps(tableStream, offset, size, 4);
			
			int length = binTable.Length;
			for (int x = 0; x < length; x++)
			{
				GenericPropertyNode node = binTable.GetProperty(x);
				
				int pageNum = LittleEndian.GetInt(node.Bytes);
				int pageOffset = POIFSConstants.BIG_BLOCK_SIZE * pageNum;
				
				CHPFormattedDiskPage cfkp = new CHPFormattedDiskPage(documentStream, pageOffset, fcMin);
				
				int fkpSize = cfkp.Size;
				
				for (int y = 0; y < fkpSize; y++)
				{
					_textRuns.Add(cfkp.GetCHPX(y));
				}
			}
		}
		
		public virtual void AdjustForDelete(int listIndex, int offset, int length)
		{
			int size = _textRuns.Count;
			int endMark = offset + length;
			int endIndex = listIndex;
			
			CHPX chpx = (CHPX) _textRuns[endIndex];
			while (chpx.End < endMark)
			{
				chpx = (CHPX) _textRuns[++endIndex];
			}
			if (listIndex == endIndex)
			{
				chpx = (CHPX) _textRuns[endIndex];
				chpx.End = (chpx.End - endMark) + offset;
			}
			else
			{
				chpx = (CHPX) _textRuns[listIndex];
				chpx.End = offset;
				for (int x = listIndex + 1; x < endIndex; x++)
				{
					chpx = (CHPX) _textRuns[x];
					chpx.Start = offset;
					chpx.End = offset;
				}
				chpx = (CHPX) _textRuns[endIndex];
				chpx.End = (chpx.End - endMark) + offset;
			}
			
			for (int x = endIndex + 1; x < size; x++)
			{
				chpx = (CHPX) _textRuns[x];
				chpx.Start = chpx.Start - length;
				chpx.End = chpx.End - length;
			}
		}
		
		public virtual void Insert(int listIndex, int cpStart, SprmBuffer buf)
		{
			CHPX insertChpx = new CHPX(cpStart, cpStart, buf);
			if (listIndex == _textRuns.Count)
			{
				_textRuns.Add(insertChpx);
			}
			else
			{
				CHPX chpx = (CHPX) _textRuns[listIndex];
				if (chpx.Start < cpStart)
				{
					CHPX clone = new CHPX(cpStart, chpx.End, chpx.SprmBuf);
					chpx.End = cpStart;
					
					_textRuns.Insert(listIndex + 1, insertChpx);
					_textRuns.Insert(listIndex + 2, clone);
				}
				else
				{
					_textRuns.Insert(listIndex, insertChpx);
				}
			}
		}
		
		public virtual void AdjustForInsert(int listIndex, int length)
		{
			int size = _textRuns.Count;
			CHPX chpx = (CHPX) _textRuns[listIndex];
			chpx.End = chpx.End + length;
			
			for (int x = listIndex + 1; x < size; x++)
			{
				chpx = (CHPX) _textRuns[x];
				chpx.Start = chpx.Start + length;
				chpx.End = chpx.End + length;
			}
		}
		
		public virtual void WriteTo(HWPFFileSystem sys, int fcMin)
		{
			
			HWPFOutputStream docStream = sys.GetStream("WordDocument");
			Stream tableStream = sys.GetStream("1Table");
			
			PlexOfCps binTable = new PlexOfCps(4);
			
			// each FKP must start on a 512 byte page.
			int docOffset = docStream.Offset;
			int mod = docOffset % POIFSConstants.BIG_BLOCK_SIZE;
			if (mod != 0)
			{
				byte[] padding = new byte[POIFSConstants.BIG_BLOCK_SIZE - mod];
				docStream.Write(padding, 0, padding.Length);
			}
			
			// get the page number for the first fkp
			docOffset = docStream.Offset;
			int pageNum = docOffset / POIFSConstants.BIG_BLOCK_SIZE;
			
			// get the ending fc
			int endingFc = ((PropertyNode) _textRuns[_textRuns.Count - 1]).End;
			endingFc += fcMin;
			
			
			ArrayList overflow = _textRuns;
			do 
			{
				PropertyNode startingProp = (PropertyNode) overflow[0];
				int start = startingProp.Start + fcMin;
				
				CHPFormattedDiskPage cfkp = new CHPFormattedDiskPage();
				cfkp.Fill(overflow);
				
				byte[] bufFkp = cfkp.ToByteArray(fcMin);
				docStream.Write(bufFkp, 0, bufFkp.Length);
				overflow = cfkp.Overflow;
				
				int end = endingFc;
				if (overflow != null)
				{
					end = ((PropertyNode) overflow[0]).Start + fcMin;
				}
				
				byte[] intHolder = new byte[4];
				LittleEndian.PutInt(intHolder, pageNum++);
				binTable.AddProperty(new GenericPropertyNode(start, end, intHolder));
			}
			while (overflow != null);
			byte[] bArray = binTable.ToByteArray();
			tableStream.Write(bArray, 0, bArray.Length);
		}
	}
}