// 
// CHPFormattedDiskPage.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Collections;

using Poi.Net.Util;

namespace Poi.Net.HWPF.Model
{
	
	/// <summary> Represents a CHP fkp. The style properties for paragraph and character runs
	/// are stored in fkps. There are PAP fkps for paragraph properties and CHP fkps
	/// for character run properties. The first part of the fkp for both CHP and PAP
	/// fkps consists of an array of 4 byte int offsets that represent a
	/// Paragraph's or Character run's text offset in the main stream. The ending
	/// offset is the next value in the array. For example, if an fkp has X number of
	/// Paragraph's stored in it then there are (x + 1) 4 byte ints in the beginning
	/// array. The number X is determined by the last byte in a 512 byte fkp.
	/// 
	/// CHP and PAP fkps also store the compressed styles(grpprl) that correspond to
	/// the offsets on the front of the fkp. The offset of the grpprls is determined
	/// differently for CHP fkps and PAP fkps.
	/// 
	/// </summary>

	public class CHPFormattedDiskPage:FormattedDiskPage
	{
	
		private const int FC_SIZE = 4;
		private ArrayList _chpxList = new ArrayList();
		private ArrayList _overFlow;

		virtual public ArrayList Overflow {
			get {
				return _overFlow;
			}
		}	
		
		public CHPFormattedDiskPage()
		{
		}
		
		/// <summary> This constructs a CHPFormattedDiskPage from a raw fkp (512 byte array
		/// read from a Word file).
		/// 
		/// </summary>
		/// <param name="fkp">The 512 byte array to read data from
		/// </param>
		public CHPFormattedDiskPage(byte[] documentStream, int offset, int fcMin):base(documentStream, offset)
		{
			
			for (int x = 0; x < _crun; x++)
			{
				_chpxList.Add(new CHPX(GetStart(x) - fcMin, GetEnd(x) - fcMin, GetGrpprl(x)));
			}
		}
		
		public virtual CHPX GetCHPX(int index)
		{
			return (CHPX) _chpxList[index];
		}
		
		public virtual void Fill(IList filler)
		{
			_chpxList.AddRange(filler);
		}
		
		/// <summary> Gets the chpx for the character run at index in this fkp.
		/// 
		/// </summary>
		/// <param name="index">The index of the chpx to get.
		/// </param>
		/// <returns> a chpx grpprl.
		/// </returns>
		protected internal override byte[] GetGrpprl(int index)
		{
			int chpxOffset = 2 * LittleEndian.GetUnsignedByte(_fkp, _offset + (((_crun + 1) * 4) + index));
			
			//optimization if offset == 0 use "Normal" style
			if (chpxOffset == 0)
			{
				return new byte[0];
			}
			
			int size = LittleEndian.GetUnsignedByte(_fkp, _offset + chpxOffset);
			
			byte[] chpx = new byte[size];
			
			Array.Copy(_fkp, _offset + ++chpxOffset, chpx, 0, size);
			return chpx;
		}
		
		protected internal virtual byte[] ToByteArray(int fcMin)
		{
			byte[] buf = new byte[512];
			int size = _chpxList.Count;
			int grpprlOffset = 511;
			int offsetOffset = 0;
			int fcOffset = 0;
			
			// total size is currently the size of one FC
			int totalSize = FC_SIZE + 2;
			
			int index = 0;
			for (; index < size; index++)
			{
				int grpprlLength = ((CHPX) _chpxList[index]).Grpprl.Length;
				
				// check to see if we have enough room for an FC, the grpprl offset,
				// the grpprl size byte and the grpprl.
				totalSize += (FC_SIZE + 2 + grpprlLength);
				// if size is uneven we will have to add one so the first grpprl falls
				// on a word boundary
				if (totalSize > 511 + (index % 2))
				{
					totalSize -= (FC_SIZE + 2 + grpprlLength);
					break;
				}
				
				// grpprls must fall on word boundaries
				if ((1 + grpprlLength) % 2 > 0)
				{
					totalSize += 1;
				}
			}
			
			// see if we couldn't fit some
			if (index != size)
			{
				_overFlow = new ArrayList();
				_overFlow.AddRange((IList) ((ArrayList) _chpxList).GetRange(index, size - index));
			}
			
			// index should equal number of CHPXs that will be in this fkp now.
			buf[511] = (byte) index;
			
			offsetOffset = (FC_SIZE * index) + FC_SIZE;
			//grpprlOffset =  offsetOffset + index + (grpprlOffset % 2);
			
			CHPX chpx = null;
			for (int x = 0; x < index; x++)
			{
				chpx = (CHPX) _chpxList[x];
				byte[] grpprl = chpx.Grpprl;
				
				LittleEndian.PutInt(buf, fcOffset, chpx.Start + fcMin);
				grpprlOffset -= (1 + grpprl.Length);
				grpprlOffset -= (grpprlOffset % 2);
				buf[offsetOffset] = (byte) (grpprlOffset / 2);
				buf[grpprlOffset] = (byte) grpprl.Length;
				Array.Copy(grpprl, 0, buf, grpprlOffset + 1, grpprl.Length);
				
				offsetOffset += 1;
				fcOffset += FC_SIZE;
			}
			// put the last chpx's end in
			LittleEndian.PutInt(buf, fcOffset, chpx.End + fcMin);
			return buf;
		}
	}
}