// 
// Ffn.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Praveen Mathew
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;

using Poi.Net.Util;

namespace Poi.Net.HWPF.Model
{
	
	/// <summary> FFN - Font Family Name. FFN is a data structure that stores the names of the Main
	/// Font and that of Alternate font as an array of characters. It has also a header
	/// that stores info about the whole structure and the fonts
	/// 
	/// </summary>

	public class Ffn
	{

		private int _cbFfnM1; //total length of FFN - 1.
		private byte _info;
		private static BitField _prq = new BitField(0x0003); // pitch request
		private static BitField _fTrueType = new BitField(0x0004); // when 1, font is a TrueType font
		private static BitField _ff = new BitField(0x0070);
		private short _wWeight; // base weight of font
		private byte _chs; // character set identifier
		private byte _ixchSzAlt; // index into ffn.szFfn to the name of
		// the alternate font
		private byte[] _panose = new byte[10]; //????
		private byte[] _fontSig = new byte[24]; //????
		
		// zero terminated string that records name of font, cuurently not
		// supporting Extended chars
		private char[] _xszFfn;
		
		// extra facilitator members
		private int _xszFfnLength;
		

		virtual public short Weight 
		{
			get {
				return _wWeight;
			}
			
		}
		virtual public byte Chs {
			get {
				return _chs;
			}
			
		}
		virtual public byte[] Panose {
			get {
				return _panose;
			}
			
		}
		virtual public byte[] FontSig {
			get {
				return _fontSig;
			}
			
		}
		virtual public int Size {
			get {
				return (_cbFfnM1 + 1);
			}
			
		}
		virtual public string MainFontName {
			get {
				int index = 0;
				for (; index < _xszFfnLength; index++)
				{
					if (_xszFfn[index] == '\x0000')
					{
						break;
					}
				}
				return new string(_xszFfn, 0, index);
			}
			
		}
		virtual public string AltFontName {
			get {
				int index = _ixchSzAlt;
				for (; index < _xszFfnLength; index++)
				{
					if (_xszFfn[index] == '\x0000')
					{
						break;
					}
				}
				return new string(_xszFfn, _ixchSzAlt, index);
			}
			
		}

		public virtual int CbFfnM1 
		{
			get { return _cbFfnM1; }
			set { this._cbFfnM1 = value; }
		}
		
		public Ffn(byte[] buf, int offset)
		{
			int offsetTmp = offset;
			
			_cbFfnM1 = LittleEndian.GetUnsignedByte(buf, offset);
			offset += LittleEndian.BYTE_SIZE;
			_info = buf[offset];
			offset += LittleEndian.BYTE_SIZE;
			_wWeight = LittleEndian.GetShort(buf, offset);
			offset += LittleEndian.SHORT_SIZE;
			_chs = buf[offset];
			offset += LittleEndian.BYTE_SIZE;
			_ixchSzAlt = buf[offset];
			offset += LittleEndian.BYTE_SIZE;
			
			// read panose and fs so we can write them back out.
			Array.Copy(buf, offset, _panose, 0, _panose.Length);
			offset += _panose.Length;
			Array.Copy(buf, offset, _fontSig, 0, _fontSig.Length);
			offset += _fontSig.Length;
			
			offsetTmp = offset - offsetTmp;
			_xszFfnLength = (this.Size - offsetTmp) / 2;
			_xszFfn = new char[_xszFfnLength];
			
			for (int i = 0; i < _xszFfnLength; i++)
			{
				_xszFfn[i] = (char) LittleEndian.GetShort(buf, offset);
				offset += LittleEndian.SHORT_SIZE;
			}
		}
		
		// changed protected to public
		public virtual byte[] ToByteArray()
		{
			int offset = 0;
			byte[] buf = new byte[this.Size];
			
			buf[offset] = (byte) _cbFfnM1;
			offset += LittleEndian.BYTE_SIZE;
			buf[offset] = _info;
			offset += LittleEndian.BYTE_SIZE;
			LittleEndian.PutShort(buf, offset, _wWeight);
			offset += LittleEndian.SHORT_SIZE;
			buf[offset] = _chs;
			offset += LittleEndian.BYTE_SIZE;
			buf[offset] = _ixchSzAlt;
			offset += LittleEndian.BYTE_SIZE;
			
			Array.Copy(_panose, 0, buf, offset, _panose.Length);
			offset += _panose.Length;
			Array.Copy(_fontSig, 0, buf, offset, _fontSig.Length);
			offset += _fontSig.Length;
			
			for (int i = 0; i < _xszFfn.Length; i++)
			{
				LittleEndian.PutShort(buf, offset, (short) _xszFfn[i]);
				offset += LittleEndian.SHORT_SIZE;
			}
			
			return buf;
		}
		
		public  override bool Equals(object o)
		{
			bool retVal = true;
			
			if (((Ffn) o).CbFfnM1 == _cbFfnM1)
			{
				if (((Ffn) o)._info == _info)
				{
					if (((Ffn) o)._wWeight == _wWeight)
					{
						if (((Ffn) o)._chs == _chs)
						{
							if (((Ffn) o)._ixchSzAlt == _ixchSzAlt)
							{
								if (ArrayHelper.Equals(((Ffn) o)._panose, _panose))
								{
									if (ArrayHelper.Equals(((Ffn) o)._fontSig, _fontSig))
									{
										if (!(ArrayHelper.Equals(((Ffn) o)._xszFfn, _xszFfn)))
											retVal = false;
									}
									else
										retVal = false;
								}
								else
									retVal = false;
							}
							else
								retVal = false;
						}
						else
							retVal = false;
					}
					else
						retVal = false;
				}
				else
					retVal = false;
			}
			else
				retVal = false;
			
			return retVal;
		}
	}
}