// 
// FontTable.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Praveen Mathew
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;

using Poi.Net.Util;
using Poi.Net.HWPF.Model;
using Poi.Net.HWPF.Model.IO;

namespace Poi.Net.HWPF.Model
{
	
	/// <summary> FontTable or in MS terminology sttbfffn is a common data structure written in all
	/// Word files. The sttbfffn is an sttbf where each string is an FFN structure instead
	/// of pascal-style strings. An sttbf is a string Table stored in file. Thus sttbffn
	/// is like an Sttbf with an array of FFN structures that stores the font name strings
	/// 
	/// </summary>

	public class FontTable
	{
		private short _stringCount; // how many strings are included in the string table
		private short _extraDataSz; // size in bytes of the extra data
		
		// added extra facilitator members
		private int lcbSttbfffn; // count of bytes in sttbfffn
		private int fcSttbfffn; // table stream offset for sttbfffn
		
		// FFN structure containing strings of font names
		private Ffn[] _fontNames = null;
		

		virtual public short StringCount 
		{
			get {
				return _stringCount;
			}
			
			set {
				this._stringCount = value;
			}
			
		}
		virtual public short ExtraDataSz {
			get {
				return _extraDataSz;
			}
			
		}
		virtual public Ffn[] FontNames {
			get {
				return _fontNames;
			}
			
		}
		virtual public int Size {
			get {
				return lcbSttbfffn;
			}
			
		}
		
		public FontTable(byte[] buf, int offset, int lcbSttbfffn)
		{
			this.lcbSttbfffn = lcbSttbfffn;
			this.fcSttbfffn = offset;
			
			_stringCount = LittleEndian.GetShort(buf, offset);
			offset += LittleEndian.SHORT_SIZE;
			_extraDataSz = LittleEndian.GetShort(buf, offset);
			offset += LittleEndian.SHORT_SIZE;
			
			_fontNames = new Ffn[_stringCount]; //Ffn corresponds to a Pascal style String in STTBF.
			
			for (int i = 0; i < _stringCount; i++)
			{
				_fontNames[i] = new Ffn(buf, offset);
				offset += _fontNames[i].Size;
			}
		}
		
		public virtual string GetMainFont(int chpFtc)
		{
			if (chpFtc >= _stringCount)
			{
				Console.WriteLine("Mismatch in chpFtc with stringCount");
				return null;
			}
			
			return _fontNames[chpFtc].MainFontName;
		}
		
		public virtual string GetAltFont(int chpFtc)
		{
			if (chpFtc >= _stringCount)
			{
				Console.WriteLine("Mismatch in chpFtc with stringCount");
				return null;
			}
			
			return _fontNames[chpFtc].AltFontName;
		}
		
		public virtual void WriteTo(HWPFFileSystem sys)
		{
			HWPFOutputStream tableStream = sys.GetStream("1Table");
			
			byte[] buf = new byte[LittleEndian.SHORT_SIZE];
			LittleEndian.PutShort(buf, _stringCount);
			tableStream.Write(buf, 0, buf.Length);
			LittleEndian.PutShort(buf, _extraDataSz);
			tableStream.Write(buf, 0, buf.Length);
			
			for (int i = 0; i < _fontNames.Length; i++)
			{
				byte[] bArray;
				bArray = _fontNames[i].ToByteArray();
				tableStream.Write(bArray, 0, bArray.Length);
			}
		}
		
		public  override bool Equals(object o)
		{
			bool retVal = true;
			
			if (((FontTable) o).StringCount == _stringCount)
			{
				if (((FontTable) o).ExtraDataSz == _extraDataSz)
				{
					Ffn[] fontNamesNew = ((FontTable) o).FontNames;
					for (int i = 0; i < _stringCount; i++)
					{
						if (!(_fontNames[i].Equals(fontNamesNew[i])))
							retVal = false;
					}
				}
				else
					retVal = false;
			}
			else
				retVal = false;
			
			
			return retVal;
		}
	}
}