// 
// ListData.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;

using Poi.Net.Util;

namespace Poi.Net.HWPF.Model
{
	
	public class ListData
	{
		private int _lsid;
		private int _tplc;
		private short[] _rgistd;
		private byte _info;
		private static BitField _fSimpleList = new BitField(0x1);
		private static BitField _fRestartHdn = new BitField(0x2);
		private byte _reserved;
		internal ListLevel[] _levels;

		virtual public int Lsid {
			get {
				return _lsid;
			}			
		}

		virtual public ListLevel[] Levels {
			get {
				return _levels;
			}			
		}

		public virtual int NumLevels {
			get { return _levels.Length; }
		}
		
		public ListData(int listID, bool numbered)
		{
			_lsid = listID;
			_rgistd = new short[9];
			
			for (int x = 0; x < 9; x++)
			{
				_rgistd[x] = (short) (StyleSheet.NIL_STYLE);
			}
			
			_levels = new ListLevel[9];
			
			for (int x = 0; x < _levels.Length; x++)
			{
				_levels[x] = new ListLevel(x, numbered);
			}
		}
		
		
		internal ListData(byte[] buf, int offset)
		{
			_lsid = LittleEndian.GetInt(buf, offset);
			offset += LittleEndian.INT_SIZE;
			_tplc = LittleEndian.GetInt(buf, offset);
			offset += LittleEndian.INT_SIZE;
			_rgistd = new short[9];
			for (int x = 0; x < 9; x++)
			{
				_rgistd[x] = LittleEndian.GetShort(buf, offset);
				offset += LittleEndian.SHORT_SIZE;
			}
			_info = buf[offset++];
			_reserved = buf[offset];
			if (_fSimpleList.GetValue(_info) > 0)
			{
				_levels = new ListLevel[1];
			}
			else
			{
				_levels = new ListLevel[9];
			}
		}
		
		public virtual void SetLevel(int index, ListLevel level)
		{
			_levels[index] = level;
		}
		
		/// <summary> Gets the level associated to a particular List at a particular index.
		/// 
		/// </summary>
		/// <param name="index">1-based index
		/// </param>
		/// <returns> a list level
		/// </returns>
		public virtual ListLevel GetLevel(int index)
		{
			return _levels[index - 1];
		}
		
		public virtual int GetLevelStyle(int index)
		{
			return _rgistd[index];
		}
		
		public virtual void SetLevelStyle(int index, int styleIndex)
		{
			_rgistd[index] = (short) styleIndex;
		}
		
		public  override bool Equals(object obj)
		{
			if (obj == null)
			{
				return false;
			}
			
			ListData lst = (ListData) obj;
			return lst._info == _info && ArrayHelper.Equals(lst._levels, _levels) && lst._lsid == _lsid && lst._reserved == _reserved && lst._tplc == _tplc && ArrayHelper.Equals(lst._rgistd, _rgistd);
		}
		
		internal virtual int ResetListID()
		{
			
			Random r = new Random();
			_lsid = (int) (r.NextDouble() * (double) ((DateTime.Now.Ticks - 621355968000000000) / 10000));
			return _lsid;
		}
		
		public virtual byte[] ToByteArray()
		{
			byte[] buf = new byte[28];
			int offset = 0;
			LittleEndian.PutInt(buf, _lsid);
			offset += LittleEndian.INT_SIZE;
			LittleEndian.PutInt(buf, offset, _tplc);
			offset += LittleEndian.INT_SIZE;
			for (int x = 0; x < 9; x++)
			{
				LittleEndian.PutShort(buf, offset, _rgistd[x]);
				offset += LittleEndian.SHORT_SIZE;
			}
			buf[offset++] = _info;
			buf[offset] = _reserved;
			return buf;
		}
	}
}