// 
// ListTables.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.Util;
using Poi.Net.HWPF.Model.IO;

namespace Poi.Net.HWPF.Model
{
	
	public class ListTables
	{
		private const int LIST_DATA_SIZE = 28;
		private const int LIST_FORMAT_OVERRIDE_SIZE = 16;
		
		
		internal Hashtable _listMap = new Hashtable();
		internal ArrayList _overrideList = new ArrayList();
		
		public ListTables()
		{
		}
		
		public ListTables(byte[] tableStream, int lstOffset, int lfoOffset)
		{
			// get the list data
			int length = LittleEndian.GetShort(tableStream, lstOffset);
			lstOffset += LittleEndian.SHORT_SIZE;
			int levelOffset = lstOffset + (length * LIST_DATA_SIZE);
			
			for (int x = 0; x < length; x++)
			{
				ListData lst = new ListData(tableStream, lstOffset);
				_listMap[(int) lst.Lsid] = lst;
				lstOffset += LIST_DATA_SIZE;
				
				int num = lst.NumLevels;
				for (int y = 0; y < num; y++)
				{
					ListLevel lvl = new ListLevel(tableStream, levelOffset);
					lst.SetLevel(y, lvl);
					levelOffset += lvl.SizeInBytes;
				}
			}
			
			// now get the list format overrides. The size is an int unlike the LST size
			length = LittleEndian.GetInt(tableStream, lfoOffset);
			lfoOffset += LittleEndian.INT_SIZE;
			int lfolvlOffset = lfoOffset + (LIST_FORMAT_OVERRIDE_SIZE * length);
			for (int x = 0; x < length; x++)
			{
				ListFormatOverride lfo = new ListFormatOverride(tableStream, lfoOffset);
				lfoOffset += LIST_FORMAT_OVERRIDE_SIZE;
				int num = lfo.NumOverrides;
				for (int y = 0; y < num; y++)
				{
					while (tableStream[lfolvlOffset] == 127 + 1)
					{
						lfolvlOffset++;
					}
					ListFormatOverrideLevel lfolvl = new ListFormatOverrideLevel(tableStream, lfolvlOffset);
					lfo.SetOverride(y, lfolvl);
					lfolvlOffset += lfolvl.SizeInBytes;
				}
				_overrideList.Add(lfo);
			}
		}
		
		public virtual int AddList(ListData lst, ListFormatOverride __override)
		{
			int lsid = lst.Lsid;
			
			while (_listMap[(int) lsid] != null)
			{
				lsid = lst.ResetListID();
				__override.Lsid = lsid;
			}
			_listMap[(int) lsid] = lst;
			_overrideList.Add(__override);
			return lsid;
		}
		
		public virtual void WriteListDataTo(HWPFOutputStream tableStream)
		{
			int[] intList = new int[_listMap.Keys.Count];
			int i = 0;

			foreach(int current in _listMap.Keys)
				intList[i++] = current;

			// use this stream as a buffer for the levels since their size varies.
			MemoryStream levelBuf = new MemoryStream();
			
			// use a byte array for the lists because we know their size.
			byte[] listBuf = new byte[intList.Length * LIST_DATA_SIZE];
			
			byte[] shortHolder = new byte[2];
			LittleEndian.PutShort(shortHolder, (short) intList.Length);
			tableStream.Write(shortHolder, 0, shortHolder.Length);
			byte[] bArray;

			for (int x = 0; x < intList.Length; x++)
			{
				
				ListData lst = (ListData) _listMap[intList[x]];
				bArray = lst.ToByteArray();
				tableStream.Write(bArray, 0, bArray.Length);
				ListLevel[] lvls = lst.Levels;
				for (int y = 0; y < lvls.Length; y++)
				{
					bArray = lvls[y].ToByteArray();
					levelBuf.Write(bArray, 0, bArray.Length);
				}
			}
			bArray = levelBuf.ToArray();
			tableStream.Write(bArray, 0, bArray.Length);
		}
		
		public virtual void WriteListOverridesTo(HWPFOutputStream tableStream)
		{
			
			// use this stream as a buffer for the levels since their size varies.
			MemoryStream levelBuf = new MemoryStream();
			
			int size = _overrideList.Count;
			
			byte[] intHolder = new byte[4];
			LittleEndian.PutInt(intHolder, size);
			tableStream.Write(intHolder, 0, intHolder.Length);

			byte[] bArray;

			for (int x = 0; x < size; x++)
			{
				ListFormatOverride lfo = (ListFormatOverride) _overrideList[x];
				bArray = lfo.ToByteArray();
				tableStream.Write(bArray, 0, bArray.Length);
				ListFormatOverrideLevel[] lfolvls = lfo.LevelOverrides;
				for (int y = 0; y < lfolvls.Length; y++)
				{
					bArray = lfolvls[y].ToByteArray();
					levelBuf.Write(bArray, 0, bArray.Length);
				}
			}
			bArray = levelBuf.ToArray();
			tableStream.Write(bArray, 0, bArray.Length);
		}
		
		public virtual ListFormatOverride Override(int lfoIndex)
		{
			return (ListFormatOverride) _overrideList[lfoIndex - 1];
		}
		
		public virtual int GetOverrideIndexFromListID(int lstid)
		{
			int returnVal = - 1;
			int size = _overrideList.Count;
			for (int x = 0; x < size; x++)
			{
				ListFormatOverride next = (ListFormatOverride) _overrideList[x];
				if (next.Lsid == lstid)
				{
					// 1-based index I think
					returnVal = x + 1;
					break;
				}
			}
			if (returnVal == - 1)
			{
				throw new ArgumentOutOfRangeException("No list found with the specified ID");
			}
			return returnVal;
		}
		
		public virtual ListLevel GetLevel(int listID, int level)
		{
			
			ListData lst = (ListData) _listMap[(int) listID];
			ListLevel lvl = lst.Levels[level];
			return lvl;
		}
		
		public virtual ListData GetListData(int listID)
		{
			
			return (ListData) _listMap[(int) listID];
		}
		
		public  override bool Equals(object obj)
		{
			if (obj == null)
			{
				return false;
			}
			
			ListTables tables = (ListTables) obj;
			
			if (_listMap.Count == tables._listMap.Count)
			{
								
				foreach (object key in _listMap)
				{				
					ListData lst1 = (ListData) _listMap[key];
					ListData lst2 = (ListData) tables._listMap[key];
					if (!lst1.Equals(lst2))
					{
						return false;
					}
				}
				int size = _overrideList.Count;
				if (size == tables._overrideList.Count)
				{
					for (int x = 0; x < size; x++)
					{
						if (!_overrideList[x].Equals(tables._overrideList[x]))
						{
							return false;
						}
					}
					return true;
				}
			}
			return false;
		}
	}
}