// 
// PAPBinTable.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.Util;
using Poi.Net.HWPF.SPRM;
using Poi.Net.HWPF.Model;
using Poi.Net.HWPF.Model.IO;
using Poi.Net.PoiFS.Common;

namespace Poi.Net.HWPF.Model
{
	
	/// <summary> This class represents the bin table of Word document but it also serves as a
	/// holder for all of the paragraphs of document that have been loaded into
	/// memory.
	/// 
	/// </summary>

	public class PAPBinTable
	{
		virtual public ArrayList Paragraphs {
			get {
				return _paragraphs;
			}
			
		}
		protected internal ArrayList _paragraphs = new ArrayList();
		internal byte[] _dataStream;
		
		public PAPBinTable()
		{
		}
		
		public PAPBinTable(byte[] documentStream, byte[] tableStream, byte[] dataStream, int offset, int size, int fcMin)
		{
			PlexOfCps binTable = new PlexOfCps(tableStream, offset, size, 4);
			
			int length = binTable.Length;
			for (int x = 0; x < length; x++)
			{
				GenericPropertyNode node = binTable.GetProperty(x);
				
				int pageNum = LittleEndian.GetInt(node.Bytes);
				int pageOffset = POIFSConstants.BIG_BLOCK_SIZE * pageNum;
				
				PAPFormattedDiskPage pfkp = new PAPFormattedDiskPage(documentStream, dataStream, pageOffset, fcMin);
				
				int fkpSize = pfkp.Size;
				
				for (int y = 0; y < fkpSize; y++)
				{
					_paragraphs.Add(pfkp.GetPAPX(y));
				}
			}
			_dataStream = dataStream;
		}
		
		public virtual void Insert(int listIndex, int cpStart, SprmBuffer buf)
		{
			PAPX forInsert = new PAPX(cpStart, cpStart, buf, _dataStream);
			if (listIndex == _paragraphs.Count)
			{
				_paragraphs.Add(forInsert);
			}
			else
			{
				PAPX currentPap = (PAPX) _paragraphs[listIndex];
				if (currentPap != null && currentPap.Start < cpStart)
				{
					SprmBuffer clonedBuf = null;
					try
					{
						clonedBuf = (SprmBuffer) currentPap.SprmBuf.Clone();
					}
					
					catch
					{
					}
					currentPap.End = cpStart;
					PAPX splitPap = new PAPX(cpStart, currentPap.End, clonedBuf, _dataStream);
					_paragraphs.Insert(++listIndex, forInsert);
					_paragraphs.Insert(++listIndex, splitPap);
				}
				else
				{
					_paragraphs.Insert(listIndex, forInsert);
				}
			}
		}
		
		public virtual void AdjustForDelete(int listIndex, int offset, int length)
		{
			int size = _paragraphs.Count;
			int endMark = offset + length;
			int endIndex = listIndex;
			
			PAPX papx = (PAPX) _paragraphs[endIndex];
			while (papx.End < endMark)
			{
				papx = (PAPX) _paragraphs[++endIndex];
			}
			if (listIndex == endIndex)
			{
				papx = (PAPX) _paragraphs[endIndex];
				papx.End = (papx.End - endMark) + offset;
			}
			else
			{
				papx = (PAPX) _paragraphs[listIndex];
				papx.End = offset;
				for (int x = listIndex + 1; x < endIndex; x++)
				{
					papx = (PAPX) _paragraphs[x];
					papx.Start = offset;
					papx.End = offset;
				}
				papx = (PAPX) _paragraphs[endIndex];
				papx.End = (papx.End - endMark) + offset;
			}
			
			for (int x = endIndex + 1; x < size; x++)
			{
				papx = (PAPX) _paragraphs[x];
				papx.Start = papx.Start - length;
				papx.End = papx.End - length;
			}
		}
		
		
		public virtual void AdjustForInsert(int listIndex, int length)
		{
			int size = _paragraphs.Count;
			PAPX papx = (PAPX) _paragraphs[listIndex];
			papx.End = papx.End + length;
			
			for (int x = listIndex + 1; x < size; x++)
			{
				papx = (PAPX) _paragraphs[x];
				papx.Start = papx.Start + length;
				papx.End = papx.End + length;
			}
		}
		
		public virtual void WriteTo(HWPFFileSystem sys, int fcMin)
		{
			
			HWPFOutputStream docStream = sys.GetStream("WordDocument");
			Stream tableStream = sys.GetStream("1Table");
			
			PlexOfCps binTable = new PlexOfCps(4);
			
			// each FKP must start on a 512 byte page.
			int docOffset = docStream.Offset;
			int mod = docOffset % POIFSConstants.BIG_BLOCK_SIZE;
			if (mod != 0)
			{
				byte[] padding = new byte[POIFSConstants.BIG_BLOCK_SIZE - mod];
				docStream.Write(padding, 0, padding.Length);
			}
			
			// get the page number for the first fkp
			docOffset = docStream.Offset;
			int pageNum = docOffset / POIFSConstants.BIG_BLOCK_SIZE;
			
			// get the ending fc
			int endingFc = ((PropertyNode) _paragraphs[_paragraphs.Count - 1]).End;
			endingFc += fcMin;
			
			
			ArrayList overflow = _paragraphs;
			do 
			{
				PropertyNode startingProp = (PropertyNode) overflow[0];
				int start = startingProp.Start + fcMin;
				
				PAPFormattedDiskPage pfkp = new PAPFormattedDiskPage(_dataStream);
				pfkp.Fill(overflow);
				
				byte[] bufFkp = pfkp.ToByteArray(fcMin);
				docStream.Write(bufFkp, 0, bufFkp.Length);
				overflow = pfkp.Overflow;
				
				int end = endingFc;
				if (overflow != null)
				{
					end = ((PropertyNode) overflow[0]).Start + fcMin;
				}
				
				byte[] intHolder = new byte[4];
				LittleEndian.PutInt(intHolder, pageNum++);
				binTable.AddProperty(new GenericPropertyNode(start, end, intHolder));
			}
			while (overflow != null);
			byte[] bArray = binTable.ToByteArray();
			tableStream.Write(bArray, 0, bArray.Length);
		}
	}
}