// 
// PropertyNode.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using Poi.Net.HWPF.SPRM;
namespace Poi.Net.HWPF.Model
{
	
	/// <summary> Represents a lightweight node in the Trees used to store content
	/// properties.
	/// 
	/// </summary>

	public abstract class PropertyNode : IComparable, ICloneable
	{
		/// <returns> The offset of this property's text.
		/// </returns>
		virtual public int Start {
			get {
				return _cpStart;
			}
			
			set {
				_cpStart = value;
			}
			
		}
		/// <returns> The offset of the end of this property's text.
		/// </returns>
		virtual public int End {
			get {
				return _cpEnd;
			}
			
			set {
				_cpEnd = value;
			}
			
		}
		protected internal object _buf;
		private int _cpStart;
		private int _cpEnd;
		
		
		/// <param name="fcStart">The start of the text for this property.
		/// </param>
		/// <param name="fcEnd">The end of the text for this property.
		/// </param>
		/// <param name="grpprl">The property description in compressed form.
		/// </param>
		protected internal PropertyNode(int fcStart, int fcEnd, object buf)
		{
			_cpStart = fcStart;
			_cpEnd = fcEnd;
			_buf = buf;
		}
		
		/// <summary> Adjust for a deletion that can span multiple PropertyNodes.</summary>
		/// <param name="">start
		/// </param>
		/// <param name="">length
		/// </param>
		public virtual void AdjustForDelete(int start, int length)
		{
			int end = start + length;
			
			if (_cpEnd > start)
			{
				if (_cpStart < end)
				{
					_cpEnd = end >= _cpEnd?start:_cpEnd - length;
					_cpStart = System.Math.Min(start, _cpStart);
				}
				else
				{
					_cpEnd -= length;
					_cpStart -= length;
				}
			}
		}
		
		protected internal virtual bool LimitsAreEqual(object o)
		{
			return ((PropertyNode) o).Start == _cpStart && ((PropertyNode) o).End == _cpEnd;
		}
		
		public  override bool Equals(object o)
		{
			if (LimitsAreEqual(o))
			{
				object testBuf = ((PropertyNode) o)._buf;
				if (testBuf is byte[] && _buf is byte[])
				{
					return ArrayHelper.Equals((byte[]) testBuf, (byte[]) _buf);
				}
				return _buf.Equals(testBuf);
			}
			return false;
		}
		
		public virtual object Clone()
		{
			return base.MemberwiseClone();
		}
		
		/// <summary> Used for sorting in collections.</summary>
		public virtual int CompareTo(object o)
		{
			int cpEnd = ((PropertyNode) o).End;
			if (_cpEnd == cpEnd)
			{
				return 0;
			}
			else if (_cpEnd < cpEnd)
			{
				return - 1;
			}
			else
			{
				return 1;
			}
		}
	}
}