// 
// SectionTable.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Collections;

using Poi.Net.Util;
using Poi.Net.HWPF.Model;
using Poi.Net.HWPF.Model.IO;

namespace Poi.Net.HWPF.Model
{

	public class SectionTable
	{
		virtual public ArrayList Sections {
			get {
				return _sections;
			}
			
		}
		private const int SED_SIZE = 12;
		
		protected internal ArrayList _sections = new ArrayList();
		protected internal IList _text;
		
		public SectionTable()
		{
		}
		
		
		public SectionTable(byte[] documentStream, byte[] tableStream, int offset, int size, int fcMin, IList tpt)
		{
			PlexOfCps sedPlex = new PlexOfCps(tableStream, offset, size, SED_SIZE);
			_text = tpt;
			
			int length = sedPlex.Length;
			
			for (int x = 0; x < length; x++)
			{
				GenericPropertyNode node = sedPlex.GetProperty(x);
				SectionDescriptor sed = new SectionDescriptor(node.Bytes, 0);
				
				int fileOffset = sed.Fc;
				
				// check for the optimization
				if (fileOffset == 0xffffffff)
				{
					_sections.Add(new SEPX(sed, CPtoFC(node.Start), CPtoFC(node.End), new byte[0]));
				}
				else
				{
					// The first short at the offset is the size of the grpprl.
					int sepxSize = LittleEndian.GetShort(documentStream, fileOffset);
					byte[] buf = new byte[sepxSize];
					fileOffset += LittleEndian.SHORT_SIZE;
					Array.Copy(documentStream, fileOffset, buf, 0, buf.Length);
					_sections.Add(new SEPX(sed, CPtoFC(node.Start), CPtoFC(node.End), buf));
				}
			}
		}
		
		public virtual void AdjustForInsert(int listIndex, int length)
		{
			int size = _sections.Count;
			SEPX sepx = (SEPX) _sections[listIndex];
			sepx.End = sepx.End + length;
			
			for (int x = listIndex + 1; x < size; x++)
			{
				sepx = (SEPX) _sections[x];
				sepx.Start = sepx.Start + length;
				sepx.End = sepx.End + length;
			}
		}
		
		// goss version of CPtoFC - this takes into account non-contiguous textpieces
		// that we have come across in real world documents. Tests against the example
		// code in HWPFDocument show no variation to Ryan's version of the code in
		// normal use, but this version works with our non-contiguous test case.
		// So far unable to get this test case to be written out as well due to
		// other issues. - piers
		private int CPtoFC(int CP)
		{
			TextPiece TP = null;
			
			for (int i = _text.Count - 1; i > - 1; i--)
			{
				TP = (TextPiece) _text[i];
				
				if (CP >= TP.CP)
					break;
			}
			int FC = TP.PieceDescriptor.FilePosition;
			int offset = CP - TP.CP;
			if (TP.UsesUnicode)
				offset *= 2;
			FC = FC + offset - ((TextPiece) _text[0]).PieceDescriptor.FilePosition;
			return FC;
		}
		
		// Ryans code
		private int FCtoCP(int fc)
		{
			int size = _text.Count;
			int cp = 0;
			for (int x = 0; x < size; x++)
			{
				TextPiece piece = (TextPiece) _text[x];
				
				if (fc <= piece.End)
				{
					cp += ((fc - piece.Start) / (piece.UsesUnicode?2:1));
					break;
				}
				else
				{
					cp += ((piece.End - piece.Start) / (piece.UsesUnicode?2:1));
				}
			}
			return cp;
		}
		
		public virtual void WriteTo(HWPFFileSystem sys, int fcMin)
		{
			HWPFOutputStream docStream = sys.GetStream("WordDocument");
			HWPFOutputStream tableStream = sys.GetStream("1Table");
			
			int offset = docStream.Offset;
			int len = _sections.Count;
			PlexOfCps plex = new PlexOfCps(SED_SIZE);
			
			for (int x = 0; x < len; x++)
			{
				SEPX sepx = (SEPX) _sections[x];
				byte[] grpprl = sepx.Grpprl;
				
				// write the sepx to the document stream. starts with a 2 byte size
				// followed by the grpprl
				byte[] shortBuf = new byte[2];
				LittleEndian.PutShort(shortBuf, (short) grpprl.Length);
				
				docStream.Write(shortBuf, 0, shortBuf.Length);
				docStream.Write(grpprl, 0, grpprl.Length);
				
				// set the fc in the section descriptor
				SectionDescriptor sed = sepx.SectionDescriptor;
				sed.Fc = offset;
				
				// add the section descriptor bytes to the PlexOfCps.
				
				
				// original line -
				//GenericPropertyNode property = new GenericPropertyNode(sepx.GetStart, sepx.GetEnd, sed.ToByteArray());
				
				// Line using Ryan's FCtoCP() conversion method -
				// unable to observe any effect on our testcases when using this code - piers
				GenericPropertyNode property = new GenericPropertyNode(FCtoCP(sepx.Start), FCtoCP(sepx.End), sed.ToByteArray());
				
				
				plex.AddProperty(property);
				
				offset = docStream.Offset;
			}
			byte[] bArray = plex.ToByteArray();
			tableStream.Write(bArray, 0, bArray.Length);
		}
	}
}