// 
// StyleDescription.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Text;

using Poi.Net.HWPF.UserModel;
using Poi.Net.Util;

namespace Poi.Net.HWPF.Model
{
	/// <summary> Comment me
	/// 
	/// </summary>

	public class StyleDescription : HDFType
	{
		virtual public int BaseStyle {
			get {
				return _baseStyle.GetValue(_infoShort2);
			}
			
		}
		virtual public byte[] CHPX {
			get {
				switch (_styleTypeCode.GetValue(_infoShort2))
				{
					
					case PARAGRAPH_STYLE: 
						if (_upxs.Length > 1)
						{
							return _upxs[1].Get;
						}
						return null;
					
					case CHARACTER_STYLE: 
						return _upxs[0].Get;
					
					default: 
						return null;
					
				}
			}
			
		}
		virtual public byte[] PAPX {
			get {
				switch (_styleTypeCode.GetValue(_infoShort2))
				{
					
					case PARAGRAPH_STYLE: 
						return _upxs[0].Get;
					
					default: 
						return null;
					
				}
			}
			
		}
		virtual public string Name {
			get {
				return _name;
			}
			
		}
		
		private const int PARAGRAPH_STYLE = 1;
		private const int CHARACTER_STYLE = 2;
		
		//unused: private int _istd;
		private int _baseLength;
		private short _infoShort;
		private static BitField _sti = new BitField(0xfff);
		private static BitField _fScratch = new BitField(0x1000);
		private static BitField _fInvalHeight = new BitField(0x2000);
		private static BitField _fHasUpe = new BitField(0x4000);
		private static BitField _fMassCopy = new BitField(0x8000);
		private short _infoShort2;
		private static BitField _styleTypeCode = new BitField(0xf);
		private static BitField _baseStyle = new BitField(0xfff0);
		private short _infoShort3;
		private static BitField _numUPX = new BitField(0xf);
		private static BitField _nextStyle = new BitField(0xfff0);
		private short _bchUpe;
		private short _infoShort4;
		private static BitField _fAutoRedef = new BitField(0x1);
		private static BitField _fHidden = new BitField(0x2);
		
		internal UPX[] _upxs;
		internal string _name;
		internal ParagraphProperties _pap;
		internal CharacterProperties _chp;
		
		public StyleDescription()
		{
			//      _pap = new ParagraphProperties();
			//      _chp = new CharacterProperties();
		}
		public StyleDescription(byte[] std, int baseLength, int offset, bool word9)
		{
			_baseLength = baseLength;
			int nameStart = offset + baseLength;
			_infoShort = LittleEndian.GetShort(std, offset);
			offset += LittleEndian.SHORT_SIZE;
			_infoShort2 = LittleEndian.GetShort(std, offset);
			offset += LittleEndian.SHORT_SIZE;
			_infoShort3 = LittleEndian.GetShort(std, offset);
			offset += LittleEndian.SHORT_SIZE;
			_bchUpe = LittleEndian.GetShort(std, offset);
			offset += LittleEndian.SHORT_SIZE;
			_infoShort4 = LittleEndian.GetShort(std, offset);
			offset += LittleEndian.SHORT_SIZE;
			
			//first byte(s) of variable length section of std is the length of the
			//style name and aliases string
			int nameLength = 0;
			int multiplier = 1;
			if (word9)
			{
				nameLength = LittleEndian.GetShort(std, nameStart);
				multiplier = 2;
				nameStart += LittleEndian.SHORT_SIZE;
			}
			else
			{
				nameLength = std[nameStart];
			}
			
			try
			{
				string tempStr;
				
				tempStr = Encoding.GetEncoding("UTF-16").GetString(std);
				_name = new string(tempStr.ToCharArray(), nameStart, nameLength * multiplier);
			}
			catch (IOException ignore)
			{
				// ignore
			}
			
			//length then null terminator.
			int grupxStart = ((nameLength + 1) * multiplier) + nameStart;
			
			// the spec only refers to two possible upxs but it mentions
			// that more may be added in the future
			int varOffset = grupxStart;
			int numUPX = _numUPX.GetValue(_infoShort3);
			_upxs = new UPX[numUPX];
			for (int x = 0; x < numUPX; x++)
			{
				int upxSize = LittleEndian.GetShort(std, varOffset);
				varOffset += LittleEndian.SHORT_SIZE;
				
				byte[] upx = new byte[upxSize];
				Array.Copy(std, varOffset, upx, 0, upxSize);
				_upxs[x] = new UPX(upx);
				varOffset += upxSize;
				
				
				// the upx will always start on a word boundary.
				if (upxSize % 2 == 1)
				{
					++varOffset;
				}
			}
		}
		public virtual ParagraphProperties GetPAP()
		{
			return _pap;
		}
		public virtual CharacterProperties GetCHP()
		{
			return _chp;
		}
		internal virtual void SetPAP(ParagraphProperties pap)
		{
			_pap = pap;
		}
		internal virtual void SetCHP(CharacterProperties chp)
		{
			_chp = chp;
		}
		
		public virtual byte[] ToByteArray()
		{
			// size equals _baseLength bytes for known variables plus 2 bytes for name
			// length plus name length * 2 plus 2 bytes for null plus upx's preceded by
			// length
			int size = _baseLength + 2 + ((_name.Length + 1) * 2);
			
			// determine the size needed for the upxs. They always fall on word
			// boundaries.
			size += _upxs[0].Size + 2;
			for (int x = 1; x < _upxs.Length; x++)
			{
				size += _upxs[x - 1].Size % 2;
				size += _upxs[x].Size + 2;
			}
			
			
			byte[] buf = new byte[size];
			
			int offset = 0;
			LittleEndian.PutShort(buf, offset, _infoShort);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, _infoShort2);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, _infoShort3);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, _bchUpe);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, _infoShort4);
			offset = _baseLength;
			
			char[] letters = _name.ToCharArray();
			LittleEndian.PutShort(buf, _baseLength, (short) letters.Length);
			offset += LittleEndian.SHORT_SIZE;
			for (int x = 0; x < letters.Length; x++)
			{
				LittleEndian.PutShort(buf, offset, (short) letters[x]);
				offset += LittleEndian.SHORT_SIZE;
			}
			// get past the null delimiter for the name.
			offset += LittleEndian.SHORT_SIZE;
			
			for (int x = 0; x < _upxs.Length; x++)
			{
				short upxSize = (short) _upxs[x].Size;
				LittleEndian.PutShort(buf, offset, upxSize);
				offset += LittleEndian.SHORT_SIZE;
				Array.Copy(_upxs[x].Get, 0, buf, offset, upxSize);
				offset += upxSize + (upxSize % 2);
			}
			
			return buf;
		}
		
		public  override bool Equals(object o)
		{
			StyleDescription sd = (StyleDescription) o;
			if (sd._infoShort == _infoShort && sd._infoShort2 == _infoShort2 && sd._infoShort3 == _infoShort3 && sd._bchUpe == _bchUpe && sd._infoShort4 == _infoShort4 && _name.Equals(sd._name))
			{
				
				if (!ArrayHelper.Equals(_upxs, sd._upxs))
				{
					return false;
				}
				return true;
			}
			return false;
		}
	}
}