// 
// StyleSheet.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;

using Poi.Net.Util;
using Poi.Net.HWPF.SPRM;
using Poi.Net.HWPF.Model.IO;
using Poi.Net.HWPF.UserModel;

namespace Poi.Net.HWPF.Model
{
	
	/// <summary> Represents a document's stylesheet. A word documents formatting is stored as
	/// compressed styles that are based on styles contained in the stylesheet. This
	/// class also contains static utility functions to uncompress different
	/// formatting properties.
	/// 
	/// </summary>

	public class StyleSheet : HDFType
	{
		
		public const int NIL_STYLE = 4095;
		private const int PAP_TYPE = 1;
		private const int CHP_TYPE = 2;
		private const int SEP_TYPE = 4;
		private const int TAP_TYPE = 5;
		
		
		
		private static readonly ParagraphProperties NIL_PAP = new ParagraphProperties();
		
		private static readonly CharacterProperties NIL_CHP = new CharacterProperties();
		
		private int _stshiLength;
		private int _baseLength;
		private int _flags;
		private int _maxIndex;
		private int _maxFixedIndex;
		private int _stylenameVersion;
		private int[] _rgftc;
		
		internal StyleDescription[] _styleDescriptions;
		
		/// <summary> StyleSheet constructor. Loads a document's stylesheet information,
		/// 
		/// </summary>
		/// <param name="styleSheet">A byte array containing a document's raw stylesheet
		/// info. Found by using FileInformationBlock.getFcStshf() and
		/// FileInformationBLock.getLcbStshf()
		/// </param>
		public StyleSheet(byte[] tableStream, int offset)
		{
			_stshiLength = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			int stdCount = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			_baseLength = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			_flags = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			_maxIndex = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			_maxFixedIndex = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			_stylenameVersion = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			
			_rgftc = new int[3];
			_rgftc[0] = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			_rgftc[1] = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			_rgftc[2] = LittleEndian.GetShort(tableStream, offset);
			offset += LittleEndian.SHORT_SIZE;
			
			offset = (LittleEndian.SHORT_SIZE + _stshiLength);
			_styleDescriptions = new StyleDescription[stdCount];
			for (int x = 0; x < stdCount; x++)
			{
				int stdSize = LittleEndian.GetShort(tableStream, offset);
				//get past the size
				offset += 2;
				if (stdSize > 0)
				{
					//byte[] std = new byte[stdSize];
					
					StyleDescription aStyle = new StyleDescription(tableStream, _baseLength, offset, true);
					
					_styleDescriptions[x] = aStyle;
				}
				
				offset += stdSize;
			}
			for (int x = 0; x < _styleDescriptions.Length; x++)
			{
				if (_styleDescriptions[x] != null)
				{
					CreatePap(x);
					CreateChp(x);
				}
			}
		}
		
		public virtual void WriteTo(HWPFOutputStream outStream)
		{
			int offset = 0;
			// add two bytes so we can prepend the stylesheet w/ its size
			byte[] buf = new byte[_stshiLength + 2];
			LittleEndian.PutShort(buf, offset, (short) _stshiLength);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, (short) _styleDescriptions.Length);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, (short) _baseLength);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, (short) _flags);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, (short) _maxIndex);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, (short) _maxFixedIndex);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, (short) _stylenameVersion);
			offset += LittleEndian.SHORT_SIZE;
			
			LittleEndian.PutShort(buf, offset, (short) _rgftc[0]);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, (short) _rgftc[1]);
			offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(buf, offset, (short) _rgftc[2]);
			
			outStream.Write(buf, 0, buf.Length);
			
			byte[] sizeHolder = new byte[2];
			for (int x = 0; x < _styleDescriptions.Length; x++)
			{
				if (_styleDescriptions[x] != null)
				{
					byte[] std = _styleDescriptions[x].ToByteArray();
					
					// adjust the size so it is always on a word boundary
					LittleEndian.PutShort(sizeHolder, (short) ((std.Length) + (std.Length % 2)));
					outStream.Write(sizeHolder, 0, sizeHolder.Length);
					outStream.Write(std, 0, std.Length);
					
					// Must always start on a word boundary.
					if (std.Length % 2 == 1)
					{
						outStream.WriteByte('\x0000');
					}
				}
				else
				{
					sizeHolder[0] = 0;
					sizeHolder[1] = 0;
					outStream.Write(sizeHolder, 0, sizeHolder.Length);
				}
			}
		}
		public  override bool Equals(object o)
		{
			StyleSheet ss = (StyleSheet) o;
			
			if (ss._baseLength == _baseLength && ss._flags == _flags && ss._maxFixedIndex == _maxFixedIndex && ss._maxIndex == _maxIndex && ss._rgftc[0] == _rgftc[0] && ss._rgftc[1] == _rgftc[1] && ss._rgftc[2] == _rgftc[2] && ss._stshiLength == _stshiLength && ss._stylenameVersion == _stylenameVersion)
			{
				if (ss._styleDescriptions.Length == _styleDescriptions.Length)
				{
					for (int x = 0; x < _styleDescriptions.Length; x++)
					{
						// check for null
						if (ss._styleDescriptions[x] != _styleDescriptions[x])
						{
							// check for equality
							if (!ss._styleDescriptions[x].Equals(_styleDescriptions[x]))
							{
								return false;
							}
						}
					}
					return true;
				}
			}
			return false;
		}
		/// <summary> Creates a PartagraphProperties object from a papx stored in the
		/// StyleDescription at the index istd in the StyleDescription array. The PAP
		/// is placed in the StyleDescription at istd after its been created. Not
		/// every StyleDescription will contain a papx. In these cases this function
		/// does nothing
		/// 
		/// </summary>
		/// <param name="istd">The index of the StyleDescription to create the
		/// ParagraphProperties  from (and also place the finished PAP in)
		/// </param>
		private void CreatePap(int istd)
		{
			StyleDescription sd = _styleDescriptions[istd];
			ParagraphProperties pap = sd.GetPAP();
			byte[] papx = sd.PAPX;
			int baseIndex = sd.BaseStyle;
			if (pap == null && papx != null)
			{
				ParagraphProperties parentPAP = new ParagraphProperties();
				if (baseIndex != NIL_STYLE)
				{
					
					parentPAP = _styleDescriptions[baseIndex].GetPAP();
					if (parentPAP == null)
					{
						CreatePap(baseIndex);
						parentPAP = _styleDescriptions[baseIndex].GetPAP();
					}
				}
				
				pap = (ParagraphProperties) ParagraphSprmUncompressor.UncompressPAP(parentPAP, papx, 2);
				sd.SetPAP(pap);
			}
		}
		/// <summary> Creates a CharacterProperties object from a chpx stored in the
		/// StyleDescription at the index istd in the StyleDescription array. The
		/// CharacterProperties object is placed in the StyleDescription at istd after
		/// its been created. Not every StyleDescription will contain a chpx. In these
		/// cases this function does nothing.
		/// 
		/// </summary>
		/// <param name="istd">The index of the StyleDescription to create the
		/// CharacterProperties object from.
		/// </param>
		private void CreateChp(int istd)
		{
			StyleDescription sd = _styleDescriptions[istd];
			CharacterProperties chp = sd.GetCHP();
			byte[] chpx = sd.CHPX;
			int baseIndex = sd.BaseStyle;
			if (chp == null && chpx != null)
			{
				CharacterProperties parentCHP = new CharacterProperties();
				if (baseIndex != NIL_STYLE)
				{
					
					parentCHP = _styleDescriptions[baseIndex].GetCHP();
					if (parentCHP == null)
					{
						CreateChp(baseIndex);
						parentCHP = _styleDescriptions[baseIndex].GetCHP();
					}
				}
				
				chp = (CharacterProperties) CharacterSprmUncompressor.UncompressCHP(parentCHP, chpx, 0);
				sd.SetCHP(chp);
			}
		}
		
		/// <summary> Gets the StyleDescription at index x.
		/// 
		/// </summary>
		/// <param name="x">the index of the desired StyleDescription.
		/// </param>
		public virtual StyleDescription GetStyleDescription(int x)
		{
			return _styleDescriptions[x];
		}
		
		public virtual CharacterProperties GetCharacterStyle(int x)
		{
			if (x == NIL_STYLE)
			{
				return NIL_CHP;
			}
			return (_styleDescriptions[x] != null?_styleDescriptions[x].GetCHP():null);
		}
		
		public virtual ParagraphProperties GetParagraphStyle(int x)
		{
			if (x == NIL_STYLE)
			{
				return NIL_PAP;
			}
			return (_styleDescriptions[x] != null?_styleDescriptions[x].GetPAP():null);
		}
	}
}