// 
// TextPiece.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Text;

namespace Poi.Net.HWPF.Model
{
	/// <summary> Lightweight representation of a text piece.
	/// 
	/// </summary>

	public class TextPiece:PropertyNode, IComparable
	{
		virtual public PieceDescriptor PieceDescriptor {
			get {
				return _pd;
			}
			
		}
		virtual public StringBuilder StringBuffer {
			get {
				return (StringBuilder) _buf;
			}
			
		}
		virtual public byte[] RawBytes {
			get {
				try
				{
					
					return Encoding.GetEncoding(_usesUnicode?"UTF-16":"windows-1252").GetBytes(((StringBuilder) _buf).ToString());
				}
				catch (IOException ignore)
				{
					// shouldn't ever happen considering we wouldn't have been able to
					// create the StringBuffer w/o getting this exception
					return UTF8Encoding.UTF8.GetBytes(((StringBuilder) _buf).ToString());
				}
			}
		}

		virtual public int CP {
			get {
				return _cpStart;
			}			
		}

		private bool _usesUnicode;
		
		private PieceDescriptor _pd;
		
		private int _cpStart;
		
		/// <param name="start">Offset in main document stream.
		/// </param>
		/// <param name="length">The total length of the text in bytes. Note: 1 character
		/// does not necessarily refer to 1 byte.
		/// </param>
		/// <param name="unicode">true if this text is unicode.
		/// </param>
		
		public TextPiece(int start, int end, byte[] text, PieceDescriptor pd, int cpStart):base(start, end, new StringBuilder(Encoding.GetEncoding(pd.Unicode?"UTF-16":"windows-1252").GetString(text)))
		{
			_usesUnicode = pd.Unicode;
			_pd = pd;
			_cpStart = cpStart;
		}
		/// <returns> If this text piece uses unicode
		/// </returns>
		public virtual bool UsesUnicode {
			get { return _usesUnicode; }
		}
		
		public virtual string Substring(int start, int end)
		{
			int denominator = _usesUnicode?2:1;
			
			return ((StringBuilder) _buf).ToString(start / denominator, end / denominator);
		}
		
		public override void AdjustForDelete(int start, int length)
		{
			
		}
		
		public int CharacterLength {
			get { return (End - Start) / (_usesUnicode?2:1); }
		}
		
		public  override bool Equals(object o)
		{
			if (LimitsAreEqual(o))
			{
				TextPiece tp = (TextPiece) o;
				return StringBuffer.ToString().Equals(tp.StringBuffer.ToString()) && tp._usesUnicode == _usesUnicode && _pd.Equals(tp._pd);
			}
			return false;
		}
		
		override public object Clone()
		{
			return null;
		}
		
		override public int CompareTo(object obj)
		{
			return 0;
		}
	}
}