// 
// TextPieceTable.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Ryan Ackley
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Collections;

using Poi.Net.PoiFS.Common;
using Poi.Net.HWPF.Model.IO;

namespace Poi.Net.HWPF.Model
{
	public class TextPieceTable
	{
		virtual public int CpMin {
			get {
				return _cpMin;
			}
			
		}
		virtual public IList TextPieces {
			get {
				return _textPieces;
			}
			
		}
		protected internal ArrayList _textPieces = new ArrayList();
		//int _multiple;
		internal int _cpMin;
		
		public TextPieceTable()
		{
		}
		
		public TextPieceTable(byte[] documentStream, byte[] tableStream, int offset, int size, int fcMin)
		{
			// get our plex of PieceDescriptors
			PlexOfCps pieceTable = new PlexOfCps(tableStream, offset, size, PieceDescriptor.SizeInBytes);
			
			//_multiple = 2;
			int length = pieceTable.Length;
			PieceDescriptor[] pieces = new PieceDescriptor[length];
			
			// iterate through piece descriptors raw bytes and create
			// PieceDescriptor objects
			for (int x = 0; x < length; x++)
			{
				GenericPropertyNode node = pieceTable.GetProperty(x);
				pieces[x] = new PieceDescriptor(node.Bytes, 0);
				
				//      if (!pieces[x].IsUnicode())
				//      {
				//        _multiple = 1;
				//      }
			}
			
			int firstPieceFilePosition = pieces[0].FilePosition;
			_cpMin = firstPieceFilePosition - fcMin;
			
			// using the PieceDescriptors, build our list of TextPieces.
			for (int x = 0; x < pieces.Length; x++)
			{
				int start = pieces[x].FilePosition;
				PropertyNode node = pieceTable.GetProperty(x);
				int nodeStart = node.Start;
				
				// multiple will be 2 if there is only one piece and its unicode. Some
				// type of optimization.
				bool unicode = pieces[x].Unicode;
				
				int multiple = 1;
				if (unicode)
				{
					multiple = 2;
				}
				int nodeEnd = ((node.End - nodeStart) * multiple) + nodeStart;
				int textSize = nodeEnd - nodeStart;
				
				
				byte[] buf = new byte[textSize];
				Array.Copy(documentStream, start, buf, 0, textSize);
				
				int startFilePosition = start - firstPieceFilePosition;
				_textPieces.Add(new TextPiece(startFilePosition, startFilePosition + textSize, buf, pieces[x], node.Start));
			}
		}
		
		public virtual byte[] WriteTo(HWPFOutputStream docStream)
		{
			
			PlexOfCps textPlex = new PlexOfCps(PieceDescriptor.SizeInBytes);
			//int fcMin = docStream.getOffset();
			
			int size = _textPieces.Count;
			int bumpDown = 0;
			for (int x = 0; x < size; x++)
			{
				TextPiece next = (TextPiece) _textPieces[x];
				PieceDescriptor pd = next.PieceDescriptor;
				
				int offset = docStream.Offset;
				int mod = (offset % POIFSConstants.BIG_BLOCK_SIZE);
				if (mod != 0)
				{
					mod = POIFSConstants.BIG_BLOCK_SIZE - mod;
					byte[] buf = new byte[mod];
					docStream.Write(buf, 0, buf.Length);
				}
				
				
				// set the text piece position to the current docStream offset.
				pd.FilePosition = docStream.Offset;
				
				// write the text to the docstream and save the piece descriptor to the
				// plex which will be written later to the tableStream.
				//if (_multiple == 1 && pd.IsUnicode() &&

				docStream.Write(next.RawBytes, 0, next.RawBytes.Length);
				
				int nodeStart = next.Start;
				int multiple = 1;
				if (pd.Unicode)
				{
					multiple = 2;
				}
				textPlex.AddProperty(new GenericPropertyNode(nodeStart - bumpDown, ((next.End - nodeStart) / multiple + nodeStart) - bumpDown, pd.ToByteArray()));
				
				if (pd.Unicode)
				{
					bumpDown += ((next.End - nodeStart) / multiple);
				}
			}
			
			return textPlex.ToByteArray();
		}
		
		
		public virtual int AdjustForInsert(int listIndex, int length)
		{
			int size = _textPieces.Count;
			
			TextPiece tp = (TextPiece) _textPieces[listIndex];
			
			//The text piece stores the length on file.
			length = length * (tp.UsesUnicode?2:1);
			tp.End = tp.End + length;
			for (int x = listIndex + 1; x < size; x++)
			{
				tp = (TextPiece) _textPieces[x];
				tp.Start = tp.Start + length;
				tp.End = tp.End + length;
			}
			return length;
		}
		
		
		public  override bool Equals(object o)
		{
			TextPieceTable tpt = (TextPieceTable) o;
			
			int size = tpt._textPieces.Count;
			if (size == _textPieces.Count)
			{
				for (int x = 0; x < size; x++)
				{
					if (!tpt._textPieces[x].Equals(_textPieces[x]))
					{
						return false;
					}
				}
				return true;
			}
			return false;
		}
	}
}