// 
// SprmBuffer.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;

using Poi.Net.Util;
namespace Poi.Net.HWPF.SPRM
{
	
	public class SprmBuffer : ICloneable
	{
		internal byte[] _buf;
		internal int _offset;
		internal bool _istd;
		
		public SprmBuffer(byte[] buf, bool istd)
		{
			_offset = buf.Length;
			_buf = buf;
			_istd = istd;
		}
		public SprmBuffer(byte[] buf):this(buf, false)
		{
		}
		public SprmBuffer()
		{
			_buf = new byte[4];
			_offset = 0;
		}
		
		private int FindSprm(short opcode)
		{
			int operation = SprmOperation.GetOperationFromOpcode(opcode);
			int type = SprmOperation.GetTypeFromOpcode(opcode);
			
			SprmIterator si = new SprmIterator(_buf, 2);
			while (si.HasNext())
			{
				SprmOperation i = si.Next();
				if (i.Operation == operation && i.Type == type)
					return i.GrpprlOffset;
			}
			return - 1;
		}
		
		public virtual void UpdateSprm(short opcode, byte operand)
		{
			int grpprlOffset = FindSprm(opcode);
			if (grpprlOffset != - 1)
			{
				_buf[grpprlOffset] = operand;
				return ;
			}
			else
				AddSprm(opcode, operand);
		}
		
		public virtual void UpdateSprm(short opcode, short operand)
		{
			int grpprlOffset = FindSprm(opcode);
			if (grpprlOffset != - 1)
			{
				LittleEndian.PutShort(_buf, grpprlOffset, operand);
				return ;
			}
			else
				AddSprm(opcode, operand);
		}
		
		public virtual void UpdateSprm(short opcode, int operand)
		{
			int grpprlOffset = FindSprm(opcode);
			if (grpprlOffset != - 1)
			{
				LittleEndian.PutInt(_buf, grpprlOffset, operand);
				return ;
			}
			else
				AddSprm(opcode, operand);
		}
		
		public virtual void AddSprm(short opcode, byte operand)
		{
			int addition = LittleEndian.SHORT_SIZE + LittleEndian.BYTE_SIZE;
			EnsureCapacity(addition);
			LittleEndian.PutShort(_buf, _offset, opcode);
			_offset += LittleEndian.SHORT_SIZE;
			_buf[_offset++] = operand;
		}
		public virtual void AddSprm(short opcode, short operand)
		{
			int addition = LittleEndian.SHORT_SIZE + LittleEndian.SHORT_SIZE;
			EnsureCapacity(addition);
			LittleEndian.PutShort(_buf, _offset, opcode);
			_offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutShort(_buf, _offset, operand);
			_offset += LittleEndian.SHORT_SIZE;
		}
		public virtual void AddSprm(short opcode, int operand)
		{
			int addition = LittleEndian.SHORT_SIZE + LittleEndian.INT_SIZE;
			EnsureCapacity(addition);
			LittleEndian.PutShort(_buf, _offset, opcode);
			_offset += LittleEndian.SHORT_SIZE;
			LittleEndian.PutInt(_buf, _offset, operand);
			_offset += LittleEndian.INT_SIZE;
		}
		public virtual void AddSprm(short opcode, byte[] operand)
		{
			int addition = LittleEndian.SHORT_SIZE + LittleEndian.BYTE_SIZE + operand.Length;
			EnsureCapacity(addition);
			LittleEndian.PutShort(_buf, _offset, opcode);
			_offset += LittleEndian.SHORT_SIZE;
			_buf[_offset++] = (byte) operand.Length;
			Array.Copy(operand, 0, _buf, _offset, operand.Length);
		}
		
		public virtual byte[] ToByteArray()
		{
			return _buf;
		}
		
		public  override bool Equals(object obj)
		{
			SprmBuffer sprmBuf = (SprmBuffer) obj;
			return (ArrayHelper.Equals(_buf, sprmBuf._buf));
		}
		
		public virtual void Append(byte[] grpprl)
		{
			EnsureCapacity(grpprl.Length);
			Array.Copy(grpprl, 0, _buf, _offset, grpprl.Length);
		}
		
		public virtual object Clone()
		{
			SprmBuffer retVal = (SprmBuffer) base.MemberwiseClone();
			retVal._buf = new byte[_buf.Length];
			Array.Copy(_buf, 0, retVal._buf, 0, _buf.Length);
			return retVal;
		}
		
		private void EnsureCapacity(int addition)
		{
			if (_offset + addition >= _buf.Length)
			{
				// add 6 more than they need for use the next iteration
				byte[] newBuf = new byte[_offset + addition + 6];
				Array.Copy(_buf, 0, newBuf, 0, _buf.Length);
				_buf = newBuf;
			}
		}
	}
}