// 
// SprmOperation.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using Poi.Net.Util;

namespace Poi.Net.HWPF.SPRM
{
	
	/// <summary> This class is used to represent a sprm operation from a Word 97/2000/XP
	/// document.
	/// </summary>
	/// <author>  Ryan Ackley
	/// </author>
	/// <version>  1.0
	/// </version>
	public class SprmOperation
	{
		virtual public int Type {
			get {
				return _type;
			}
			
		}
		virtual public int Operation {
			get {
				return _operation;
			}
			
		}
		virtual public int GrpprlOffset {
			get {
				return _gOffset;
			}
			
		}
		virtual public int Operand {
			get {
				switch (_sizeCode)
				{
					
					case 0: 
					case 1: 
						return _grpprl[_gOffset];
					
					case 2: 
					case 4: 
					case 5: 
						return LittleEndian.GetShort(_grpprl, _gOffset);
					
					case 3: 
						return LittleEndian.GetInt(_grpprl, _gOffset);
					
					case 6: 
						throw new System.NotSupportedException("This SPRM contains a variable length operand");
					
					case 7: 
						byte[] threeByteInt = new byte[4];
						threeByteInt[0] = _grpprl[_gOffset];
						threeByteInt[1] = _grpprl[_gOffset + 1];
						threeByteInt[2] = _grpprl[_gOffset + 2];
						threeByteInt[3] = (byte) 0;
						return LittleEndian.GetInt(threeByteInt, 0);
					
					default: 
						throw new System.ArgumentException("SPRM contains an invalid size code");
					
				}
			}
			
		}
		virtual public int SizeCode {
			get {
				return _sizeCode;
			}
			
		}
		virtual public byte[] Grpprl {
			get {
				return _grpprl;
			}
			
		}
		
		private static readonly BitField OP_BITFIELD = new BitField(0x1ff);
		
		private static readonly BitField SPECIAL_BITFIELD = new BitField(0x200);
		
		private static readonly BitField TYPE_BITFIELD = new BitField(0x1c00);
		
		private static readonly BitField SIZECODE_BITFIELD = new BitField(0xe000);
		
		private static short LONG_SPRM_TABLE = -10744;
		private static short LONG_SPRM_PARAGRAPH = -14827;
		
		public const int PAP_TYPE = 1;
		public const int TAP_TYPE = 5;
		
		private int _type;
		private int _operation;
		private int _gOffset;
		private byte[] _grpprl;
		private int _sizeCode;
		private int _size;
		
		public SprmOperation(byte[] grpprl, int offset)
		{
			_grpprl = grpprl;
			
			short sprmStart = LittleEndian.GetShort(grpprl, offset);
			
			_gOffset = offset + 2;
			
			_operation = OP_BITFIELD.GetValue(sprmStart);
			_type = TYPE_BITFIELD.GetValue(sprmStart);
			_sizeCode = SIZECODE_BITFIELD.GetValue(sprmStart);
			_size = InitSize(sprmStart);
		}
		
		public static int GetOperationFromOpcode(short opcode)
		{
			return OP_BITFIELD.GetValue(opcode);
		}
		
		public static int GetTypeFromOpcode(short opcode)
		{
			return TYPE_BITFIELD.GetValue(opcode);
		}
		
		public virtual int Size {
			get { return _size; }
		 }
		private int InitSize(short sprm)
		{
			switch (_sizeCode)
			{
				
				case 0: 
				case 1: 
					return 3;
				
				case 2: 
				case 4: 
				case 5: 
					return 4;
				
				case 3: 
					return 6;
				
				case 6: 
					if (sprm == LONG_SPRM_TABLE || sprm == LONG_SPRM_PARAGRAPH)
					{
						int retVal = (0x0000ffff & LittleEndian.GetShort(_grpprl, _gOffset)) + 3;
						_gOffset += 2;
						return retVal;
					}
					else
					{
						return (0x000000ff & _grpprl[_gOffset++]) + 3;
					}

				case 7: 
					return 5;
				
				default: 
					throw new System.ArgumentException("SPRM contains an invalid size code");
				
			}
		}
	}
}