// 
// TableSprmUncompressor.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;

using Poi.Net.HWPF.UserModel;
using Poi.Net.Util;

namespace Poi.Net.HWPF.SPRM
{
	
	public class TableSprmUncompressor:SprmUncompressor
	{
		public TableSprmUncompressor()
		{
		}
		
		public static TableProperties UncompressTAP(byte[] grpprl, int offset)
		{
			TableProperties newProperties = new TableProperties();
			
			SprmIterator sprmIt = new SprmIterator(grpprl, offset);
			
			while (sprmIt.HasNext())
			{
				SprmOperation sprm = (SprmOperation) sprmIt.Next();
				
				//TAPXs are actually PAPXs so we have to make sure we are only trying to
				//uncompress the right type of sprm.
				if (sprm.Type == SprmOperation.TAP_TYPE)
				{
					UnCompressTAPOperation(newProperties, sprm);
				}
			}
			
			return newProperties;
		}
		
		/// <summary> Used to uncompress a table property. Performs an operation defined
		/// by a sprm stored in a tapx.
		/// 
		/// </summary>
		/// <param name="newTAP">The TableProperties object to perform the operation on.
		/// </param>
		/// <param name="operand">The operand that defines this operation.
		/// </param>
		/// <param name="param">The parameter for this operation.
		/// </param>
		/// <param name="varParam">Variable length parameter for this operation.
		/// </param>
		internal static void UnCompressTAPOperation(TableProperties newTAP, SprmOperation sprm)
		{
			switch (sprm.Operation)
			{
				
				case 0: 
					newTAP.Jc = (short) sprm.Operand;
					break;
				
				case 0x01: 
				{
					short[] rgdxaCenter = newTAP.RgdxaCenter;
					short itcMac = newTAP.ItcMac;
					int adjust = sprm.Operand - (rgdxaCenter[0] + newTAP.DxaGapHalf);
					for (int x = 0; x < itcMac; x++)
					{
						rgdxaCenter[x] = (short) (rgdxaCenter[x] + adjust);
					}
					break;
				}
				
				case 0x02: 
				{
					short[] rgdxaCenter = newTAP.RgdxaCenter;
					if (rgdxaCenter != null)
					{
						int adjust = newTAP.DxaGapHalf - sprm.Operand;
						rgdxaCenter[0] = (short) (rgdxaCenter[0] + adjust);
					}
					newTAP.DxaGapHalf = sprm.Operand;
					break;
				}
				
				case 0x03: 
					newTAP.FCantSplit = GetFlag(sprm.Operand);
					break;
				
				case 0x04: 
					newTAP.FTableHeader = GetFlag(sprm.Operand);
					break;
				
				case 0x05: 
				{
					byte[] buf = sprm.Grpprl;
					int offset = sprm.GrpprlOffset;
					newTAP.BrcTop = new BorderCode(buf, offset);
					offset += BorderCode.SIZE;
					newTAP.BrcLeft = new BorderCode(buf, offset);
					offset += BorderCode.SIZE;
					newTAP.BrcBottom = new BorderCode(buf, offset);
					offset += BorderCode.SIZE;
					newTAP.BrcRight = new BorderCode(buf, offset);
					offset += BorderCode.SIZE;
					newTAP.BrcHorizontal = new BorderCode(buf, offset);
					offset += BorderCode.SIZE;
					newTAP.BrcVertical = new BorderCode(buf, offset);
					break;
				}
				
				case 0x06: 
					
					//obsolete, used in word 1.x
					break;
				
				case 0x07: 
					newTAP.DyaRowHeight = sprm.Operand;
					break;
				
				case 0x08: 
				{
					byte[] grpprl = sprm.Grpprl;
					int offset = sprm.GrpprlOffset;
					short itcMac = grpprl[offset];
					short[] rgdxaCenter = new short[itcMac + 1];
					TableCellDescriptor[] rgtc = new TableCellDescriptor[itcMac];
					//I use varParam[0] and newTAP._itcMac interchangably
					newTAP.ItcMac = itcMac;
					newTAP.RgdxaCenter = rgdxaCenter;
					newTAP.Rgtc = rgtc;
					
					// get the rgdxaCenters
					for (int x = 0; x < itcMac; x++)
					{
						rgdxaCenter[x] = LittleEndian.GetShort(grpprl, offset + (1 + (x * 2)));
					}
					
					// only try to get the TC entries if they exist...
					int endOfSprm = offset + sprm.Size - 6; // -2 bytes for sprm - 2 for size short - 2 to correct offsets being 0 based
					int startOfTCs = offset + (1 + (itcMac + 1) * 2);
					
					bool hasTCs = startOfTCs < endOfSprm;
					
					for (int x = 0; x < itcMac; x++)
					{
						if (hasTCs)
							rgtc[x] = TableCellDescriptor.ConvertBytesToTC(grpprl, offset + (1 + ((itcMac + 1) * 2) + (x * 20)));
						else
							rgtc[x] = new TableCellDescriptor();
					}
					
					rgdxaCenter[itcMac] = LittleEndian.GetShort(grpprl, offset + (1 + (itcMac * 2)));
					break;
				}
				
				case 0x09: 
					
					/// <todo>  handle cell shading </todo>
					break;
				
				case 0x0a: 
					
					/// <todo>  handle word defined table styles </todo>
					break;
				
				case 0x20: 
					//      {
					//        TableCellDescriptor[] rgtc = newTAP.getRgtc();
					//
					//        for (int x = varParam[0]; x < varParam[1]; x++)
					//        {
					//
					//          if ((varParam[2] & 0x08) > 0)
					//          {
					//            short[] brcRight = rgtc[x].getBrcRight ();
					//            brcRight[0] = LittleEndian.GetShort (varParam, 6);
					//            brcRight[1] = LittleEndian.GetShort (varParam, 8);
					//          }
					//          else if ((varParam[2] & 0x04) > 0)
					//          {
					//            short[] brcBottom = rgtc[x].getBrcBottom ();
					//            brcBottom[0] = LittleEndian.GetShort (varParam, 6);
					//            brcBottom[1] = LittleEndian.GetShort (varParam, 8);
					//          }
					//          else if ((varParam[2] & 0x02) > 0)
					//          {
					//            short[] brcLeft = rgtc[x].getBrcLeft ();
					//            brcLeft[0] = LittleEndian.GetShort (varParam, 6);
					//            brcLeft[1] = LittleEndian.GetShort (varParam, 8);
					//          }
					//          else if ((varParam[2] & 0x01) > 0)
					//          {
					//            short[] brcTop = rgtc[x].getBrcTop ();
					//            brcTop[0] = LittleEndian.GetShort (varParam, 6);
					//            brcTop[1] = LittleEndian.GetShort (varParam, 8);
					//          }
					//        }
					//        break;
					//      }
					break;
				
				case 0x21: 
					{
						int param = sprm.Operand;
						int index = (param & unchecked((int) 0xff000000)) >> 24;
						int count = (param & 0x00ff0000) >> 16;
						int width = (param & 0x0000ffff);
						int itcMac = newTAP.ItcMac;
						
						short[] rgdxaCenter = new short[itcMac + count + 1];
						TableCellDescriptor[] rgtc = new TableCellDescriptor[itcMac + count];
						if (index >= itcMac)
						{
							index = itcMac;
							Array.Copy(newTAP.RgdxaCenter, 0, rgdxaCenter, 0, itcMac + 1);
							Array.Copy(newTAP.Rgtc, 0, rgtc, 0, itcMac);
						}
						else
						{
							//copy rgdxaCenter
							Array.Copy(newTAP.RgdxaCenter, 0, rgdxaCenter, 0, index + 1);
							Array.Copy(newTAP.RgdxaCenter, index + 1, rgdxaCenter, index + count, itcMac - (index));
							//copy rgtc
							Array.Copy(newTAP.Rgtc, 0, rgtc, 0, index);
							Array.Copy(newTAP.Rgtc, index, rgtc, index + count, itcMac - index);
						}
						
						for (int x = index; x < index + count; x++)
						{
							rgtc[x] = new TableCellDescriptor();
							rgdxaCenter[x] = (short) (rgdxaCenter[x - 1] + width);
						}
						rgdxaCenter[index + count] = (short) (rgdxaCenter[(index + count) - 1] + width);
						break;
					}
					/// <todo>  handle table sprms from complex files </todo>
				
				case 0x22: 
				case 0x23: 
				case 0x24: 
				case 0x25: 
				case 0x26: 
				case 0x27: 
				case 0x28: 
				case 0x29: 
				case 0x2a: 
				case 0x2b: 
				case 0x2c: 
					break;
				
				default: 
					break;
				
			}
		}
	}
}