// 
// HWPFList.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;

using Poi.Net.HWPF;
using Poi.Net.HWPF.Model;
using Poi.Net.HWPF.SPRM;

namespace Poi.Net.HWPF.UserModel
{
	
	/// <summary> This class is used to create a list in a Word document. It is used in
	/// conjunction with 
	/// Poi.Net.HWPF.HWPFDocument#RegisterList(HWPFList) registerList} in
	///  Poi.Net.HWPF.HWPFDocument HWPFDocument}.
	/// 
	/// In Word, lists are not ranged entities, meaning you can't actually add one
	/// to the document. Lists only act as properties for list entries. Once you
	/// register a list, you can add list entries to a document that are a part of
	/// the list.
	/// 
	/// The only benefit of this that I see, is that you can add a list entry
	/// anywhere in the document and continue numbering from the previous list.
	/// 
	/// </summary>
	/// <author>  Ryan Ackley
	/// </author>
	public class HWPFList
	{
		virtual public ListData ListData {
			get {
				return _listData;
			}
			
		}
		virtual public ListFormatOverride Override {
			get {
				return _override;
			}
			
		}
		private ListData _listData;
		private ListFormatOverride _override;
		//unused: private bool _registered;
		private StyleSheet _styleSheet;
		
		/// <summary> </summary>
		/// <param name="numbered">true if the list should be numbered; false if it should be
		/// bulleted.
		/// </param>
		/// <param name="styleSheet">The document's stylesheet.
		/// </param>
		public HWPFList(bool numbered, StyleSheet styleSheet)
		{
			
			Random r = new Random();
			_listData = new ListData((int) (r.NextDouble() * (double) ((System.DateTime.Now.Ticks - 621355968000000000) / 10000)), numbered);
			_override = new ListFormatOverride(_listData.Lsid);
			_styleSheet = styleSheet;
		}
		
		/// <summary> Sets the character properties of the list numbers.
		/// 
		/// </summary>
		/// <param name="level">the level number that the properties should apply to.
		/// </param>
		/// <param name="chp">The character properties.
		/// </param>
		public virtual void SetLevelNumberProperties(int level, CharacterProperties chp)
		{
			ListLevel listLevel = _listData.GetLevel(level);
			int styleIndex = _listData.GetLevelStyle(level);
			CharacterProperties _base = _styleSheet.GetCharacterStyle(styleIndex);
			
			byte[] grpprl = CharacterSprmCompressor.CompressCharacterProperty(chp, _base);
			listLevel.NumberProperties = grpprl;
		}
		
		/// <summary> Sets the paragraph properties for a particular level of the list.
		/// 
		/// </summary>
		/// <param name="level">The level number.
		/// </param>
		/// <param name="pap">The paragraph properties
		/// </param>
		public virtual void SetLevelParagraphProperties(int level, ParagraphProperties pap)
		{
			ListLevel listLevel = _listData.GetLevel(level);
			int styleIndex = _listData.GetLevelStyle(level);
			ParagraphProperties _base = _styleSheet.GetParagraphStyle(styleIndex);
			
			byte[] grpprl = ParagraphSprmCompressor.CompressParagraphProperty(pap, _base);
			listLevel.LevelProperties = grpprl;
		}
		
		public virtual void SetLevelStyle(int level, int styleIndex)
		{
			_listData.SetLevelStyle(level, styleIndex);
		}
	}
}