// 
// Range.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Collections;

using Poi.Net.Util;
using Poi.Net.HWPF;
using Poi.Net.HWPF.Model;
using Poi.Net.HWPF.SPRM;

namespace Poi.Net.HWPF.UserModel
{
	
	/// <summary> This class is the central class of the HWPF object model. All properties
	/// that apply to a range of characters in a Word document extend this class.
	/// 
	/// It is possible to insert text and/or properties at the beginning or end of a
	/// range.
	/// 
	/// Ranges are only valid if there hasn't been an insert in a prior Range since
	/// the Range's creation. Once an element (text, paragraph, etc.) has been
	/// inserted into a Range, subsequent Ranges become unstable.
	/// 
	/// </summary>
	/// <author>  Ryan Ackley
	/// </author>
	public class Range
	{
		
		public const int TYPE_PARAGRAPH = 0;
		public const int TYPE_CHARACTER = 1;
		public const int TYPE_SECTION = 2;
		public const int TYPE_TEXT = 3;
		public const int TYPE_LISTENTRY = 4;
		public const int TYPE_TABLE = 5;
		public const int TYPE_UNDEFINED = 6;
		
		/// <summary>Needed so inserts and deletes will ripple up through containing Ranges </summary>
		private System.WeakReference _parent;
		
		/// <summary>The starting character offset of this range.</summary>
		protected internal int _start;
		
		/// <summary>The ending character offset of this range.</summary>
		protected internal int _end;
		
		/// <summary>The document this range blongs to.</summary>
		protected internal HWPFDocument _doc;
		
		/// <summary>Have we loaded the section indexes yet</summary>
		internal bool _sectionRangeFound;
		
		/// <summary>All sections that belong to the document this Range belongs to.</summary>
		protected internal IList _sections;
		
		/// <summary>The start index in the sections list for this Range</summary>
		protected internal int _sectionStart;
		
		/// <summary>The end index in the sections list for this Range.</summary>
		protected internal int _sectionEnd;
		
		/// <summary>Have we loaded the paragraph indexes yet.</summary>
		protected internal bool _parRangeFound;
		
		/// <summary>All paragraphs that belong to the document this Range belongs to.</summary>
		protected internal IList _paragraphs;
		
		/// <summary>The start index in the paragraphs list for this Range</summary>
		protected internal int _parStart;
		
		/// <summary>The end index in the paragraphs list for this Range.</summary>
		protected internal int _parEnd;
		
		/// <summary>Have we loaded the characterRun indexes yet.</summary>
		protected internal bool _charRangeFound;
		
		/// <summary>All CharacterRuns that belong to the document this Range belongs to.</summary>
		protected internal IList _characters;
		
		/// <summary>The start index in the characterRuns list for this Range</summary>
		protected internal int _charStart;
		
		/// <summary>The end index in the characterRuns list for this Range. </summary>
		protected internal int _charEnd;
		
		/// <summary>Have we loaded the Text indexes yet</summary>
		protected internal bool _textRangeFound;
		
		/// <summary>All text pieces that belong to the document this Range belongs to.</summary>
		protected internal IList _text;
		
		/// <summary>The start index in the text list for this Range.</summary>
		protected internal int _textStart;
		
		/// <summary>The end index in the text list for this Range.</summary>
		protected internal int _textEnd;
		
		//  protected Range()
		//  {
		//
		//  }
		
		/// <summary> Used to construct a Range from a document. This is generally used to
		/// create a Range that spans the whole document.
		/// 
		/// </summary>
		/// <param name="start">Starting character offset of the range.
		/// </param>
		/// <param name="end">Ending character offset of the range.
		/// </param>
		/// <param name="doc">The HWPFDocument the range is based on.
		/// </param>
		public Range(int start, int end, HWPFDocument doc)
		{
			_start = start;
			_end = end;
			_doc = doc;
			_sections = _doc.SectionTable.Sections;
			_paragraphs = _doc.ParagraphTable.Paragraphs;
			_characters = _doc.CharacterTable.TextRuns;
			_text = _doc.TextTable.TextPieces;
			_parent = new System.WeakReference(null);
		}
		
		
		/// <summary> Used to create Ranges that are children of other Ranges.
		/// 
		/// </summary>
		/// <param name="start">Starting character offset of the range.
		/// </param>
		/// <param name="end">Ending character offset of the range.
		/// </param>
		/// <param name="parent">The parent this range belongs to.
		/// </param>
		protected internal Range(int start, int end, Range parent)
		{
			_start = start;
			_end = end;
			_doc = parent._doc;
			_sections = parent._sections;
			_paragraphs = parent._paragraphs;
			_characters = parent._characters;
			_text = parent._text;
			_parent = new System.WeakReference(parent);
		}
		
		/// <summary> Constructor used to build a Range from indexes in one of its internal
		/// lists.
		/// 
		/// </summary>
		/// <param name="startIdx">The starting index in the list.
		/// </param>
		/// <param name="endIdx">The ending index in the list.
		/// </param>
		/// <param name="idxType">The list type.
		/// </param>
		/// <param name="parent">The parent Range this range belongs to.
		/// </param>
		protected internal Range(int startIdx, int endIdx, int idxType, Range parent)
		{
			_doc = parent._doc;
			_sections = parent._sections;
			_paragraphs = parent._paragraphs;
			_characters = parent._characters;
			_text = parent._text;
			_parent = new System.WeakReference(parent);
			
			switch (idxType)
			{
				
				case TYPE_PARAGRAPH: 
					_parStart = parent._parStart + startIdx;
					_parEnd = parent._parStart + endIdx;
					_start = ((PropertyNode) _paragraphs[_parStart]).Start;
					_end = ((PropertyNode) _paragraphs[_parEnd]).End;
					_parRangeFound = true;
					break;
				
				case TYPE_CHARACTER: 
					_charStart = parent._charStart + startIdx;
					_charEnd = parent._charStart + endIdx;
					_start = ((PropertyNode) _characters[_charStart]).Start;
					_end = ((PropertyNode) _characters[_charEnd]).End;
					_charRangeFound = true;
					break;
				
				case TYPE_SECTION: 
					_sectionStart = parent._sectionStart + startIdx;
					_sectionEnd = parent._sectionStart + endIdx;
					_start = ((PropertyNode) _sections[_sectionStart]).Start;
					_end = ((PropertyNode) _sections[_sectionEnd]).End;
					_sectionRangeFound = true;
					break;
				
				case TYPE_TEXT: 
					_textStart = parent._textStart + startIdx;
					_textEnd = parent._textStart + endIdx;
					_start = ((PropertyNode) _text[_textStart]).Start;
					_end = ((PropertyNode) _text[_textEnd]).End;
					_textRangeFound = true;
					break;
				}
		}
		
		/// <summary> Gets the text that this Range contains.
		/// 
		/// </summary>
		/// <returns> The text for this range.
		/// </returns>
		public virtual string Text {
			get {

				InitText();
			
				System.Text.StringBuilder sb = new System.Text.StringBuilder();

				for (int x = _textStart; x < _textEnd; x++)
				{
					TextPiece piece = (TextPiece) _text[x];
					int start = _start > piece.Start?_start - piece.Start:0;
					int end = _end <= piece.End?_end - piece.Start:piece.End - piece.Start;
				
					if (piece.UsesUnicode)
						// convert the byte pointers to char pointers
					{
						start /= 2;
						end /= 2;
					}
					sb.Append(piece.StringBuffer.ToString(start, end));
				}

				return sb.ToString();
			}
		}
		
		/// <summary> Used to get the number of sections in a range. If this range is smaller
		/// than a section, it will return 1 for its containing section.
		/// 
		/// </summary>
		/// <returns> The number of sections in this range.
		/// </returns>
		public virtual int NumSections {
			get {
				InitSections();
				return _sectionEnd - _sectionStart;
			}

		}
		
		/// <summary> Used to get the number of paragraphs in a range. If this range is smaller
		/// than a paragraph, it will return 1 for its containing paragraph.
		/// 
		/// </summary>
		/// <returns> The number of paragraphs in this range.
		/// </returns>
		
		public virtual int NumParagraphs()
		{
			InitParagraphs();
			return _parEnd - _parStart;
		}
		
		/// <summary> </summary>
		/// <returns> The number of characterRuns in this range.
		/// </returns>
		
		public virtual int NumCharacterRuns {
			get {
				InitCharacterRuns();
				return _charEnd - _charStart;
			}
		}
		
		/// <summary> Inserts text into the front of this range.
		/// 
		/// </summary>
		/// <param name="text">The text to insert
		/// </param>
		/// <returns> The character run that text was inserted into.
		/// </returns>
		public virtual CharacterRun InsertBefore(string text)
		//throws UnsupportedEncodingException
		{
			InitAll();
			
			TextPiece tp = (TextPiece) _text[_textStart];
			System.Text.StringBuilder sb = (System.Text.StringBuilder) tp.StringBuffer;
			
			// Since this is the first item in our list, it is safe to assume that
			// _start >= tp.GetStart
			int insertIndex = _start - tp.Start;
			sb.Insert(insertIndex, text);
			int adjustedLength = _doc.TextTable.AdjustForInsert(_textStart, text.Length);
			_doc.CharacterTable.AdjustForInsert(_charStart, adjustedLength);
			_doc.ParagraphTable.AdjustForInsert(_parStart, adjustedLength);
			_doc.SectionTable.AdjustForInsert(_sectionStart, adjustedLength);
			AdjustForInsert(text.Length);
			
			return GetCharacterRun(0);
		}
		
		/// <summary> Inserts text onto the end of this range
		/// 
		/// </summary>
		/// <param name="text">The text to insert
		/// </param>
		/// <returns> The character run the text was inserted into.
		/// </returns>
		public virtual CharacterRun InsertAfter(string text)
		{
			InitAll();
			
			int listIndex = _textEnd - 1;
			TextPiece tp = (TextPiece) _text[listIndex];
			System.Text.StringBuilder sb = (System.Text.StringBuilder) tp.StringBuffer;
			
			int insertIndex = _end - tp.Start;
			
			if (tp.StringBuffer[_end - 1] == '\r' && text[0] != '\u0007')
			{
				insertIndex--;
			}
			sb.Insert(insertIndex, text);
			int adjustedLength = _doc.TextTable.AdjustForInsert(listIndex, text.Length);
			_doc.CharacterTable.AdjustForInsert(_charEnd - 1, adjustedLength);
			_doc.ParagraphTable.AdjustForInsert(_parEnd - 1, adjustedLength);
			_doc.SectionTable.AdjustForInsert(_sectionEnd - 1, adjustedLength);
			AdjustForInsert(text.Length);
			
			return GetCharacterRun(NumCharacterRuns - 1);
		}
		
		/// <summary> Inserts text into the front of this range and it gives that text the
		/// CharacterProperties specified in props.
		/// 
		/// </summary>
		/// <param name="text">The text to insert.
		/// </param>
		/// <param name="props">The CharacterProperties to give the text.
		/// </param>
		/// <returns> A new CharacterRun that has the given text and properties and is n
		/// ow a part of the document.
		/// </returns>
		public virtual CharacterRun InsertBefore(string text, CharacterProperties props)
		//throws UnsupportedEncodingException
		{
			InitAll();
			PAPX papx = (PAPX) _paragraphs[_parStart];
			short istd = papx.Istd;
			
			StyleSheet ss = _doc.StyleSheet;
			CharacterProperties baseStyle = ss.GetCharacterStyle(istd);
			byte[] grpprl = CharacterSprmCompressor.CompressCharacterProperty(props, baseStyle);
			SprmBuffer buf = new SprmBuffer(grpprl);
			_doc.CharacterTable.Insert(_charStart, _start, buf);
			
			return InsertBefore(text);
		}
		
		/// <summary> Inserts text onto the end of this range and gives that text the
		/// CharacterProperties specified in props.
		/// 
		/// </summary>
		/// <param name="text">The text to insert.
		/// </param>
		/// <param name="props">The CharacterProperties to give the text.
		/// </param>
		/// <returns> A new CharacterRun that has the given text and properties and is n
		/// ow a part of the document.
		/// </returns>
		public virtual CharacterRun InsertAfter(string text, CharacterProperties props)
		//throws UnsupportedEncodingException
		{
			InitAll();
			PAPX papx = (PAPX) _paragraphs[_parEnd - 1];
			short istd = papx.Istd;
			
			StyleSheet ss = _doc.StyleSheet;
			CharacterProperties baseStyle = ss.GetCharacterStyle(istd);
			byte[] grpprl = CharacterSprmCompressor.CompressCharacterProperty(props, baseStyle);
			SprmBuffer buf = new SprmBuffer(grpprl);
			_doc.CharacterTable.Insert(_charEnd, _end, buf);
			_charEnd++;
			return InsertAfter(text);
		}
		
		/// <summary> Inserts and empty paragraph into the front of this range.
		/// 
		/// </summary>
		/// <param name="props">The properties that the new paragraph will have.
		/// </param>
		/// <param name="styleIndex">The index into the stylesheet for the new paragraph.
		/// </param>
		/// <returns> The newly inserted paragraph.
		/// </returns>
		public virtual Paragraph InsertBefore(ParagraphProperties props, int styleIndex)
		//throws UnsupportedEncodingException
		{
			return this.InsertBefore(props, styleIndex, "\r");
		}
		
		/// <summary> Inserts a paragraph into the front of this range. The paragraph will
		/// contain one character run that has the default properties for the
		/// paragraph's style.
		/// 
		/// It is necessary for the text to end with the character '\r'
		/// 
		/// </summary>
		/// <param name="props">The paragraph's properties.
		/// </param>
		/// <param name="styleIndex">The index of the paragraph's style in the style sheet.
		/// </param>
		/// <param name="text">The text to insert.
		/// </param>
		/// <returns> A newly inserted paragraph.
		/// </returns>
		protected internal virtual Paragraph InsertBefore(ParagraphProperties props, int styleIndex, string text)
		//throws UnsupportedEncodingException
		{
			InitAll();
			StyleSheet ss = _doc.StyleSheet;
			ParagraphProperties baseStyle = ss.GetParagraphStyle(styleIndex);
			CharacterProperties baseChp = ss.GetCharacterStyle(styleIndex);
			
			byte[] grpprl = ParagraphSprmCompressor.CompressParagraphProperty(props, baseStyle);
			byte[] withIndex = new byte[grpprl.Length + LittleEndian.SHORT_SIZE];
			LittleEndian.PutShort(withIndex, (short) styleIndex);
			Array.Copy(grpprl, 0, withIndex, LittleEndian.SHORT_SIZE, grpprl.Length);
			SprmBuffer buf = new SprmBuffer(withIndex);
			
			_doc.ParagraphTable.Insert(_parStart, _start, buf);
			InsertBefore(text, baseChp);
			return GetParagraph(0);
		}
		
		/// <summary> Inserts and empty paragraph into the end of this range.
		/// 
		/// </summary>
		/// <param name="props">The properties that the new paragraph will have.
		/// </param>
		/// <param name="styleIndex">The index into the stylesheet for the new paragraph.
		/// </param>
		/// <returns> The newly inserted paragraph.
		/// </returns>
		
		public virtual Paragraph InsertAfter(ParagraphProperties props, int styleIndex)
		//throws UnsupportedEncodingException
		{
			return this.InsertAfter(props, styleIndex, "\r");
		}
		
		/// <summary> Inserts a paragraph into the end of this range. The paragraph will
		/// contain one character run that has the default properties for the
		/// paragraph's style.
		/// 
		/// It is necessary for the text to end with the character '\r'
		/// 
		/// </summary>
		/// <param name="props">The paragraph's properties.
		/// </param>
		/// <param name="styleIndex">The index of the paragraph's style in the style sheet.
		/// </param>
		/// <param name="text">The text to insert.
		/// </param>
		/// <returns> A newly inserted paragraph.
		/// </returns>
		protected internal virtual Paragraph InsertAfter(ParagraphProperties props, int styleIndex, string text)
		//throws UnsupportedEncodingException
		{
			InitAll();
			StyleSheet ss = _doc.StyleSheet;
			ParagraphProperties baseStyle = ss.GetParagraphStyle(styleIndex);
			CharacterProperties baseChp = ss.GetCharacterStyle(styleIndex);
			
			byte[] grpprl = ParagraphSprmCompressor.CompressParagraphProperty(props, baseStyle);
			byte[] withIndex = new byte[grpprl.Length + LittleEndian.SHORT_SIZE];
			LittleEndian.PutShort(withIndex, (short) styleIndex);
			Array.Copy(grpprl, 0, withIndex, LittleEndian.SHORT_SIZE, grpprl.Length);
			SprmBuffer buf = new SprmBuffer(withIndex);
			
			_doc.ParagraphTable.Insert(_parEnd, _end, buf);
			_parEnd++;
			InsertAfter(text, baseChp);
			return GetParagraph(NumParagraphs() - 1);
		}
		
		public virtual void Delete()
		{
			InitAll();
			
			int numSections = _sections.Count;
			int numRuns = _characters.Count;
			int NumParagraphs = _paragraphs.Count;
			
			for (int x = _charStart; x < numRuns; x++)
			{
				CHPX chpx = (CHPX) _characters[x];
				chpx.AdjustForDelete(_start, _end - _start);
			}
			
			for (int x = _parStart; x < NumParagraphs; x++)
			{
				PAPX papx = (PAPX) _paragraphs[x];
				papx.AdjustForDelete(_start, _end - _start);
			}
			
			for (int x = _sectionStart; x < numSections; x++)
			{
				SEPX sepx = (SEPX) _sections[x];
				sepx.AdjustForDelete(_start, _end - _start);
			}
		}
		
		/// <summary> Inserts a simple table into the beginning of this range. The number of
		/// columns is determined by the TableProperties passed into this function.
		/// 
		/// </summary>
		/// <param name="props">The table properties for the table.
		/// </param>
		/// <param name="rows">The number of rows.
		/// </param>
		/// <returns> The empty Table that is now part of the document.
		/// </returns>
		public virtual Table InsertBefore(TableProperties props, int rows)
		{
			ParagraphProperties parProps = new ParagraphProperties();
			parProps.FInTable = (byte) 1;
			parProps.TableLevel = (byte) 1;
			
			int columns = props.ItcMac;
			for (int x = 0; x < rows; x++)
			{
				Paragraph cell = this.InsertBefore(parProps, StyleSheet.NIL_STYLE);
				cell.InsertAfter(System.Convert.ToString('\u0007'));
				for (int y = 1; y < columns; y++)
				{
					cell = cell.InsertAfter(parProps, StyleSheet.NIL_STYLE);
					cell.InsertAfter(System.Convert.ToString('\u0007'));
				}
				cell = cell.InsertAfter(parProps, StyleSheet.NIL_STYLE, System.Convert.ToString('\u0007'));
				cell.SetTableRowEnd(props);
			}
			return new Table(_start, _start + (rows * (columns + 1)), this, 1);
		}
		
		/// <summary> Inserts a list into the beginning of this range.
		/// 
		/// </summary>
		/// <param name="props">The properties of the list entry. All list entries are
		/// paragraphs.
		/// </param>
		/// <param name="listID">The id of the list that contains the properties.
		/// </param>
		/// <param name="level">The indentation level of the list.
		/// </param>
		/// <param name="styleIndex">The base style's index in the stylesheet.
		/// </param>
		/// <returns> The empty ListEntry that is now part of the document.
		/// </returns>
		public virtual ListEntry InsertBefore(ParagraphProperties props, int listID, int level, int styleIndex)
		{
			ListTables lt = _doc.ListTables;
			if (lt.GetLevel(listID, level) == null)
			{
				throw new System.ArgumentOutOfRangeException("The specified list and level do not exist");
			}
			
			int ilfo = lt.GetOverrideIndexFromListID(listID);
			props.Ilfo = ilfo;
			props.Ilvl = (byte) level;
			
			return (ListEntry) InsertBefore(props, styleIndex);
		}
		
		/// <summary> Inserts a list into the beginning of this range.
		/// 
		/// </summary>
		/// <param name="props">The properties of the list entry. All list entries are
		/// paragraphs.
		/// </param>
		/// <param name="listID">The id of the list that contains the properties.
		/// </param>
		/// <param name="level">The indentation level of the list.
		/// </param>
		/// <param name="styleIndex">The base style's index in the stylesheet.
		/// </param>
		/// <returns> The empty ListEntry that is now part of the document.
		/// </returns>
		public virtual ListEntry InsertAfter(ParagraphProperties props, int listID, int level, int styleIndex)
		{
			ListTables lt = _doc.ListTables;
			if (lt.GetLevel(listID, level) == null)
			{
				throw new System.ArgumentOutOfRangeException("The specified list and level do not exist");
			}
			int ilfo = lt.GetOverrideIndexFromListID(listID);
			props.Ilfo = ilfo;
			props.Ilvl = (byte) level;
			
			return (ListEntry) InsertAfter(props, styleIndex);
		}
		
		
		/// <summary> Gets the character run at index. The index is relative to this range.
		/// 
		/// </summary>
		/// <param name="index">The index of the character run to get.
		/// </param>
		/// <returns> The character run at the specified index in this range.
		/// </returns>
		public virtual CharacterRun GetCharacterRun(int index)
		{
			InitCharacterRuns();
			CHPX chpx = (CHPX) _characters[index + _charStart];
			
			int[] point = FindRange(_paragraphs, _parStart, System.Math.Max(chpx.Start, _start), chpx.End);
			PAPX papx = (PAPX) _paragraphs[point[0]];
			short istd = papx.Istd;
			
			CharacterRun chp = new CharacterRun(chpx, _doc.StyleSheet, istd, this);
			
			return chp;
		}
		
		/// <summary> Gets the section at index. The index is relative to this range.
		/// 
		/// </summary>
		/// <param name="index">The index of the section to get.
		/// </param>
		/// <returns> The section at the specified index in this range.
		/// </returns>
		public virtual Section GetSection(int index)
		{
			InitSections();
			SEPX sepx = (SEPX) _sections[index + _sectionStart];
			Section sep = new Section(sepx, this);
			return sep;
		}
		
		/// <summary> Gets the paragraph at index. The index is relative to this range.
		/// 
		/// </summary>
		/// <param name="index">The index of the paragraph to get.
		/// </param>
		/// <returns> The paragraph at the specified index in this range.
		/// </returns>
		
		public virtual Paragraph GetParagraph(int index)
		{
			InitParagraphs();
			PAPX papx = (PAPX) _paragraphs[index + _parStart];
			
			ParagraphProperties props = papx.GetParagraphProperties(_doc.StyleSheet);
			Paragraph pap = null;
			if (props.Ilfo > 0)
			{
				pap = new ListEntry(papx, this, _doc.ListTables);
			}
			else
			{
				pap = new Paragraph(papx, this);
			}
			
			return pap;
		}
		
		/// <summary> This method is used to determine the type. Handy for switch statements
		/// compared to the instanceof operator.
		/// 
		/// </summary>
		/// <returns> A TYPE constant.
		/// </returns>
		public virtual int Type {
			get {	return TYPE_UNDEFINED; }
		}
		
		/// <summary> Gets the table that starts with paragraph. In a Word file, a table consists
		/// of a group of paragraphs with certain flags set.
		/// 
		/// </summary>
		/// <param name="paragraph">The paragraph that is the first paragraph in the table.
		/// </param>
		/// <returns> The table that starts with paragraph
		/// </returns>
		public virtual Table GetTable(Paragraph paragraph)
		{
			if (!paragraph.InTable)
			{
				throw new System.ArgumentException("This paragraph doesn't belong to a table");
			}
			
			Range r = (Range) paragraph;
			
			if (r._parent.Target != this)
			{
				throw new System.ArgumentException("This paragraph is not a child of this range");
			}
			
			r.InitAll();
			int tableEnd = r._parEnd;
			
			if (r._parStart != 0 && GetParagraph(r._parStart - 1).InTable && GetParagraph(r._parStart - 1)._sectionEnd >= r._sectionStart)
			{
				throw new System.ArgumentException("This paragraph is not the first one in the table");
			}
			
			int limit = _paragraphs.Count;
			for (; tableEnd < limit; tableEnd++)
			{
				if (!GetParagraph(tableEnd).InTable)
				{
					break;
				}
			}
			
			InitAll();
			if (tableEnd > _parEnd)
			{
				throw new System.IndexOutOfRangeException("The table's bounds fall outside of this Range");
			}
			return new Table(r._parStart, tableEnd, r._doc.Range, paragraph.TableLevel);
		}
		
		/// <summary> loads all of the list indexes.</summary>
		protected internal virtual void InitAll()
		{
			InitText();
			InitCharacterRuns();
			InitParagraphs();
			InitSections();
		}
		
		/// <summary> inits the paragraph list indexes.</summary>
		private void InitParagraphs()
		{
			if (!_parRangeFound)
			{
				int[] point = FindRange(_paragraphs, _parStart, _start, _end);
				_parStart = point[0];
				_parEnd = point[1];
				_parRangeFound = true;
			}
		}
		
		/// <summary> inits the character run list indexes.</summary>
		private void InitCharacterRuns()
		{
			if (!_charRangeFound)
			{
				int[] point = FindRange(_characters, _charStart, _start, _end);
				_charStart = point[0];
				_charEnd = point[1];
				_charRangeFound = true;
			}
		}
		
		/// <summary> inits the text piece list indexes.</summary>
		private void InitText()
		{
			if (!_textRangeFound)
			{
				int[] point = FindRange(_text, _textStart, _start, _end);
				_textStart = point[0];
				_textEnd = point[1];
				_textRangeFound = true;
			}
		}
		
		/// <summary> inits the section list indexes.</summary>
		private void InitSections()
		{
			if (!_sectionRangeFound)
			{
				int[] point = FindRange(_sections, _sectionStart, _start, _end);
				_sectionStart = point[0];
				_sectionEnd = point[1];
				_sectionRangeFound = true;
			}
		}
		
		/// <summary> Used to find the list indexes of a particular property.
		/// 
		/// </summary>
		/// <param name="rpl">A list of property nodes.
		/// </param>
		/// <param name="min">A hint on where to start looking.
		/// </param>
		/// <param name="start">The starting character offset.
		/// </param>
		/// <param name="end">The ending character offset.
		/// </param>
		/// <returns> An int array of length 2. The first int is the start index and the
		/// second int is the end index.
		/// </returns>
		private int[] FindRange(IList rpl, int min, int start, int end)
		{
			int x = min;
			PropertyNode node = (PropertyNode) rpl[x];
			while (node.End <= start && x < rpl.Count - 1)
			{
				x++;
				node = (PropertyNode) rpl[x];
			}
			
			int y = x;
			node = (PropertyNode) rpl[y];
			while (node.End < end && y < rpl.Count - 1)
			{
				y++;
				node = (PropertyNode) rpl[y];
			}
			return new int[]{x, y + 1};
		}
		
		/// <summary> resets the list indexes.</summary>
		private void Reset()
		{
			_textRangeFound = false;
			_charRangeFound = false;
			_parRangeFound = false;
			_sectionRangeFound = false;
		}
		
		/// <summary> adjust this range after an insert happens.</summary>
		/// <param name="length">the length to adjust for
		/// </param>
		private void AdjustForInsert(int length)
		{
			_end += length;
			Reset();
			
			Range parent = (Range) _parent.Target;
			if (parent != null)
			{
				parent.AdjustForInsert(length);
			}
		}

		public virtual object Clone() 
		{
			return this.MemberwiseClone();
		}
	}
}