// 
// POIFSReader.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Text;
using System.Collections;

namespace Poi.Net.PoiFS.Dev
{
	
	/// <summary> This class contains methods used to inspect POIFSViewable objects
	/// 
	/// </summary>
	
	public class POIFSViewEngine
	{
		
		private static readonly string _EOL = Environment.NewLine;
		
		/// <summary> Inspect an object that may be viewable, and drill down if told
		/// to
		/// 
		/// </summary>
		/// <param name="viewable">the object to be viewed
		/// </param>
		/// <param name="drilldown">if true, and the object implements
		/// POIFSViewable, inspect the objects' contents
		/// (recursively)
		/// </param>
		/// <param name="indentLevel">how far in to indent each string
		/// </param>
		/// <param name="indentString">string to use for indenting
		/// 
		/// </param>
		/// <returns> a List of Strings holding the content
		/// </returns>
		
		public static IList InspectViewable(object viewable, bool drilldown, int indentLevel, string indentString)
		{
			IList objects = new ArrayList();
			
			if (viewable is POIFSViewable)
			{
				POIFSViewable inspected = (POIFSViewable) viewable;
				
				objects.Add(Indent(indentLevel, indentString, inspected.ShortDescription));
				if (drilldown)
				{
					if (inspected.PreferArray)
					{
						object[] data = inspected.ViewableArray;
						
						for (int j = 0; j < data.Length; j++)
						{
							((ArrayList)objects).AddRange(InspectViewable(data[j], drilldown, indentLevel + 1, indentString));
						}
					}
					else
					{
						IEnumerator iter = inspected.ViewableIterator;
						
						while (iter.MoveNext())
						{
							((ArrayList)objects).AddRange(InspectViewable(iter.Current, drilldown, indentLevel + 1, indentString));
						}
					}
				}
			}
			else
			{
				
				objects.Add(Indent(indentLevel, indentString, viewable.ToString()));
			}
			return objects;
		}
		
		private static string Indent(int indentLevel, string indentString, string data)
		{
			StringBuilder finalBuffer = new StringBuilder();
			StringBuilder indentPrefix = new StringBuilder();
			
			for (int j = 0; j < indentLevel; j++)
			{
				indentPrefix.Append(indentString);
			}
			
			StreamReader reader = new StreamReader(data);
			
			try
			{
				
				string line = reader.ReadLine();
				
				while (line != null)
				{
					finalBuffer.Append(indentPrefix).Append(line).Append(_EOL);
					
					line = reader.ReadLine();
				}
			}
			catch (IOException e)
			{
				finalBuffer.Append(indentPrefix).Append(e.Message).Append(_EOL);
			}
			return finalBuffer.ToString();
		}
	} 
}