// 
// POIFSReader.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.FileSystem;
using Poi.Net.PoiFS.Properties;
using Poi.Net.PoiFS.Storage;

namespace Poi.Net.PoiFS.EventFileSystem
{
	
	/// <summary>
	/// An event-driven reader for POIFS file systems. Users of this class
	/// first create an instance of it, then use the registerListener
	/// methods to register POIFSReaderListener instances for specific
	/// documents. Once all the listeners have been registered, the read()
	/// method is called, which results in the listeners being notified as
	/// their documents are read.
	/// 
	/// </summary>
	
	public class POIFSReader
	{
		private POIFSReaderRegistry registry;
		private bool registryClosed;
		
		/// <summary> Create a POIFSReader</summary>
		
		public POIFSReader()
		{
			registry = new POIFSReaderRegistry();
			registryClosed = false;
		}
		
		/// <summary> Read from an InputStream and process the documents we get
		/// 
		/// </summary>
		/// <param name="stream">the InputStream from which to read the data
		/// 
		/// </param>
		/// <exception cref=""> IOException on errors reading, or on invalid data
		/// </exception>
		
		public virtual void Read(Stream stream)
		{
			registryClosed = true;
			
			// read the header block from the stream
			HeaderBlockReader header_block_reader = new HeaderBlockReader(stream);
			// read the rest of the stream into blocks
			RawDataBlockList data_blocks = new RawDataBlockList(stream);
			
			// set up the block allocation table (necessary for the
			// data_blocks to be manageable
			new BlockAllocationTableReader(header_block_reader.BATCount,
											header_block_reader.BATArray,
											header_block_reader.XBATCount,
											header_block_reader.XBATIndex, 
											data_blocks);
			
			// get property table from the document
			PropertyTable properties = new PropertyTable(header_block_reader.PropertyStart, 
															data_blocks);
			
			// process documents
			ProcessProperties(
				SmallBlockTableReader
					.GetSmallDocumentBlocks(data_blocks,
											properties.Root,
											header_block_reader.SBATStart),
				data_blocks, properties.Root.Children,
				new POIFSDocumentPath());
		}
		
		/// <summary> Register a POIFSReaderListener for all documents
		/// 
		/// </summary>
		/// <param name="listener">the listener to be registered
		/// 
		/// </param>
		/// <exception cref=""> NullPointerException if listener is null
		/// </exception>
		/// <exception cref=""> SystemException if Read() has already been
		/// called
		/// </exception>
		
		public virtual void RegisterListener(POIFSReaderListener listener)
		{
			if (listener == null)
				throw new NullReferenceException();

			if (registryClosed)
				throw new SystemException("Illegal State");

			registry.RegisterListener(listener);
		}
		
		/// <summary>
		/// Register a POIFSReaderListener for a document in the root
		/// directory
		/// 
		/// </summary>
		/// <param name="listener">the listener to be registered
		/// </param>
		/// <param name="name">the document name
		/// </param>
		/// <exception cref=""> NullPointerException if listener is null or name is
		/// null or empty
		/// </exception>
		/// <exception cref=""> SystemException if Read() has already been
		/// called
		/// </exception>
		
		public virtual void RegisterListener (POIFSReaderListener listener, string name)
		{
			RegisterListener(listener, null, name);
		}
		
		/// <summary> Register a POIFSReaderListener for a document in the specified
		/// directory
		/// 
		/// </summary>
		/// <param name="listener">the listener to be registered
		/// </param>
		/// <param name="path">the document path; if null, the root directory is
		/// assumed
		/// </param>
		/// <param name="name">the document name
		/// 
		/// </param>
		/// <exception cref=""> NullPointerException if listener is null or name is
		/// null or empty
		/// </exception>
		/// <exception cref=""> SystemException if Read() has already been
		/// called
		/// </exception>
		
		public virtual void RegisterListener(POIFSReaderListener listener, POIFSDocumentPath path, string name)
		{
			if ((listener == null) || ((object) name == null) || (name.Length == 0))
				throw new NullReferenceException();

			if (registryClosed)
				throw new SystemException();

			registry.RegisterListener(listener, (path == null)?new POIFSDocumentPath():path, name);
		}
		private void ProcessProperties(BlockList small_blocks, BlockList big_blocks, IEnumerator properties, POIFSDocumentPath path)
		{
			
			while (properties.MoveNext())
			{
				
				Property property = (Property) properties.Current;
				string name = property.Name;
				
				if (property.Directory)
				{
					POIFSDocumentPath new_path = new POIFSDocumentPath(path, new string[]{name});
					
					ProcessProperties(small_blocks, big_blocks, ((DirectoryProperty) property).Children, new_path);
				}
				else
				{	
					int startBlock = property.StartBlock;
					IEnumerator listeners = registry.GetListeners(path, name);
					
					
					if (listeners.MoveNext())
					{
						int size = property.Size;
						POIFSDocument document = null;
						
						if (property.ShouldUseSmallBlocks())
						{
							document = new POIFSDocument(name, small_blocks.FetchBlocks(startBlock), size);
						}
						else
						{
							document = new POIFSDocument(name, big_blocks.FetchBlocks(startBlock), size);
						}
						
						do
						{
							
							POIFSReaderListener listener = (POIFSReaderListener) listeners.Current;
							
							listener.ProcessPOIFSReaderEvent(new POIFSReaderEvent(new DocumentInputStream(document), path, name));
						} while (listeners.MoveNext());
					}
					else
					{
						
						// consume the document's data and discard it
						if (property.ShouldUseSmallBlocks())
						{
							small_blocks.FetchBlocks(startBlock);
						}
						else
						{
							big_blocks.FetchBlocks(startBlock);
						}
					}
				}
			}
		}
		/// <summary> read in files
		/// 
		/// </summary>
		/// <param name="args">names of the files
		/// 
		/// </param>
		/// <exception cref=""> IOException
		/// </exception>
		
		public static void _Main(string[] args)
		{
			if (args.Length == 0)
			{
				Console.Error.WriteLine("at least one argument required: input filename(s)");
				Environment.Exit(1);
			}
			
			// register for all
			for (int j = 0; j < args.Length; j++)
			{
				POIFSReader reader = new POIFSReader();
				POIFSReaderListener listener = new SampleListener();
				
				reader.RegisterListener(listener);
				Console.WriteLine("reading " + args[j]);
				
				FileStream stream = new FileStream(args[j], FileMode.Open, FileAccess.Read);
				
				reader.Read(stream);
				stream.Close();
			}
		}
		
		private class SampleListener : POIFSReaderListener
		{
			
			/// <summary> Constructor SampleListener</summary>
			
			internal SampleListener()
			{
			}
			
			/// <summary> Method processPOIFSReaderEvent
			/// 
			/// </summary>
			/// <param name="">event
			/// </param>
			
			public virtual void ProcessPOIFSReaderEvent(POIFSReaderEvent e)
			{
				DocumentInputStream stream = e.Stream;
				POIFSDocumentPath path = e.Path;
				string name = e.Name;
				
				try
				{
					byte[] data = new byte[stream.Length - stream.Position];
					
					stream.Read(data);
					int pathLength = path.Length;
					
					for (int k = 0; k < pathLength; k++)
					{
						Console.Write("/" + path.GetComponent(k));
					}
					Console.WriteLine("/" + name + ": " + data.Length + " bytes read");
				}
				catch (IOException ignored)
				{
				}
			}
		}
	}
}