// 
// POIFSReaderRegistry.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Collections;

using Poi.Net.PoiFS.FileSystem;

namespace Poi.Net.PoiFS.EventFileSystem
{
	
	/// <summary> A registry for POIFSReaderListeners and the DocumentDescriptors of
	/// the documents those listeners are interested in
	/// 
	/// </summary>
	/// <version>  %I%, %G%
	/// </version>
	
	class POIFSReaderRegistry
	{
		
		// the POIFSReaderListeners who listen to all POIFSReaderEvents
		// HS
		private ArrayList omnivorousListeners;
		
		// Each mapping in this Map has a key consisting of a
		// POIFSReaderListener and a value cosisting of a Set of
		// DocumentDescriptors for the documents that POIFSReaderListener
		// is interested in; used to efficiently manage the registry
		private IDictionary selectiveListeners;
		
		// Each mapping in this Map has a key consisting of a
		// DocumentDescriptor and a value consisting of a Set of
		// POIFSReaderListeners for the document matching that
		// DocumentDescriptor; used when a document is found, to quickly
		// get the listeners interested in that document
		private IDictionary chosenDocumentDescriptors;
		
		/// <summary> Construct the registry</summary>
		
		internal POIFSReaderRegistry()
		{
			
			omnivorousListeners = new ArrayList();
			
			selectiveListeners = new Hashtable();
			
			chosenDocumentDescriptors = new Hashtable();
		}
		
		/// <summary> register a POIFSReaderListener for a particular document
		/// 
		/// </summary>
		/// <param name="listener">the listener
		/// </param>
		/// <param name="path">the path of the document of interest
		/// </param>
		/// <param name="documentName">the name of the document of interest
		/// </param>
		
		internal virtual void RegisterListener(POIFSReaderListener listener, POIFSDocumentPath path, string documentName)
		{
			if (!omnivorousListeners.Contains(listener))
			{
				
			

				// not an omnivorous listener (if it was, this method is a
				// no-op)

                // HS
				ArrayList descriptors = (ArrayList) selectiveListeners[listener];
				
				if (descriptors == null)
				{
					
					// this listener has not registered before
					
					descriptors = new ArrayList();
					selectiveListeners[listener] = descriptors;
				}
				DocumentDescriptor descriptor = new DocumentDescriptor(path, documentName);
				
				if(!descriptors.Contains(descriptor))
				{
					descriptors.Add(descriptor);
					
					// this listener wasn't already listening for this
					// document -- add the listener to the set of
					// listeners for this document
					// HS
					ArrayList listeners = (ArrayList) chosenDocumentDescriptors[descriptor];
					
					if (listeners == null)
					{
						// nobody was listening for this document before
						
						listeners = new ArrayList();
						chosenDocumentDescriptors[descriptor] = listeners;
					}
					if(!listeners.Contains(listener))
						listeners.Add(listener);
				}
			}
		}
		
		/// <summary> register for all documents
		/// 
		/// </summary>
		/// <param name="listener">the listener who wants to get all documents
		/// </param>
		
		internal virtual void RegisterListener(POIFSReaderListener listener)
		{
			if (!omnivorousListeners.Contains(listener))
			{
				
				// wasn't already listening for everything, so drop
				// anything listener might have been listening for and
				// then add the listener to the set of omnivorous
				// listeners
				RemoveSelectiveListener(listener);
				omnivorousListeners.Add(listener);
			}
		}
		
		/// <summary> get am iterator of listeners for a particular document
		/// 
		/// </summary>
		/// <param name="path">the document path
		/// </param>
		/// <param name="name">the name of the document
		/// 
		/// </param>
		/// <returns> an Iterator POIFSReaderListeners; may be empty
		/// </returns>
		
		internal virtual IEnumerator GetListeners(POIFSDocumentPath path, string name)
		{
			// HS
			ArrayList rval = new ArrayList();

			foreach (Object current in omnivorousListeners)
			{
				if (!rval.Contains(current))
					rval.Add(current);
			}
			// HS
			ArrayList selectiveListeners = (ArrayList) chosenDocumentDescriptors[new DocumentDescriptor(path, name)];
			
			if (selectiveListeners != null)
			{
				foreach (Object current in selectiveListeners)
					 if (!rval.Contains(current))
						rval.Add(current);

			}
			return rval.GetEnumerator();
		}
		
		private void RemoveSelectiveListener(POIFSReaderListener listener)
		{
			object tempObject;
			tempObject = selectiveListeners[listener];
			selectiveListeners.Remove(listener);
			// HS
			ArrayList selectedDescriptors = (ArrayList) tempObject;
			
			if (selectedDescriptors != null)
			{
				foreach (DocumentDescriptor docDescriptor in selectedDescriptors)
					DropDocument(listener, docDescriptor);
			}
		}
		
		private void DropDocument(POIFSReaderListener listener, DocumentDescriptor descriptor)
		{
			ArrayList listeners = (ArrayList) chosenDocumentDescriptors[descriptor];
			
			listeners.Remove(listener);
			if (listeners.Count == 0)
			{
				chosenDocumentDescriptors.Remove(descriptor);
			}
		}
	}
}