// 
// DirectoryNode.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.HPSF;
using Poi.Net.PoiFS.Dev;
using Poi.Net.PoiFS.Properties;

namespace Poi.Net.PoiFS.FileSystem
{
	
	/// <summary> Simple implementation of DirectoryEntry
	/// 
	/// </summary>
	
	public class DirectoryNode:EntryNode, DirectoryEntry, POIFSViewable
	{
		/// <returns> this directory's path representation
		/// </returns>
		virtual public POIFSDocumentPath Path
		{
			
			
			get
			{
				return _path;
			}
			
		}
		/// <summary> get an iterator of the Entry instances contained directly in
		/// this instance (in other words, children only; no grandchildren
		/// etc.)
		/// 
		/// </summary>
		/// <returns> iterator; never null, but HasNext() may return false
		/// immediately (i.e., this DirectoryEntry is empty). All
		/// objects retrieved by Next() are guaranteed to be
		/// implementations of Entry.
		/// </returns>
		virtual public IEnumerator Entries
		{
			
			
			get
			{
				return _entries.Values.GetEnumerator();
			}
			
		}
		/// <summary> is this DirectoryEntry empty?
		/// 
		/// </summary>
		/// <returns> true if this instance contains no Entry instances
		/// </returns>
		virtual public bool Empty
		{
			
			
			get
			{
				return (_entries.Count == 0);
			}
			
		}
		/// <summary> find out how many Entry instances are contained directly within
		/// this DirectoryEntry
		/// 
		/// </summary>
		/// <returns> number of immediately (no grandchildren etc.) contained
		/// Entry instances
		/// </returns>
		virtual public int EntryCount
		{
			
			
			get
			{
				return _entries.Count;
			}
			
		}
		
		/// <summary> Gets the storage clsid of the directory entry
		/// 
		/// </summary>
		/// <returns> storage Class ID
		/// </returns>
		/// <summary> Sets the storage clsid for the directory entry
		/// 
		/// </summary>
		/// <param name="clsidStorage">storage Class ID
		/// </param>
		virtual public ClassID StorageClsid {
			get {
				return Property.StorageClsid;
			}
			
			set {
				Property.StorageClsid = value;
			}
			
		}
		/// <summary> is this a DirectoryEntry?
		/// 
		/// </summary>
		/// <returns> true if the Entry is a DirectoryEntry, else false
		/// </returns>
		override public bool DirectoryEntry
		{
			
			
			get
			{
				return true;
			}
			
		}
		/// <summary> extensions use this method to verify internal rules regarding
		/// deletion of the underlying store.
		/// 
		/// </summary>
		/// <returns> true if it's ok to delete the underlying store, else
		/// false
		/// </returns>
		override protected internal bool DeleteOK
		{
			
			
			get
			{
				
				// if this directory is empty, we can delete it
				return Empty;
			}
			
		}
		/// <summary> Get an array of objects, some of which may implement
		/// POIFSViewable
		/// 
		/// </summary>
		/// <returns> an array of Object; may not be null, but may be empty
		/// </returns>
		virtual public object[] ViewableArray
		{
			
			
			get
			{
				return new object[0];
			}
			
		}
		/// <summary> Get an Iterator of objects, some of which may implement
		/// POIFSViewable
		/// 
		/// </summary>
		/// <returns> an Iterator; may not be null, but may have an empty
		/// back end store
		/// </returns>
		virtual public IEnumerator ViewableIterator
		{
			
			
			get
			{
				IList components = new ArrayList();
				
				components.Add(Property);
				
				
				
				SortedList sortedEntries = new SortedList(_entries);
				
				foreach (Object current in sortedEntries.Values)
					components.Add(current);

				return components.GetEnumerator();
			}
			
		}
		/// <summary> Provides a short description of the object, to be used when a
		/// POIFSViewable object has not provided its contents.
		/// 
		/// </summary>
		/// <returns> short description
		/// </returns>
		virtual public string ShortDescription
		{
			
			
			get
			{
				return Name;
			}
			
			/* **********  END  begin implementation of POIFSViewable ********** */
			
		}
		
		// Map of Entry instances, keyed by their names
		private IDictionary _entries;
		
		// the POIFSFileSystem we belong to
		private POIFSFileSystem _filesystem;
		
		// the path described by this document
		private POIFSDocumentPath _path;
		
		/// <summary> create a DirectoryNode. This method is not public by design; it
		/// is intended strictly for the internal use of this package
		/// 
		/// </summary>
		/// <param name="property">the DirectoryProperty for this DirectoryEntry
		/// </param>
		/// <param name="filesystem">the POIFSFileSystem we belong to
		/// </param>
		/// <param name="parent">the parent of this entry
		/// </param>
		
		internal DirectoryNode(DirectoryProperty property, POIFSFileSystem filesystem, DirectoryNode parent):base(property, parent)
		{
			if (parent == null)
			{
				_path = new POIFSDocumentPath();
			}
			else
			{
				_path = new POIFSDocumentPath(parent._path, new string[]{property.Name});
			}
			_filesystem = filesystem;
			
			_entries = new Hashtable();
			IEnumerator iter = property.Children;
					
			while (iter.MoveNext())
			{
				
				Properties.Property child = (Properties.Property) iter.Current;
				Entry childNode = null;
				
				if (child.Directory)
				{
					childNode = new DirectoryNode((DirectoryProperty) child, _filesystem, this);
				}
				else
				{
					childNode = new DocumentNode((DocumentProperty) child, this);
				}
				_entries[childNode.Name] = childNode;
			}
		}
		
		/// <summary> create a new DocumentEntry
		/// 
		/// </summary>
		/// <param name="document">the new document
		/// 
		/// </param>
		/// <returns> the new DocumentEntry
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		
		internal virtual DocumentEntry CreateDocument(POIFSDocument document)
		{
			DocumentProperty property = document.DocumentProperty;
			DocumentNode rval = new DocumentNode(property, this);
			
			((DirectoryProperty) Property).AddChild(property);
			_filesystem.AddDocument(document);
			_entries[property.Name] = rval;
			return rval;
		}
		
		/// <summary> Change a contained Entry's name
		/// 
		/// </summary>
		/// <param name="oldName">the original name
		/// </param>
		/// <param name="newName">the new name
		/// 
		/// </param>
		/// <returns> true if the operation succeeded, else false
		/// </returns>
		
		internal virtual bool ChangeName(string oldName, string newName)
		{
			bool rval = false;
			EntryNode child = (EntryNode) _entries[oldName];
			
			if (child != null)
			{
				rval = ((DirectoryProperty) Property).ChangeName(child.Property, newName);
				if (rval)
				{
					_entries.Remove(oldName);
					_entries[child.Property.Name] = child;
				}
			}
			return rval;
		}
		
		/// <summary> Delete an entry
		/// 
		/// </summary>
		/// <param name="entry">the EntryNode to be deleted
		/// 
		/// </param>
		/// <returns> true if the entry was deleted, else false
		/// </returns>
		
		internal virtual bool DeleteEntry(EntryNode entry)
		{
			bool rval = ((DirectoryProperty) Property).DeleteChild(entry.Property);
			
			if (rval)
			{
				_entries.Remove(entry.Name);
				_filesystem.Remove(entry);
			}
			return rval;
		}
		
		/* ********** START implementation of DirectoryEntry ********** */
		
		/// <summary> get a specified Entry by name
		/// 
		/// </summary>
		/// <param name="name">the name of the Entry to obtain.
		/// 
		/// </param>
		/// <returns> the specified Entry, if it is directly contained in
		/// this DirectoryEntry
		/// 
		/// </returns>
		/// <exception cref=""> FileNotFoundException if no Entry with the specified
		/// name exists in this DirectoryEntry
		/// </exception>
		
		public virtual Entry GetEntry(string name)
		{
			Entry rval = null;
			
			if ((object) name != null)
			{
				rval = (Entry) _entries[name];
			}
			if (rval == null)
			{
				
				// either a null name was given, or there is no such name
				throw new FileNotFoundException("no such entry: \"" + name + "\"");
			}
			return rval;
		}
		
		/// <summary> create a new DocumentEntry
		/// 
		/// </summary>
		/// <param name="name">the name of the new DocumentEntry
		/// </param>
		/// <param name="stream">the InputStream from which to create the new
		/// DocumentEntry
		/// 
		/// </param>
		/// <returns> the new DocumentEntry
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		
		public virtual DocumentEntry CreateDocument(string name, Stream stream)
		{
			return CreateDocument(new POIFSDocument(name, stream));
		}
		
		/// <summary> create a new DocumentEntry; the data will be provided later
		/// 
		/// </summary>
		/// <param name="name">the name of the new DocumentEntry
		/// </param>
		/// <param name="size">the size of the new DocumentEntry
		/// </param>
		/// <param name="writer">the writer of the new DocumentEntry
		/// 
		/// </param>
		/// <returns> the new DocumentEntry
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		
		public virtual DocumentEntry CreateDocument(string name, int size, POIFSWriterListener writer)
		{
			return CreateDocument(new POIFSDocument(name, size, _path, writer));
		}
		
		/// <summary> create a new DirectoryEntry
		/// 
		/// </summary>
		/// <param name="name">the name of the new DirectoryEntry
		/// 
		/// </param>
		/// <returns> the new DirectoryEntry
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		
		public virtual DirectoryEntry CreateDirectory(string name)
		{
			DirectoryProperty property = new DirectoryProperty(name);
			DirectoryNode rval = new DirectoryNode(property, _filesystem, this);
			
			((DirectoryProperty) Property).AddChild(property);
			_filesystem.AddDirectory(property);
			_entries[name] = rval;
			return rval;
		}
		
		/* **********  END  implementation of DirectoryEntry ********** */
		/* ********** START implementation of Entry ********** */
		
		/* **********  END  implementation of Entry ********** */
		/* ********** START extension of Entry ********** */
		
		/* **********  END  extension of Entry ********** */
		/* ********** START begin implementation of POIFSViewable ********** */
		
		/// <summary> Give viewers a hint as to whether to call getViewableArray or
		/// getViewableIterator
		/// 
		/// </summary>
		/// <returns> true if a viewer should call getViewableArray, false if
		/// a viewer should call getViewableIterator
		/// </returns>
		
		public virtual bool PreferArray {
			get 
			{
				return false;
			}
		}
	} 
}