// 
// DocumentInputStream.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

namespace Poi.Net.PoiFS.FileSystem
{
	
	/// <summary> This class provides methods to read a DocumentEntry managed by a
	/// Filesystem instance.
	/// 
	/// </summary>
	
	public class DocumentInputStream:Stream
	{
		
		// current offset into the Document
		private int _current_offset;
				
		// the Document's size
		private int _document_size;
		
		// have we been closed?
		private bool _closed;
		
		// the actual Document
		private POIFSDocument _document;
		
		// buffer used to read one byte at a time
		private byte[] _tiny_buffer;
		
		// returned by read operations if we're at end of document
		private const int EOD = 0;
		
		/// <summary> Create an InputStream from the specified DocumentEntry
		/// 
		/// </summary>
		/// <param name="document">the DocumentEntry to be read
		/// 
		/// </param>
		/// <exception cref=""> IOException if the DocumentEntry cannot be opened
		/// (like, maybe it has been deleted?)
		/// </exception>
		
		public DocumentInputStream(DocumentEntry document)
		{
			_current_offset = 0;
			_document_size  = document.Size;
			_closed         = false;
			_tiny_buffer    = null;

			if (document is DocumentNode)
			{
				_document = ((DocumentNode) document).Document;
			}
			else
			{
				throw new IOException("Cannot open internal document storage");
			}
		}
		
		/// <summary> Create an InputStream from the specified Document
		/// 
		/// </summary>
		/// <param name="document">the Document to be read
		/// 
		/// </param>
		/// <exception cref=""> IOException if the DocumentEntry cannot be opened
		/// (like, maybe it has been deleted?)
		/// </exception>
		
		public DocumentInputStream(POIFSDocument document)
		{
			_current_offset = 0;
			_document_size = document.Size;
			_closed = false;
			_tiny_buffer = null;
			_document = document;
		}
		
		/// <summary> Returns the number of bytes that can be read (or skipped over)
		/// from this input stream without blocking by the next caller of a
		/// method for this input stream. The next caller might be the same
		/// thread or or another thread.
		/// 
		/// </summary>
		/// <returns> the number of bytes that can be read from this input
		/// stream without blocking.
		/// 
		/// </returns>
		/// <exception cref=""> IOException on error (such as the stream has been
		/// closed)
		/// </exception>
		
		
		/// <summary> Closes this input stream and releases any system resources
		/// associated with the stream.
		/// 
		/// </summary>
		/// <exception cref=""> IOException
		/// </exception>
		
		public override void Close()
		{
			_closed = true;
		}
		
		/// <summary> Marks the current position in this input stream. A subsequent
		/// call to the reset method repositions this stream at the last
		/// marked position so that subsequent reads re-read the same
		/// bytes.
		/// 
		/// The readlimit arguments tells this input stream to allow that
		/// many bytes to be read before the mark position gets
		/// invalidated. This implementation, however, does not care.
		/// 
		/// The general contract of mark is that, if the method
		/// markSupported returns true, the stream somehow remembers all
		/// the bytes read after the call to mark and stands ready to
		/// supply those same bytes again if and whenever the method reset
		/// is called. However, the stream is not required to remember any
		/// data at all if more than readlimit bytes are read from the
		/// stream before reset is called. But this stream will.
		/// 
		/// </summary>
		/// <param name="ignoredReadlimit">the maximum limit of bytes that can be
		/// read before the mark position becomes
		/// invalid. Ignored by this
		/// implementation.
		/// </param>
		
		
		/// <summary> Tests if this input stream supports the mark and reset methods.
		/// 
		/// </summary>
		/// <returns> true
		/// </returns>
		
		
		public override bool CanSeek
		{
			get {
				return true;
			}
		}
		
		/// <summary> Reads the next byte of data from the input stream. The value
		/// byte is returned as an int in the range 0 to 255. If no byte is
		/// available because the end of the stream has been reached, the
		/// value -1 is returned. The definition of this method in
		/// java.io.InputStream allows this method to block, but it won't.
		/// 
		/// </summary>
		/// <returns> the next byte of data, or -1 if the end of the stream
		/// is reached.
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		
		public override int ReadByte()
		{
			dieIfClosed();
			if (atEOD())
			{
				return EOD;
			}
			if (_tiny_buffer == null)
			{
				_tiny_buffer = new byte[1];
			}
			_document.Read(_tiny_buffer, _current_offset++);
			return ((int) _tiny_buffer[0]) & 0x000000FF;
		}
		
		/// <summary> Reads some number of bytes from the input stream and stores
		/// them into the buffer array b. The number of bytes actually read
		/// is returned as an integer. The definition of this method in
		/// java.io.InputStream allows this method to block, but it won't.
		/// 
		/// If b is null, a NullPointerException is thrown. If the length
		/// of b is zero, then no bytes are read and 0 is returned;
		/// otherwise, there is an attempt to read at least one byte. If no
		/// byte is available because the stream is at end of file, the
		/// value -1 is returned; otherwise, at least one byte is read and
		/// stored into b.
		/// 
		/// The first byte read is stored into element b[0], the next one
		/// into b[1], and so on. The number of bytes read is, at most,
		/// equal to the length of b. Let k be the number of bytes actually
		/// read; these bytes will be stored in elements b[0] through
		/// b[k-1], leaving elements b[k] through b[b.length-1] unaffected.
		/// 
		/// If the first byte cannot be read for any reason other than end
		/// of file, then an IOException is thrown. In particular, an
		/// IOException is thrown if the input stream has been closed.
		/// 
		/// The read(b) method for class InputStream has the same effect as:
		/// 
		/// <code>read(b, 0, b.length)</code>
		/// 
		/// </summary>
		/// <param name="b">the buffer into which the data is read.
		/// 
		/// </param>
		/// <returns> the total number of bytes read into the buffer, or -1
		/// if there is no more data because the end of the stream
		/// has been reached.
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		/// <exception cref=""> NullPointerException
		/// </exception>
		
		
		public int Read(byte[] b)
		{
			return Read(b, 0, b.Length);
		}
		
		/// <summary> Reads up to len bytes of data from the input stream into an
		/// array of bytes. An attempt is made to read as many as len
		/// bytes, but a smaller number may be read, possibly zero. The
		/// number of bytes actually read is returned as an integer.
		/// 
		/// The definition of this method in java.io.InputStream allows it
		/// to block, but it won't.
		/// 
		/// If b is null, a NullPointerException is thrown.
		/// 
		/// If off is negative, or len is negative, or off+len is greater
		/// than the length of the array b, then an
		/// IndexOutOfBoundsException is thrown.
		/// 
		/// If len is zero, then no bytes are read and 0 is returned;
		/// otherwise, there is an attempt to read at least one byte. If no
		/// byte is available because the stream is at end of file, the
		/// value -1 is returned; otherwise, at least one byte is read and
		/// stored into b.
		/// 
		/// The first byte read is stored into element b[off], the next one
		/// into b[off+1], and so on. The number of bytes read is, at most,
		/// equal to len. Let k be the number of bytes actually read; these
		/// bytes will be stored in elements b[off] through b[off+k-1],
		/// leaving elements b[off+k] through b[off+len-1] unaffected.
		/// 
		/// In every case, elements b[0] through b[off] and elements
		/// b[off+len] through b[b.length-1] are unaffected.
		/// 
		/// If the first byte cannot be read for any reason other than end
		/// of file, then an IOException is thrown. In particular, an
		/// IOException is thrown if the input stream has been closed.
		/// 
		/// </summary>
		/// <param name="b">the buffer into which the data is read.
		/// </param>
		/// <param name="off">the start offset in array b at which the data is
		/// written.
		/// </param>
		/// <param name="len">the maximum number of bytes to read.
		/// 
		/// </param>
		/// <returns> the total number of bytes read into the buffer, or -1
		/// if there is no more data because the end of the stream
		/// has been reached.
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		/// <exception cref=""> NullPointerException
		/// </exception>
		/// <exception cref=""> IndexOutOfBoundsException
		/// </exception>
		
		public override int Read(byte[] b, int off, int len)
		{
			dieIfClosed();
			if (b == null)
			{
				throw new NullReferenceException("buffer is null");
			}
			if ((off < 0) || (len < 0) || (b.Length < (off + len)))
			{
				throw new IndexOutOfRangeException("can't read past buffer boundaries");
			}
			if (len == 0)
			{
				return 0;
			}
			if (atEOD())
			{
				return EOD;
			}
			int limit = System.Math.Min((int)(Length - Position), len);
			
			if ((off == 0) && (limit == b.Length))
			{
				_document.Read(b, _current_offset);
			}
			else
			{
				byte[] buffer = new byte[limit];
				
				_document.Read(buffer, _current_offset);
				Array.Copy(buffer, 0, b, off, limit);
			}
			_current_offset += limit;
			return limit;
		}
		
		/// <summary> Repositions this stream to the position at the time the mark
		/// method was last called on this input stream.
		/// 
		/// The general contract of reset is:
		/// 
		/// <ul>
		/// <li>
		/// If the method markSupported returns true, then:
		/// <ul>
		/// <li>
		/// If the method mark has not been called since the
		/// stream was created, or the number of bytes read
		/// from the stream since mark was last called is
		/// larger than the argument to mark at that last
		/// call, then an IOException might be thrown.
		/// </li>
		/// <li>
		/// If such an IOException is not thrown, then the
		/// stream is reset to a state such that all the
		/// bytes read since the most recent call to mark
		/// (or since the start of the file, if mark has not
		/// been called) will be resupplied to subsequent
		/// callers of the read method, followed by any
		/// bytes that otherwise would have been the next
		/// input data as of the time of the call to reset.
		/// </li>
		/// </ul>
		/// </li>
		/// <li>
		/// If the method markSupported returns false, then:
		/// <ul>
		/// <li>
		/// The call to reset may throw an IOException.
		/// </li>
		/// <li>
		/// If an IOException is not thrown, then the
		/// stream is reset to a fixed state that depends
		/// on the particular type of the input and how it
		/// was created. The bytes that will be supplied to
		/// subsequent callers of the read method depend on
		/// the particular type of the input stream.
		/// </li>
		/// </ul>
		/// </li>
		/// </ul>
		/// 
		/// All well and good ... this class's markSupported method returns
		/// true and this method does not care whether you've called mark
		/// at all, or whether you've exceeded the number of bytes
		/// specified in the last call to mark. We're basically walking a
		/// byte array ... mark and reset to your heart's content.
		/// </summary>
		
		
		/// <summary> Skips over and discards n bytes of data from this input
		/// stream. The skip method may, for a variety of reasons, end up
		/// skipping over some smaller number of bytes, possibly 0. This
		/// may result from any of a number of conditions; reaching end of
		/// file before n bytes have been skipped is only one
		/// possibility. The actual number of bytes skipped is returned. If
		/// n is negative, no bytes are skipped.
		/// 
		/// </summary>
		/// <param name="n">the number of bytes to be skipped.
		/// 
		/// </param>
		/// <returns> the actual number of bytes skipped.
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		
		
		public long Skip(long n)
		{
			dieIfClosed();
			if (n < 0)
			{
				return 0;
			}
			int new_offset = _current_offset + (int) n;
			
			if (new_offset < _current_offset)
			{
				
				// wrap around in converting a VERY large long to an int
				new_offset = _document_size;
			}
			else if (new_offset > _document_size)
			{
				new_offset = _document_size;
			}
			long rval = new_offset - _current_offset;
			
			_current_offset = new_offset;
			return rval;
		}
		
		private void dieIfClosed()
		{
			if (_closed)
			{
				throw new IOException("cannot perform requested operation on a closed stream");
			}
		}
		
		private bool atEOD()
		{
			return _current_offset == _document_size;
		}
		
		public override void Write(byte[] buffer, int offset, int count)
		{
		}

		
		public override void SetLength(long value)
		{
		}
		
		public override long Seek(long offset, SeekOrigin origin)
		{
			return 0;
		}
		
		public override void Flush()
		{
		}
		public override bool CanRead {
			get {
				return true;
			}
			
		}
		
		public override bool CanWrite {
			get {
				return false;
			}
			
		}
		public override long Length {
			get {
				return _document_size;
			}
			
		}
		
		public override long Position {
			get {
				return _current_offset;
			}
			
			set {
				_current_offset = Convert.ToInt32(value);
			}
			
		}
	} 
}