// 
// DocumentOutputStream.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

namespace Poi.Net.PoiFS.FileSystem
{
	
	/// <summary> This class provides a wrapper over an OutputStream so that Document
	/// writers can't accidently go over their size limits
	/// 
	/// </summary>
	
	public class DocumentOutputStream:Stream
	{
		private Stream stream;
		private int limit;
		private int written;
		
		/// <summary> Create a DocumentOutputStream
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the data is actually
		/// read
		/// </param>
		/// <param name="limit">the maximum number of bytes that can be written
		/// </param>
		
		internal DocumentOutputStream(Stream stream, int limit)
		{
			this.stream = stream;
			this.limit = limit;
			this.written = 0;
		}
		
		/// <summary> Writes the specified byte to this output stream. The general
		/// contract for write is that one byte is written to the output
		/// stream. The byte to be written is the eight low-order bits of
		/// the argument b. The 24 high-order bits of b are ignored.
		/// 
		/// </summary>
		/// <param name="b">the byte.
		/// </param>
		/// <exception cref=""> IOException if an I/O error occurs. In particular,
		/// an IOException may be thrown if the
		/// output stream has been closed, or if the
		/// writer tries to write too much data.
		/// </exception>
		
		
		public override  void WriteByte(byte b)
		{
			limitCheck(1);
			stream.WriteByte(b);
		}
		
		/// <summary> Writes b.length bytes from the specified byte array
		/// to this output stream.
		/// 
		/// </summary>
		/// <param name="b">the data.
		/// </param>
		/// <exception cref=""> IOException if an I/O error occurs.
		/// </exception>
		
		
		public void Write(byte[] b)
		{
			Write(b, 0, b.Length);
		}
		
		/// <summary> Writes len bytes from the specified byte array starting at
		/// offset off to this output stream.  The general contract for
		/// Write(b, off, len) is that some of the bytes in the array b are
		/// written to the output stream in order; element b[off] is the
		/// first byte written and b[off+len-1] is the last byte written by
		/// this operation.
		/// If b is null, a NullPointerException is thrown.
		/// If off is negative, or len is negative, or off+len is greater
		/// than the length of the array b, then an
		/// IndexOutOfBoundsException is thrown.
		/// 
		/// </summary>
		/// <param name="b">the data.
		/// </param>
		/// <param name="off">the start offset in the data.
		/// </param>
		/// <param name="len">the number of bytes to write.
		/// </param>
		/// <exception cref=""> IOException if an I/O error occurs. In particular,
		/// an IOException</code> is thrown if the
		/// output stream is closed or if the writer
		/// tries to write too many bytes.
		/// </exception>
		
		public override void Write(byte[] b, int off, int len)
		{
			limitCheck(len);
			stream.Write(b, off, len);
		}
		
		/// <summary> Flushes this output stream and forces any buffered output bytes
		/// to be written out.
		/// 
		/// </summary>
		/// <exception cref=""> IOException if an I/O error occurs.
		/// </exception>
		
		public override void Flush()
		{
			stream.Flush();
		}
		
		/// <summary> Closes this output stream and releases any system resources
		/// associated with this stream. The general contract of close is
		/// that it closes the output stream. A closed stream cannot
		/// perform output operations and cannot be reopened.
		/// 
		/// </summary>
		/// <exception cref=""> IOException if an I/O error occurs.
		/// </exception>
		
		public override void Close()
		{
			stream.Close();
			// ignore this call
		}
		
		/// <summary> write the rest of the document's data (fill in at the end)
		/// 
		/// </summary>
		/// <param name="totalLimit">the actual number of bytes the corresponding
		/// document must fill
		/// </param>
		/// <param name="fill">the byte to fill remaining space with
		/// 
		/// </param>
		/// <exception cref=""> IOException on I/O error
		/// </exception>
		
		internal virtual void writeFiller(int totalLimit, byte fill)
		{
			if (totalLimit > written)
			{
				byte[] filler = new byte[totalLimit - written];
				
				// Fill
				for (int i = 0; i < filler.Length; i++)
					filler[i] = (byte) fill;

				stream.Write(filler, 0, filler.Length);
			}
		}
		
		private void limitCheck(int toBeWritten)
		{
			if ((written + toBeWritten) > limit)
			{
				throw new IOException("tried to write too much data");
			}
			written += toBeWritten;
		}
		
		public override int Read(byte[] buffer, int offset, int count)
		{
			throw new NotImplementedException();
		}
		
		public override void SetLength(long value)
		{
			throw new NotImplementedException();
		}
		
		public override long Seek(long offset, SeekOrigin origin)
		{
			return 0;
		}
		
		public override bool CanRead {
			get {
				return false;
			}
			
		}
		
		public override bool CanSeek {
			get {
				return false;
			}
			
		}
		
		public override bool CanWrite {
			get {
				return true;
			}
			
		}
		
		public override long Length {
			get {
				return stream.Length;
			}
			
		}
		
		public override long Position {
			get {
				return stream.Position;
			}
			
			set {
			}
			
		}
	} 
}