// 
// EntryNode.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


using System;
using Poi.Net.PoiFS.Properties;

namespace Poi.Net.PoiFS.FileSystem
{
	
	/// <summary> Abstract implementation of Entry
	/// 
	/// Extending classes should override isDocument() or isDirectory(), as
	/// appropriate
	/// 
	/// Extending classes must override isDeleteOK()
	/// 
	/// </summary>
	
	public abstract class EntryNode : Entry
	{
		/// <summary> grant access to the property
		/// 
		/// </summary>
		/// <returns> the property backing this entry
		/// </returns>
		virtual protected internal Property Property
		{
			
			
			get
			{
				return _property;
			}
			
		}
		/// <summary> is this the root of the tree?
		/// 
		/// </summary>
		/// <returns> true if so, else false
		/// </returns>
		virtual protected internal bool Root
		{
			
			
			get
			{
				
				// only the root Entry has no parent ...
				return (_parent == null);
			}
			
		}
		/// <summary> extensions use this method to verify internal rules regarding
		/// deletion of the underlying store.
		/// 
		/// </summary>
		/// <returns> true if it's ok to delete the underlying store, else
		/// false
		/// </returns>
		protected internal abstract bool DeleteOK{get;}
		/// <summary> get the name of the Entry
		/// 
		/// </summary>
		/// <returns> name
		/// </returns>
		virtual public string Name
		{
			
			
			get
			{
				return _property.Name;
			}
			
		}
		/// <summary> is this a DirectoryEntry?
		/// 
		/// </summary>
		/// <returns> true if the Entry is a DirectoryEntry, else false
		/// </returns>
		virtual public bool DirectoryEntry
		{
			
			
			get
			{
				return false;
			}
			
		}
		/// <summary> is this a DocumentEntry?
		/// 
		/// </summary>
		/// <returns> true if the Entry is a DocumentEntry, else false
		/// </returns>
		virtual public bool DocumentEntry
		{
			
			
			get
			{
				return false;
			}
			
		}
		/// <summary> get this Entry's parent (the DocumentEntry that owns this
		/// Entry). All Entry objects, except the root Entry, has a parent.
		/// 
		/// </summary>
		/// <returns> this Entry's parent; null iff this is the root Entry
		/// </returns>
		virtual public DirectoryEntry Parent
		{
			
			
			get
			{
				return _parent;
			}
			
		}
		
		// the DocumentProperty backing this object
		private Property _property;
		
		// this object's parent Entry
		private DirectoryNode _parent;
		
		/// <summary> create a DocumentNode. This method is not public by design; it
		/// is intended strictly for the internal use of extending classes
		/// 
		/// </summary>
		/// <param name="property">the Property for this Entry
		/// </param>
		/// <param name="parent">the parent of this entry
		/// </param>
		
		protected internal EntryNode(Property property, DirectoryNode parent)
		{
			_property = property;
			_parent = parent;
		}
		
		/* ********** START implementation of Entry ********** */
		
		/// <summary> Delete this Entry. This operation should succeed, but there are
		/// special circumstances when it will not:
		/// 
		/// If this Entry is the root of the Entry tree, it cannot be
		/// deleted, as there is no way to create another one.
		/// 
		/// If this Entry is a directory, it cannot be deleted unless it is
		/// empty.
		/// 
		/// </summary>
		/// <returns> true if the Entry was successfully deleted, else false
		/// </returns>
		
		public virtual bool Delete()
		{
			bool rval = false;
			
			if ((!Root) && DeleteOK)
			{
				rval = _parent.DeleteEntry(this);
			}
			return rval;
		}
		
		/// <summary> Rename this Entry. This operation will fail if:
		/// 
		/// There is a sibling Entry (i.e., an Entry whose parent is the
		/// same as this Entry's parent) with the same name.
		/// 
		/// This Entry is the root of the Entry tree. Its name is dictated
		/// by the Filesystem and many not be changed.
		/// 
		/// </summary>
		/// <param name="newName">the new name for this Entry
		/// 
		/// </param>
		/// <returns> true if the operation succeeded, else false
		/// </returns>
		
		public virtual bool RenameTo(string newName)
		{
			bool rval = false;
			
			if (!Root)
			{
				rval = _parent.ChangeName(Name, newName);
			}
			return rval;
		}
		
		/* **********  END  implementation of Entry ********** */
	} 
}