// 
// POIFSDocument.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Text;
using System.Collections;

using Poi.Net.PoiFS.Common;
using Poi.Net.PoiFS.Dev;
using Poi.Net.PoiFS.Properties;
using Poi.Net.PoiFS.Storage;
using Poi.Net.Util;
namespace Poi.Net.PoiFS.FileSystem
{
	
	/// <summary> This class manages a document in the POIFS filesystem.
	/// 
	/// </summary>
	
	public class POIFSDocument : BATManaged, BlockWritable, POIFSViewable
	{
		/// <summary> return the array of SmallDocumentBlocks used
		/// 
		/// </summary>
		/// <returns> array of SmallDocumentBlocks; may be empty, cannot be null
		/// </returns>
		virtual public BlockWritable[] SmallBlocks
		{
			
			
			get
			{
				return _small_store.Blocks;
			}
			
		}
		/// <returns> size of the document
		/// </returns>
		virtual public int Size
		{
			
			
			get
			{
				return _size;
			}
			
		}
		/// <summary> Get the DocumentProperty
		/// 
		/// </summary>
		/// <returns> the instance's DocumentProperty
		/// </returns>
		virtual internal DocumentProperty DocumentProperty
		{
			
			
			get
			{
				return _property;
			}
			
		}
		/// <summary> Get an array of objects, some of which may implement
		/// POIFSViewable
		/// 
		/// </summary>
		/// <returns> an array of Object; may not be null, but may be empty
		/// </returns>
		virtual public object[] ViewableArray
		{
			
			
			get
			{
				object[] results = new object[1];
				string result;
				
				try
				{
					MemoryStream output = new MemoryStream();
					BlockWritable[] blocks = null;
					
					if (_big_store.Valid)
					{
						blocks = _big_store.Blocks;
					}
					else if (_small_store.Valid)
					{
						blocks = _small_store.Blocks;
					}
					if (blocks != null)
					{
						for (int k = 0; k < blocks.Length; k++)
						{
							blocks[k].WriteBlocks(output);
						}
						
						byte[] data = output.ToArray();

						if (data.Length > _property.Size)
						{
							byte[] tmp = new byte[_property.Size];
							
							Array.Copy(data, 0, tmp, 0, tmp.Length);
							data = tmp;
						}
						output = new MemoryStream();
						HexDump.Dump(data, 0, output, 0);
						char[] tmpChar;
						byte[] tmpByte;
						tmpByte = output.GetBuffer();
						tmpChar = new char[output.Length];
						Array.Copy(tmpByte, 0, tmpChar, 0, tmpChar.Length);
						result = new string(tmpChar);
					}
					else
					{
						result = "<NO DATA>";
					}
				}
				catch (IOException e)
				{
					
					result = e.Message;
				}
				results[0] = result;
				return results;
			}
			
		}
		/// <summary> Get an Iterator of objects, some of which may implement
		/// POIFSViewable
		/// 
		/// </summary>
		/// <returns> an Iterator; may not be null, but may have an empty
		/// back end store
		/// </returns>
		virtual public IEnumerator ViewableIterator
		{
			
			
			get
			{
				return ((IList) ArrayList.ReadOnly(new ArrayList())).GetEnumerator();
			}
			
		}
		/// <summary> Provides a short description of the object, to be used when a
		/// POIFSViewable object has not provided its contents.
		/// 
		/// </summary>
		/// <returns> short description
		/// </returns>
		virtual public string ShortDescription {			
			get {
				StringBuilder buffer = new StringBuilder();
				
				buffer.Append("Document: \"").Append(_property.Name).Append("\"");
				buffer.Append(" size = ").Append(Size);
				return buffer.ToString();
			}		
		}
		private DocumentProperty _property;
		private int _size;
		
		// one of these stores will be valid
		private SmallBlockStore _small_store;
		private BigBlockStore _big_store;
		
		/// <summary> Constructor from large blocks
		/// 
		/// </summary>
		/// <param name="name">the name of the POIFSDocument
		/// </param>
		/// <param name="blocks">the big blocks making up the POIFSDocument
		/// </param>
		/// <param name="length">the actual length of the POIFSDocument
		/// 
		/// </param>
		/// <exception cref=""> IOException
		/// </exception>
		
		public POIFSDocument(string name, RawDataBlock[] blocks, int length)
		{
			_size = length;
			_big_store = new BigBlockStore(this, blocks);
			_property = new DocumentProperty(name, _size);
			_small_store = new SmallBlockStore(this, new BlockWritable[0]);
			_property.Document = this;
		}
		
		/// <summary> Constructor from small blocks
		/// 
		/// </summary>
		/// <param name="name">the name of the POIFSDocument
		/// </param>
		/// <param name="blocks">the small blocks making up the POIFSDocument
		/// </param>
		/// <param name="length">the actual length of the POIFSDocument
		/// </param>
		
		public POIFSDocument(string name, SmallDocumentBlock[] blocks, int length)
		{
			_size = length;
			try
			{
				_big_store = new BigBlockStore(this, new RawDataBlock[0]);
			}
			catch (IOException ignored)
			{
				
				// can't happen with that constructor
			}
			_property = new DocumentProperty(name, _size);
			_small_store = new SmallBlockStore(this, blocks);
			_property.Document = this;
		}
		
		/// <summary> Constructor from small blocks
		/// 
		/// </summary>
		/// <param name="name">the name of the POIFSDocument
		/// </param>
		/// <param name="blocks">the small blocks making up the POIFSDocument
		/// </param>
		/// <param name="length">the actual length of the POIFSDocument
		/// 
		/// </param>
		/// <exception cref=""> IOException
		/// </exception>
		
		public POIFSDocument(string name, ListManagedBlock[] blocks, int length)
		{
			_size = length;
			_property = new DocumentProperty(name, _size);
			_property.Document = this;
			if (Property.IsSmall(_size))
			{
				_big_store = new BigBlockStore(this, new RawDataBlock[0]);
				_small_store = new SmallBlockStore(this, blocks);
			}
			else
			{
				_big_store = new BigBlockStore(this, blocks);
				_small_store = new SmallBlockStore(this, new BlockWritable[0]);
			}
		}
		
		/// <summary> Constructor
		/// 
		/// </summary>
		/// <param name="name">the name of the POIFSDocument
		/// </param>
		/// <param name="stream">the InputStream we read data from
		/// 
		/// </param>
		/// <exception cref=""> IOException thrown on read errors
		/// </exception>
		
		public POIFSDocument(string name,Stream stream)
		{
			IList blocks = new ArrayList();
			
			_size = 0;
			while (true)
			{
				DocumentBlock block = new DocumentBlock(stream);
				int blockSize = block.Size;
				
				if (blockSize > 0)
				{
					blocks.Add(block);
					_size += blockSize;
				}
				if (block.PartiallyRead())
				{
					break;
				}
			}
			DocumentBlock[] bigBlocks = (DocumentBlock[]) ((ArrayList)blocks).ToArray(typeof(DocumentBlock));

			_big_store = new BigBlockStore(this, bigBlocks);
			_property = new DocumentProperty(name, _size);
			_property.Document = this;
			if (_property.ShouldUseSmallBlocks())
			{
				_small_store = new SmallBlockStore(this, SmallDocumentBlock.Convert(bigBlocks, _size));
				_big_store = new BigBlockStore(this, new DocumentBlock[0]);
			}
			else
			{
				_small_store = new SmallBlockStore(this, new BlockWritable[0]);
			}
		}
		
		/// <summary> Constructor
		/// 
		/// </summary>
		/// <param name="name">the name of the POIFSDocument
		/// </param>
		/// <param name="size">the length of the POIFSDocument
		/// </param>
		/// <param name="path">the path of the POIFSDocument
		/// </param>
		/// <param name="writer">the writer who will eventually write the document
		/// contents
		/// 
		/// </param>
		/// <exception cref=""> IOException thrown on read errors
		/// </exception>
		
		public POIFSDocument(string name, int size, POIFSDocumentPath path, POIFSWriterListener writer)
		{
			_size = size;
			_property = new DocumentProperty(name, _size);
			_property.Document = this;
			if (_property.ShouldUseSmallBlocks())
			{
				_small_store = new SmallBlockStore(this, path, name, size, writer);
				_big_store = new BigBlockStore(this, new object[0]);
			}
			else
			{
				_small_store = new SmallBlockStore(this, new BlockWritable[0]);
				_big_store = new BigBlockStore(this, path, name, size, writer);
			}
		}
		
		/// <summary> read data from the internal stores
		/// 
		/// </summary>
		/// <param name="buffer">the buffer to write to
		/// </param>
		/// <param name="offset">the offset into our storage to read from
		/// </param>
		
		internal virtual void Read(byte[] buffer, int offset)
		{
			if (_property.ShouldUseSmallBlocks())
			{
				SmallDocumentBlock.Read(_small_store.Blocks, buffer, offset);
			}
			else
			{
				DocumentBlock.Read(_big_store.Blocks, buffer, offset);
			}
		}
		
		/* ********** START implementation of BlockWritable ********** */
		
		/// <summary> Write the storage to an OutputStream
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the stored data should
		/// be written
		/// 
		/// </param>
		/// <exception cref=""> IOException on problems writing to the specified
		/// stream
		/// </exception>
		
		public virtual void WriteBlocks(Stream stream)
		{
			_big_store.WriteBlocks(stream);
		}
		
		/* **********  END  implementation of BlockWritable ********** */
		/* ********** START implementation of BATManaged ********** */
		
		/// <summary> Return the number of BigBlock's this instance uses
		/// 
		/// </summary>
		/// <returns> count of BigBlock instances
		/// </returns>
		
		public virtual int CountBlocks {
			get { return _big_store.CountBlocks; }
		}
		
		/// <summary> Set the start block for this instance
		/// 
		/// </summary>
		/// <param name="index">index into the array of blocks making up the
		/// filesystem
		/// </param>
		
		public virtual int StartBlock {
			set {_property.StartBlock = value; }
		}
		
		/* **********  END  implementation of BATManaged ********** */
		/* ********** START begin implementation of POIFSViewable ********** */
		
		/// <summary> Give viewers a hint as to whether to call getViewableArray or
		/// getViewableIterator
		/// 
		/// </summary>
		/// <returns> true if a viewer should call getViewableArray, false if
		/// a viewer should call getViewableIterator
		/// </returns>
		
		public virtual bool PreferArray
		{
			get 
			{
				return true;
			}

		}
		
		
		/* **********  END  begin implementation of POIFSViewable ********** */
		private class SmallBlockStore
		{
			private void InitBlock(POIFSDocument enclosingInstance)
			{
				this.enclosingInstance = enclosingInstance;
			}
			private POIFSDocument enclosingInstance;
			/// <returns> true if this store is a valid source of data
			/// </returns>
			virtual internal bool Valid
			{
				
				
				get
				{
					return ((smallBlocks.Length > 0) || (writer != null));
				}
				
			}
			/// <returns> the SmallDocumentBlocks
			/// </returns>
			virtual internal BlockWritable[] Blocks
			{
				
				
				get
				{
					if (Valid && (writer != null))
					{
						MemoryStream stream = new MemoryStream(size);
						DocumentOutputStream dstream = new DocumentOutputStream(stream, size);
						
						writer.ProcessPOIFSWriterEvent(new POIFSWriterEvent(dstream, path, name, size));
						
						smallBlocks = SmallDocumentBlock.Convert(stream.ToArray(), size);
					}
					return smallBlocks;
				}
				
			}
			public POIFSDocument Enclosing_Instance
			{
				get
				{
					return enclosingInstance;
				}
				
			}
			private SmallDocumentBlock[] smallBlocks;
			private POIFSDocumentPath path;
			private string name;
			private int size;
			private POIFSWriterListener writer;
			
			/// <summary> Constructor
			/// 
			/// </summary>
			/// <param name="blocks">blocks to construct the store from
			/// </param>
			
			internal SmallBlockStore(POIFSDocument enclosingInstance, object[] blocks)
			{
				InitBlock(enclosingInstance);
				smallBlocks = new SmallDocumentBlock[blocks.Length];
				for (int j = 0; j < blocks.Length; j++)
				{
					smallBlocks[j] = (SmallDocumentBlock) blocks[j];
				}
				this.path = null;
				this.name = null;
				this.size = - 1;
				this.writer = null;
			}
			
			/// <summary> Constructor for a small block store that will be written
			/// later
			/// 
			/// </summary>
			/// <param name="path">path of the document
			/// </param>
			/// <param name="name">name of the document
			/// </param>
			/// <param name="size">length of the document
			/// </param>
			/// <param name="writer">the object that will eventually write the document
			/// </param>
			
			internal SmallBlockStore(POIFSDocument enclosingInstance, POIFSDocumentPath path, string name, int size, POIFSWriterListener writer)
			{
				InitBlock(enclosingInstance);
				smallBlocks = new SmallDocumentBlock[0];
				this.path = path;
				this.name = name;
				this.size = size;
				this.writer = writer;
			}
		} 
		
		
		private class BigBlockStore
		{
			private void InitBlock(POIFSDocument enclosingInstance)
			{
				this.enclosingInstance = enclosingInstance;
			}
			private POIFSDocument enclosingInstance;
			/// <returns> true if this store is a valid source of data
			/// </returns>
			virtual internal bool Valid
			{
				
				
				get
				{
					return ((bigBlocks.Length > 0) || (writer != null));
				}
				
			}
			/// <returns> the DocumentBlocks
			/// </returns>
			virtual internal DocumentBlock[] Blocks
			{
				
				
				get
				{
					if (Valid && (writer != null))
					{
						MemoryStream stream = new MemoryStream(size);
						DocumentOutputStream dstream = new DocumentOutputStream(stream, size);
						
						writer.ProcessPOIFSWriterEvent(new POIFSWriterEvent(dstream, path, name, size));
						
						bigBlocks = DocumentBlock.Convert(stream.ToArray(), size);
					}
					return bigBlocks;
				}
				
			}
			public POIFSDocument Enclosing_Instance
			{
				get
				{
					return enclosingInstance;
				}
				
			}
			private DocumentBlock[] bigBlocks;
			private POIFSDocumentPath path;
			private string name;
			private int size;
			private POIFSWriterListener writer;
			
			/// <summary> Constructor
			/// 
			/// </summary>
			/// <param name="blocks">the blocks making up the store
			/// 
			/// </param>
			/// <exception cref=""> IOException on I/O error
			/// </exception>
			
			internal BigBlockStore(POIFSDocument enclosingInstance, object[] blocks)
			{
				InitBlock(enclosingInstance);
				bigBlocks = new DocumentBlock[blocks.Length];
				for (int j = 0; j < blocks.Length; j++)
				{
					if (blocks[j] is DocumentBlock)
					{
						bigBlocks[j] = (DocumentBlock) blocks[j];
					}
					else
					{
						bigBlocks[j] = new DocumentBlock((RawDataBlock) blocks[j]);
					}
				}
				this.path = null;
				this.name = null;
				this.size = - 1;
				this.writer = null;
			}
			
			/// <summary> Constructor for a big block store that will be written
			/// later
			/// 
			/// </summary>
			/// <param name="path">path of the document
			/// </param>
			/// <param name="name">name of the document
			/// </param>
			/// <param name="size">length of the document
			/// </param>
			/// <param name="writer">the object that will eventually write the
			/// document
			/// </param>
			
			internal BigBlockStore(POIFSDocument enclosingInstance, POIFSDocumentPath path, string name, int size, POIFSWriterListener writer)
			{
				InitBlock(enclosingInstance);
				bigBlocks = new DocumentBlock[0];
				this.path = path;
				this.name = name;
				this.size = size;
				this.writer = writer;
			}
			
			/// <summary> write the blocks to a stream
			/// 
			/// </summary>
			/// <param name="stream">the stream to which the data is to be written
			/// 
			/// </param>
			/// <exception cref=""> IOException on error
			/// </exception>
			
			internal virtual void WriteBlocks(Stream stream)
			{
				if (Valid)
				{
					if (writer != null)
					{
						DocumentOutputStream dstream = new DocumentOutputStream(stream, size);
						
						writer.ProcessPOIFSWriterEvent(new POIFSWriterEvent(dstream, path, name, size));
						dstream.writeFiller(CountBlocks * POIFSConstants.BIG_BLOCK_SIZE, DocumentBlock.FillByte);
					}
					else
					{
						for (int k = 0; k < bigBlocks.Length; k++)
						{
							bigBlocks[k].WriteBlocks(stream);
						}
					}
				}
			}
			
			/// <returns> number of big blocks making up this document
			/// </returns>
			
			internal virtual int CountBlocks 
			{

				get 
				{

					int rval = 0;
				
					if (Valid)
					{
						if (writer != null)
						{
							rval = (size + POIFSConstants.BIG_BLOCK_SIZE - 1) / POIFSConstants.BIG_BLOCK_SIZE;
						}
						else
						{
							rval = bigBlocks.Length;
						}
					}
					return rval;
				}
			}
		}
	}
}