// 
// POIFSDocumentPath.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Text;
using System.IO;

namespace Poi.Net.PoiFS.FileSystem
{
	
	/// <summary> Class POIFSDocumentPath
	/// 
	/// </summary>
	
	public class POIFSDocumentPath
	{
		/// <summary> Returns the path's parent or <code>null</code> if this path
		/// is the root path.
		/// 
		/// </summary>
		/// <returns> path of parent, or null if this path is the root path
		/// </returns>
		virtual public POIFSDocumentPath Parent
		{
			
			
			get
			{
				
				int length = components.Length - 1;
				
				if (length < 0)
				{
					return null;
				}
				POIFSDocumentPath parent = new POIFSDocumentPath(null);
				
				parent.components = new string[length];
				Array.Copy(components, 0, parent.components, 0, length);
				return parent;
			}
			
		}
		private string[] components;
		private int hashcode = 0;
		
		/// <summary> constructor for the path of a document that is not in the root
		/// of the POIFSFileSystem
		/// 
		/// </summary>
		/// <param name="components">the Strings making up the path to a document.
		/// The Strings must be ordered as they appear in
		/// the directory hierarchy of the the document
		/// -- the first string must be the name of a
		/// directory in the root of the POIFSFileSystem,
		/// and every Nth (for N > 1) string thereafter
		/// must be the name of a directory in the
		/// directory identified by the (N-1)th string.
		/// 
		/// If the components parameter is null or has
		/// zero length, the POIFSDocumentPath is
		/// appropriate for a document that is in the
		/// root of a POIFSFileSystem
		/// 
		/// </param>
		/// <exception cref=""> IllegalArgumentException if any of the elements in
		/// the components parameter
		/// are null or have zero
		/// length
		/// </exception>
		
		public POIFSDocumentPath(string[] components)
		{
			if (components == null)
			{
				this.components = new string[0];
			}
			else
			{
				this.components = new string[components.Length];
				for (int j = 0; j < components.Length; j++)
				{
					if (((object) components[j] == null) || (components[j].Length == 0))
					{
						throw new ArgumentException("components cannot contain null or empty strings");
					}
					this.components[j] = components[j];
				}
			}
		}
		
		/// <summary> simple constructor for the path of a document that is in the
		/// root of the POIFSFileSystem. The constructor that takes an
		/// array of Strings can also be used to create such a
		/// POIFSDocumentPath by passing it a null or empty String array
		/// </summary>
		
		public POIFSDocumentPath()
		{
			this.components = new string[0];
		}
		
		/// <summary> constructor that adds additional subdirectories to an existing
		/// path
		/// 
		/// </summary>
		/// <param name="path">the existing path
		/// </param>
		/// <param name="components">the additional subdirectory names to be added
		/// 
		/// </param>
		/// <exception cref=""> IllegalArgumentException if any of the Strings in
		/// components is null or zero
		/// length
		/// </exception>
		
		public POIFSDocumentPath(POIFSDocumentPath path, string[] components)
		{
			if (components == null)
			{
				this.components = new string[path.components.Length];
			}
			else
			{
				this.components = new string[path.components.Length + components.Length];
			}
			for (int j = 0; j < path.components.Length; j++)
			{
				this.components[j] = path.components[j];
			}
			if (components != null)
			{
				for (int j = 0; j < components.Length; j++)
				{
					if (((object) components[j] == null) || (components[j].Length == 0))
					{
						throw new ArgumentException("components cannot contain null or empty strings");
					}
					this.components[j + path.components.Length] = components[j];
				}
			}
		}
		
		/// <summary> equality. Two POIFSDocumentPath instances are equal if they
		/// have the same number of component Strings, and if each
		/// component String is equal to its coresponding component String
		/// 
		/// </summary>
		/// <param name="o">the object we're checking equality for
		/// 
		/// </param>
		/// <returns> true if the object is equal to this object
		/// </returns>
		
		public  override bool Equals(object o)
		{
			bool rval = false;
			
			if ((o != null) && (o.GetType() == this.GetType()))
			{
				if (this == o)
				{
					rval = true;
				}
				else
				{
					POIFSDocumentPath path = (POIFSDocumentPath) o;
					
					if (path.components.Length == this.components.Length)
					{
						rval = true;
						for (int j = 0; j < this.components.Length; j++)
						{
							if (!path.components[j].Equals(this.components[j]))
							{
								rval = false;
								break;
							}
						}
					}
				}
			}
			return rval;
		}
		
		/// <summary> calculate and return the hashcode
		/// 
		/// </summary>
		/// <returns> hashcode
		/// </returns>
		
		public override int GetHashCode()
		{
			if (hashcode == 0)
			{
				for (int j = 0; j < components.Length; j++)
				{
					hashcode += components[j].GetHashCode();
				}
			}
			return hashcode;
		}
		
		/// <returns> the number of components
		/// </returns>
		
		public virtual int Length
		{
			get 
			{
				return components.Length;
			}
		}
		
		/// <summary> get the specified component
		/// 
		/// </summary>
		/// <param name="n">which component (0 ... length() - 1)
		/// 
		/// </param>
		/// <returns> the nth component;
		/// 
		/// </returns>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if n < 0 or n >=
		/// length()
		/// </exception>
		
		public virtual string GetComponent(int n)
		{
			return components[n];
		}
		
		/// <summary> Returns a string representation of the path. Components are
		/// separated by the platform-specific file separator.
		/// 
		/// </summary>
		/// <returns> string representation
		/// </returns>
		
		public override string ToString()
		{
			
			StringBuilder b = new StringBuilder();
			
			int l = Length;
			
			b.Append(Path.DirectorySeparatorChar);
			for (int i = 0; i < l; i++)
			{
				b.Append(GetComponent(i));
				if (i < l - 1)
				{
					b.Append(Path.DirectorySeparatorChar);
				}
			}
			return b.ToString();
		}
	} 
}