// 
// POIFSFileSystem.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.Common;
using Poi.Net.PoiFS.Dev;
using Poi.Net.PoiFS.Properties;
using Poi.Net.PoiFS.Storage;

namespace Poi.Net.PoiFS.FileSystem
{
	
	/// <summary> This is the main class of the POIFS system; it manages the entire
	/// life cycle of the filesystem.
	/// 
	/// </summary>

	public class POIFSFileSystem : POIFSViewable
	{
		/// <summary> get the root entry
		/// 
		/// </summary>
		/// <returns> the root entry
		/// </returns>
		virtual public DirectoryEntry Root
		{
			
			
			get
			{
				if (_root == null)
				{
					_root = new DirectoryNode(_property_table.Root, this, null);
				}
				return _root;
			}
			
		}
		/// <summary> Get an array of objects, some of which may implement
		/// POIFSViewable
		/// 
		/// </summary>
		/// <returns> an array of Object; may not be null, but may be empty
		/// </returns>
		virtual public object[] ViewableArray
		{
			
			
			get
			{
				if (PreferArray)
				{
					return ((POIFSViewable) Root).ViewableArray;
				}
				else
				{
					return new object[0];
				}
			}
			
		}
		/// <summary> Get an Iterator of objects, some of which may implement
		/// POIFSViewable
		/// 
		/// </summary>
		/// <returns> an Iterator; may not be null, but may have an empty
		/// back end store
		/// </returns>
		virtual public IEnumerator ViewableIterator
		{
			
			
			get
			{
				if (!PreferArray)
				{
					return ((POIFSViewable) Root).ViewableIterator;
				}
				else
				{
					return ((IList) ArrayList.ReadOnly(new ArrayList())).GetEnumerator();
				}
			}
			
		}
		/// <summary> Provides a short description of the object, to be used when a
		/// POIFSViewable object has not provided its contents.
		/// 
		/// </summary>
		/// <returns> short description
		/// </returns>
		virtual public string ShortDescription
		{
			
			
			get
			{
				return "POIFS FileSystem";
			}
			
			/* **********  END  begin implementation of POIFSViewable ********** */
			
		}
		private PropertyTable _property_table;
		private IList _documents;
		private DirectoryNode _root;
		
		/// <summary> Constructor, intended for writing</summary>
		
		public POIFSFileSystem()
		{
			_property_table = new PropertyTable();
			_documents = new ArrayList();
			_root = null;
		}
		
		/// <summary> Create a POIFSFileSystem from an InputStream
		/// 
		/// </summary>
		/// <param name="stream">the InputStream from which to read the data
		/// 
		/// </param>
		/// <exception cref=""> IOException on errors reading, or on invalid data
		/// </exception>
		
		public POIFSFileSystem(Stream stream):this()
		{
			
			// read the header block from the stream
			HeaderBlockReader header_block_reader = new HeaderBlockReader(stream);
			
			// read the rest of the stream into blocks
			RawDataBlockList data_blocks = new RawDataBlockList(stream);
			
			// set up the block allocation table (necessary for the
			// data_blocks to be manageable
			new BlockAllocationTableReader(header_block_reader.BATCount, header_block_reader.BATArray, header_block_reader.XBATCount, header_block_reader.XBATIndex, data_blocks);
			
			// get property table from the document
			PropertyTable properties = new PropertyTable(header_block_reader.PropertyStart, data_blocks);
			
			// init documents
			ProcessProperties(SmallBlockTableReader.GetSmallDocumentBlocks(data_blocks, 
																			properties.Root, 
																			header_block_reader.SBATStart), 
								data_blocks, 
								properties.Root.Children,
								null);
		}
		
		/// <summary> Create a new document to be added to the root directory
		/// 
		/// </summary>
		/// <param name="stream">the InputStream from which the document's data
		/// will be obtained
		/// </param>
		/// <param name="name">the name of the new POIFSDocument
		/// 
		/// </param>
		/// <returns> the new DocumentEntry
		/// 
		/// </returns>
		/// <exception cref=""> IOException on error creating the new POIFSDocument
		/// </exception>
		
		public virtual DocumentEntry CreateDocument(Stream stream, string name)
		{
			return Root.CreateDocument(name, stream);
		}
		
		/// <summary> create a new DocumentEntry in the root entry; the data will be
		/// provided later
		/// 
		/// </summary>
		/// <param name="name">the name of the new DocumentEntry
		/// </param>
		/// <param name="size">the size of the new DocumentEntry
		/// </param>
		/// <param name="writer">the writer of the new DocumentEntry
		/// 
		/// </param>
		/// <returns> the new DocumentEntry
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		
		public virtual DocumentEntry CreateDocument(string name, int size, POIFSWriterListener writer)
		{
			return Root.CreateDocument(name, size, writer);
		}
		
		/// <summary> create a new DirectoryEntry in the root directory
		/// 
		/// </summary>
		/// <param name="name">the name of the new DirectoryEntry
		/// 
		/// </param>
		/// <returns> the new DirectoryEntry
		/// 
		/// </returns>
		/// <exception cref=""> IOException on name duplication
		/// </exception>
		
		public virtual DirectoryEntry CreateDirectory(string name)
		{
			return Root.CreateDirectory(name);
		}
		
		/// <summary> Write the filesystem out
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the filesystem will be
		/// written
		/// 
		/// </param>
		/// <exception cref=""> IOException thrown on errors writing to the stream
		/// </exception>
		
		public virtual void WriteFileSystem(Stream stream)
		{
			
			// get the property table ready
			_property_table.preWrite();
			
			// create the small block store, and the SBAT
			SmallBlockTableWriter sbtw = new SmallBlockTableWriter(_documents, _property_table.Root);
			
			// create the block allocation table
			BlockAllocationTableWriter bat = new BlockAllocationTableWriter();
			
			// create a list of BATManaged objects: the documents plus the
			// property table and the small block table
			IList bm_objects = new ArrayList();

			((ArrayList)bm_objects).AddRange(_documents);
			bm_objects.Add(_property_table);
			bm_objects.Add(sbtw);
			bm_objects.Add(sbtw.SBAT);
			
			// walk the list, allocating space for each and assigning each
			// a starting block number

			foreach(BATManaged bmo in bm_objects)
			{
				int block_count = bmo.CountBlocks;
				
				if (block_count != 0)
				{
					bmo.StartBlock = bat.AllocateSpace(block_count);
				}
				else
				{
					
					// Either the BATManaged object is empty or its data
					// is composed of SmallBlocks; in either case,
					// allocating space in the BAT is inappropriate
				}
			}
			
			// allocate space for the block allocation table and take its
			// starting block
			int batStartBlock = bat.CreateBlocks();
			
			// get the extended block allocation table blocks
			HeaderBlockWriter header_block_writer = new HeaderBlockWriter();
			BATBlock[] xbat_blocks = header_block_writer.SetBATBlocks(bat.CountBlocks, batStartBlock);
			
			// set the property table start block
			header_block_writer.PropertyStart = _property_table.StartBlock;
			
			// set the small block allocation table start block
			header_block_writer.SBATStart = sbtw.SBAT.StartBlock;
			
			// set the small block allocation table block count
			header_block_writer.SBATBlockCount = sbtw.SBATBlockCount;
			
			// the header is now properly initialized. Make a list of
			// writers (the header block, followed by the documents, the
			// property table, the small block store, the small block
			// allocation table, the block allocation table, and the
			// extended block allocation table blocks)
			IList writers = new ArrayList();
			
			writers.Add(header_block_writer);
			((ArrayList)writers).AddRange(_documents);
			writers.Add(_property_table);
			writers.Add(sbtw);
			writers.Add(sbtw.SBAT);
			writers.Add(bat);
			for (int j = 0; j < xbat_blocks.Length; j++)
			{
				writers.Add(xbat_blocks[j]);
			}
			
			// now, write everything out
			
			foreach(BlockWritable writer in bm_objects)
			{
				writer.WriteBlocks(stream);
			}
		}
		
		/// <summary> read in a file and write it back out again
		/// 
		/// </summary>
		/// <param name="args">names of the files; arg[ 0 ] is the input file,
		/// arg[ 1 ] is the output file
		/// 
		/// </param>
		/// <exception cref=""> IOException
		/// </exception>
		
		public static void Main2(string[] args)
		{
			if (args.Length != 2)
			{
				Console.Error.WriteLine("two arguments required: input filename and output filename");
				Environment.Exit(1);
			}
			
			FileStream stream = new FileStream(args[0], FileMode.Open, FileAccess.Read);
			
			FileStream ostream = new FileStream(args[1], FileMode.Create);
			
			new POIFSFileSystem(stream).WriteFileSystem(ostream);
			stream.Close();
			ostream.Close();
		}
		
		/// <summary> open a document in the root entry's list of entries
		/// 
		/// </summary>
		/// <param name="documentName">the name of the document to be opened
		/// 
		/// </param>
		/// <returns> a newly opened DocumentInputStream
		/// 
		/// </returns>
		/// <exception cref=""> IOException if the document does not exist or the
		/// name is that of a DirectoryEntry
		/// </exception>
		
		public virtual DocumentInputStream CreateDocumentInputStream(string documentName)
		{
			Entry document = Root.GetEntry(documentName);
			
			if (!document.DocumentEntry)
			{
				throw new IOException("Entry '" + documentName + "' is not a DocumentEntry");
			}
			return new DocumentInputStream((DocumentEntry) document);
		}
		
		/// <summary> add a new POIFSDocument
		/// 
		/// </summary>
		/// <param name="document">the POIFSDocument being added
		/// </param>
		
		internal virtual void AddDocument(POIFSDocument document)
		{
			_documents.Add(document);
			_property_table.AddProperty(document.DocumentProperty);
		}
		
		/// <summary> add a new DirectoryProperty
		/// 
		/// </summary>
		/// <param name="directory">the DirectoryProperty being added
		/// </param>
		
		internal virtual void AddDirectory(DirectoryProperty directory)
		{
			_property_table.AddProperty(directory);
		}
		
		/// <summary> Remove an entry
		/// 
		/// </summary>
		/// <param name="entry">to be Removed
		/// </param>
		
		internal virtual void Remove(EntryNode entry)
		{
			_property_table.RemoveProperty(entry.Property);
			if (entry.DocumentEntry)
			{
				_documents.Remove(((DocumentNode) entry).Document);
			}
		}
		
		private void ProcessProperties(BlockList small_blocks, BlockList big_blocks, IEnumerator properties, DirectoryNode dir)
		{
			
			while (properties.MoveNext())
			{
				
				Property property = (Property) properties.Current;
				string name = property.Name;
				DirectoryNode parent = (dir == null)?((DirectoryNode) Root):dir;
				
				if (property.Directory)
				{
					DirectoryNode new_dir = (DirectoryNode) parent.CreateDirectory(name);
					
					new_dir.StorageClsid = property.StorageClsid;
					
					ProcessProperties(small_blocks, big_blocks, ((DirectoryProperty) property).Children, new_dir);
				}
				else
				{
					int startBlock = property.StartBlock;
					int size = property.Size;
					POIFSDocument document = null;
					
					if (property.ShouldUseSmallBlocks())
					{
						document = new POIFSDocument(name, small_blocks.FetchBlocks(startBlock), size);
					}
					else
					{
						document = new POIFSDocument(name, big_blocks.FetchBlocks(startBlock), size);
					}
					parent.CreateDocument(document);
				}
			}
		}
		
		/* ********** START begin implementation of POIFSViewable ********** */
		
		/// <summary> Give viewers a hint as to whether to call getViewableArray or
		/// getViewableIterator
		/// 
		/// </summary>
		/// <returns> true if a viewer should call getViewableArray, false if
		/// a viewer should call getViewableIterator
		/// </returns>
		
		public virtual bool PreferArray
		{
			get 
			{
				return ((POIFSViewable) Root).PreferArray;
			}
		}
	} 
}