// 
// DirectoryProperty.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.Storage;

namespace Poi.Net.PoiFS.Properties
{
	
	/// <summary> Directory property
	/// 
	/// </summary>
	
	public class DirectoryProperty:Property, Parent
	{
		/// <returns> true if a directory type Property
		/// </returns>
		override public bool Directory
		{
			
			
			get
			{
				return true;
			}
			
		}
		/// <summary> Get an iterator over the children of this Parent; all elements
		/// are instances of Property.
		/// 
		/// </summary>
		/// <returns> Iterator of children; may refer to an empty collection
		/// </returns>
		virtual public IEnumerator Children
		{
			
			
			get
			{
				return _children.GetEnumerator();
			}
			
		}
		
		// List of Property instances
		private IList _children;
		
		// set of children's names
		// HS
		private ArrayList _children_names;
		
		/// <summary> Default constructor
		/// 
		/// </summary>
		/// <param name="name">the name of the directory
		/// </param>
		
		public DirectoryProperty(string name):base()
		{
			_children = new ArrayList();
			
			_children_names = new ArrayList();
			Name = name;
			Size = 0;
			PropertyType = PropertyConstants.DIRECTORY_TYPE;
			StartBlock = 0;
			NodeColor = _NODE_BLACK; // simplification
		}
		
		/// <summary> reader constructor
		/// 
		/// </summary>
		/// <param name="index">index number
		/// </param>
		/// <param name="array">byte data
		/// </param>
		/// <param name="offset">offset into byte data
		/// </param>
		
		protected internal DirectoryProperty(int index, byte[] array, int offset):base(index, array, offset)
		{
			_children = new ArrayList();
			
			_children_names = new ArrayList();
		}
		
		/// <summary> Change a Property's name
		/// 
		/// </summary>
		/// <param name="property">the Property whose name is being changed
		/// </param>
		/// <param name="newName">the new name for the Property
		/// 
		/// </param>
		/// <returns> true if the name change could be made, else false
		/// </returns>
		
		public virtual bool ChangeName(Property property, string newName)
		{
			bool result;
			string oldName = property.Name;
			
			property.Name = newName;
			string cleanNewName = property.Name;
			
			if (_children_names.Contains(cleanNewName))
			{
				
				// revert the change
				property.Name = oldName;
				result = false;
			}
			else
			{
				_children_names.Add(cleanNewName);
				_children_names.Remove(oldName);
				result = true;
			}
			return result;
		}
		
		/// <summary> Delete a Property
		/// 
		/// </summary>
		/// <param name="property">the Property being deleted
		/// 
		/// </param>
		/// <returns> true if the Property could be deleted, else false
		/// </returns>
		
		public virtual bool DeleteChild (Property property)
		{
			bool tempBoolean;
			tempBoolean = _children.Contains(property);
			_children.Remove(property);
			bool result = tempBoolean;
			
			if (result)
			{
				_children_names.Remove(property.Name);
			}
			return result;
		}
		
		
		private class PropertyComparator : IComparer
		{
			public PropertyComparator(DirectoryProperty enclosingInstance)
			{
				InitBlock(enclosingInstance);
			}
			private void InitBlock(DirectoryProperty enclosingInstance)
			{
				this.enclosingInstance = enclosingInstance;
			}
			private DirectoryProperty enclosingInstance;
			public DirectoryProperty Enclosing_Instance
			{
				get
				{
					return enclosingInstance;
				}
				
			}
			
			/// <summary> Object equality, implemented as object identity
			/// 
			/// </summary>
			/// <param name="o">Object we're being compared to
			/// 
			/// </param>
			/// <returns> true if identical, else false
			/// </returns>
			
			public  override bool Equals(object o)
			{
				return this == o;
			}
			
			/// <summary> compare method. Assumes both parameters are non-null
			/// instances of Property. One property is less than another if
			/// its name is shorter than the other property's name. If the
			/// names are the same length, the property whose name comes
			/// before the other property's name, alphabetically, is less
			/// than the other property.
			/// 
			/// </summary>
			/// <param name="o1">first object to compare, better be a Property
			/// </param>
			/// <param name="o2">second object to compare, better be a Property
			/// 
			/// </param>
			/// <returns> negative value if o1 <  o2,
			/// zero           if o1 == o2,
			/// positive value if o1 >  o2.
			/// </returns>
			
			public virtual int Compare(object o1, object o2)
			{
				string name1 = ((Property) o1).Name;
				string name2 = ((Property) o2).Name;
				int result = name1.Length - name2.Length;
				
				if (result == 0)
				{
					result = String.CompareOrdinal(name1, name2);
				}
				return result;
			}
		} 
		
		/* ********** START extension of Property ********** */
		
		/// <summary> Perform whatever activities need to be performed prior to
		/// writing
		/// </summary>
		
		protected internal override void PreWrite()
		{
			if (_children.Count > 0)
			{
				Property[] temp = new Property[_children.Count];
				_children.CopyTo(temp,0);
				Property[] children = (Property[]) temp;
				
				Array.Sort(children, new PropertyComparator(this));
				int midpoint = children.Length / 2;
				
				ChildProperty = children[midpoint].Index;
				children[0].SetPreviousChild(null);
				children[0].NextChild = null;
				for (int j = 1; j < midpoint; j++)
				{
					children[j].SetPreviousChild(children[j - 1]);
					children[j].NextChild = null;
				}
				if (midpoint != 0)
				{
					children[midpoint].SetPreviousChild(children[midpoint - 1]);
				}
				if (midpoint != (children.Length - 1))
				{
					children[midpoint].NextChild = children[midpoint + 1];
					for (int j = midpoint + 1; j < children.Length - 1; j++)
					{
						children[j].SetPreviousChild(null);
						children[j].NextChild = children[j + 1];
					}
					children[children.Length - 1].SetPreviousChild(null);
					children[children.Length - 1].NextChild = null;
				}
				else
				{
					children[midpoint].NextChild = null;
				}
			}
		}
		
		/* **********  END  extension of Property ********** */
		/* ********** START implementation of Parent ********** */
		
		/// <summary> Add a new child to the collection of children
		/// 
		/// </summary>
		/// <param name="property">the new child to be added; must not be null
		/// 
		/// </param>
		/// <exception cref=""> IOException if we already have a child with the same
		/// name
		/// </exception>
		
		public virtual void AddChild (Property property)
		{
			string name = property.Name;
			
			if (_children_names.Contains(name))
			{
				throw new IOException("Duplicate name \"" + name + "\"");
			}
			else 
			{
				_children_names.Add(name);
				_children.Add(property);
			}
		}
		
		/* **********  END  implementation of Parent ********** */
	} 
}