// 
// Property.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Text;
using System.Collections;

using Poi.Net.HPSF;
using Poi.Net.Util;
using Poi.Net.PoiFS.Dev;
using Poi.Net.PoiFS.Common;

namespace Poi.Net.PoiFS.Properties
{
	
	/// <summary> This abstract base class is the ancestor of all classes
	/// implementing POIFS Property behavior.
	/// 
	/// </summary>
	
	public abstract class Property : Child, POIFSViewable
	{
		
		/// <returns> the start block
		/// </returns>
		/// <summary> Set the start block for the document referred to by this
		/// Property.
		/// 
		/// </summary>
		/// <param name="startBlock">the start block index
		/// </param>
		virtual public int StartBlock
		{
			
			
			get
			{
				return _start_block.Get();
			}
			
			
			
			set {
				_start_block.Set(value, _raw_data);
			}
			
		}
		/// <returns> true if a directory type Property
		/// </returns>
		abstract public bool Directory{get;}
		
		/// <summary> Sets the storage clsid, which is the Class ID of a COM object which
		/// reads and writes this stream
		/// </summary>
		/// <returns> storage Class ID for this property stream
		/// </returns>
		/// <summary> Sets the storage class ID for this property stream. This is the Class ID
		/// of the COM object which can read and write this property stream
		/// </summary>
		/// <param name="clsidStorage">Storage Class ID
		/// </param>
		virtual public ClassID StorageClsid {
			get {
				return _storage_clsid;
			}
			
			set {
				_storage_clsid = value;
				if (value == null)
				{
					// Fill
					for (int i = (_storage_clsid_offset <= 0) ? _storage_clsid_offset : _storage_clsid_offset - 1 ; i < _storage_clsid_offset + ClassID.LENGTH; i++)
						_raw_data[i] = (byte) 0;
				}
				else
				{
					value.Write(_raw_data, _storage_clsid_offset);
				}
			}
			/// <summary> Set the property type. Makes no attempt to validate the value.
			/// 
			/// </summary>
			/// <param name="propertyType">the property type (root, file, directory)
			/// </param>
			
		}
		virtual protected internal byte PropertyType
		{
			set {
				_property_type.Set(value, _raw_data);
			}
			
		}
		/// <summary> Set the node color.
		/// 
		/// </summary>
		/// <param name="nodeColor">the node color (red or black)
		/// </param>
		virtual protected internal byte NodeColor
		{
			
			
			set {
				_node_color.Set(value, _raw_data);
			}
			
		}
		/// <summary> Set the child property.
		/// 
		/// </summary>
		/// <param name="child">the child property's index in the Property Table
		/// </param>
		virtual protected internal int ChildProperty
		{
			
			
			set {
				_child_property.Set(value, _raw_data);
			}
			
		}
		/// <summary> Get the child property (its index in the Property Table)
		/// 
		/// </summary>
		/// <returns> child property index
		/// </returns>
		virtual protected internal int ChildIndex {
			get { return _child_property.Get(); }
			
		}
		
		/// <summary> get the index for this Property
		/// 
		/// </summary>
		/// <returns> the index of this Property within its Property Table
		/// </returns>
		/// <summary> Set the index for this Property
		/// 
		/// </summary>
		/// <param name="index">this Property's index within its containing
		/// Property Table
		/// </param>
		virtual protected internal int Index
		{
			
			
			get
			{
				return _index;
			}
			
			
			
			set {
				_index = value;
			}
			
		}
		/// <summary> get the next sibling
		/// 
		/// </summary>
		/// <returns> index of next sibling
		/// </returns>
		virtual internal int NextChildIndex
		{
			
			
			get
			{
				return _next_property.Get();
			}
			
		}
		/// <summary> get the previous sibling
		/// 
		/// </summary>
		/// <returns> index of previous sibling
		/// </returns>
		virtual internal int PreviousChildIndex
		{
			
			
			get
			{
				return _previous_property.Get();
			}
			
		}
		/// <summary> Get an array of objects, some of which may implement
		/// POIFSViewable
		/// 
		/// </summary>
		/// <returns> an array of Object; may not be null, but may be empty
		/// </returns>
		virtual public object[] ViewableArray
		{
			
			
			get
			{
				object[] results = new object[5];
				
				results[0] = "Name          = \"" + Name + "\"";
				results[1] = "Property Type = " + _property_type.Get();
				results[2] = "Node Color    = " + _node_color.Get();
				long time = _days_1.Get();
				
				time <<= 32;
				time += (((long) _seconds_1.Get()) & 0x0000FFFFL);
				results[3] = "Time 1        = " + time;
				time = _days_2.Get();
				time <<= 32;
				time += (((long) _seconds_2.Get()) & 0x0000FFFFL);
				results[4] = "Time 2        = " + time;
				return results;
			}
			
		}
		/// <summary> Get an Iterator of objects, some of which may implement
		/// POIFSViewable
		/// 
		/// </summary>
		/// <returns> an Iterator; may not be null, but may have an empty
		/// back end store
		/// </returns>
		virtual public IEnumerator ViewableIterator
		{
			
			
			get
			{
				return ((IList) ArrayList.ReadOnly(new ArrayList())).GetEnumerator();
			}
			
		}
		/// <summary> Provides a short description of the object, to be used when a
		/// POIFSViewable object has not provided its contents.
		/// 
		/// </summary>
		/// <returns> short description
		/// </returns>
		virtual public string ShortDescription
		{
			
			
			get
			{
				StringBuilder buffer = new StringBuilder();
				
				buffer.Append("Property: \"").Append(Name).Append("\"");
				return buffer.ToString();
			}
			
			/* **********  END  begin implementation of POIFSViewable ********** */
			
		}
		private static byte _default_fill = (byte) 0x00;
		private const int _name_size_offset = 0x40;
		
		private static readonly int _max_name_length = (_name_size_offset / LittleEndian.SHORT_SIZE) - 1;
		protected internal const int _NO_INDEX = - 1;
		
		// useful offsets
		private const int _node_color_offset = 0x43;
		private const int _previous_property_offset = 0x44;
		private const int _next_property_offset = 0x48;
		private const int _child_property_offset = 0x4C;
		private const int _storage_clsid_offset = 0x50;
		private const int _user_flags_offset = 0x60;
		private const int _seconds_1_offset = 0x64;
		private const int _days_1_offset = 0x68;
		private const int _seconds_2_offset = 0x6C;
		private const int _days_2_offset = 0x70;
		private const int _start_block_offset = 0x74;
		private const int _size_offset = 0x78;
		
		// node colors
		protected internal const byte _NODE_BLACK = 1;
		protected internal const byte _NODE_RED = 0;
		
		// documents must be at least this size to be stored in big blocks
		private const int _big_block_minimum_bytes = 4096;
		private string _name;
		private ShortField _name_size;
		private ByteField _property_type;
		private ByteField _node_color;
		private IntegerField _previous_property;
		private IntegerField _next_property;
		private IntegerField _child_property;
		private ClassID _storage_clsid;
		private IntegerField _user_flags;
		private IntegerField _seconds_1;
		private IntegerField _days_1;
		private IntegerField _seconds_2;
		private IntegerField _days_2;
		private IntegerField _start_block;
		private IntegerField _size;
		private byte[] _raw_data;
		private int _index;
		private Child _next_child;
		private Child _previous_child;
		
		/// <summary> Default constructor</summary>
		
		protected internal Property()
		{
			_raw_data = new byte[POIFSConstants.PROPERTY_SIZE];

			// Fill
			for (int i = 0; i < _raw_data.Length; i++)
				_raw_data[i]= (byte) _default_fill;

			_name_size = new ShortField(_name_size_offset);
			_property_type = new ByteField(PropertyConstants.PROPERTY_TYPE_OFFSET);
			_node_color = new ByteField(_node_color_offset);
			_previous_property = new IntegerField(_previous_property_offset, _NO_INDEX, _raw_data);
			_next_property = new IntegerField(_next_property_offset, _NO_INDEX, _raw_data);
			_child_property = new IntegerField(_child_property_offset, _NO_INDEX, _raw_data);
			_storage_clsid = new ClassID(_raw_data, _storage_clsid_offset);
			_user_flags = new IntegerField(_user_flags_offset, 0, _raw_data);
			_seconds_1 = new IntegerField(_seconds_1_offset, 0, _raw_data);
			_days_1 = new IntegerField(_days_1_offset, 0, _raw_data);
			_seconds_2 = new IntegerField(_seconds_2_offset, 0, _raw_data);
			_days_2 = new IntegerField(_days_2_offset, 0, _raw_data);
			_start_block = new IntegerField(_start_block_offset);
			_size = new IntegerField(_size_offset, 0, _raw_data);
			_index = _NO_INDEX;
			Name = "";
			NextChild = null;
			SetPreviousChild(null);
		}
		
		/// <summary> Constructor from byte data
		/// 
		/// </summary>
		/// <param name="index">index number
		/// </param>
		/// <param name="array">byte data
		/// </param>
		/// <param name="offset">offset into byte data
		/// </param>
		
		protected internal Property(int index, byte[] array, int offset)
		{
			_raw_data = new byte[POIFSConstants.PROPERTY_SIZE];
			Array.Copy(array, offset, _raw_data, 0, POIFSConstants.PROPERTY_SIZE);
			_name_size = new ShortField(_name_size_offset, _raw_data);
			_property_type = new ByteField(PropertyConstants.PROPERTY_TYPE_OFFSET, _raw_data);
			_node_color = new ByteField(_node_color_offset, _raw_data);
			_previous_property = new IntegerField(_previous_property_offset, _raw_data);
			_next_property = new IntegerField(_next_property_offset, _raw_data);
			_child_property = new IntegerField(_child_property_offset, _raw_data);
			_storage_clsid = new ClassID(_raw_data, _storage_clsid_offset);
			_user_flags = new IntegerField(_user_flags_offset, 0, _raw_data);
			_seconds_1 = new IntegerField(_seconds_1_offset, _raw_data);
			_days_1 = new IntegerField(_days_1_offset, _raw_data);
			_seconds_2 = new IntegerField(_seconds_2_offset, _raw_data);
			_days_2 = new IntegerField(_days_2_offset, _raw_data);
			_start_block = new IntegerField(_start_block_offset, _raw_data);
			_size = new IntegerField(_size_offset, _raw_data);
			_index = index;
			int name_length = (_name_size.Get() / LittleEndian.SHORT_SIZE) - 1;
			
			if (name_length < 1)
			{
				_name = "";
			}
			else
			{
				char[] char_array = new char[name_length];
				int name_offset = 0;
				
				for (int j = 0; j < name_length; j++)
				{
					char_array[j] = (char) new ShortField(name_offset, _raw_data).Get();
					name_offset += LittleEndian.SHORT_SIZE;
				}
				_name = new string(char_array, 0, name_length);
			}
			_next_child = null;
			_previous_child = null;
		}
		
		/// <summary> Write the raw data to an OutputStream.
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the data should be
		/// written.
		/// 
		/// </param>
		/// <exception cref=""> IOException on problems writing to the specified
		/// stream.
		/// </exception>
		
		public virtual void WriteData (Stream stream)
		{
				stream.Write(_raw_data, 0, _raw_data.Length);
		}
		
		/// <summary> find out the document size
		/// 
		/// </summary>
		/// <returns> size in bytes
		/// </returns>
		
		public virtual int Size {
			get { return _size.Get(); }
			set { _size.Set(value, _raw_data); }
		}
		
		/// <summary> Based on the currently defined size, should this property use
		/// small blocks?
		/// 
		/// </summary>
		/// <returns> true if the size is less than _big_block_minimum_bytes
		/// </returns>
		
		public virtual bool ShouldUseSmallBlocks()
		{
			return Property.IsSmall(_size.Get());
		}
		
		/// <summary> does the length indicate a small document?
		/// 
		/// </summary>
		/// <param name="length">length in bytes
		/// 
		/// </param>
		/// <returns> true if the length is less than
		/// _big_block_minimum_bytes
		/// </returns>
		
		public static bool IsSmall(int length)
		{
			return length < _big_block_minimum_bytes;
		}
		
		/// <summary> Get the name of this property
		/// 
		/// </summary>
		/// <returns> property name as String
		/// </returns>
		
		public virtual string Name 
		{
			get {return _name;}

			// protected internal?
			set {
				char[] char_array = value.ToCharArray();
				int limit = System.Math.Min(char_array.Length, _max_name_length);
			
				_name = new string(char_array, 0, limit);
				short offset = 0;
				int j = 0;
			
				for (; j < limit; j++)
				{
					new ShortField(offset, (short) char_array[j], _raw_data);
					offset = (short) (offset + LittleEndian.SHORT_SIZE);
				}
				
				for (; j < _max_name_length + 1; j++)
				{
					new ShortField(offset, (short) 0, _raw_data);
					offset = (short) (offset + LittleEndian.SHORT_SIZE);
				}
			
				// double the count, and include the null at the end
				_name_size.Set((short) ((limit + 1) * LittleEndian.SHORT_SIZE), _raw_data);
			}
		}
		
		/// <summary> Set the name; silently truncates the name if it's too long.
		/// 
		/// </summary>
		/// <param name="name">the new name
		/// </param>

		
		/// <summary> Set the size of the document associated with this Property
		/// 
		/// </summary>
		/// <param name="size">the size of the document, in bytes
		/// </param>
		
			
		/// <summary> Perform whatever activities need to be performed prior to
		/// writing
		/// </summary>
		
		abstract protected internal void PreWrite();
		
		/// <summary> determine whether the specified index is valid
		/// 
		/// </summary>
		/// <param name="index">value to be checked
		/// 
		/// </param>
		/// <returns> true if the index is valid
		/// </returns>
		
		internal static bool IsValidIndex(int index)
		{
			return index != _NO_INDEX;
		}
		
		/* ********** START implementation of Child ********** */
		
		/// <summary> Get the next Child, if any
		/// 
		/// </summary>
		/// <returns> the next Child; may return null
		/// </returns>
		
		public virtual Child NextChild {
			get { return _next_child; }
			set {
				_next_child = value;
				_next_property.Set((value == null)?_NO_INDEX:((Property) value).Index, _raw_data);
			}
		}
		
		/// <summary> Get the previous Child, if any
		/// 
		/// </summary>
		/// <returns> the previous Child; may return null
		/// </returns>
		
		public virtual Child GetPreviousChild()
		{
			return _previous_child;
		}
		
		/// <summary> Set the next Child
		/// 
		/// </summary>
		/// <param name="child">the new 'next' child; may be null, which has the
		/// effect of saying there is no 'next' child
		/// </param>
				
		/// <summary> Set the previous Child
		/// 
		/// </summary>
		/// <param name="child">the new 'previous' child; may be null, which has
		/// the effect of saying there is no 'previous' child
		/// </param>
		
		public virtual void SetPreviousChild(Child child)
		{
			_previous_child = child;
			_previous_property.Set((child == null)?_NO_INDEX:((Property) child).Index, _raw_data);
		}
		
		/* **********  END  implementation of Child ********** */
		/* ********** START begin implementation of POIFSViewable ********** */
		
		/// <summary> Give viewers a hint as to whether to call getViewableArray or
		/// getViewableIterator
		/// 
		/// </summary>
		/// <returns> true if a viewer should call getViewableArray, false if
		/// a viewer should call getViewableIterator
		/// </returns>
		
		public virtual bool PreferArray 
		{
			get 
			{
				return true;
			}

		}
	} 
}