// 
// PropertyFactory.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Collections;

using Poi.Net.PoiFS.Common;
using Poi.Net.PoiFS.Storage;

namespace Poi.Net.PoiFS.Properties
{
	
	/// <summary> Factory for turning an array of RawDataBlock instances containing
	/// Proprty data into an array of proper Property objects.
	/// 
	/// The array produced may be sparse, in that any portion of data that
	/// should correspond to a Property, but which does not map to a proper
	/// Property (i.e., a DirectoryProperty, DocumentProperty, or
	/// RootProperty) will get mapped to a null Property in the array.
	/// 
	/// </summary>
	/// 
	class PropertyFactory
	{
		
		// no need for an accessible constructor
		private PropertyFactory()
		{
		}
		
		/// <summary> Convert raw data blocks to an array of Property's
		/// 
		/// </summary>
		/// <param name="blocks">to be converted
		/// 
		/// </param>
		/// <returns> the converted List of Property objects. May contain
		/// nulls, but will not be null
		/// 
		/// </returns>
		/// <exception cref=""> IOException if any of the blocks are empty
		/// </exception>
		
		internal static IList ConvertToProperties(ListManagedBlock[] blocks)
		{
			IList properties = new ArrayList();
			
			for (int j = 0; j < blocks.Length; j++)
			{
				byte[] data = blocks[j].Data;
				int property_count = data.Length / POIFSConstants.PROPERTY_SIZE;
				int offset = 0;
				
				for (int k = 0; k < property_count; k++)
				{
					switch (data[offset + PropertyConstants.PROPERTY_TYPE_OFFSET])
					{
						
						
						case PropertyConstants.DIRECTORY_TYPE: 
							properties.Add(new DirectoryProperty(properties.Count, data, offset));
							break;
						
						
						case PropertyConstants.DOCUMENT_TYPE: 
							properties.Add(new DocumentProperty(properties.Count, data, offset));
							break;
						
						
						case PropertyConstants.ROOT_TYPE: 
							properties.Add(new RootProperty(properties.Count, data, offset));
							break;
						
						
						default: 
							properties.Add(null);
							break;
						
					}
					offset += POIFSConstants.PROPERTY_SIZE;
				}
			}
			return properties;
		}
	} 
}