// 
// PropertyTable.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.Common;
using Poi.Net.PoiFS.Storage;
using Poi.Net.PoiFS.FileSystem;

namespace Poi.Net.PoiFS.Properties
{
	
	/// <summary> This class embodies the Property Table for the filesystem; this is
	/// basically the dsirectory for all of the documents in the
	/// filesystem.
	/// 
	/// </summary>

	public class PropertyTable : BATManaged, BlockWritable
	{
		/// <summary> Get the root property
		/// 
		/// </summary>
		/// <returns> the root property
		/// </returns>
		virtual public RootProperty Root
		{
			
			
			get
			{
				
				// it's always the first element in the List
				return (RootProperty) _properties[0];
			}
			
		}
		private int _start_block;
		private IList _properties;
		private BlockWritable[] _blocks;
		
		/// <summary> Default constructor</summary>
		
		public PropertyTable()
		{
			_start_block = POIFSConstants.END_OF_CHAIN;
			_properties = new ArrayList();
			AddProperty(new RootProperty());
			_blocks = null;
		}
		
		/// <summary> reading constructor (used when we've read in a file and we want
		/// to extract the property table from it). Populates the
		/// properties thoroughly
		/// 
		/// </summary>
		/// <param name="startBlock">the first block of the property table
		/// </param>
		/// <param name="blockList">the list of blocks
		/// 
		/// </param>
		/// <exception cref=""> IOException if anything goes wrong (which should be
		/// a result of the input being NFG)
		/// </exception>
		
		public PropertyTable(int startBlock, RawDataBlockList blockList)
		{
			_start_block = POIFSConstants.END_OF_CHAIN;
			_blocks = null;
			_properties = PropertyFactory.ConvertToProperties(blockList.FetchBlocks(startBlock));
			populatePropertyTree((DirectoryProperty) _properties[0]);
		}
		
		/// <summary> Add a property to the list of properties we manage
		/// 
		/// </summary>
		/// <param name="property">the new Property to manage
		/// </param>
		
		public virtual void AddProperty(Property property)
		{
			_properties.Add(property);
		}
		
		/// <summary> Remove a property from the list of properties we manage
		/// 
		/// </summary>
		/// <param name="property">the Property to be Removed
		/// </param>
		
		public virtual void RemoveProperty(Property property)
		{
			_properties.Remove(property);
		}
		
		/// <summary> Prepare to be written</summary>
		
		public virtual void preWrite()
		{

			
			Property[] temp = new Property[_properties.Count];
			_properties.CopyTo(temp,0);
			Property[] properties = (Property[]) temp;
			
			// give each property its index
			for (int k = 0; k < properties.Length; k++)
			{
				properties[k].Index = k;
			}
			
			// allocate the blocks for the property table
			_blocks = PropertyBlock.CreatePropertyBlockArray(_properties);
			
			// prepare each property for writing
			for (int k = 0; k < properties.Length; k++)
			{
				properties[k].PreWrite();
			}
		}
		
		/// <summary> Get the start block for the property table
		/// 
		/// </summary>
		/// <returns> start block index
		/// </returns>
		
		private void populatePropertyTree(DirectoryProperty root)
		{
			int index = root.ChildIndex;
			
			if (!Property.IsValidIndex(index))
			{
				
				// property has no children
				return ;
			}
			ArrayList children = new ArrayList();
			
			children.Add(_properties[index]);
			while (!(children.Count == 0))
			{

				//HelperClass
				
				Property property = (Property) children[children.Count - 1];
				children.RemoveAt(children.Count - 1);

				root.AddChild(property);
				if (property.Directory)
				{
					populatePropertyTree((DirectoryProperty) property);
				}
				index = property.PreviousChildIndex;
				if (Property.IsValidIndex(index))
				{
					children.Add(_properties[index]);
				}
				index = property.NextChildIndex;
				if (Property.IsValidIndex(index))
				{
					children.Add(_properties[index]);
				}
			}
		}
		
		/* ********** START implementation of BATManaged ********** */
		
		/// <summary> Return the number of BigBlock's this instance uses
		/// 
		/// </summary>
		/// <returns> count of BigBlock instances
		/// </returns>
		
		public virtual int CountBlocks {
			get { return (_blocks == null)?0:_blocks.Length; }
		}
		
		/// <summary> Set the start block for this instance
		/// 
		/// </summary>
		/// <param name="index">index into the array of BigBlock instances making
		/// up the the filesystem
		/// </param>
		
		public virtual int StartBlock {
			get { return _start_block; }
			set { _start_block = value; }
		}
		
		/* **********  END  implementation of BATManaged ********** */
		/* ********** START implementation of BlockWritable ********** */
		
		/// <summary> Write the storage to an OutputStream
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the stored data should
		/// be written
		/// 
		/// </param>
		/// <exception cref=""> IOException on problems writing to the specified
		/// stream
		/// </exception>
		
		public virtual void WriteBlocks(Stream stream)
		{
			if (_blocks != null)
			{
				for (int j = 0; j < _blocks.Length; j++)
				{
					_blocks[j].WriteBlocks(stream);
				}
			}
		}
		
		/* **********  END  implementation of BlockWritable ********** */
	} 
}