// 
// BlockAllocationTableReader.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.Util;
using Poi.Net.PoiFS.Common;

namespace Poi.Net.PoiFS.Storage
{
	
	/// <summary> This class manages and creates the Block Allocation Table, which is
	/// basically a set of linked lists of block indices.
	/// 
	/// Each block of the filesystem has an index. The first block, the
	/// header, is skipped; the first block after the header is index 0,
	/// the next is index 1, and so on.
	/// 
	/// A block's index is also its index into the Block Allocation
	/// Table. The entry that it finds in the Block Allocation Table is the
	/// index of the next block in the linked list of blocks making up a
	/// file, or it is set to -2: end of list.
	/// 
	/// </summary>
	
	public class BlockAllocationTableReader
	{
		private IntList _entries;
		
		/// <summary> create a BlockAllocationTableReader for an existing filesystem. Side
		/// effect: when this method finishes, the BAT blocks will have
		/// been Removed from the raw block list, and any blocks labeled as
		/// 'unused' in the block allocation table will also have been
		/// Removed from the raw block list.
		/// 
		/// </summary>
		/// <param name="block_count">the number of BAT blocks making up the block
		/// allocation table
		/// </param>
		/// <param name="block_array">the array of BAT block indices from the
		/// filesystem's header
		/// </param>
		/// <param name="xbat_count">the number of XBAT blocks
		/// </param>
		/// <param name="xbat_index">the index of the first XBAT block
		/// </param>
		/// <param name="raw_block_list">the list of RawDataBlocks
		/// 
		/// </param>
		/// <exception cref=""> IOException if, in trying to create the table, we
		/// encounter Logic errors
		/// </exception>
		
		public BlockAllocationTableReader(int block_count, int[] block_array, int xbat_count, int xbat_index, BlockList raw_block_list):this()
		{
			if (block_count <= 0)
			{
				throw new IOException("Illegal block count; minimum count is 1, got " + block_count + " instead");
			}
			
			// acquire raw data blocks containing the BAT block data
			RawDataBlock[] blocks = new RawDataBlock[block_count];
			int limit = System.Math.Min(block_count, block_array.Length);
			int block_index;
			
			for (block_index = 0; block_index < limit; block_index++)
			{
				blocks[block_index] = (RawDataBlock) raw_block_list.Remove(block_array[block_index]);
			}
			if (block_index < block_count)
			{
				
				// must have extended blocks
				if (xbat_index < 0)
				{
					throw new IOException("BAT count exceeds limit, yet XBAT index indicates no valid entries");
				}
				int chain_index = xbat_index;
				int max_entries_per_block = BATBlock.EntriesPerXBATBlock();
				int chain_index_offset = BATBlock.XBATChainOffset;
				
				for (int j = 0; j < xbat_count; j++)
				{
					limit = System.Math.Min(block_count - block_index, max_entries_per_block);
					byte[] data = raw_block_list.Remove(chain_index).Data;
					int offset = 0;
					
					for (int k = 0; k < limit; k++)
					{
						blocks[block_index++] = (RawDataBlock) raw_block_list.Remove(LittleEndian.GetInt(data, offset));
						offset += LittleEndian.INT_SIZE;
					}
					chain_index = LittleEndian.GetInt(data, chain_index_offset);
					if (chain_index == POIFSConstants.END_OF_CHAIN)
					{
						break;
					}
				}
			}
			if (block_index != block_count)
			{
				throw new IOException("Could not find all blocks");
			}
			
			// now that we have all of the raw data blocks, go through and
			// create the indices
			SetEntries(blocks, raw_block_list);
		}
		
		/// <summary> create a BlockAllocationTableReader from an array of raw data blocks
		/// 
		/// </summary>
		/// <param name="blocks">the raw data
		/// </param>
		/// <param name="raw_block_list">the list holding the managed blocks
		/// 
		/// </param>
		/// <exception cref=""> IOException
		/// </exception>
		
		internal BlockAllocationTableReader(ListManagedBlock[] blocks, BlockList raw_block_list):this()
		{
			SetEntries(blocks, raw_block_list);
		}
		
		/// <summary> Constructor BlockAllocationTableReader
		/// 
		/// 
		/// </summary>
		
		internal BlockAllocationTableReader()
		{
			_entries = new IntList();
		}
		
		/// <summary> walk the entries from a specified point and return the
		/// associated blocks. The associated blocks are Removed from the
		/// block list
		/// 
		/// </summary>
		/// <param name="startBlock">the first block in the chain
		/// </param>
		/// <param name="blockList">the raw data block list
		/// 
		/// </param>
		/// <returns> array of ListManagedBlocks, in their correct order
		/// 
		/// </returns>
		/// <exception cref=""> IOException if there is a problem acquiring the blocks
		/// </exception>
		
		internal virtual ListManagedBlock[] FetchBlocks(int startBlock, BlockList blockList)
		{
			IList blocks = new ArrayList();
			int currentBlock = startBlock;
			
			while (currentBlock != POIFSConstants.END_OF_CHAIN)
			{
				blocks.Add(blockList.Remove(currentBlock));
				currentBlock = _entries.Get(currentBlock);
			}
			
			ListManagedBlock[] temp = new ListManagedBlock[blocks.Count];
			blocks.CopyTo(temp,0);
			return (ListManagedBlock[]) temp;
		}
		
		// methods for debugging reader
		
		/// <summary> determine whether the block specified by index is used or not
		/// 
		/// </summary>
		/// <param name="index">index of block in question
		/// 
		/// </param>
		/// <returns> true if the specific block is used, else false
		/// </returns>
		
		internal virtual bool IsUsed(int index)
		{
			bool rval = false;
			
			try
			{
				rval = _entries.Get(index) != - 1;
			}
			catch (IndexOutOfRangeException ignored)
			{
			}
			return rval;
		}
		
		/// <summary> return the next block index
		/// 
		/// </summary>
		/// <param name="index">of the current block
		/// 
		/// </param>
		/// <returns> index of the next block (may be
		/// POIFSConstants.END_OF_CHAIN, indicating end of chain
		/// (duh))
		/// 
		/// </returns>
		/// <exception cref=""> IOException if the current block is unused
		/// </exception>
		
		internal virtual int GetNextBlockIndex(int index)
		{
			if (IsUsed(index))
			{
				return _entries.Get(index);
			}
			else
			{
				throw new IOException("index " + index + " is unused");
			}
		}
		
		/// <summary> Convert an array of blocks into a set of integer indices
		/// 
		/// </summary>
		/// <param name="blocks">the array of blocks containing the indices
		/// </param>
		/// <param name="raw_blocks">the list of blocks being managed. Unused
		/// blocks will be eliminated from the list
		/// 
		/// </param>
		/// <exception cref=""> IOException
		/// </exception>
		
		private void SetEntries(ListManagedBlock[] blocks, BlockList raw_blocks)
		{
			int limit = BATBlock.EntriesPerBlock();
			
			for (int block_index = 0; block_index < blocks.Length; block_index++)
			{
				byte[] data = blocks[block_index].Data;
				int offset = 0;
				
				for (int k = 0; k < limit; k++)
				{
					int entry = LittleEndian.GetInt(data, offset);
					
					if (entry == POIFSConstants.UNUSED_BLOCK)
					{
						raw_blocks.Zap(_entries.Size);
					}
					_entries.Add(entry);
					offset += LittleEndian.INT_SIZE;
				}
				
				// discard block
				blocks[block_index] = null;
			}
			raw_blocks.BAT = this;
		}
	} // end class BlockAllocationTableReader
}