// 
// BlockAllocationTableWriter.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

using Poi.Net.Util;
using Poi.Net.PoiFS.Common;
using Poi.Net.PoiFS.FileSystem;

namespace Poi.Net.PoiFS.Storage
{
	
	/// <summary> This class manages and creates the Block Allocation Table, which is
	/// basically a set of linked lists of block indices.
	/// 
	/// Each block of the filesystem has an index. The first block, the
	/// header, is skipped; the first block after the header is index 0,
	/// the next is index 1, and so on.
	/// 
	/// A block's index is also its index into the Block Allocation
	/// Table. The entry that it finds in the Block Allocation Table is the
	/// index of the next block in the linked list of blocks making up a
	/// file, or it is set to -2: end of list.
	/// 
	/// </summary>
	
	public class BlockAllocationTableWriter : BlockWritable, BATManaged
	{
		private IntList _entries;
		private BATBlock[] _blocks;
		private int _start_block;
		
		/// <summary> create a BlockAllocationTableWriter</summary>
		
		public BlockAllocationTableWriter()
		{
			_start_block = POIFSConstants.END_OF_CHAIN;
			_entries = new IntList();
			_blocks = new BATBlock[0];
		}
		
		/// <summary> Create the BATBlocks we need
		/// 
		/// </summary>
		/// <returns> start block index of BAT blocks
		/// </returns>
		
		public virtual int CreateBlocks()
		{
			int xbat_blocks = 0;
			int bat_blocks = 0;
			
			while (true)
			{
				int calculated_bat_blocks = BATBlock.CalculateStorageRequirements(bat_blocks + xbat_blocks + _entries.Size);
				int calculated_xbat_blocks = HeaderBlockWriter.CalculateXBATStorageRequirements(calculated_bat_blocks);
				
				if ((bat_blocks == calculated_bat_blocks) && (xbat_blocks == calculated_xbat_blocks))
				{
					
					// stable ... we're OK
					break;
				}
				else
				{
					bat_blocks = calculated_bat_blocks;
					xbat_blocks = calculated_xbat_blocks;
				}
			}
			int startBlock = AllocateSpace(bat_blocks);
			
			AllocateSpace(xbat_blocks);
			SimpleCreateBlocks();
			return startBlock;
		}
		
		/// <summary> Allocate space for a block of indices
		/// 
		/// </summary>
		/// <param name="blockCount">the number of blocks to allocate space for
		/// 
		/// </param>
		/// <returns> the starting index of the blocks
		/// </returns>
		
		public virtual int AllocateSpace(int blockCount)
		{
			int startBlock = _entries.Size;
			
			if (blockCount > 0)
			{
				int limit = blockCount - 1;
				int index = startBlock + 1;
				
				for (int k = 0; k < limit; k++)
				{
					_entries.Add(index++);
				}
				_entries.Add(POIFSConstants.END_OF_CHAIN);
			}
			return startBlock;
		}
		
		/// <summary> get the starting block
		/// 
		/// </summary>
		/// <returns> the starting block index
		/// </returns>
		
		
		/// <summary> create the BATBlocks</summary>
		
		internal virtual void SimpleCreateBlocks()
		{
			_blocks = BATBlock.CreateBATBlocks(_entries.ToArray());
		}
		
		/* ********** START implementation of BlockWritable ********** */
		
		/// <summary> Write the storage to an OutputStream
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the stored data should
		/// be written
		/// 
		/// </param>
		/// <exception cref=""> IOException on problems writing to the specified
		/// stream
		/// </exception>
		
		public virtual void WriteBlocks(Stream stream)
		{
			for (int j = 0; j < _blocks.Length; j++)
			{
				_blocks[j].WriteBlocks(stream);
			}
		}
		
		/* **********  END  implementation of BlockWritable ********** */
		/* ********** START implementation of BATManaged ********** */
		
		/// <summary> Return the number of BigBlock's this instance uses
		/// 
		/// </summary>
		/// <returns> count of BigBlock instances
		/// </returns>
		
		public virtual int CountBlocks {
			get { return _blocks.Length; }
			}
		
		/// <summary> Set the start block for this instance
		/// 
		/// </summary>
		/// <param name="index">index into the array of BigBlock instances making
		/// up the the filesystem
		/// 
		/// </param>
		/// <param name="">start_block
		/// </param>
		
		public virtual int StartBlock {
			get { return _start_block; }
			set { _start_block = value; }
		}
		
		/* **********  END  implementation of BATManaged ********** */
	} // end class BlockAllocationTableWriter
}