// 
// DocumentBlock.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

using Poi.Net.Util;
using Poi.Net.PoiFS.Common;

namespace Poi.Net.PoiFS.Storage
{
	
	/// <summary> A block of document data.
	/// 
	/// </summary>
	
	public class DocumentBlock:BigBlock
	{
		/// <returns> the fill byte used
		/// </returns>
		public static byte FillByte
		{
			
			
			get
			{
				return _default_value;
			}
			
		}
		private static byte _default_value = unchecked((byte) 0xFF);
		private byte[] _data;
		private int _bytes_read;
		
		/// <summary> create a document block from a raw data block
		/// 
		/// </summary>
		/// <param name="block">the raw data block
		/// 
		/// </param>
		/// <exception cref=""> IOException
		/// </exception>
		
		public DocumentBlock(RawDataBlock block)
		{
			_data = block.Data;
			_bytes_read = _data.Length;
		}
		
		/// <summary> Create a single instance initialized with data.
		/// 
		/// </summary>
		/// <param name="stream">the InputStream delivering the data.
		/// 
		/// </param>
		/// <exception cref=""> IOException
		/// </exception>
		
		public DocumentBlock(Stream stream):this()
		{
			int count = IOUtils.ReadFully(stream, _data);
			
			_bytes_read = (count == - 1)?0:count;
		}
		
		/// <summary> Create a single instance initialized with default values</summary>
		
		private DocumentBlock()
		{
			_data = new byte[POIFSConstants.BIG_BLOCK_SIZE];
			
			// Fill
			for (int i = 0; i < _data.Length; i++)
				_data[i] = (byte) _default_value;
		}
		
		/// <summary> Get the number of bytes read for this block
		/// 
		/// </summary>
		/// <returns> bytes read into the block
		/// </returns>
		
		public virtual int Size {
			get{ return _bytes_read; }
		}
		
		/// <summary> Was this a partially read block?
		/// 
		/// </summary>
		/// <returns> true if the block was only partially filled with data
		/// </returns>
		
		public virtual bool PartiallyRead()
		{
			return _bytes_read != POIFSConstants.BIG_BLOCK_SIZE;
		}
		
		/// <summary> convert a single long array into an array of DocumentBlock
		/// instances
		/// 
		/// </summary>
		/// <param name="array">the byte array to be converted
		/// </param>
		/// <param name="size">the intended size of the array (which may be smaller)
		/// 
		/// </param>
		/// <returns> an array of DocumentBlock instances, filled from the
		/// input array
		/// </returns>
		
		public static DocumentBlock[] Convert(byte[] array, int size)
		{
			DocumentBlock[] rval = new DocumentBlock[(size + POIFSConstants.BIG_BLOCK_SIZE - 1) / POIFSConstants.BIG_BLOCK_SIZE];
			int offset = 0;
			
			for (int k = 0; k < rval.Length; k++)
			{
				rval[k] = new DocumentBlock();
				if (offset < array.Length)
				{
					int length = System.Math.Min(POIFSConstants.BIG_BLOCK_SIZE, array.Length - offset);
					
					Array.Copy(array, offset, rval[k]._data, 0, length);
					if (length != POIFSConstants.BIG_BLOCK_SIZE)
					{
						// Fill
						for (int i = (length <= 0) ? length : length - 1 ; i < POIFSConstants.BIG_BLOCK_SIZE; i++)
							rval[k]._data[i] = (byte) _default_value;

					}
				}
				else
				{					
					// Fill
					for (int i = 0; i < rval[k]._data.Length; i++)
						rval[k]._data[i] = (byte) _default_value;
				}
				offset += POIFSConstants.BIG_BLOCK_SIZE;
			}
			return rval;
		}
		
		/// <summary> read data from an array of DocumentBlocks
		/// 
		/// </summary>
		/// <param name="blocks">the blocks to read from
		/// </param>
		/// <param name="buffer">the buffer to write the data into
		/// </param>
		/// <param name="offset">the offset into the array of blocks to read from
		/// </param>
		
		public static void Read(DocumentBlock[] blocks, byte[] buffer, int offset)
		{
			int firstBlockIndex = offset / POIFSConstants.BIG_BLOCK_SIZE;
			int firstBlockOffset = offset % POIFSConstants.BIG_BLOCK_SIZE;
			int lastBlockIndex = (offset + buffer.Length - 1) / POIFSConstants.BIG_BLOCK_SIZE;
			
			if (firstBlockIndex == lastBlockIndex)
			{
				Array.Copy(blocks[firstBlockIndex]._data, firstBlockOffset, buffer, 0, buffer.Length);
			}
			else
			{
				int buffer_offset = 0;
				
				Array.Copy(blocks[firstBlockIndex]._data, firstBlockOffset, buffer, buffer_offset, POIFSConstants.BIG_BLOCK_SIZE - firstBlockOffset);
				buffer_offset += POIFSConstants.BIG_BLOCK_SIZE - firstBlockOffset;
				for (int j = firstBlockIndex + 1; j < lastBlockIndex; j++)
				{
					Array.Copy(blocks[j]._data, 0, buffer, buffer_offset, POIFSConstants.BIG_BLOCK_SIZE);
					buffer_offset += POIFSConstants.BIG_BLOCK_SIZE;
				}
				Array.Copy(blocks[lastBlockIndex]._data, 0, buffer, buffer_offset, buffer.Length - buffer_offset);
			}
		}
		
		/* ********** START extension of BigBlock ********** */
		
		/// <summary> Write the block's data to an OutputStream
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the stored data should
		/// be written
		/// 
		/// </param>
		/// <exception cref=""> IOException on problems writing to the specified
		/// stream
		/// </exception>
		
		internal override void WriteData(Stream stream)
		{
			DoWriteData(stream, _data);
		}
		
		/* **********  END  extension of BigBlock ********** */
	} 
}