// 
// HeaderBlockReader.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

using Poi.Net.PoiFS.Common;
using Poi.Net.Util;

namespace Poi.Net.PoiFS.Storage
{
	
	/// <summary> The block containing the archive header
	/// 
	/// </summary>
	
	public class HeaderBlockReader
	{
		/// <summary> get start of Property Table
		/// 
		/// </summary>
		/// <returns> the index of the first block of the Property Table
		/// </returns>
		virtual public int PropertyStart
		{
			
			
			get
			{
				return _property_start.Get();
			}
			
		}
		/// <returns> start of small block allocation table
		/// </returns>
		virtual public int SBATStart
		{
			
			
			get
			{
				return _sbat_start.Get();
			}
			
		}
		/// <returns> number of BAT blocks
		/// </returns>
		virtual public int BATCount
		{
			
			
			get
			{
				return _bat_count.Get();
			}
			
		}
		/// <returns> BAT array
		/// </returns>
		virtual public int[] BATArray
		{
			
			
			get
			{
				int[] result = new int[HeaderBlockConstants._max_bats_in_header];
				int offset = HeaderBlockConstants._bat_array_offset;
				
				for (int j = 0; j < HeaderBlockConstants._max_bats_in_header; j++)
				{
					result[j] = LittleEndian.GetInt(_data, offset);
					offset += LittleEndian.INT_SIZE;
				}
				return result;
			}
			
		}
		/// <returns> XBAT count
		/// </returns>
		virtual public int XBATCount
		{
			
			
			get
			{
				return _xbat_count.Get();
			}
			
		}
		/// <returns> XBAT index
		/// </returns>
		virtual public int XBATIndex
		{
			
			
			get
			{
				return _xbat_start.Get();
			}
			
		}
		
		// number of big block allocation table blocks (int)
		private IntegerField _bat_count;
		
		// start of the property set block (int index of the property set
		// chain's first big block)
		private IntegerField _property_start;
		
		// start of the small block allocation table (int index of small
		// block allocation table's first big block)
		private IntegerField _sbat_start;
		
		// big block index for extension to the big block allocation table
		private IntegerField _xbat_start;
		private IntegerField _xbat_count;
		private byte[] _data;
		
		/// <summary> create a new HeaderBlockReader from an InputStream
		/// 
		/// </summary>
		/// <param name="stream">the source InputStream
		/// 
		/// </param>
		/// <exception cref=""> IOException on errors or bad data
		/// </exception>
		
		public HeaderBlockReader(Stream stream)
		{
			_data = new byte[POIFSConstants.BIG_BLOCK_SIZE];
			int byte_count = IOUtils.ReadFully(stream, _data);
			
			if (byte_count != POIFSConstants.BIG_BLOCK_SIZE)
			{
				string type = " byte" + ((byte_count == 1)?(""):("s"));
				
				throw new IOException("Unable to read entire header; " + byte_count + type + " read; expected " + POIFSConstants.BIG_BLOCK_SIZE + " bytes");
			}
			
			// verify signature
			LongField signature = new LongField(HeaderBlockConstants._signature_offset, _data);
			
			if (signature.Get() != HeaderBlockConstants._signature)
			{
				throw new IOException("Invalid header signature; read " + signature.Get() + ", expected " + HeaderBlockConstants._signature);
			}
			_bat_count = new IntegerField(HeaderBlockConstants._bat_count_offset, _data);
			_property_start = new IntegerField(HeaderBlockConstants._property_start_offset, _data);
			_sbat_start = new IntegerField(HeaderBlockConstants._sbat_start_offset, _data);
			_xbat_start = new IntegerField(HeaderBlockConstants._xbat_start_offset, _data);
			_xbat_count = new IntegerField(HeaderBlockConstants._xbat_count_offset, _data);
		}
	} 
}