// 
// PropertyBlock.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.Common;
using Poi.Net.PoiFS.Properties;
using Poi.Net.Util;

namespace Poi.Net.PoiFS.Storage
{
	
	/// <summary> A block of Property instances
	/// 
	/// </summary>
	
	public class PropertyBlock:BigBlock
	{		
		private class AnonymousProperty:Property
		{
			protected internal override void PreWrite()
			{
			}
			override public bool Directory {
				get { return false; }				
			}
		}

		private static readonly int _properties_per_block = POIFSConstants.BIG_BLOCK_SIZE / POIFSConstants.PROPERTY_SIZE;
		private Property[] _properties;
		
		/// <summary> Create a single instance initialized with default values
		/// 
		/// </summary>
		/// <param name="properties">the properties to be inserted
		/// </param>
		/// <param name="offset">the offset into the properties array
		/// </param>
		
		private PropertyBlock(Property[] properties, int offset)
		{
			_properties = new Property[_properties_per_block];
			for (int j = 0; j < _properties_per_block; j++)
			{
				_properties[j] = properties[j + offset];
			}
		}
		
		/// <summary> Create an array of PropertyBlocks from an array of Property
		/// instances, creating empty Property instances to make up any
		/// shortfall
		/// 
		/// </summary>
		/// <param name="properties">the Property instances to be converted into
		/// PropertyBlocks, in a java List
		/// 
		/// </param>
		/// <returns> the array of newly created PropertyBlock instances
		/// </returns>
		
		public static BlockWritable[] CreatePropertyBlockArray(IList properties)
		{
			int block_count = (properties.Count + _properties_per_block - 1) / _properties_per_block;
			Property[] to_be_written = new Property[block_count * _properties_per_block];
			
			Property[] temp = new Property[properties.Count];
			properties.CopyTo(temp,0);

			Array.Copy(temp, 0, to_be_written, 0, properties.Count);
			for (int j = properties.Count; j < to_be_written.Length; j++)
			{
				
				// create an instance of an anonymous inner class that
				// extends Propert

				to_be_written[j] = new AnonymousProperty();
			}
			BlockWritable[] rvalue = new BlockWritable[block_count];
			
			for (int j = 0; j < block_count; j++)
			{
				rvalue[j] = new PropertyBlock(to_be_written, j * _properties_per_block);
			}
			return rvalue;
		}
		
		/* ********** START extension of BigBlock ********** */
		
		/// <summary> Write the block's data to an OutputStream
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the stored data should
		/// be written
		/// 
		/// </param>
		/// <exception cref=""> IOException on problems writing to the specified
		/// stream
		/// </exception>
		
		internal override void WriteData (Stream stream)
		{
			for (int j = 0; j < _properties_per_block; j++)
			{
				_properties[j].WriteData(stream);
			}
		}
		
		/* **********  END  extension of BigBlock ********** */
	} 
}