// 
// SmallBlockTableWriter.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.Properties;
using Poi.Net.PoiFS.FileSystem;

namespace Poi.Net.PoiFS.Storage
{
	
	/// <summary> This class implements storage for writing the small blocks used by
	/// small documents.
	/// 
	/// </summary>
	
	public class SmallBlockTableWriter : BlockWritable, BATManaged
	{
		/// <summary> Get the number of SBAT blocks
		/// 
		/// </summary>
		/// <returns> number of SBAT big blocks
		/// </returns>
		virtual public int SBATBlockCount
		{
			
			
			get
			{
				return (_big_block_count + 15) / 16;
			}
			
		}
		/// <summary> Get the SBAT
		/// 
		/// </summary>
		/// <returns> the Small Block Allocation Table
		/// </returns>
		virtual public BlockAllocationTableWriter SBAT
		{
			
			
			get
			{
				return _sbat;
			}
			
		}
		private BlockAllocationTableWriter _sbat;
		private IList _small_blocks;
		private int _big_block_count;
		private RootProperty _root;
		
		/// <summary> Creates new SmallBlockTable
		/// 
		/// </summary>
		/// <param name="documents">a List of POIFSDocument instances
		/// </param>
		/// <param name="root">the Filesystem's root property
		/// </param>
		
		public SmallBlockTableWriter(IList documents, RootProperty root)
		{
			_sbat = new BlockAllocationTableWriter();
			_small_blocks = new ArrayList();
			_root = root;

			foreach(POIFSDocument doc in documents)
			{
				BlockWritable[] blocks = doc.SmallBlocks;
				
				if (blocks.Length != 0)
				{
					doc.StartBlock = _sbat.AllocateSpace(blocks.Length);
					for (int j = 0; j < blocks.Length; j++)
					{
						_small_blocks.Add(blocks[j]);
					}
				}
			}
			_sbat.SimpleCreateBlocks();
			_root.Size = _small_blocks.Count;
			_big_block_count = SmallDocumentBlock.Fill(_small_blocks);
		}
		
		/* ********** START implementation of BATManaged ********** */
		
		/// <summary> Return the number of BigBlock's this instance uses
		/// 
		/// </summary>
		/// <returns> count of BigBlock instances
		/// </returns>
		
		public virtual int CountBlocks {
				get {return _big_block_count; }
		}
		
		/// <summary> Set the start block for this instance
		/// 
		/// </summary>
		/// <param name="index">index into the array of BigBlock instances making
		/// up the the filesystem
		/// 
		/// </param>
		/// <param name="">start_block
		/// </param>
		
		public virtual int StartBlock {
			set { _root.StartBlock = value; }
		}
		
		/* **********  END  implementation of BATManaged ********** */
		/* ********** START implementation of BlockWritable ********** */
		
		/// <summary> Write the storage to an OutputStream
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the stored data should
		/// be written
		/// 
		/// </param>
		/// <exception cref=""> IOException on problems writing to the specified
		/// stream
		/// </exception>
		
		public virtual void WriteBlocks(Stream stream)
		{
			
			foreach (BlockWritable current in _small_blocks)
				current.WriteBlocks(stream);
		}
		
		/* **********  END  implementation of BlockWritable ********** */
	}
}