// 
// SmallDocumentBlock.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.Common;

namespace Poi.Net.PoiFS.Storage
{
	
	/// <summary> Storage for documents that are too small to use regular
	/// DocumentBlocks for their data
	/// 
	/// </summary>
	
	public class SmallDocumentBlock : BlockWritable, ListManagedBlock
	{
		/// <summary> Get the data from the block
		/// 
		/// </summary>
		/// <returns> the block's data as a byte array
		/// 
		/// </returns>
		/// <exception cref=""> IOException if there is no data
		/// </exception>
		virtual public byte[] Data
		{
			
			
			get
			{
				return _data;
			}
			
			/* **********  END  implementation of ListManagedBlock ********** */
			
		}
		private byte[] _data;
		private static byte _default_fill = unchecked((byte) 0xff);
		private const int _block_size = 64;
		
		private static readonly int _blocks_per_big_block = POIFSConstants.BIG_BLOCK_SIZE / _block_size;
		
		private SmallDocumentBlock(byte[] data, int index):this()
		{
			Array.Copy(data, index * _block_size, _data, 0, _block_size);
		}
		
		private SmallDocumentBlock()
		{
			_data = new byte[_block_size];
		}
		
		/// <summary> convert a single long array into an array of SmallDocumentBlock
		/// instances
		/// 
		/// </summary>
		/// <param name="array">the byte array to be converted
		/// </param>
		/// <param name="size">the intended size of the array (which may be smaller)
		/// 
		/// </param>
		/// <returns> an array of SmallDocumentBlock instances, filled from
		/// the array
		/// </returns>
		
		public static SmallDocumentBlock[] Convert(byte[] array, int size)
		{
			SmallDocumentBlock[] rval = new SmallDocumentBlock[(size + _block_size - 1) / _block_size];
			int offset = 0;
			
			for (int k = 0; k < rval.Length; k++)
			{
				rval[k] = new SmallDocumentBlock();
				if (offset < array.Length)
				{
					int length = System.Math.Min(_block_size, array.Length - offset);
					
					Array.Copy(array, offset, rval[k]._data, 0, length);
					if (length != _block_size)
					{
						
						// Fill
						for (int i = (length <= 0) ? length : length - 1 ; i < _block_size; i++)
							rval[k]._data[i] = (byte) _default_fill;
					}
				}
				else
				{					
					// Fill
					for (int i = 0; i < rval[k]._data.Length; i++)
						rval[k]._data[i] = (byte) _default_fill;
				}
				offset += _block_size;
			}
			return rval;
		}
		
		/// <summary> fill out a List of SmallDocumentBlocks so that it fully occupies
		/// a set of big blocks
		/// 
		/// </summary>
		/// <param name="blocks">the List to be filled out
		/// 
		/// </param>
		/// <returns> number of big blocks the list encompasses
		/// </returns>
		
		public static int Fill(IList blocks)
		{
			int count = blocks.Count;
			int big_block_count = (count + _blocks_per_big_block - 1) / _blocks_per_big_block;
			int full_count = big_block_count * _blocks_per_big_block;
			
			for (; count < full_count; count++)
			{
				blocks.Add(MakeEmptySmallDocumentBlock());
			}
			return big_block_count;
		}
		
		/// <summary> Factory for creating SmallDocumentBlocks from DocumentBlocks
		/// 
		/// </summary>
		/// <param name="store">the original DocumentBlocks
		/// </param>
		/// <param name="size">the total document size
		/// 
		/// </param>
		/// <returns> an array of new SmallDocumentBlocks instances
		/// 
		/// </returns>
		/// <exception cref=""> IOException on errors reading from the DocumentBlocks
		/// </exception>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if, somehow, the store
		/// contains less data than size indicates
		/// </exception>
		
		public static SmallDocumentBlock[] Convert(BlockWritable[] store, int size)
		{
			MemoryStream stream = new MemoryStream();
			
			for (int j = 0; j < store.Length; j++)
			{
				store[j].WriteBlocks(stream);
			}
			
			byte[] data = stream.ToArray();
			
			SmallDocumentBlock[] rval = new SmallDocumentBlock[ConvertToBlockCount(size)];
			
			for (int index = 0; index < rval.Length; index++)
			{
				rval[index] = new SmallDocumentBlock(data, index);
			}
			return rval;
		}
		
		/// <summary> create a list of SmallDocumentBlock's from raw data
		/// 
		/// </summary>
		/// <param name="blocks">the raw data containing the SmallDocumentBlock
		/// data
		/// 
		/// </param>
		/// <returns> a List of SmallDocumentBlock's extracted from the input
		/// 
		/// </returns>
		/// <exception cref=""> IOException
		/// </exception>
		
		public static IList Extract(ListManagedBlock[] blocks)
		{
			IList sdbs = new ArrayList();
			
			for (int j = 0; j < blocks.Length; j++)
			{
				byte[] data = blocks[j].Data;
				
				for (int k = 0; k < _blocks_per_big_block; k++)
				{
					sdbs.Add(new SmallDocumentBlock(data, k));
				}
			}
			return sdbs;
		}
		
		/// <summary> read data from an array of SmallDocumentBlocks
		/// 
		/// </summary>
		/// <param name="blocks">the blocks to read from
		/// </param>
		/// <param name="buffer">the buffer to write the data into
		/// </param>
		/// <param name="offset">the offset into the array of blocks to read from
		/// </param>
		
		public static void Read(BlockWritable[] blocks, byte[] buffer, int offset)
		{
			int firstBlockIndex = offset / _block_size;
			int firstBlockOffset = offset % _block_size;
			int lastBlockIndex = (offset + buffer.Length - 1) / _block_size;
			
			if (firstBlockIndex == lastBlockIndex)
			{
				Array.Copy(((SmallDocumentBlock) blocks[firstBlockIndex])._data, firstBlockOffset, buffer, 0, buffer.Length);
			}
			else
			{
				int buffer_offset = 0;
				
				Array.Copy(((SmallDocumentBlock) blocks[firstBlockIndex])._data, firstBlockOffset, buffer, buffer_offset, _block_size - firstBlockOffset);
				buffer_offset += _block_size - firstBlockOffset;
				for (int j = firstBlockIndex + 1; j < lastBlockIndex; j++)
				{
					Array.Copy(((SmallDocumentBlock) blocks[j])._data, 0, buffer, buffer_offset, _block_size);
					buffer_offset += _block_size;
				}
				Array.Copy(((SmallDocumentBlock) blocks[lastBlockIndex])._data, 0, buffer, buffer_offset, buffer.Length - buffer_offset);
			}
		}
		
		/// <summary> Calculate the storage size of a set of SmallDocumentBlocks
		/// 
		/// </summary>
		/// <param name="size">number of SmallDocumentBlocks
		/// 
		/// </param>
		/// <returns> total size
		/// </returns>
		
		public static int CalcSize(int size)
		{
			return size * _block_size;
		}
		
		private static SmallDocumentBlock MakeEmptySmallDocumentBlock()
		{
			SmallDocumentBlock block = new SmallDocumentBlock();
			
			
			// Fill
			for (int i = 0; i < block._data.Length; i++)
				block._data[i] = (byte) _default_fill;
			return block;
		}
		
		private static int ConvertToBlockCount(int size)
		{
			return (size + _block_size - 1) / _block_size;
		}
		
		/* ********** START implementation of BlockWritable ********** */
		
		/// <summary> Write the storage to an OutputStream
		/// 
		/// </summary>
		/// <param name="stream">the OutputStream to which the stored data should
		/// be written
		/// 
		/// </param>
		/// <exception cref=""> IOException on problems writing to the specified
		/// stream
		/// </exception>
		
		public virtual void WriteBlocks(Stream stream)
		{
				stream.Write(_data, 0, _data.Length);
		}
		
		/* **********  END  implementation of BlockWritable ********** */
		/* ********** START implementation of ListManagedBlock ********** */
	} 
}