// 
// BitField.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//   Andrew C. Oliver (acoliver at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;

namespace Poi.Net.Util
{
	
	/// <summary> Manage operations dealing with bit-mapped fields.
	/// 
	/// </summary>
	
	public class BitField
	{
		private int _mask;		
		private int _shift_count;
		
		/// <summary> Create a BitField instance
		/// 
		/// </summary>
		/// <param name="mask">the mask specifying which bits apply to this
		/// BitField. Bits that are set in this mask are the
		/// bits that this BitField operates on
		/// </param>
		
		public BitField(int mask)
		{
			_mask = mask;
			int count = 0;
			int bit_pattern = mask;
			
			if (bit_pattern != 0)
			{
				while ((bit_pattern & 1) == 0)
				{
					count++;
					bit_pattern >>= 1;
				}
			}
			_shift_count = count;
		}
		
		/// <summary> Obtain the value for the specified BitField, appropriately
		/// shifted right. Many users of a BitField will want to treat the
		/// specified bits as an int value, and will not want to be aware
		/// that the value is stored as a BitField (and so shifted left so
		/// many bits)
		/// 
		/// </summary>
		/// <param name="holder">the int data containing the bits we're interested
		/// in
		/// 
		/// </param>
		/// <returns> the selected bits, shifted right appropriately
		/// </returns>
		
		public virtual int GetValue(int holder)
		{
			return GetRawValue(holder) >> _shift_count;
		}
		
		/// <summary> Obtain the value for the specified BitField, appropriately
		/// shifted right, as a short. Many users of a BitField will want
		/// to treat the specified bits as an int value, and will not want
		/// to be aware that the value is stored as a BitField (and so
		/// shifted left so many bits)
		/// 
		/// </summary>
		/// <param name="holder">the short data containing the bits we're
		/// interested in
		/// 
		/// </param>
		/// <returns> the selected bits, shifted right appropriately
		/// </returns>
		
		public virtual short GetShortValue(short holder)
		{
			return (short) GetValue(holder);
		}
		
		/// <summary> Obtain the value for the specified BitField, unshifted
		/// 
		/// </summary>
		/// <param name="holder">the int data containing the bits we're interested
		/// in
		/// 
		/// </param>
		/// <returns> the selected bits
		/// </returns>
		
		public virtual int GetRawValue(int holder)
		{
			return (holder & _mask);
		}
		
		/// <summary> Obtain the value for the specified BitField, unshifted
		/// 
		/// </summary>
		/// <param name="holder">the short data containing the bits we're
		/// interested in
		/// 
		/// </param>
		/// <returns> the selected bits
		/// </returns>
		
		public virtual short GetShortRawValue(short holder)
		{
			return (short) GetRawValue(holder);
		}
		
		/// <summary> Is the field set or not? This is most commonly used for a
		/// single-bit field, which is often used to represent a boolean
		/// value; the results of using it for a multi-bit field is to
		/// determine whether *any* of its bits are set
		/// 
		/// </summary>
		/// <param name="holder">the int data containing the bits we're interested
		/// in
		/// 
		/// </param>
		/// <returns> true if any of the bits are set, else false
		/// </returns>
		
		public virtual bool IsSet(int holder)
		{
			return (holder & _mask) != 0;
		}
		
		/// <summary> Are all of the bits set or not? This is a stricter test than
		/// IsSet, in that all of the bits in a multi-bit set must be set
		/// for this method to return true
		/// 
		/// </summary>
		/// <param name="holder">the int data containing the bits we're interested
		/// in
		/// 
		/// </param>
		/// <returns> true if all of the bits are set, else false
		/// </returns>
		
		public virtual bool IsAllSet(int holder)
		{
			return (holder & _mask) == _mask;
		}
		
		/// <summary> Replace the bits with new values.
		/// 
		/// </summary>
		/// <param name="holder">the int data containint the bits we're interested
		/// in
		/// </param>
		/// <param name="value">the new value for the specified bits
		/// 
		/// </param>
		/// <returns> the value of holder with the bits from the value
		/// parameter replacing the old bits
		/// </returns>
		
		public virtual int SetValue(int holder, int _value)
		{
			return (holder & ~ _mask) | ((_value << _shift_count) & _mask);
		}
		
		/// <summary> Replace the bits with new values.
		/// 
		/// </summary>
		/// <param name="holder">the short data containing the bits we're
		/// interested in
		/// </param>
		/// <param name="value">the new value for the specified bits
		/// 
		/// </param>
		/// <returns> the value of holder with the bits from the value
		/// parameter replacing the old bits
		/// </returns>
		
		public virtual short SetShortValue(short holder, short _value)
		{
			return (short) SetValue(holder, _value);
		}
		
		/// <summary> Clear the bits.
		/// 
		/// </summary>
		/// <param name="holder">the int data containing the bits we're interested
		/// in
		/// 
		/// </param>
		/// <returns> the value of holder with the specified bits Cleared
		/// (set to 0)
		/// </returns>
		
		public virtual int Clear(int holder)
		{
			return holder & ~ _mask;
		}
		
		/// <summary> Clear the bits.
		/// 
		/// </summary>
		/// <param name="holder">the short data containing the bits we're
		/// interested in
		/// 
		/// </param>
		/// <returns> the value of holder with the specified bits Cleared
		/// (set to 0)
		/// </returns>
		
		public virtual short ClearShort(short holder)
		{
			return (short) Clear(holder);
		}
		
		/// <summary> Clear the bits.
		/// 
		/// </summary>
		/// <param name="holder">the byte data containing the bits we're
		/// interested in
		/// 
		/// </param>
		/// <returns> the value of holder with the specified bits Cleared
		/// (set to 0)
		/// </returns>
		
		public virtual byte ClearByte(byte holder)
		{
			return (byte) Clear(holder);
		}
		
		/// <summary> Set the bits.
		/// 
		/// </summary>
		/// <param name="holder">the int data containing the bits we're interested
		/// in
		/// 
		/// </param>
		/// <returns> the value of holder with the specified bits set to 1
		/// </returns>
		
		public virtual int Set(int holder)
		{
			return holder | _mask;
		}
		
		/// <summary> Set the bits.
		/// 
		/// </summary>
		/// <param name="holder">the short data containing the bits we're
		/// interested in
		/// 
		/// </param>
		/// <returns> the value of holder with the specified bits set to 1
		/// </returns>
		
		public virtual short SetShort(short holder)
		{
			return (short) Set(holder);
		}
		
		/// <summary> Set the bits.
		/// 
		/// </summary>
		/// <param name="holder">the byte data containing the bits we're
		/// interested in
		/// 
		/// </param>
		/// <returns> the value of holder with the specified bits set to 1
		/// </returns>
		
		public virtual byte SetByte(byte holder)
		{
			return (byte) Set(holder);
		}
		
		/// <summary> Set a boolean BitField
		/// 
		/// </summary>
		/// <param name="holder">the int data containing the bits we're interested
		/// in
		/// </param>
		/// <param name="flag">indicating whether to set or Clear the bits
		/// 
		/// </param>
		/// <returns> the value of holder with the specified bits set or
		/// Cleared
		/// </returns>
		
		public virtual int SetBoolean(int holder, bool flag)
		{
			return flag?Set(holder):Clear(holder);
		}
		
		/// <summary> Set a boolean BitField
		/// 
		/// </summary>
		/// <param name="holder">the short data containing the bits we're
		/// interested in
		/// </param>
		/// <param name="flag">indicating whether to set or Clear the bits
		/// 
		/// </param>
		/// <returns> the value of holder with the specified bits set or
		/// Cleared
		/// </returns>
		
		public virtual short SetShortBoolean(short holder, bool flag)
		{
			return flag?SetShort(holder):ClearShort(holder);
		}
		
		/// <summary> Set a boolean BitField
		/// 
		/// </summary>
		/// <param name="holder">the byte data containing the bits we're
		/// interested in
		/// </param>
		/// <param name="flag">indicating whether to set or Clear the bits
		/// 
		/// </param>
		/// <returns> the value of holder with the specified bits set or
		/// Cleared
		/// </returns>
		
		public virtual byte SetByteBoolean(byte holder, bool flag)
		{
			return flag?SetByte(holder):ClearByte(holder);
		}
	} 
}