// 
// BlockingInputStream.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Jens Gerhard
//   aviks
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

namespace Poi.Net.Util
{
	
	/// <summary> Implementation of a BlockingInputStream to provide data to 
	/// RawDataBlock that expects data in 512 byte chunks.  Useful to read
	/// data from slow (ie, non FileInputStream) sources, for example when 
	/// reading an OLE2 Document over a network. 
	/// 
	/// Possible extentions: add a timeout. Curently a call to read(byte[]) on this
	/// class is blocking, so use at your own peril if your underlying stream blocks. 
	/// 
	/// </summary>

	public class BlockingInputStream:Stream
	{
		protected internal Stream stream;
		
		public BlockingInputStream(Stream stream)
		{
			this.stream = stream;
		}
		
		
		public int available()
		{
			long available;
			available = stream.Length - stream.Position;

			return (int) available;
		}
		
		public override void Close()
		{
			stream.Close();
		}
		int _mark = 0;
		
		public void mark(int readLimit)
		{
			
			_mark = readLimit;
		}
		
		
		public bool markSupported()
		{
			
			return stream.CanSeek;
		}
		
		public override int ReadByte()
		{
			return stream.ReadByte();
		}
		
		/// <summary> We had to revert to byte per byte reading to keep
		/// with slow network connections on one hand, without
		/// missing the end-of-file. 
		/// This is the only method that does its own thing in this class
		/// everything else is delegated to aggregated stream. 
		/// THIS IS A BLOCKING BLOCK READ!!!
		/// </summary>
		
		public int Read(byte[] bf)
		{
			
			int i = 0;
			int b = 4611;
			while (i < bf.Length)
			{
				b = stream.ReadByte();
				if (b == - 1)
					break;
				bf[i++] = (byte) b;
			}
			if (i == 0 && b == - 1)
				return - 1;
			return i;
		}
		
		
		public override int Read(byte[] bf, int s, int l)
		{
				int rval = stream.Read(bf,s, l);
				return rval;
		}
		
		
		public void reset()
		{
			
			stream.Seek(_mark, SeekOrigin.Begin);
		}
		
		
		public long Skip(long n)
		{
			Stream temp_Stream;
			long temp_Int64;
			temp_Stream = stream;
			temp_Int64 = temp_Stream.Position;
			temp_Int64 = temp_Stream.Seek(n, SeekOrigin.Current) - temp_Int64;

			if(stream is Poi.Net.Util.BlockingInputStream) 
			{
				return ((Poi.Net.Util.BlockingInputStream)stream).Skip(n);
			}
			else if(stream is Poi.Net.PoiFS.FileSystem.DocumentInputStream) 
			{
				return ((Poi.Net.PoiFS.FileSystem.DocumentInputStream)stream).Skip(n);
			}
			else 
			{
				return temp_Int64;
			}
		}
		
		public override void Write(byte[] buffer, int offset, int count)
		{
		}
				
		public override void SetLength(long value)
		{
		}
		
		public override long Seek(long offset, SeekOrigin origin)
		{
			return 0;
		}
		
		public override void Flush()
		{
		}
		
		public override bool CanRead {
			get {
				return false;
			}
			
		}
		
		public override bool CanSeek {
			get {
				return false;
			}
			
		}
		
		public override bool CanWrite {
			get {
				return false;
			}
			
		}
		
		public override long Length {
			get {
				return stream.Length;
			}
			
		}
		
		public override long Position {
			get {
				return stream.Position;
			}
			
			set {
				stream.Position = value;
			}
			
		}
	}
}