// 
// ByteField.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

using Poi.Net.Util;
namespace Poi.Net.Util
{
	
	/// <summary> representation of a byte (8-bit) field at a fixed location within a
	/// byte array
	/// 
	/// </summary>
	
	public class ByteField : FixedField
	{
		private const byte _default_value = 0;
		private byte _value;
		
		private int _offset;
		
		/// <summary> construct the ByteField with its offset into its containing
		/// byte array and a default value of 0
		/// 
		/// </summary>
		/// <param name="offset">of the field within its byte array
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if offset is negative
		/// </exception>
		
		public ByteField(int offset):this(offset, _default_value)
		{
		}
		
		/// <summary> construct the ByteField with its offset into its containing
		/// byte array and initialize its value
		/// 
		/// </summary>
		/// <param name="offset">of the field within its byte array
		/// </param>
		/// <param name="value">the initial value
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if offset is negative
		/// </exception>
		
		public ByteField(int offset, byte __value)
		{
			if (offset < 0)
			{
				throw new IndexOutOfRangeException("offset cannot be negative");
			}
			_offset = offset;
			Set(__value);
		}
		
		/// <summary> Construct the ByteField with its offset into its containing
		/// byte array and initialize its value from its byte array
		/// 
		/// </summary>
		/// <param name="offset">of the field within its byte array
		/// </param>
		/// <param name="data">the byte array to read the value from
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is not
		/// within the range of 0..(data.length - 1)
		/// </exception>
		
		public ByteField(int offset, byte[] data):this(offset)
		{
			ReadFromBytes(data);
		}
		
		/// <summary> construct the ByteField with its offset into its containing
		/// byte array, initialize its value, and write its value to its
		/// byte array
		/// 
		/// </summary>
		/// <param name="offset">of the field within its byte array
		/// </param>
		/// <param name="value">the initial value
		/// </param>
		/// <param name="data">the byte array to write the value to
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is not
		/// within the range of 0..(data.length - 1)
		/// </exception>
		
		public ByteField(int offset, byte _value, byte[] data):this(offset, _value)
		{
			WriteToBytes(data);
		}
		
		/// <summary> get the ByteField's current value
		/// 
		/// </summary>
		/// <returns> current value
		/// </returns>
		
		public virtual byte Get()
		{
			return _value;
		}
		
		/// <summary> set the ByteField's current value
		/// 
		/// </summary>
		/// <param name="value">to be set
		/// </param>
		
		public virtual void Set(byte __value)
		{
			_value = __value;
		}
		
		/// <summary> set the ByteField's current value and write it to a byte array
		/// 
		/// </summary>
		/// <param name="value">to be set
		/// </param>
		/// <param name="data">the byte array to write the value to
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is out
		/// of the byte array's range
		/// </exception>
		
		public virtual void Set(byte __value, byte[] data)
		{
			Set(__value);
			WriteToBytes(data);
		}
		
		/* ********** START implementation of FixedField ********** */
		
		/// <summary> set the value from its offset into an array of bytes
		/// 
		/// </summary>
		/// <param name="data">the byte array from which the value is to be read
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is out
		/// of range of the bte array
		/// </exception>
		
		public virtual void ReadFromBytes(byte[] data)
		{
			_value = data[_offset];
		}
		
		/// <summary> set the value from an InputStream
		/// 
		/// </summary>
		/// <param name="stream">the InputStream from which the value is to be
		/// read
		/// 
		/// </param>
		/// <exception cref=""> BufferUnderrunException if there is not enough data
		/// available from the InputStream
		/// </exception>
		/// <exception cref=""> IOException if an IOException is thrown from reading
		/// the InputStream
		/// </exception>
		
		public virtual void ReadFromStream(Stream stream)
		{
			_value = (LittleEndian.ReadFromStream(stream, LittleEndian.BYTE_SIZE))[0];
		}
		
		/// <summary> write the value out to an array of bytes at the appropriate
		/// offset
		/// 
		/// </summary>
		/// <param name="data">the array of bytes to which the value is to be
		/// written
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is out
		/// of the byte array's range
		/// </exception>
		
		public virtual void WriteToBytes(byte[] data)
		{
			data[_offset] = _value;
		}
		
		/// <summary> return the value as a String
		/// 
		/// </summary>
		/// <returns> the value as a String
		/// </returns>
		
		public override string ToString()
		{
			return Convert.ToString((byte) _value);
		}
		
		/* **********  END  implementation of FixedField ********** */
	} 
}