// 
// HexDump.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//   Glen Stampoultzis  (glens at apache.org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Text;

namespace Poi.Net.Util
{
	
	/// <summary> Dump data in hexadecimal format; derived from a HexDump utility I
	/// wrote in June 2001.
	/// 
	/// </summary>
	
	public class HexDump
	{
		
		public static readonly string EOL = Environment.NewLine;
		//    private static final StringBuffer _lbuffer    = new StringBuffer(8);
		//    private static final StringBuffer _cbuffer    = new StringBuffer(2);
		
		private static readonly char[] _hexcodes = new char[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
		
		private static readonly int[] _shifts = new int[]{28, 24, 20, 16, 12, 8, 4, 0};
		
		
		// all static methods, so no need for a public constructor
		private HexDump()
		{
		}
		
		/// <summary> Dump an array of bytes to an OutputStream
		/// 
		/// </summary>
		/// <param name="data">the byte array to be Dumped
		/// </param>
		/// <param name="offset">its offset, whatever that might mean
		/// </param>
		/// <param name="stream">the OutputStream to which the data is to be
		/// written
		/// </param>
		/// <param name="index">initial index into the byte array
		/// </param>
		/// <param name="length">number of characters to output
		/// 
		/// </param>
		/// <exception cref=""> IOException is thrown if anything goes wrong writing
		/// the data to stream
		/// </exception>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the index is
		/// outside the data array's bounds
		/// </exception>
		/// <exception cref=""> IllegalArgumentException if the output stream is
		/// null
		/// </exception>
		
		public static void Dump(byte[] data, long offset,Stream stream, int index, int length)
		{
			lock (typeof(Poi.Net.Util.HexDump))
			{
				if ((index < 0) || (data.Length != 0 && index >= data.Length))
				{
					throw new IndexOutOfRangeException("illegal index: " + index + " into array of length " + data.Length);
				}
				if (data.Length == 0)
					return ; // nothing more to do.
				if (stream == null)
				{
					throw new ArgumentException("cannot write to nullstream");
				}
				long display_offset = offset + index;
				StringBuilder buffer = new StringBuilder(74);
				
				
				int data_length = System.Math.Min(data.Length, index + length);
				for (int j = index; j < data_length; j += 16)
				{
					int chars_read = data_length - j;
					
					if (chars_read > 16)
					{
						chars_read = 16;
					}
					buffer.Append(Dump(display_offset)).Append(' ');
					for (int k = 0; k < 16; k++)
					{
						if (k < chars_read)
						{
							buffer.Append(Dump(data[k + j]));
						}
						else
						{
							buffer.Append("  ");
						}
						buffer.Append(' ');
					}
					for (int k = 0; k < chars_read; k++)
					{
						if ((data[k + j] >= ' ') && (data[k + j] < 127))
						{
							buffer.Append((char) data[k + j]);
						}
						else
						{
							buffer.Append('.');
						}
					}
					buffer.Append(EOL);

					byte[] buf = UTF8Encoding.UTF8.GetBytes(buffer.ToString());
					stream.Write(buf, 0, buf.Length);

					stream.Flush();
					buffer.Length = 0;
					display_offset += chars_read;
				}
			}
		}
		
		/// <summary> Dump an array of bytes to an OutputStream
		/// 
		/// </summary>
		/// <param name="data">the byte array to be Dumped
		/// </param>
		/// <param name="offset">its offset, whatever that might mean
		/// </param>
		/// <param name="stream">the OutputStream to which the data is to be
		/// written
		/// </param>
		/// <param name="index">initial index into the byte array
		/// 
		/// </param>
		/// <exception cref=""> IOException is thrown if anything goes wrong writing
		/// the data to stream
		/// </exception>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the index is
		/// outside the data array's bounds
		/// </exception>
		/// <exception cref=""> IllegalArgumentException if the output stream is
		/// null
		/// </exception>
		
		
		public static void Dump(byte[] data, long offset,Stream stream, int index)
		{
			lock (typeof(Poi.Net.Util.HexDump))
			{
				Dump(data, offset, stream, index, data.Length - index);
			}
		}
		
		/// <summary> Dump an array of bytes to a String
		/// 
		/// </summary>
		/// <param name="data">the byte array to be Dumped
		/// </param>
		/// <param name="offset">its offset, whatever that might mean
		/// </param>
		/// <param name="index">initial index into the byte array
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the index is
		/// outside the data array's bounds
		/// </exception>
		/// <returns> output string
		/// </returns>
		
		public static string Dump(byte[] data, long offset, int index)
		{
			StringBuilder buffer;
			if ((index < 0) || (index >= data.Length))
			{
				throw new IndexOutOfRangeException("illegal index: " + index + " into array of length " + data.Length);
			}
			long display_offset = offset + index;
			buffer = new StringBuilder(74);
			
			for (int j = index; j < data.Length; j += 16)
			{
				int chars_read = data.Length - j;
				
				if (chars_read > 16)
				{
					chars_read = 16;
				}
				buffer.Append(Dump(display_offset)).Append(' ');
				for (int k = 0; k < 16; k++)
				{
					if (k < chars_read)
					{
						buffer.Append(Dump(data[k + j]));
					}
					else
					{
						buffer.Append("  ");
					}
					buffer.Append(' ');
				}
				for (int k = 0; k < chars_read; k++)
				{
					if ((data[k + j] >= ' ') && (data[k + j] < 127))
					{
						buffer.Append((char) data[k + j]);
					}
					else
					{
						buffer.Append('.');
					}
				}
				buffer.Append(EOL);
				display_offset += chars_read;
			}
			return buffer.ToString();
		}
		
		
		private static string Dump(long _value)
		{
			StringBuilder buf = new StringBuilder();
			buf.Length = 0;
			for (int j = 0; j < 8; j++)
			{
				buf.Append(_hexcodes[((int) (_value >> _shifts[j])) & 15]);
			}
			return buf.ToString();
		}
		
		private static string Dump(byte _value)
		{
			StringBuilder buf = new StringBuilder();
			buf.Length = 0;
			for (int j = 0; j < 2; j++)
			{
				buf.Append(_hexcodes[(_value >> _shifts[j + 6]) & 15]);
			}
			return buf.ToString();
		}
		
		/// <summary> Converts the parameter to a hex value.
		/// 
		/// </summary>
		/// <param name="value">    The value to convert
		/// </param>
		/// <returns>          A String representing the array of bytes
		/// </returns>
		public static string ToHex(byte[] _value)
		{
			StringBuilder retVal = new StringBuilder();
			retVal.Append('[');
			for (int x = 0; x < _value.Length; x++)
			{
				retVal.Append(ToHex(_value[x]));
				retVal.Append(", ");
			}
			retVal.Append(']');
			return retVal.ToString();
		}
		
		/// <summary> Converts the parameter to a hex value.
		/// 
		/// </summary>
		/// <param name="value">    The value to convert
		/// </param>
		/// <returns>          The result right padded with 0
		/// </returns>
		public static string ToHex(short _value)
		{
			return ToHex(_value, 4);
		}
		
		/// <summary> Converts the parameter to a hex value.
		/// 
		/// </summary>
		/// <param name="value">    The value to convert
		/// </param>
		/// <returns>          The result right padded with 0
		/// </returns>
		public static string ToHex(byte _value)
		{
			return ToHex(_value, 2);
		}
		
		/// <summary> Converts the parameter to a hex value.
		/// 
		/// </summary>
		/// <param name="value">    The value to convert
		/// </param>
		/// <returns>          The result right padded with 0
		/// </returns>
		public static string ToHex(int _value)
		{
			return ToHex(_value, 8);
		}
		
		
		private static string ToHex(long _value, int digits)
		{
			StringBuilder result = new StringBuilder(digits);
			for (int j = 0; j < digits; j++)
			{
				result.Append(_hexcodes[(int) ((_value >> _shifts[j + (8 - digits)]) & 15)]);
			}
			return result.ToString();
		}
		
		/// <summary> Dumps <code>bytesToDump</code> bytes to an output stream.
		/// 
		/// </summary>
		/// <param name="in">         The stream to read from
		/// </param>
		/// <param name="out">        The output stream
		/// </param>
		/// <param name="start">      The index to use as the starting position for the left hand side label
		/// </param>
		/// <param name="bytesToDump">The number of bytes to output.  Use -1 to read until the end of file.
		/// </param>
		
		public static void Dump(Stream inStream,Stream outStream, int start, int bytesToDump)
		{
			MemoryStream buf = new MemoryStream();
			if (bytesToDump == - 1)
			{
				int c = inStream.ReadByte();
				while (c != - 1)
				{
					buf.WriteByte((byte) c);
					c = inStream.ReadByte();
				}
			}
			else
			{
				int bytesRemaining = bytesToDump;
				while (bytesRemaining-- > 0)
				{
					int c = inStream.ReadByte();
					if (c == - 1)
						break;
					else
						buf.WriteByte((byte) c);
				}
			}
			
			byte[] data = buf.ToArray();

			Dump(data, 0, outStream, start, data.Length);
		}
	}
}