// 
// HexRead.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//   Glen Stampoultzis (glens at apache.org) 
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Text;
using System.Collections;

namespace Poi.Net.Util
{
	
	/// <summary> Utilities to read hex from files.
	/// 
	/// </summary>

	public class HexRead
	{
		/// <summary> This method reads hex data from a filename and returns a byte array.
		/// The file may contain line comments that are preceeded with a # symbol.
		/// 
		/// </summary>
		/// <param name="filename"> The filename to read
		/// </param>
		/// <returns> The bytes read from the file.
		/// </returns>
		/// <throws>  IOException If there was a problem while reading the file.      </throws>
		public static byte[] ReadData(string filename)
		{
			FileInfo file = new FileInfo(filename);
			
			FileStream stream = new FileStream(file.FullName, FileMode.Open, FileAccess.Read);
			try
			{
				return ReadData(stream, - 1);
			}
			finally
			{
				stream.Close();
			}
		}
		
		/// <summary> Same as ReadData(String) except that this method allows you to specify sections within
		/// a file.  Sections are referenced using section headers in the form:
		/// <pre>
		/// [sectioname]
		/// </pre>
		/// 
		/// </summary>
		/// <seealso cref="#ReadData(String)">
		/// </seealso>
		public static byte[] ReadData(string filename, string section)
		{
			FileInfo file = new FileInfo(filename);
			
			FileStream stream = new FileStream(file.FullName, FileMode.Open, FileAccess.Read);
			try
			{
				StringBuilder sectionText = new StringBuilder();
				bool inSection = false;
				int c = stream.ReadByte();
				while (c != - 1)
				{
					switch (c)
					{
						
						case '[': 
							inSection = true;
							break;
						
						case '\n': 
						case '\r': 
							inSection = false;
							sectionText = new StringBuilder();
							break;
						
						case ']': 
							inSection = false;
							if (sectionText.ToString().Equals(section))
								return ReadData(stream, '[');
							sectionText = new StringBuilder();
							break;
						
						default: 
							if (inSection)
								sectionText.Append((char) c);
							break;
						
					}
					c = stream.ReadByte();
				}
			}
			finally
			{
				stream.Close();
			}
			throw new IOException("Section '" + section + "' not found");
		}
		
		static public byte[] ReadData(Stream stream, int eofChar)
		{
			int characterCount = 0;
			byte b = (byte) 0;
			IList bytes = new ArrayList();
			bool done = false;
			while (!done)
			{
				int count = stream.ReadByte();
				char baseChar = 'a';
				if (count == eofChar)
					break;
				switch (count)
				{
					
					case '#': 
						ReadToEOL(stream);
						break;
					
					case '0': 
					case '1': 
					case '2': 
					case '3': 
					case '4': 
					case '5': 
					case '6': 
					case '7': 
					case '8': 
					case '9': 
						b <<= 4;
						b = (byte) (b + (byte) (count - '0'));
						characterCount++;
						if (characterCount == 2)
						{
							bytes.Add((byte) b);
							characterCount = 0;
							b = (byte) 0;
						}
						break;
					
					case 'A': 
					case 'B': 
					case 'C': 
					case 'D': 
					case 'E': 
					case 'F': 
						baseChar = 'A';
						goto case 'a';
					
					case 'a': 
					case 'b': 
					case 'c': 
					case 'd': 
					case 'e': 
					case 'f': 
						b <<= 4;
						b = (byte) (b + (byte) (count + 10 - baseChar));
						characterCount++;
						if (characterCount == 2)
						{
							bytes.Add((byte) b);
							characterCount = 0;
							b = (byte) 0;
						}
						break;
					
					case - 1: 
						done = true;
						break;
					
					default: 
						break;
					
				}
			}
			
			byte[] polished = (byte[]) ((ArrayList)bytes).ToArray(typeof(byte[])); //bytes;
			byte[] rval = new byte[polished.Length];
			for (int j = 0; j < polished.Length; j++)
			{
				rval[j] = (byte) polished[j];
			}
			return rval;
		}
		
		static public byte[] ReadFromString(string data)
		{
			return ReadData(new MemoryStream(UTF8Encoding.UTF8.GetBytes(data)), - 1);
		}
		
		static private void ReadToEOL(Stream stream)
		{
			int c = stream.ReadByte();
			while (c != - 1 && c != '\n' && c != '\r')
			{
				c = stream.ReadByte();
			}
		}
	}
}