// 
// IntList.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
namespace Poi.Net.Util
{
	
	/// <summary> A List of int's; as full an implementation of the java.util.List
	/// interface as possible, with an eye toward minimal creation of
	/// objects
	/// 
	/// the mimicry of List is as follows:
	/// <ul>
	/// <li> if possible, operations designated 'optional' in the List
	/// interface are attempted
	/// <li> wherever the List interface refers to an Object, substitute
	/// int
	/// <li> wherever the List interface refers to a Collection or List,
	/// substitute IntList
	/// </ul>
	/// 
	/// the mimicry is not perfect, however:
	/// <ul>
	/// <li> operations involving Iterators or ListIterators are not
	/// supported
	/// <li> Remove(Object) becomes RemoveValue to distinguish it from
	/// Remove(int index)
	/// <li> subList is not supported
	/// </ul>
	/// 
	/// </summary>

	public class IntList
	{
		/// <summary> Returns true if this list contains no elements.
		/// 
		/// </summary>
		/// <returns> true if this list contains no elements.
		/// </returns>
		virtual public bool Empty
		{
			
			
			get
			{
				return _limit == 0;
			}
			
		}
		private int[] _array;
		private int _limit;
		private const int _default_size = 128;
		
		/// <summary> create an IntList of default size</summary>
		
		public IntList():this(_default_size)
		{
		}
		
		/// <summary> create a copy of an existing IntList
		/// 
		/// </summary>
		/// <param name="list">the existing IntList
		/// </param>
		
		public IntList(IntList list):this(list._array.Length)
		{
			Array.Copy(list._array, 0, _array, 0, _array.Length);
			_limit = list._limit;
		}
		
		/// <summary> create an IntList with a predefined initial size
		/// 
		/// </summary>
		/// <param name="initialCapacity">the size for the internal array
		/// </param>
		
		public IntList(int initialCapacity)
		{
			_array = new int[initialCapacity];
			_limit = 0;
		}
		
		/// <summary> add the specfied value at the specified index
		/// 
		/// </summary>
		/// <param name="index">the index where the new value is to be added
		/// </param>
		/// <param name="value">the new value
		/// 
		/// </param>
		/// <exception cref=""> IndexOutOfBoundsException if the index is out of
		/// range (index < 0 || index > size()).
		/// </exception>
		
		public virtual void Add(int index, int _value)
		{
			if (index > _limit)
			{
				throw new IndexOutOfRangeException();
			}
			else if (index == _limit)
			{
				Add(_value);
			}
			else
			{
				
				// index < limit -- insert into the middle
				if (_limit == _array.Length)
				{
					GrowArray(_limit * 2);
				}
				Array.Copy(_array, index, _array, index + 1, _limit - index);
				_array[index] = _value;
				_limit++;
			}
		}
		
		/// <summary> Appends the specified element to the end of this list
		/// 
		/// </summary>
		/// <param name="value">element to be appended to this list.
		/// 
		/// </param>
		/// <returns> true (as per the general contract of the Collection.add
		/// method).
		/// </returns>
		
		public virtual bool Add(int _value)
		{
			if (_limit == _array.Length)
			{
				GrowArray(_limit * 2);
			}
			_array[_limit++] = _value;
			return true;
		}
		
		/// <summary> Appends all of the elements in the specified collection to the
		/// end of this list, in the order that they are returned by the
		/// specified collection's iterator.  The behavior of this
		/// operation is unspecified if the specified collection is
		/// modified while the operation is in progress.  (Note that this
		/// will occur if the specified collection is this list, and it's
		/// nonempty.)
		/// 
		/// </summary>
		/// <param name="c">collection whose elements are to be added to this
		/// list.
		/// 
		/// </param>
		/// <returns> true if this list changed as a result of the call.
		/// </returns>
		
		public virtual bool AddAll(IntList c)
		{
			if (c._limit != 0)
			{
				if ((_limit + c._limit) > _array.Length)
				{
					GrowArray(_limit + c._limit);
				}
				Array.Copy(c._array, 0, _array, _limit, c._limit);
				_limit += c._limit;
			}
			return true;
		}
		
		/// <summary> Inserts all of the elements in the specified collection into
		/// this list at the specified position.  Shifts the element
		/// currently at that position (if any) and any subsequent elements
		/// to the right (increases their indices).  The new elements will
		/// appear in this list in the order that they are returned by the
		/// specified collection's iterator.  The behavior of this
		/// operation is unspecified if the specified collection is
		/// modified while the operation is in progress.  (Note that this
		/// will occur if the specified collection is this list, and it's
		/// nonempty.)
		/// 
		/// </summary>
		/// <param name="index">index at which to insert first element from the
		/// specified collection.
		/// </param>
		/// <param name="c">elements to be inserted into this list.
		/// 
		/// </param>
		/// <returns> true if this list changed as a result of the call.
		/// 
		/// </returns>
		/// <exception cref=""> IndexOutOfBoundsException if the index is out of
		/// range (index < 0 || index > size())
		/// </exception>
		
		public virtual bool AddAll(int index, IntList c)
		{
			if (index > _limit)
			{
				throw new IndexOutOfRangeException();
			}
			if (c._limit != 0)
			{
				if ((_limit + c._limit) > _array.Length)
				{
					GrowArray(_limit + c._limit);
				}
				
				// make a hole
				Array.Copy(_array, index, _array, index + c._limit, _limit - index);
				
				// fill it in
				Array.Copy(c._array, 0, _array, index, c._limit);
				_limit += c._limit;
			}
			return true;
		}
		
		/// <summary> Removes all of the elements from this list.  This list will be
		/// empty after this call returns (unless it throws an exception).
		/// </summary>
		
		public virtual void Clear()
		{
			_limit = 0;
		}
		
		/// <summary> Returns true if this list contains the specified element.  More
		/// formally, returns true if and only if this list contains at
		/// least one element e such that o == e
		/// 
		/// </summary>
		/// <param name="o">element whose presence in this list is to be tested.
		/// 
		/// </param>
		/// <returns> true if this list contains the specified element.
		/// </returns>
		
		public virtual bool Contains(int o)
		{
			bool rval = false;
			
			for (int j = 0; !rval && (j < _limit); j++)
			{
				if (_array[j] == o)
				{
					rval = true;
				}
			}
			return rval;
		}
		
		/// <summary> Returns true if this list contains all of the elements of the
		/// specified collection.
		/// 
		/// </summary>
		/// <param name="c">collection to be checked for containment in this list.
		/// 
		/// </param>
		/// <returns> true if this list contains all of the elements of the
		/// specified collection.
		/// </returns>
		
		public virtual bool ContainsAll(IntList c)
		{
			bool rval = true;
			
			if (this != c)
			{
				for (int j = 0; rval && (j < c._limit); j++)
				{
					if (!Contains(c._array[j]))
					{
						rval = false;
					}
				}
			}
			return rval;
		}
		
		/// <summary> Compares the specified object with this list for equality.
		/// Returns true if and only if the specified object is also a
		/// list, both lists have the same size, and all corresponding
		/// pairs of elements in the two lists are equal.  (Two elements e1
		/// and e2 are equal if e1 == e2.)  In other words, two lists are
		/// defined to be equal if they contain the same elements in the
		/// same order.  This definition ensures that the equals method
		/// works properly across different implementations of the List
		/// interface.
		/// 
		/// </summary>
		/// <param name="o">the object to be compared for equality with this list.
		/// 
		/// </param>
		/// <returns> true if the specified object is equal to this list.
		/// </returns>
		
		public  override bool Equals(object o)
		{
			bool rval = this == o;
			
			if (!rval && (o != null) && (o.GetType() == this.GetType()))
			{
				IntList other = (IntList) o;
				
				if (other._limit == _limit)
				{
					
					// assume match
					rval = true;
					for (int j = 0; rval && (j < _limit); j++)
					{
						rval = _array[j] == other._array[j];
					}
				}
			}
			return rval;
		}
		
		/// <summary> Returns the element at the specified position in this list.
		/// 
		/// </summary>
		/// <param name="index">index of element to return.
		/// 
		/// </param>
		/// <returns> the element at the specified position in this list.
		/// 
		/// </returns>
		/// <exception cref=""> IndexOutOfBoundsException if the index is out of
		/// range (index < 0 || index >= size()).
		/// </exception>
		
		public virtual int Get(int index)
		{
			if (index >= _limit)
			{
				throw new IndexOutOfRangeException();
			}
			return _array[index];
		}
		
		/// <summary> Returns the hash code value for this list.  The hash code of a
		/// list is defined to be the result of the following calculation:
		/// 
		/// <code>
		/// hashCode = 1;
		/// Iterator i = list.iterator();
		/// while (i.HasNext()) {
		/// Object obj = i.Next();
		/// hashCode = 31*hashCode + (obj==null ? 0 : obj.hashCode());
		/// }
		/// </code>
		/// 
		/// This ensures that list1.equals(list2) implies that
		/// list1.hashCode()==list2.hashCode() for any two lists, list1 and
		/// list2, as required by the general contract of Object.hashCode.
		/// 
		/// </summary>
		/// <returns> the hash code value for this list.
		/// </returns>
		
		public override int GetHashCode()
		{
			int hash = 0;
			
			for (int j = 0; j < _limit; j++)
			{
				hash = (31 * hash) + _array[j];
			}
			return hash;
		}
		
		/// <summary> Returns the index in this list of the first occurrence of the
		/// specified element, or -1 if this list does not contain this
		/// element.  More formally, returns the lowest index i such that
		/// (o == get(i)), or -1 if there is no such index.
		/// 
		/// </summary>
		/// <param name="o">element to search for.
		/// 
		/// </param>
		/// <returns> the index in this list of the first occurrence of the
		/// specified element, or -1 if this list does not contain
		/// this element.
		/// </returns>
		
		public virtual int IndexOf(int o)
		{
			int rval = 0;
			
			for (; rval < _limit; rval++)
			{
				if (o == _array[rval])
				{
					break;
				}
			}
			if (rval == _limit)
			{
				rval = - 1; // didn't find it
			}
			return rval;
		}
		
		/// <summary> Returns the index in this list of the last occurrence of the
		/// specified element, or -1 if this list does not contain this
		/// element.  More formally, returns the highest index i such that
		/// (o == get(i)), or -1 if there is no such index.
		/// 
		/// </summary>
		/// <param name="o">element to search for.
		/// 
		/// </param>
		/// <returns> the index in this list of the last occurrence of the
		/// specified element, or -1 if this list does not contain
		/// this element.
		/// </returns>
		
		public virtual int LastIndexOf(int o)
		{
			int rval = _limit - 1;
			
			for (; rval >= 0; rval--)
			{
				if (o == _array[rval])
				{
					break;
				}
			}
			return rval;
		}
		
		/// <summary> Removes the element at the specified position in this list.
		/// Shifts any subsequent elements to the left (subtracts one from
		/// their indices).  Returns the element that was Removed from the
		/// list.
		/// 
		/// </summary>
		/// <param name="index">the index of the element to Removed.
		/// 
		/// </param>
		/// <returns> the element previously at the specified position.
		/// 
		/// </returns>
		/// <exception cref=""> IndexOutOfBoundsException if the index is out of
		/// range (index < 0 || index >= size()).
		/// </exception>
		
		public virtual int Remove(int index)
		{
			if (index >= _limit)
			{
				throw new IndexOutOfRangeException();
			}
			int rval = _array[index];
			
			Array.Copy(_array, index + 1, _array, index, _limit - index);
			_limit--;
			return rval;
		}
		
		/// <summary> Removes the first occurrence in this list of the specified
		/// element (optional operation).  If this list does not contain
		/// the element, it is unchanged.  More formally, Removes the
		/// element with the lowest index i such that (o.equals(get(i)))
		/// (if such an element exists).
		/// 
		/// </summary>
		/// <param name="o">element to be Removed from this list, if present.
		/// 
		/// </param>
		/// <returns> true if this list contained the specified element.
		/// </returns>
		
		public virtual bool RemoveValue(int o)
		{
			bool rval = false;
			
			for (int j = 0; !rval && (j < _limit); j++)
			{
				if (o == _array[j])
				{
					Array.Copy(_array, j + 1, _array, j, _limit - j);
					_limit--;
					rval = true;
				}
			}
			return rval;
		}
		
		/// <summary> Removes from this list all the elements that are contained in
		/// the specified collection
		/// 
		/// </summary>
		/// <param name="c">collection that defines which elements will be Removed
		/// from this list.
		/// 
		/// </param>
		/// <returns> true if this list changed as a result of the call.
		/// </returns>
		
		public virtual bool RemoveAll(IntList c)
		{
			bool rval = false;
			
			for (int j = 0; j < c._limit; j++)
			{
				if (RemoveValue(c._array[j]))
				{
					rval = true;
				}
			}
			return rval;
		}
		
		/// <summary> Retains only the elements in this list that are contained in
		/// the specified collection.  In other words, Removes from this
		/// list all the elements that are not contained in the specified
		/// collection.
		/// 
		/// </summary>
		/// <param name="c">collection that defines which elements this set will
		/// retain.
		/// 
		/// </param>
		/// <returns> true if this list changed as a result of the call.
		/// </returns>
		
		public virtual bool RetainAll(IntList c)
		{
			bool rval = false;
			
			for (int j = 0; j < _limit; )
			{
				if (!c.Contains(_array[j]))
				{
					Remove(j);
					rval = true;
				}
				else
				{
					j++;
				}
			}
			return rval;
		}
		
		/// <summary> Replaces the element at the specified position in this list
		/// with the specified element
		/// 
		/// </summary>
		/// <param name="index">index of element to replace.
		/// </param>
		/// <param name="element">element to be stored at the specified position.
		/// 
		/// </param>
		/// <returns> the element previously at the specified position.
		/// 
		/// </returns>
		/// <exception cref=""> IndexOutOfBoundsException if the index is out of
		/// range (index < 0 || index >= size()).
		/// </exception>
		
		public virtual int Set(int index, int element)
		{
			if (index >= _limit)
			{
				throw new IndexOutOfRangeException();
			}
			int rval = _array[index];
			
			_array[index] = element;
			return rval;
		}
		
		/// <summary> Returns the number of elements in this list. If this list
		/// contains more than Integer.MAX_VALUE elements, returns
		/// Integer.MAX_VALUE.
		/// 
		/// </summary>
		/// <returns> the number of elements in this IntList
		/// </returns>
		
		public virtual int Size { 
			get { return _limit; }
		}
		
		/// <summary> Returns an array containing all of the elements in this list in
		/// proper sequence.  Obeys the general contract of the
		/// Collection.ToArray method.
		/// 
		/// </summary>
		/// <returns> an array containing all of the elements in this list in
		/// proper sequence.
		/// </returns>
		
		public virtual int[] ToArray()
		{
			int[] rval = new int[_limit];
			
			Array.Copy(_array, 0, rval, 0, _limit);
			return rval;
		}
		
		/// <summary> Returns an array containing all of the elements in this list in
		/// proper sequence.  Obeys the general contract of the
		/// Collection.ToArray(Object[]) method.
		/// 
		/// </summary>
		/// <param name="a">the array into which the elements of this list are to
		/// be stored, if it is big enough; otherwise, a new array
		/// is allocated for this purpose.
		/// 
		/// </param>
		/// <returns> an array containing the elements of this list.
		/// </returns>
		
		public virtual int[] ToArray(int[] a)
		{
			int[] rval;
			
			if (a.Length == _limit)
			{
				Array.Copy(_array, 0, a, 0, _limit);
				rval = a;
			}
			else
			{
				rval = ToArray();
			}
			return rval;
		}
		
		private void GrowArray(int new_size)
		{
			int size = (new_size == _array.Length)?new_size + 1:new_size;
			int[] new_array = new int[size];
			
			Array.Copy(_array, 0, new_array, 0, _limit);
			_array = new_array;
		}
	} 
}