// 
// IntegerField.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

namespace Poi.Net.Util
{
	
	/// <summary> representation of an integer (32-bit) field at a fixed location
	/// within a byte array
	/// 
	/// </summary>

	public class IntegerField : FixedField
	{
		private int _value;
		
		private int _offset;
		
		/// <summary> construct the IntegerField with its offset into its containing
		/// byte array
		/// 
		/// </summary>
		/// <param name="offset">of the field within its byte array
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is
		/// negative
		/// </exception>
		
		public IntegerField(int offset)
		{
			if (offset < 0)
			{
				throw new IndexOutOfRangeException("negative offset");
			}
			_offset = offset;
		}
		
		/// <summary> construct the IntegerField with its offset into its containing
		/// byte array and initialize its value
		/// 
		/// </summary>
		/// <param name="offset">of the field within its byte array
		/// </param>
		/// <param name="value">the initial value
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is
		/// negative
		/// </exception>
		
		public IntegerField(int offset, int __value):this(offset)
		{
			Set(__value);
		}
		
		/// <summary> Construct the IntegerField with its offset into its containing
		/// byte array and initialize its value from its byte array
		/// 
		/// </summary>
		/// <param name="offset">of the field within its byte array
		/// </param>
		/// <param name="data">the byte array to read the value from
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is not
		/// within the range of 0..(data.length - 1)
		/// </exception>
		
		public IntegerField(int offset, byte[] data):this(offset)
		{
			ReadFromBytes(data);
		}
		
		/// <summary> construct the IntegerField with its offset into its containing
		/// byte array, initialize its value, and write the value to a byte
		/// array
		/// 
		/// </summary>
		/// <param name="offset">of the field within its byte array
		/// </param>
		/// <param name="value">the initial value
		/// </param>
		/// <param name="data">the byte array to write the value to
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is
		/// negative or too large
		/// </exception>
		
		public IntegerField(int offset, int __value, byte[] data):this(offset)
		{
			Set(__value, data);
		}
		
		/// <summary> get the IntegerField's current value
		/// 
		/// </summary>
		/// <returns> current value
		/// </returns>
		
		public virtual int Get()
		{
			return _value;
		}
		
		/// <summary> set the IntegerField's current value
		/// 
		/// </summary>
		/// <param name="value">to be set
		/// </param>
		
		public virtual void Set(int __value)
		{
			_value = __value;
		}
		
		/// <summary> set the IntegerField's current value and write it to a byte
		/// array
		/// 
		/// </summary>
		/// <param name="value">to be set
		/// </param>
		/// <param name="data">the byte array to write the value to
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is too
		/// large
		/// </exception>
		
		public virtual void Set(int __value, byte[] data)
		{
			_value = __value;
			WriteToBytes(data);
		}
		
		/* ********** START implementation of FixedField ********** */
		
		/// <summary> set the value from its offset into an array of bytes
		/// 
		/// </summary>
		/// <param name="data">the byte array from which the value is to be read
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is too
		/// large
		/// </exception>
		
		public virtual void ReadFromBytes(byte[] data)
		{
			_value = LittleEndian.GetInt(data, _offset);
		}
		
		/// <summary> set the value from an InputStream
		/// 
		/// </summary>
		/// <param name="stream">the InputStream from which the value is to be
		/// read
		/// 
		/// </param>
		/// <exception cref=""> BufferUnderrunException if there is not enough data
		/// available from the InputStream
		/// </exception>
		/// <exception cref=""> IOException if an IOException is thrown from reading
		/// the InputStream
		/// </exception>
		
		public virtual void ReadFromStream(Stream stream)
		{
			_value = LittleEndian.ReadInt(stream);
		}
		
		/// <summary> write the value out to an array of bytes at the appropriate
		/// offset
		/// 
		/// </summary>
		/// <param name="data">the array of bytes to which the value is to be
		/// written
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException if the offset is too
		/// large
		/// </exception>
		
		public virtual void WriteToBytes(byte[] data)
		{
			LittleEndian.PutInt(data, _offset, _value);
		}
		
		/// <summary> return the value as a String
		/// 
		/// </summary>
		/// <returns> the value as a String
		/// </returns>
		
		public override string ToString()
		{
			return Convert.ToString(_value);
		}
		
		/* **********  END  implementation of FixedField ********** */
	} 
}