// 
// LittleEndian.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//   Andrew Oliver (acoliver at apache dot org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

namespace Poi.Net.Util
{
	
	/// <summary>  a utility class for handling little-endian numbers, which the 80x86 world is
	/// replete with. The methods are all static, and input/output is from/to byte
	/// arrays, or from InputStreams.
	/// 
	/// </summary>

	public class LittleEndian : LittleEndianConsts
	{
		
		// all methods are static, so an accessible constructor makes no
		// sense
		/// <summary>  Constructor for the LittleEndian object</summary>
		private LittleEndian()
		{
		}
		
		
		/// <summary>  get a short value from a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <returns>         the short (16-bit) value
		/// </returns>
		
		public static short GetShort(byte[] data, int offset)
		{
			return (short) GetNumber(data, offset, LittleEndian.SHORT_SIZE);
		}
		
		
		/// <summary>  get an unsigned short value from a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <returns>         the unsigned short (16-bit) value in an integer
		/// </returns>
		public static int GetUShort(byte[] data, int offset)
		{
			short num = (short) GetNumber(data, offset, LittleEndian.SHORT_SIZE);
			int retNum;
			if (num < 0)
			{
				retNum = ((int) Int16.MaxValue + 1) * 2 + (int) num;
			}
			else
			{
				retNum = (int) num;
			}
			return retNum;
		}
		
		
		/// <summary>  get a short array from a byte array.
		/// 
		/// </summary>
		/// <param name="data">   Description of the Parameter
		/// </param>
		/// <param name="offset"> Description of the Parameter
		/// </param>
		/// <param name="size">   Description of the Parameter
		/// </param>
		/// <returns>         The simpleShortArray value
		/// </returns>
		public static short[] GetSimpleShortArray(byte[] data, int offset, int size)
		{
			short[] results = new short[size];
			for (int i = 0; i < size; i++)
			{
				results[i] = GetShort(data, offset + 2 + (i * 2));
			}
			return results;
		}
		
		
		/// <summary>  get a short array from a byte array. The short array is assumed to start
		/// with a word describing the length of the array.
		/// 
		/// </summary>
		/// <param name="data">   Description of the Parameter
		/// </param>
		/// <param name="offset"> Description of the Parameter
		/// </param>
		/// <returns>         The shortArray value
		/// </returns>
		public static short[] GetShortArray(byte[] data, int offset)
		{
			int size = (short) GetNumber(data, offset, LittleEndian.SHORT_SIZE);
			short[] results = GetSimpleShortArray(data, offset, size);
			return results;
		}
		
		
		/// <summary>  get a short value from the beginning of a byte array
		/// 
		/// </summary>
		/// <param name="data"> the byte array
		/// </param>
		/// <returns>       the short (16-bit) value
		/// </returns>
		
		public static short GetShort(byte[] data)
		{
			return GetShort(data, 0);
		}
		
		
		/// <summary>  get an unsigned short value from the beginning of a byte array
		/// 
		/// </summary>
		/// <param name="data"> the byte array
		/// </param>
		/// <returns>       the unsigned short (16-bit) value in an int
		/// </returns>
		public static int GetUShort(byte[] data)
		{
			return GetUShort(data, 0);
		}
		
		
		/// <summary>  get an int value from a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <returns>         the int (32-bit) value
		/// </returns>
		
		public static int GetInt(byte[] data, int offset)
		{
			return (int) GetNumber(data, offset, LittleEndian.INT_SIZE);
		}
		
		
		/// <summary>  get an int value from the beginning of a byte array
		/// 
		/// </summary>
		/// <param name="data"> the byte array
		/// </param>
		/// <returns>       the int (32-bit) value
		/// </returns>
		
		public static int GetInt(byte[] data)
		{
			return GetInt(data, 0);
		}
		
		
		/// <summary>  get an unsigned int value from a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <returns>         the unsigned int (32-bit) value in a long
		/// </returns>
		public static long GetUInt(byte[] data, int offset)
		{
			int num = (int) GetNumber(data, offset, LittleEndian.INT_SIZE);
			long retNum;
			if (num < 0)
			{
				retNum = ((long) int.MaxValue + 1) * 2 + (long) num;
			}
			else
			{
				retNum = (int) num;
			}
			return retNum;
		}
		
		/// <summary>  get an unsigned int value from a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <returns>         the unsigned int (32-bit) value in a long
		/// </returns>
		public static long GetUInt(byte[] data)
		{
			return GetUInt(data, 0);
		}
		
		/// <summary>  get a long value from a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <returns>         the long (64-bit) value
		/// </returns>
		
		public static ulong GetLong(byte[] data, int offset)
		{
			return GetNumber(data, offset, LittleEndian.LONG_SIZE, true);
		}
		
		
		/// <summary>  get a long value from the beginning of a byte array
		/// 
		/// </summary>
		/// <param name="data"> the byte array
		/// </param>
		/// <returns>       the long (64-bit) value
		/// </returns>
		
		public static ulong GetLong(byte[] data)
		{
			return GetLong(data, 0);
		}
		
		
		/// <summary>  get a double value from a byte array, reads it in little endian format
		/// then converts the resulting revolting IEEE 754 (curse them) floating
		/// point number to a happy java double
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <returns>         the double (64-bit) value
		/// </returns>
		
		public static double GetDouble(byte[] data, int offset)
		{
			
			return Convert.ToDouble(GetNumber(data, offset, LittleEndian.DOUBLE_SIZE));
		}
		
		
		/// <summary>  get a double value from the beginning of a byte array
		/// 
		/// </summary>
		/// <param name="data"> the byte array
		/// </param>
		/// <returns>       the double (64-bit) value
		/// </returns>
		
		public static double GetDouble(byte[] data)
		{
			return GetDouble(data, 0);
		}
		
		
		/// <summary>  put a short value into a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <param name="value">  the short (16-bit) value
		/// </param>
		public static void PutShort(byte[] data, int offset, short _value)
		{
			PutNumber(data, offset, Convert.ToInt64(_value), LittleEndian.SHORT_SIZE);
		}
		
		
		/// <summary>  put a array of shorts into a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <param name="value">  the short array
		/// </param>
		public static void PutShortArray(byte[] data, int offset, short[] _value)
		{
			PutNumber(data, offset, Convert.ToInt64(_value.Length), LittleEndian.SHORT_SIZE);
			for (int i = 0; i < _value.Length; i++)
			{
				PutNumber(data, offset + 2 + (i * 2), Convert.ToInt64(_value[i]), LittleEndian.SHORT_SIZE);
			}
		}
		
		/// <summary> put an unsigned short value into a byte array
		/// 
		/// </summary>
		/// <param name="data">the byte array
		/// </param>
		/// <param name="offset">a starting offset into the byte array
		/// </param>
		/// <param name="value">the short (16-bit) value
		/// 
		/// </param>
		/// <exception cref=""> ArrayIndexOutOfBoundsException may be thrown
		/// </exception>
		public static void PutUShort(byte[] data, int offset, int _value)
		{
			PutNumber(data, offset, Convert.ToInt64(_value), LittleEndian.SHORT_SIZE);
		}
		
		/// <summary>  put a short value into beginning of a byte array
		/// 
		/// </summary>
		/// <param name="data">  the byte array
		/// </param>
		/// <param name="value"> the short (16-bit) value
		/// </param>
		
		public static void PutShort(byte[] data, short _value)
		{
			PutShort(data, 0, _value);
		}
		
		
		/// <summary>  put an int value into a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <param name="value">  the int (32-bit) value
		/// </param>
		
		public static void PutInt(byte[] data, int offset, int _value)
		{
			PutNumber(data, offset, Convert.ToInt64(_value), LittleEndian.INT_SIZE);
		}
		
		
		/// <summary>  put an int value into beginning of a byte array
		/// 
		/// </summary>
		/// <param name="data">  the byte array
		/// </param>
		/// <param name="value"> the int (32-bit) value
		/// </param>
		
		public static void PutInt(byte[] data, int _value)
		{
			PutInt(data, 0, _value);
		}
		
		
		/// <summary>  put a long value into a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <param name="value">  the long (64-bit) value
		/// </param>
		
		public static void PutLong(byte[] data, int offset, long _value)
		{
			PutNumber(data, offset, _value, LittleEndian.LONG_SIZE);
		}
				
		public static void PutLong(byte[] data, int offset, ulong _value)
		{
			PutNumber(data, offset, _value, LittleEndian.LONG_SIZE);
		}
		
		
		/// <summary>  put a long value into beginning of a byte array
		/// 
		/// </summary>
		/// <param name="data">  the byte array
		/// </param>
		/// <param name="value"> the long (64-bit) value
		/// </param>
		
		public static void PutLong(byte[] data, long _value)
		{
			PutLong(data, 0, _value);
		}

		public static void PutLong(byte[] data, ulong _value)
		{
			PutLong(data, 0, _value);
		}
		
		
		/// <summary>  put a double value into a byte array
		/// 
		/// </summary>
		/// <param name="data">   the byte array
		/// </param>
		/// <param name="offset"> a starting offset into the byte array
		/// </param>
		/// <param name="value">  the double (64-bit) value
		/// </param>
		
		public static void PutDouble(byte[] data, int offset, double _value)
		{
			// Excel likes NaN to be a specific value.
			if (double.IsNaN(_value))
				PutNumber(data, offset, - 276939487313920L, LittleEndian.DOUBLE_SIZE);
			else
			{
				
				PutNumber(data, offset, Convert.ToInt64(_value), LittleEndian.DOUBLE_SIZE);
			}
		}
		
		
		/// <summary>  put a double value into beginning of a byte array
		/// 
		/// </summary>
		/// <param name="data">  the byte array
		/// </param>
		/// <param name="value"> the double (64-bit) value
		/// </param>
		
		public static void PutDouble(byte[] data, double _value)
		{
			PutDouble(data, 0, _value);
		}
		
		
		/// <summary>  Exception to handle buffer underruns
		/// 
		/// </summary>
		/// <author>      Marc Johnson (mjohnson at apache dot org)
		/// </author>
		
		[Serializable]
		public class BufferUnderrunException:IOException
		{
			
			/// <summary>  simple constructor</summary>
			
			internal BufferUnderrunException():base("buffer underrun")
			{
			}
		}
		
		
		/// <summary>  get a short value from an InputStream
		/// 
		/// </summary>
		/// <param name="stream">                      the InputStream from which the short
		/// is to be read
		/// </param>
		/// <returns>                              the short (16-bit) value
		/// </returns>
		/// <exception cref="">  IOException              will be propagated back to the caller
		/// </exception>
		/// <exception cref="">  BufferUnderrunException  if the stream cannot provide enough
		/// bytes
		/// </exception>
		
		public static short ReadShort(Stream stream)
		{
			return GetShort(ReadFromStream(stream, LittleEndian.SHORT_SIZE));
		}
		
		
		/// <summary>  get an int value from an InputStream
		/// 
		/// </summary>
		/// <param name="stream">                      the InputStream from which the int is
		/// to be read
		/// </param>
		/// <returns>                              the int (32-bit) value
		/// </returns>
		/// <exception cref="">  IOException              will be propagated back to the caller
		/// </exception>
		/// <exception cref="">  BufferUnderrunException  if the stream cannot provide enough
		/// bytes
		/// </exception>
		
		public static int ReadInt(Stream stream)
		{
			return GetInt(ReadFromStream(stream, LittleEndian.INT_SIZE));
		}
		
		
		/// <summary>  get a long value from an InputStream
		/// 
		/// </summary>
		/// <param name="stream">                      the InputStream from which the long
		/// is to be read
		/// </param>
		/// <returns>                              the long (64-bit) value
		/// </returns>
		/// <exception cref="">  IOException              will be propagated back to the caller
		/// </exception>
		/// <exception cref="">  BufferUnderrunException  if the stream cannot provide enough
		/// bytes
		/// </exception>
		
		public static ulong ReadLong(Stream stream)
		{
			return GetLong(ReadFromStream(stream, LittleEndian.LONG_SIZE));
		}
		
		/// <summary>  Read the appropriate number of bytes from the stream and return them to
		/// the caller. 
		/// 
		/// However, for the purposes of the POI project, this risk is deemed
		/// negligible. It is, however, so noted.
		/// 
		/// </summary>
		/// <param name="stream">                      the InputStream we're reading from
		/// </param>
		/// <param name="size">                        the number of bytes to read; in
		/// 99.99% of cases, this will be SHORT_SIZE, INT_SIZE, or LONG_SIZE --
		/// but it doesn't have to be.
		/// </param>
		/// <returns>                              the byte array containing the
		/// required number of bytes. The array will contain all zero's on end
		/// of stream
		/// </returns>
		/// <exception cref="">  IOException              will be propagated back to the caller
		/// </exception>
		/// <exception cref="">  BufferUnderrunException  if the stream cannot provide enough
		/// bytes
		/// </exception>
		
		public static byte[] ReadFromStream(Stream stream, int size)
		{
			byte[] buffer = new byte[size];
			
			int count = stream.Read(buffer, 0, buffer.Length);

			if (count == 0)
			{
				
				// return a zero-filled buffer
				// Fill
				for (int i = 0; i < buffer.Length; i++)
					buffer[i] = (byte) 0;
			}
			else if (count != size)
			{
				throw new BufferUnderrunException();
			}
			return buffer;
		}
		
		
		/// <summary>  Gets the number attribute of the LittleEndian class
		/// 
		/// </summary>
		/// <param name="data">   Description of the Parameter
		/// </param>
		/// <param name="offset"> Description of the Parameter
		/// </param>
		/// <param name="size">   Description of the Parameter
		/// </param>
		/// <returns>         The number value
		/// </returns>
		/// 

		private static ulong GetNumber(byte[] data, int offset, int size, bool ul)
		{
			ulong result = 0;

			for (int j = offset + size - 1; j >= offset; j--)
			{
				result <<= 8;
				result |= (ulong) 0xff & data[j];
			}
			return result;
		}	
		

		private static long GetNumber(byte[] data, int offset, int size)
		{
	
			long result = 0;
			
			for (int j = offset + size - 1; j >= offset; j--)
			{
				
				result <<= 8;
				result |= 0xff & data[j];
			}
			return result;
		}
		
		
		/// <summary>  Description of the Method
		/// 
		/// </summary>
		/// <param name="data">   Description of the Parameter
		/// </param>
		/// <param name="offset"> Description of the Parameter
		/// </param>
		/// <param name="value">  Description of the Parameter
		/// </param>
		/// <param name="size">   Description of the Parameter
		/// </param>
		private static void PutNumber(byte[] data, int offset, ulong _value, int size)
		{
			int limit = size + offset;
			ulong v = _value;
			
			for (int j = offset; j < limit; j++)
			{
				data[j] = (byte) (v & 0xFF);
				v >>= 8;
			}
		}
		private static void PutNumber(byte[] data, int offset, long _value, int size)
		{
			int limit = size + offset;
			long v = _value;
			
			for (int j = offset; j < limit; j++)
			{
				data[j] = (byte) (v & 0xFF);
				v >>= 8;
			}
		}
		
		
		/// <summary>  Convert an 'unsigned' byte to an integer. ie, don't carry across the
		/// sign.
		/// 
		/// </summary>
		/// <param name="b"> Description of the Parameter
		/// </param>
		/// <returns>    Description of the Return Value
		/// </returns>
		public static int UByteToInt(byte b)
		{
			return ((b & 0x80) == 0?(int) b:(int) (b & (byte) 0x7f) + 0x80);
		}
		
		
		/// <summary>  get the unsigned value of a byte.
		/// 
		/// </summary>
		/// <param name="data">   the byte array.
		/// </param>
		/// <param name="offset"> a starting offset into the byte array.
		/// </param>
		/// <returns>         the unsigned value of the byte as a 32 bit integer
		/// </returns>
		public static int GetUnsignedByte(byte[] data, int offset)
		{
			return (int) GetNumber(data, offset, LittleEndian.BYTE_SIZE);
		}
		
		
		/// <summary>  get the unsigned value of a byte.
		/// 
		/// </summary>
		/// <param name="data"> the byte array
		/// </param>
		/// <returns>       the unsigned value of the byte as a 32 bit integer
		/// </returns>
		public static int GetUnsignedByte(byte[] data)
		{
			return GetUnsignedByte(data, 0);
		}
		
		
		/// <summary>  Copy a portion of a byte array
		/// 
		/// </summary>
		/// <param name="data">                       the original byte array
		/// </param>
		/// <param name="offset">                     Where to start copying from.
		/// </param>
		/// <param name="size">                       Number of bytes to copy.
		/// </param>
		/// <returns>                             The byteArray value
		/// </returns>
		/// <throws>   IndexOutOfBoundsException  - if copying would cause access of </throws>
		/// <summary>      data outside array bounds.
		/// </summary>
		public static byte[] GetByteArray(byte[] data, int offset, int size)
		{
			byte[] copy = new byte[size];
			Array.Copy(data, offset, copy, 0, size);
			
			return copy;
		}
	}
}