// 
// POILogger.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Marc Johnson (mjohnson at apache dot org)
//   Glen Stampoultzis (glens at apache.org)
//   Nicola Ken Barozzi (nicolaken at apache.org)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.Text;
using System.Collections;

namespace Poi.Net.Util
{
	
	/// <summary> A Logger interface that strives to make it as easy as possible for
	/// developers to write Log calls, while simultaneously making those
	/// calls as cheap as possible by performing lazy evaluation of the Log
	/// message.
	/// 
	/// </summary>
	
	public abstract class POILogger
	{
		
		public const int DEBUG = 1;
		public const int INFO = 3;
		public const int WARN = 5;
		public const int ERROR = 7;
		public const int FATAL = 9;
		
		/// <summary> package scope so it cannot be instantiated outside of the util
		/// package. You need a POILogger? Go to the POILogFactory for one
		/// 
		/// </summary>
		internal POILogger()
		{
		}
		
		abstract public void Initialize(string cat);
		
		abstract public void Log(int level, object obj1);
		
		/// <summary> Check if a Logger is enabled to Log at the specified level
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		abstract public bool Check(int level);
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first object to place in the message
		/// </param>
		/// <param name="obj2">second object to place in the message
		/// </param>
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first object to place in the message
		/// </param>
		/// <param name="obj2">second object to place in the message
		/// </param>
		
		public virtual void Log(int level, object obj1, object obj2)
		{
			if (Check(level))
			{
				Log(level, new StringBuilder(32).Append(obj1).Append(obj2));
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third Object to place in the message
		/// </param>
		
		public virtual void Log(int level, object obj1, object obj2, object obj3)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(48).Append(obj1).Append(obj2).Append(obj3));
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third Object to place in the message
		/// </param>
		/// <param name="obj4">fourth Object to place in the message
		/// </param>
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(64).Append(obj1).Append(obj2).Append(obj3).Append(obj4));
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third Object to place in the message
		/// </param>
		/// <param name="obj4">fourth Object to place in the message
		/// </param>
		/// <param name="obj5">fifth Object to place in the message
		/// </param>
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4, object obj5)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(80).Append(obj1).Append(obj2).Append(obj3).Append(obj4).Append(obj5));
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third Object to place in the message
		/// </param>
		/// <param name="obj4">fourth Object to place in the message
		/// </param>
		/// <param name="obj5">fifth Object to place in the message
		/// </param>
		/// <param name="obj6">sixth Object to place in the message
		/// </param>
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4, object obj5, object obj6)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(96).Append(obj1).Append(obj2).Append(obj3).Append(obj4).Append(obj5).Append(obj6));
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third Object to place in the message
		/// </param>
		/// <param name="obj4">fourth Object to place in the message
		/// </param>
		/// <param name="obj5">fifth Object to place in the message
		/// </param>
		/// <param name="obj6">sixth Object to place in the message
		/// </param>
		/// <param name="obj7">seventh Object to place in the message
		/// </param>
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4, object obj5, object obj6, object obj7)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(112).Append(obj1).Append(obj2).Append(obj3).Append(obj4).Append(obj5).Append(obj6).Append(obj7));
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third Object to place in the message
		/// </param>
		/// <param name="obj4">fourth Object to place in the message
		/// </param>
		/// <param name="obj5">fifth Object to place in the message
		/// </param>
		/// <param name="obj6">sixth Object to place in the message
		/// </param>
		/// <param name="obj7">seventh Object to place in the message
		/// </param>
		/// <param name="obj8">eighth Object to place in the message
		/// </param>
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4, object obj5, object obj6, object obj7, object obj8)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(128).Append(obj1).Append(obj2).Append(obj3).Append(obj4).Append(obj5).Append(obj6).Append(obj7).Append(obj8));
			}
		}
		
		/// <summary> Log a message
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">The object to Log.  This is converted to a string.
		/// </param>
		/// <param name="exception">An exception to be Logged
		/// </param>
		
		
		public virtual void Log(int level, object obj1, Exception exception)
		{
			Log(level, obj1, exception);
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="exception">An exception to be Logged
		/// </param>
		
		
		public virtual void Log(int level, object obj1, object obj2, Exception exception)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(32).Append(obj1).Append(obj2), exception);
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third object to place in the message
		/// </param>
		/// <param name="exception">An error message to be Logged
		/// </param>
		
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, Exception exception)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(48).Append(obj1).Append(obj2).Append(obj3), exception);
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third object to place in the message
		/// </param>
		/// <param name="obj4">fourth object to place in the message
		/// </param>
		/// <param name="exception">An exception to be Logged
		/// </param>
		
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4, Exception exception)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(64).Append(obj1).Append(obj2).Append(obj3).Append(obj4), exception);
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third object to place in the message
		/// </param>
		/// <param name="obj4">fourth object to place in the message
		/// </param>
		/// <param name="obj5">fifth object to place in the message
		/// </param>
		/// <param name="exception">An exception to be Logged
		/// </param>
		
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4, object obj5, Exception exception)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(80).Append(obj1).Append(obj2).Append(obj3).Append(obj4).Append(obj5), exception);
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third object to place in the message
		/// </param>
		/// <param name="obj4">fourth object to place in the message
		/// </param>
		/// <param name="obj5">fifth object to place in the message
		/// </param>
		/// <param name="obj6">sixth object to place in the message
		/// </param>
		/// <param name="exception">An exception to be Logged
		/// </param>
		
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4, object obj5, object obj6, Exception exception)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(96).Append(obj1).Append(obj2).Append(obj3).Append(obj4).Append(obj5).Append(obj6), exception);
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third object to place in the message
		/// </param>
		/// <param name="obj4">fourth object to place in the message
		/// </param>
		/// <param name="obj5">fifth object to place in the message
		/// </param>
		/// <param name="obj6">sixth object to place in the message
		/// </param>
		/// <param name="obj7">seventh object to place in the message
		/// </param>
		/// <param name="exception">An exception to be Logged
		/// </param>
		
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4, object obj5, object obj6, object obj7, Exception exception)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(112).Append(obj1).Append(obj2).Append(obj3).Append(obj4).Append(obj5).Append(obj6).Append(obj7), exception);
			}
		}
		
		/// <summary> Log a message. Lazily appends Object parameters together.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="obj1">first Object to place in the message
		/// </param>
		/// <param name="obj2">second Object to place in the message
		/// </param>
		/// <param name="obj3">third object to place in the message
		/// </param>
		/// <param name="obj4">fourth object to place in the message
		/// </param>
		/// <param name="obj5">fifth object to place in the message
		/// </param>
		/// <param name="obj6">sixth object to place in the message
		/// </param>
		/// <param name="obj7">seventh object to place in the message
		/// </param>
		/// <param name="obj8">eighth object to place in the message
		/// </param>
		/// <param name="exception">An exception to be Logged
		/// </param>
		
		
		public virtual void Log(int level, object obj1, object obj2, object obj3, object obj4, object obj5, object obj6, object obj7, object obj8, Exception exception)
		{
			
			
			if (Check(level))
			{
				Log(level, new StringBuilder(128).Append(obj1).Append(obj2).Append(obj3).Append(obj4).Append(obj5).Append(obj6).Append(obj7).Append(obj8), exception);
			}
		}
		
		/// <summary> Logs a formated message. The message itself may contain %
		/// characters as place holders. This routine will attempt to match
		/// the placeholder by looking at the type of parameter passed to
		/// obj1.
		/// 
		/// If the parameter is an array, it traverses the array first and
		/// matches parameters sequentially against the array items.
		/// Otherwise the parameters after <code>message</code> are matched
		/// in order.
		/// 
		/// If the place holder matches against a number it is printed as a
		/// whole number. This can be overridden by specifying a precision
		/// in the form %n.m where n is the padding for the whole part and
		/// m is the number of decimal places to display. n can be excluded
		/// if desired. n and m may not be more than 9.
		/// 
		/// If the last parameter (after flattening) is a Throwable it is
		/// Logged specially.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="message">The message to Log.
		/// </param>
		/// <param name="obj1">The first object to match against.
		/// </param>
		
		public virtual void LogFormatted(int level, string message, object obj1)
		{
			CommonLogFormatted(level, message, new object[]{obj1});
		}
		
		/// <summary> Logs a formated message. The message itself may contain %
		/// characters as place holders. This routine will attempt to match
		/// the placeholder by looking at the type of parameter passed to
		/// obj1.
		/// 
		/// If the parameter is an array, it traverses the array first and
		/// matches parameters sequentially against the array items.
		/// Otherwise the parameters after <code>message</code> are matched
		/// in order.
		/// 
		/// If the place holder matches against a number it is printed as a
		/// whole number. This can be overridden by specifying a precision
		/// in the form %n.m where n is the padding for the whole part and
		/// m is the number of decimal places to display. n can be excluded
		/// if desired. n and m may not be more than 9.
		/// 
		/// If the last parameter (after flattening) is a Throwable it is
		/// Logged specially.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="message">The message to Log.
		/// </param>
		/// <param name="obj1">The first object to match against.
		/// </param>
		/// <param name="obj2">The second object to match against.
		/// </param>
		
		public virtual void LogFormatted(int level, string message, object obj1, object obj2)
		{
			CommonLogFormatted(level, message, new object[]{obj1, obj2});
		}
		
		/// <summary> Logs a formated message. The message itself may contain %
		/// characters as place holders. This routine will attempt to match
		/// the placeholder by looking at the type of parameter passed to
		/// obj1.
		/// 
		/// If the parameter is an array, it traverses the array first and
		/// matches parameters sequentially against the array items.
		/// Otherwise the parameters after <code>message</code> are matched
		/// in order.
		/// 
		/// If the place holder matches against a number it is printed as a
		/// whole number. This can be overridden by specifying a precision
		/// in the form %n.m where n is the padding for the whole part and
		/// m is the number of decimal places to display. n can be excluded
		/// if desired. n and m may not be more than 9.
		/// 
		/// If the last parameter (after flattening) is a Throwable it is
		/// Logged specially.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="message">The message to Log.
		/// </param>
		/// <param name="obj1">The first object to match against.
		/// </param>
		/// <param name="obj2">The second object to match against.
		/// </param>
		/// <param name="obj3">The third object to match against.
		/// </param>
		
		public virtual void LogFormatted(int level, string message, object obj1, object obj2, object obj3)
		{
			CommonLogFormatted(level, message, new object[]{obj1, obj2, obj3});
		}
		
		/// <summary> Logs a formated message. The message itself may contain %
		/// characters as place holders. This routine will attempt to match
		/// the placeholder by looking at the type of parameter passed to
		/// obj1.
		/// 
		/// If the parameter is an array, it traverses the array first and
		/// matches parameters sequentially against the array items.
		/// Otherwise the parameters after <code>message</code> are matched
		/// in order.
		/// 
		/// If the place holder matches against a number it is printed as a
		/// whole number. This can be overridden by specifying a precision
		/// in the form %n.m where n is the padding for the whole part and
		/// m is the number of decimal places to display. n can be excluded
		/// if desired. n and m may not be more than 9.
		/// 
		/// If the last parameter (after flattening) is a Throwable it is
		/// Logged specially.
		/// 
		/// </summary>
		/// <param name="level">One of DEBUG, INFO, WARN, ERROR, FATAL
		/// </param>
		/// <param name="message">The message to Log.
		/// </param>
		/// <param name="obj1">The first object to match against.
		/// </param>
		/// <param name="obj2">The second object to match against.
		/// </param>
		/// <param name="obj3">The third object to match against.
		/// </param>
		/// <param name="obj4">The forth object to match against.
		/// </param>
		
		public virtual void LogFormatted(int level, string message, object obj1, object obj2, object obj3, object obj4)
		{
			CommonLogFormatted(level, message, new object[]{obj1, obj2, obj3, obj4});
		}
		
		private void CommonLogFormatted(int level, string message, object[] unflatParams)
		{
			
			
			if (Check(level))
			{
				object[] parameters = FlattenArrays(unflatParams);
				
				
				if (parameters[parameters.Length - 1] is Exception)
				{
					
					Log(level, StringUtil.format(message, parameters), (Exception) parameters[parameters.Length - 1]);
				}
				else
				{
					Log(level, StringUtil.format(message, parameters));
				}
			}
		}
		
		/// <summary> Flattens any contained objects. Only tranverses one level deep.</summary>
		
		private object[] FlattenArrays(object[] objects)
		{
			IList results = new ArrayList();
			
			for (int i = 0; i < objects.Length; i++)
				((ArrayList)results).AddRange(ObjectToObjectArray(objects[i]));
			
			object[] temp = new object[results.Count];
			results.CopyTo(temp,0);
			return (object[]) temp;
		}
		
		private IList ObjectToObjectArray(object obj)
		{
			IList results = new ArrayList();
			
			if (obj is byte[])
			{
				byte[] array = (byte[]) obj;
				
				for (int j = 0; j < array.Length; j++)
				{
					results.Add((byte) array[j]);
				}
			}
			if (obj is char[])
			{
				char[] array = (char[]) obj;
				
				for (int j = 0; j < array.Length; j++)
				{
					results.Add(array[j]);
				}
			}
			else if (obj is short[])
			{
				short[] array = (short[]) obj;
				
				for (int j = 0; j < array.Length; j++)
				{
					results.Add((short) array[j]);
				}
			}
			else if (obj is int[])
			{
				int[] array = (int[]) obj;
				
				for (int j = 0; j < array.Length; j++)
				{
					results.Add((int) array[j]);
				}
			}
			else if (obj is long[])
			{
				long[] array = (long[]) obj;
				
				for (int j = 0; j < array.Length; j++)
				{
					results.Add((long) array[j]);
				}
			}
			else if (obj is float[])
			{
				float[] array = (float[]) obj;
				
				for (int j = 0; j < array.Length; j++)
				{
					results.Add((float) array[j]);
				}
			}
			else if (obj is double[])
			{
				double[] array = (double[]) obj;
				
				for (int j = 0; j < array.Length; j++)
				{
					results.Add((double) array[j]);
				}
			}
			else if (obj is object[])
			{
				object[] array = (object[]) obj;
				
				for (int j = 0; j < array.Length; j++)
				{
					results.Add(array[j]);
				}
			}
			else
			{
				results.Add(obj);
			}
			return results;
		}
	} 
}