// 
// StringUtil.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Andrew C. Oliver
//   Sergei Kozello (sergeikozello at mail.ru)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Text;

namespace Poi.Net.Util
{
	
	/// <summary>  Title: String Utility Description: Collection of string handling utilities
	/// 
	/// Now it is quite confusing: the method pairs, in which
	/// one of them write data and other read written data are:
	/// putUncompressedUnicodeHigh and GetFromUnicode
	/// putUncompressedUnicode     and GetFromUnicodeHigh
	/// 
	/// </summary>
	
	public class StringUtil
	{
		/// <returns> the encoding we want to use (ISO-8859-1)
		/// </returns>
		public static string PreferredEncoding {
			get {
				return ENCODING;
			}
			
		}
		
		private const string ENCODING = "ISO-8859-1";
		/// <summary>  Constructor for the StringUtil object</summary>
		private StringUtil()
		{
		}
		
		
		/// <summary>  given a byte array of 16-bit unicode characters, compress to 8-bit and
		/// return a string
		/// 
		/// { 0x16, 0x00 } -> 0x16
		/// 
		/// </summary>
		/// <param name="string">                             the byte array to be converted
		/// </param>
		/// <param name="offset">                             the initial offset into the
		/// byte array. it is assumed that string[ offset ] and string[ offset +
		/// 1 ] contain the first 16-bit unicode character
		/// </param>
		/// <param name="">len
		/// </param>
		/// <returns>                                     the converted string
		/// </returns>
		/// <exception cref="">  ArrayIndexOutOfBoundsException  if offset is out of bounds for
		/// the byte array (i.e., is negative or is greater than or equal to
		/// string.length)
		/// </exception>
		/// <exception cref="">  IllegalArgumentException        if len is too large (i.e.,
		/// there is not enough data in string to create a String of that
		/// length)
		/// </exception>
		/// <len>                                         the length of the final string </len>
		
		public static string GetFromUnicodeHigh(byte[] str, int offset, int len)
		{
			
			if ((offset < 0) || (offset >= str.Length))
			{
				throw new IndexOutOfRangeException("Illegal offset");
			}
			if ((len < 0) || (((str.Length - offset) / 2) < len))
			{
				throw new ArgumentException("Illegal length");
			}
			
			char[] chars = new char[len];
			for (int i = 0; i < chars.Length; i++)
			{
				chars[i] = (char) (str[offset + (2 * i)] & 0xFF | (str[offset + (2 * i + 1)] << 8));
			}
			
			return new string(chars);
		}
		
		
		/// <summary>  given a byte array of 16-bit unicode characters, compress to 8-bit and
		/// return a string
		/// 
		/// { 0x16, 0x00 } -> 0x16
		/// 
		/// </summary>
		/// <param name="string"> the byte array to be converted
		/// </param>
		/// <returns>         the converted string
		/// </returns>
		
		public static string GetFromUnicodeHigh(byte[] str)
		{
			return GetFromUnicodeHigh(str, 0, str.Length / 2);
		}
		
		
		/// <summary>  given a byte array of 16-bit unicode characters, compress to 8-bit and
		/// return a string
		/// 
		/// { 0x00, 0x16 } -> 0x16
		/// 
		/// </summary>
		/// <param name="string">                             the byte array to be converted
		/// </param>
		/// <param name="offset">                             the initial offset into the
		/// byte array. it is assumed that string[ offset ] and string[ offset +
		/// 1 ] contain the first 16-bit unicode character
		/// </param>
		/// <param name="">len
		/// </param>
		/// <returns>                                     the converted string
		/// </returns>
		/// <exception cref="">  ArrayIndexOutOfBoundsException  if offset is out of bounds for
		/// the byte array (i.e., is negative or is greater than or equal to
		/// string.length)
		/// </exception>
		/// <exception cref="">  IllegalArgumentException        if len is too large (i.e.,
		/// there is not enough data in string to create a String of that
		/// length)
		/// </exception>
		/// <len>                                         the length of the final string </len>
		
		public static string GetFromUnicode(byte[] str, int offset, int len)
		{
			if ((offset < 0) || (offset >= str.Length))
			{
				throw new IndexOutOfRangeException("Illegal offset");
			}
			if ((len < 0) || (((str.Length - offset) / 2) < len))
			{
				throw new ArgumentException("Illegal length");
			}
			
			
			char[] chars = new char[len];
			for (int i = 0; i < chars.Length; i++)
			{
				chars[i] = (char) ((str[offset + (2 * i)] << 8) + str[offset + (2 * i + 1)]);
			}
			
			return new string(chars);
		}
		
		
		/// <summary>  given a byte array of 16-bit unicode characters, compress to 8-bit and
		/// return a string
		/// 
		/// { 0x00, 0x16 } -> 0x16
		/// 
		/// </summary>
		/// <param name="string"> the byte array to be converted
		/// </param>
		/// <returns>         the converted string
		/// </returns>
		
		public static string GetFromUnicode(byte[] str)
		{
			return GetFromUnicode(str, 0, str.Length / 2);
		}
		
		
		/// <summary> read compressed unicode(8bit)
		/// 
		/// </summary>
		/// <author>  Toshiaki Kamoshida(kamoshida.toshiaki at future dot co dot jp)
		/// 
		/// </author>
		/// <param name="string">byte array to read
		/// </param>
		/// <param name="offset">offset to read byte array
		/// </param>
		/// <param name="len">   length to read byte array
		/// </param>
		/// <returns> String generated String instance by reading byte array
		/// </returns>
		public static string GetFromCompressedUnicode(byte[] str, int offset, int len)
		{
			try
			{
				string tempStr;

				tempStr = Encoding.GetEncoding("ISO-8859-1").GetString(str);
				return new string(tempStr.ToCharArray(), offset, len);
			}
			catch (IOException ignored)
			{
				throw new ApplicationException(); /* unreachable */
			}
		}
		
		/// <summary>  write compressed unicode
		/// 
		/// </summary>
		/// <param name="input">  the String containing the data to be written
		/// </param>
		/// <param name="output"> the byte array to which the data is to be written
		/// </param>
		/// <param name="offset"> an offset into the byte arrat at which the data is start
		/// when written
		/// </param>
		
		public static void PutCompressedUnicode(string input, byte[] output, int offset)
		{
			int strlen = input.Length;
			
			for (int k = 0; k < strlen; k++)
			{
				output[offset + k] = (byte) input[k];
			}
		}
		
		
		/// <summary>  Write uncompressed unicode
		/// 
		/// </summary>
		/// <param name="input">  the String containing the unicode data to be written
		/// </param>
		/// <param name="output"> the byte array to hold the uncompressed unicode
		/// </param>
		/// <param name="offset"> the offset to start writing into the byte array
		/// </param>
		
		public static void PutUncompressedUnicode(string input, byte[] output, int offset)
		{
			int strlen = input.Length;
			
			for (int k = 0; k < strlen; k++)
			{
				char c = input[k];
				
				output[offset + (2 * k)] = (byte) c;
				output[offset + (2 * k) + 1] = (byte) (c >> 8);
			}
		}
		
		/// <summary>  Write uncompressed unicode
		/// 
		/// </summary>
		/// <param name="input">  the String containing the unicode data to be written
		/// </param>
		/// <param name="output"> the byte array to hold the uncompressed unicode
		/// </param>
		/// <param name="offset"> the offset to start writing into the byte array
		/// </param>
		
		public static void PutUncompressedUnicodeHigh(string input, byte[] output, int offset)
		{
			int strlen = input.Length;
			
			for (int k = 0; k < strlen; k++)
			{
				char c = input[k];
				
				output[offset + (2 * k)] = (byte) (c >> 8);
				output[offset + (2 * k)] = (byte) c;
			}
		}
		
		
		
		
		/// <summary>  Description of the Method
		/// 
		/// </summary>
		/// <param name="message"> Description of the Parameter
		/// </param>
		/// <param name="params">  Description of the Parameter
		/// </param>
		/// <returns>          Description of the Return Value
		/// </returns>
		public static string format(string message, object[] parameters)
		{
			int currentParamNumber = 0;
			StringBuilder formattedMessage = new StringBuilder();
			
			for (int i = 0; i < message.Length; i++)
			{
				if (message[i] == '%')
				{
					if (currentParamNumber >= parameters.Length)
					{
						formattedMessage.Append("?missing data?");
					}
					else if ((parameters[currentParamNumber] is ValueType) && (i + 1 < message.Length))
					{
						//BUG
						//i += matchOptionalFormatting((System.ValueType) parameters[currentParamNumber++], message.Substring(i + 1), formattedMessage);
					}
					else
					{
						
						formattedMessage.Append(parameters[currentParamNumber++].ToString());
					}
				}
				else
				{
					if ((message[i] == '\\') && (i + 1 < message.Length) && (message[i + 1] == '%'))
					{
						formattedMessage.Append('%');
						i++;
					}
					else
					{
						formattedMessage.Append(message[i]);
					}
				}
			}
			return formattedMessage.ToString();
		}
		
		
		/// <summary>  Description of the Method
		/// 
		/// </summary>
		/// <param name="number">     Description of the Parameter
		/// </param>
		/// <param name="formatting"> Description of the Parameter
		/// </param>
		/// <param name="outputTo">   Description of the Parameter
		/// </param>
		/// <returns>             Description of the Return Value
		/// </returns>
/*		
		private static int matchOptionalFormatting(System.ValueType number, string formatting, StringBuilder outputTo)
		{
			HelperClass.TextNumberFormat numberFormat = HelperClass.TextNumberFormat.getTextNumberInstance();
			
			if ((0 < formatting.Length) && System.Char.IsDigit(formatting[0]))
			{
				numberFormat.SetMinimumIntegerDigits(int.Parse(formatting[0] + ""));
				if ((2 < formatting.Length) && (formatting[1] == '.') && System.Char.IsDigit(formatting[2]))
				{
					numberFormat.SetMaximumFractionDigits(int.Parse(formatting[2] + ""));
					
					
					numberFormat.format(number, outputTo, new FieldPosition(0));
					return 3;
				}
				
				
				numberFormat.format(number, outputTo, new FieldPosition(0));
				return 1;
			}
			else if ((0 < formatting.Length) && (formatting[0] == '.'))
			{
				if ((1 < formatting.Length) && System.Char.IsDigit(formatting[1]))
				{
					numberFormat.SetMaximumFractionDigits(int.Parse(formatting[1] + ""));
					
					
					numberFormat.format(number, outputTo, new FieldPosition(0));
					return 2;
				}
			}
			
			
			numberFormat.format(number, outputTo, new FieldPosition(0));
			return 1;
		}
*/
	}
}