// 
// TestBasic.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

using NUnit.Framework;
using Poi.Net.HPSF;

namespace Poi.Net.HPSF.Basic
{
	/// <summary> Tests the basic HPSF functionality.
	/// 
	/// </summary>

	[TestFixture] 
	public class TestBasic
	{
		
		internal const string POI_FS = "TestGermanWord90.doc";
		internal static readonly string[] POI_FILES = new string[]{"\x0005SummaryInformation", 
				"\x0005DocumentSummaryInformation", "WordDocument", "\x0001CompObj", "1Table"};
		internal const int BYTE_ORDER = 0xfffe;
		internal const int FORMAT = 0x0000;
		internal const int OS_VERSION = 0x00020A04;
		internal static byte[] CLASS_ID = new byte[]{(byte) 0x00, (byte) 0x00, (byte) 0x00, 
				(byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, 
				(byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00};
		internal static readonly int[] SECTION_COUNT = new int[]{1, 2};
		internal static readonly bool[] IS_SUMMARY_INFORMATION = new bool[]{true, false};
		internal static readonly bool[] IS_DOCUMENT_SUMMARY_INFORMATION = new bool[]{false, true};
		internal POIFile[] poiFiles;
		
		
		
		public TestBasic()
		{
		}		
		
		/// <summary> Read a the test file from the "data" directory.</summary>

		[SetUp]
		public void SetUp()
		{			
			FileInfo dataDir = new FileInfo("Poi.Net\\Tests\\hpsf\\data");			
			FileInfo data = new FileInfo(dataDir.FullName + "\\" + POI_FS);			
			poiFiles = Util.ReadPOIFiles(data);
		}
		
		
		
		/// <summary> Checks the names of the files in the POI filesystem. They
		/// are expected to be in a certain order.
		/// </summary>

		[Test]
		public void ReadFiles()
		{
			string[] expected = POI_FILES;
			for (int i = 0; i < expected.Length; i++)
				Assert.AreEqual(poiFiles[i].Name, expected[i]);
		}		
		
		
		/// <summary> Tests whether property sets can be created from the POI
		/// files in the POI file system. This test case expects the first
		/// file to be a  SummaryInformation}, the second file to be
		/// a  DocumentSummaryInformation} and the rest to be no
		/// property sets. In the latter cases a 
		/// NoPropertySetStreamException} will be thrown when trying to
		/// create a  PropertySet}.
		/// </summary>

		[Test]
		public void CreatePropertySets()
		{
			System.Type[] expected = new System.Type[]{typeof(SummaryInformation), typeof(DocumentSummaryInformation), 
				typeof(NoPropertySetStreamException), typeof(NoPropertySetStreamException), typeof(NoPropertySetStreamException)};
			for (int i = 0; i < expected.Length; i++)
			{
				Stream inStream = new MemoryStream(poiFiles[i].Bytes);
				object o;
				try
				{
					o = PropertySetFactory.Create(inStream);
				}
				catch (NoPropertySetStreamException ex)
				{
					o = ex;
				}
				catch (UnexpectedPropertySetTypeException ex)
				{
					o = ex;
				}
				catch (MarkUnsupportedException ex)
				{
					o = ex;
				}
				inStream.Close();
				Assert.AreEqual(o.GetType(), expected[i]);
			}
		}
		
		
		
		/// <summary> Tests the  PropertySet} methods. The test file has two
		/// property sets: the first one is a  SummaryInformation},
		/// the second one is a  DocumentSummaryInformation}.
		/// </summary>

		[Test]
		public void PropertySetMethods()
		{
			
			/* Loop over the two property sets. */
			for (int i = 0; i < 2; i++)
			{
				byte[] b = poiFiles[i].Bytes;
				PropertySet ps = PropertySetFactory.Create(new MemoryStream(b));
				Assert.AreEqual(ps.ByteOrder, BYTE_ORDER);
				Assert.AreEqual(ps.Format, FORMAT);
				Assert.AreEqual(ps.OSVersion, OS_VERSION);
				Assert.AreEqual(new string(System.Text.Encoding.Default.GetChars(ps.ClassID.Bytes)), 
					new string(System.Text.Encoding.Default.GetChars(CLASS_ID)));
				Assert.AreEqual(ps.SectionCount, SECTION_COUNT[i]);
				Assert.AreEqual(ps.SummaryInformation, IS_SUMMARY_INFORMATION[i]);
				Assert.AreEqual(ps.DocumentSummaryInformation, IS_DOCUMENT_SUMMARY_INFORMATION[i]);
			}
		}
		
		
	}
}