// 
// TestBasic.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;

using NUnit.Framework;
using Poi.Net.HPSF;

namespace Poi.Net.HPSF.Basic
{
	
	/// <summary> Test case for OLE2 files with empty properties. An empty property's type
	/// is  Variant.VT_EMPTY}.
	/// 
	/// </summary>

	[TestFixture]
	public class TestEmptyProperties
	{
		
		/// <summary> This test file's summary information stream contains some empty
		/// properties.
		/// </summary>
		internal const string POI_FS = "TestCorel.shw";
		
		
		internal static readonly string[] POI_FILES = new string[]{"PerfectOffice_MAIN", "\x0005SummaryInformation", "Main"};
		
		internal POIFile[] poiFiles;
		
		
		
		/// <summary> Constructor
		/// 
		/// </summary>
		/// <param name="name">The name of the test case
		/// </param>
		public TestEmptyProperties()
		{
		}
		
		
		
		/// <summary> Read a the test file from the "data" directory.
		/// 
		/// </summary>
		/// <exception cref=""> FileNotFoundException if the file containing the test data
		/// does not exist
		/// </exception>
		/// <exception cref=""> IOException if an I/O exception occurs
		/// </exception>

		[SetUp]
		public virtual void SetUp()
		{
			
			
			FileInfo dataDir = new FileInfo("Poi.Net\\Tests\\hpsf\\data");		
			FileInfo data = new FileInfo(dataDir.FullName + "\\" + POI_FS);
			
			poiFiles = Util.ReadPOIFiles(data);
		}
		
		
		
		/// <summary> Checks the names of the files in the POI filesystem. They
		/// are expected to be in a certain order.
		/// 
		/// </summary>
		/// <exception cref=""> IOException if an I/O exception occurs
		/// </exception>

		[Test]
		public virtual void ReadFiles()
		{
			string[] expected = POI_FILES;
			for (int i = 0; i < expected.Length; i++)
				Assert.AreEqual(poiFiles[i].Name, expected[i]);
		}
		
		
		
		/// <summary> Tests whether property sets can be created from the POI
		/// files in the POI file system. This test case expects the first
		/// file to be a  SummaryInformation}, the second file to be
		/// a  DocumentSummaryInformation} and the rest to be no
		/// property sets. In the latter cases a 
		/// NoPropertySetStreamException} will be thrown when trying to
		/// create a  PropertySet}.
		/// 
		/// </summary>
		/// <exception cref=""> IOException if an I/O exception occurs
		/// </exception>

		[Test]
		public virtual void CreatePropertySets()
		{
			System.Type[] expected = new System.Type[]{typeof(NoPropertySetStreamException), typeof(SummaryInformation), typeof(NoPropertySetStreamException)};
			for (int i = 0; i < expected.Length; i++)
			{
				Stream inStream = new MemoryStream(poiFiles[i].Bytes);
				object o;
				try
				{
					o = PropertySetFactory.Create(inStream);
				}
				catch (NoPropertySetStreamException ex)
				{
					o = ex;
				}
				catch (UnexpectedPropertySetTypeException ex)
				{
					o = ex;
				}
				catch (MarkUnsupportedException ex)
				{
					o = ex;
				}
				inStream.Close();
				Assert.AreEqual(o.GetType(), expected[i]);
			}
		}
		
		
		
		/// <summary> Tests the  PropertySet} methods. The test file has two
		/// property sets: the first one is a  SummaryInformation},
		/// the second one is a  DocumentSummaryInformation}.
		/// 
		/// </summary>
		/// <exception cref=""> IOException if an I/O exception occurs
		/// </exception>
		/// <exception cref=""> HPSFException if an HPSF operation fails
		/// </exception>

		[Test]
		public virtual void PropertySetMethods()
		{
			byte[] b = poiFiles[1].Bytes;
			PropertySet ps = PropertySetFactory.Create(new MemoryStream(b));
			SummaryInformation s = (SummaryInformation) ps;
			Assert.IsNull(s.Title);
			Assert.IsNull(s.Subject);
			Assert.IsNotNull(s.Author);
			Assert.IsNull(s.Keywords);
			Assert.IsNull(s.Comments);
			Assert.IsNotNull(s.Template);
			Assert.IsNotNull(s.LastAuthor);
			Assert.IsNotNull(s.RevNumber);
			Assert.IsNull(s.EditTime);
			Assert.IsNull(s.LastPrinted);
			Assert.IsNull(s.CreateDateTime);
			Assert.IsNull(s.LastSaveDateTime);
			Assert.AreEqual(s.PageCount, 0);
			Assert.AreEqual(s.WordCount, 0);
			Assert.AreEqual(s.CharCount, 0);
			Assert.IsNull(s.Thumbnail);
			Assert.IsNull(s.ApplicationName);
		}
		
	}
}