// 
// Util.cs
//  
// Copyright 2002-2004  Apache Software Foundation
//
// Jakarta Poi Authors:
//   Rainer Klute (klute@rainer-klute.de)
//
// Poi.Net Authors:
//   Johannes Roith(johannes at jroith dot de)
//  
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System;
using System.IO;
using System.Collections;

using Poi.Net.PoiFS.EventFileSystem;

namespace Poi.Net.HPSF.Basic
{
	
	
	
	/// <summary> Static utility methods needed by the HPSF test cases.
	/// 
	/// </summary>

	public class Util
	{
		private class AnonymousPOIFSReaderListener : POIFSReaderListener
		{
			public AnonymousPOIFSReaderListener(IList files)
			{
				InitBlock(files);
			}
			private void InitBlock(IList files)
			{
				this.files = files;
			}
			
			private IList files;
			public virtual void ProcessPOIFSReaderEvent(POIFSReaderEvent e)
			{
				try
				{
					POIFile f = new POIFile();
					f.Name = e.Name;
					f.Path = e.Path;
					Stream inStream = e.Stream;
					MemoryStream outStream = new MemoryStream();
					Util.copy(inStream, outStream);
					outStream.Close();
					f.Bytes = outStream.ToArray();
					files.Add(f);
				}
				catch (IOException ex)
				{
					//SupportClass.WriteStackTrace(ex, Console.Error);
					
					throw new System.SystemException(ex.Message);
				}
			}
		}
		
		/// <summary> Reads bytes from an input stream and writes them to an
		/// output stream until end of file is encountered.
		/// 
		/// </summary>
		/// <param name="in">the input stream to read from
		/// </param>
		/// <param name="out">the output stream to write to
		/// </param>
		public static void copy(Stream inStream, Stream outStream)
		{
			
			int BUF_SIZE = 1000;
			byte[] b = new byte[BUF_SIZE];
			int read;
			bool eof = false;
			while (!eof)
			{
				try
				{
					read = inStream.Read(b, 0, BUF_SIZE);
					if (read > 0)
						outStream.Write(b, 0, read);
					else
						eof = true;
				}
				catch (EndOfStreamException ignored)
				{
					eof = true;
				}
			}
		}
		
		
		
		/// <summary> Reads all files from a POI filesystem and returns them as an
		/// array of  POIFile} instances. This method loads all files
		/// into memory and thus does not cope well with large POI
		/// filessystems.
		/// 
		/// </summary>
		/// <param name="poiFs">The name of the POI filesystem as seen by the
		/// operating system. (This is the "filename".)
		/// 
		/// </param>
		/// <returns> The POI files. The elements are ordered in the same way
		/// as the files in the POI filesystem.
		/// </returns>
		public static POIFile[] ReadPOIFiles(FileInfo poiFs)
		{
			return ReadPOIFiles(poiFs, null);
		}
		
		
		
		/// <summary> Reads a set of files from a POI filesystem and returns them
		/// as an array of  POIFile} instances. This method loads all
		/// files into memory and thus does not cope well with large POI
		/// filessystems.
		/// 
		/// </summary>
		/// <param name="poiFs">The name of the POI filesystem as seen by the
		/// operating system. (This is the "filename".)
		/// 
		/// </param>
		/// <param name="poiFiles">The names of the POI files to be read.
		/// 
		/// </param>
		/// <returns> The POI files. The elements are ordered in the same way
		/// as the files in the POI filesystem.
		/// </returns>
		public static POIFile[] ReadPOIFiles(FileInfo poiFs, string[] poiFiles)
		{
			
			IList files = new ArrayList();
			POIFSReader r = new POIFSReader();
			POIFSReaderListener pfl = new AnonymousPOIFSReaderListener(files);
			if (poiFiles == null)
			/* Register the listener for all POI files. */
				r.RegisterListener(pfl);
			/* Register the listener for the specified POI files
			* only. */
			else
				for (int i = 0; i < poiFiles.Length; i++)
					r.RegisterListener(pfl, poiFiles[i]);
			
			/* Read the POI filesystem. */
			
			r.Read(new FileStream(poiFs.FullName, FileMode.Open, FileAccess.Read));
			POIFile[] result = new POIFile[files.Count];
			for (int i = 0; i < result.Length; i++)
				result[i] = (POIFile) files[i];
			return result;
		}
		
	}
}